package com.arsdigita.acs;

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;

/**
 * Filter for /admin directory.  Only allows logged-in users
 * who are site-wide admins to view files in the /admin directory.
 */

public class AdminFilter extends Filter {

    static {
	RequestProcessor.registerFilter("/admin", new AdminFilter());
    }

    public int filter(HttpServletRequest req, 
		      HttpServletResponse resp,
		      ServletContext ctx) throws ACSException {
	try {
	    // get the user object , but don't redirect
	    UserInfo user = Security.getVerifiedUserInfo(req, resp, false);
	    if (user != null && user.isSiteWideAdmin()) { 
		// good to go
		return FILTER_OK;
	    } else {
		resp.setStatus(resp.SC_FORBIDDEN);
		resp.setContentType("text/html");
		OutputStream out = resp.getOutputStream();
		PrintWriter pw = new PrintWriter(out);
		pw.print("<html>\n<head>\n<title>You are not an administrator</title></head>\n");
		pw.print("<body bgcolor=\"white\">\n");
		pw.print("Sorry, but you must be logged on as a site wide administrator to talk to the admin pages.");
		pw.print("<p>\nVisit <a href=\"/register/?return_url=");
		pw.print(Utilities.URLEncode(req.getRequestURI()));
		pw.print("\">/register/</a> to log in now.\n");
		pw.print("</body>\n</html>\n");
		pw.flush();
		out.flush();
		return FILTER_RETURN;
	    }
	}
	catch (IOException e) { 
	    throw new ACSException(e.getMessage());
	}
    }
}
