package com.arsdigita.acs;

import com.arsdigita.acs.db.*;
import org.apache.turbine.util.*;
import javax.servlet.http.*;
import javax.servlet.*;
import java.sql.*;

/**
 * Logs clickthroughs from this site to foreign sites
 * @author Philip Greenspun (philg@mit.edu) 
 * @creation-date 1997
 * @cvs-id $Id: Clickthrough.java,v 1.2 2000/10/25 19:39:41 jbank Exp $
 * 
 *    part of the ArsDigita Community System 
 * documented in /doc/clickthrough.tcl
 * fixed by philg@mit.edu on November 24, 1999 to protect against
 * inserting multiple rows on the first insert of the day 
 **/

public class Clickthrough extends Filter {

    static {
	RequestProcessor.registerFilter("/ct/", new Clickthrough());
    }

    /**
     * Handles a URL of the form /ct/local/URL?send_to=foreignURL.
     * usage: drop something that looks like this into your static .html
     * files 
     *
     * /ct/**local_url**?send_to=**foreign_url**
     *
     * the initial /ct is what tells the AOLserver to come to this
     * system of code.
     *
     * the rest of this system (reports) is located in /admin/click/
     * if either the local URL or the foreign URL aren't known 
     * in the database, the clickthough server creates a row
     *
     * in normal operation, the user is redirected immediately to the URL
     * specified by the send_to variable.  Then the server thread
     * requests a database connection and sends the insert to the database
     * logs the clickthrough in the clickthough 
     *
     * @return FILTER_RETURN
     *
     **/
    public int filter  (HttpServletRequest req, 
			HttpServletResponse resp,
			ServletContext ctx) 
    throws ACSException { 
	// rip off the "/ct/" from the URL 
	String url = req.getRequestURI();
	System.out.println("URL: " + url);
	url = url.substring(4);
	System.out.println("URL: " + url);
	String sendTo = req.getParameter("send_to");
	if (sendTo == null) {
	    throw new ACSException("Clickthroughs are supposed to look like <pre><code>\n/ct/**local_url**?send_to=**foreign_url**</pre></code>\nThis request looked like <pre><code>\n" + req.getRequestURI() + "?" + req.getQueryString());
	}
	// ok, we've got a valid sendTo.
	// do the redirect first
	
	// check malformed
	if (sendTo.equals("http")) {
	    throw new ACSException("You were redirected to just \"http\"; the way that we do clickthroughs here requires your browser to be a little bit liberal about how it handles URLs.  Your browser apparently isn't.  If you go back to the preceding page and View Source, you can probably tease out the target URL and go there manually with an Open command.");
	}
	
	try {
	    resp.sendRedirect(sendTo);
	} catch (java.io.IOException e) { 
	    throw new ACSException(e.getMessage());
	}
	    
	// now do the update in the database

	Database db;
	try {
	    db = new Database();
	    final String update_sql = "update clickthrough_log " + 
		"set click_count = click_count + 1 " + 
		"where local_url = :local_url " + 
		"and foreign_url = :foreign_url " +
		"and trunc(entry_date) = trunc(sysdate)";
	    int ct = db.dml(update_sql, new Object[] {url, sendTo});
	    if  (ct  == 0) {
		// there wasn't already a row there
		// let's insert one (but only one; in the very rare case
		// that another thread is executing this same code, we 
		// don't want to be left with two rows in the database for 
		// the same tuple)
		final String insert_sql =  "insert into clickthrough_log " + 
		    "( local_url, foreign_url, entry_date, click_count) " +
		    "select :local_url, :foreign_url, trunc(sysdate), 1 " + 
		    "from dual " + 
		    "where 0 = (select count(*) " + 
		    "from clickthrough_log " + 
		    "where local_url = :local_url  " + 
		    "and foreign_url = :foreign_url " + 
		    "and trunc(entry_date) = trunc(sysdate))";
		db.dml(insert_sql, new Object[] {url, sendTo, url, sendTo});
	    }
	}
	catch (SQLException sqle) { 
	    Log.note("Clickthrough insert failed: " + sqle.getMessage());
	}
	return FILTER_RETURN;
    }	    
}
