package com.arsdigita.acs;

/**
 * Interface for representing data objects that we will display in
 * templates.  Provides a generic interface for iterating over lists
 * of items and getting properties from the currently-active item.
 */

public interface DataSource  {

    /**
     * Returns true if <code>column</code> is a valid column in 
     * this datasource; returns false otherwise.
     */
    public boolean contains(String column) throws DataSourceException;

    /** 
     * Generic get-property method; returns the value of the column named
     * by <code>key</code> for the currently active 
     */
    public String get(String key) throws DataSourceException;

    /**
     * Advances to the next item, if there is one, and returns true; 
     * returns false otherwise.
     */
    public boolean next() throws DataSourceException;

    /**
     * Returns a list of valid keys in this datasource, for use with get().
     */
    public String[] keys() throws DataSourceException;

    /**
     * Returns the current row number in the datasource.
     */
    public int rownum() throws DataSourceException;
}
