package com.arsdigita.acs;

import javax.servlet.*;
import javax.servlet.http.*;

/**
 * Filter base class
 * @see com.arsdigita.acs.Handler
 * @see com.arsdigita.acs.RequestProcessor
 * @author Bill Schneider (bschneid@arsdigita.com)
 */
public abstract class Filter implements Handler {
    /**
     * Indicates that the caller should not continue processing
     * the request 
     */
    public final static int FILTER_RETURN = HANDLER_RETURN;

    /**
     * Indicates the caller should continue procesing the request
     */
    public final static int FILTER_OK = HANDLER_OK;

    /**
     * Indicates that the caller should stop procesing the request, 
     * and not process any more filters.
     */
    public final static int FILTER_BREAK = HANDLER_BREAK;

    public int handle(HttpServletRequest req, HttpServletResponse resp, ServletContext ctx) throws ServletException, java.io.IOException {
        try {
            return filter(req, resp, ctx);
        } catch (ACSException e) {
            throw new ServletException(e.getMessage());
        }
    }

    abstract public int filter(HttpServletRequest req, HttpServletResponse resp, ServletContext ctx) throws ACSException;
}
