// $Id: Handler.java,v 1.1 2000/10/26 14:27:01 jbank Exp $ 

package com.arsdigita.acs;

import javax.servlet.*;
import javax.servlet.http.*;

/**
 * 
 * Interface for a generic handler, to test incoming requests and
 * screen them out or pass them through as appropriate.
 * <p>
 * Defines a single method, 
 * <code>handle(HttpServletRequest, HttpServletResponse, ServletContext)</code>.
 * Implementing classes will check parameters or cookies in the request
 * and may either write to the response output stream directly, or 
 * pass the response object on to another request dispatcher.
 * 
 * @author Bill Schneider (bschneid@arsdigita.com)
 * @see com.arsdigita.acs.RequestProcessor
 * @see com.arsdigita.acs.Filter
 * 
 */
public interface Handler {

    // constants 
    /**
     * Indicates that the caller should not continue processing
     * the request 
     */
    public final static int HANDLER_RETURN = 1;

    /**
     * Indicates the caller should continue procesing the request
     */
    public final static int HANDLER_OK = 2;

    /**
     * Indicates that the caller should stop procesing the request, 
     * and not process any more handlers.
     */
    public final static int HANDLER_BREAK = 3;


    /** The request handler interface.  
     * @param req The servlet request
     * @param resp The servlet response
     * @param ctx The servlet context
     * @return HANDLER_RETURN if the caller should not continue processing 
     * the request after the handler is called; HANDLER_OK if the
     * caller should continue procesing the request; HANDLER_BREAK if the
     * caller should not process any more
     * handlers.
     */
    public int handle(HttpServletRequest req, HttpServletResponse resp, ServletContext ctx) throws ServletException, java.io.IOException;
}


