// $Id: RobotFilter.java,v 1.1 2000/10/31 09:39:40 deison Exp $

package com.arsdigita.acs;

import javax.servlet.*;
import javax.servlet.http.*;

import java.io.*;
import java.util.*;

import com.arsdigita.acs.db.*;
import org.apache.turbine.util.Log;
import com.oroinc.text.perl.Perl5Util;
import java.sql.*;


/**
 * This filter is used to detect robots and redirect
 * them to a different part of the site.
 */
public class RobotFilter extends Filter {
    private static Hashtable robotUserAgentCache = null;
    static {
        // We don't build the cache yet since we want the rest of the system to come online first...
        // it will be build by the scheduled procedure or by the isRobot method if needed sooner.

        // Check to see if the robots table needs to be updated
        // when the server starts (5 seconds after to be precise).
	Scheduler s = Scheduler.getInstance();
	ScheduledProcedure sp = new ScheduledProcedure() {
	    public void run() {
                try {
                    updateRobotList();
                } catch (SQLException e) {
                    Log.error("Error updating robot list: " + e.toString());
                }
	    }
	};
	s.scheduleOnce(sp, 5);

        // Install ad_robot_filter for all specified patterns
        Vector filterPatterns = Parameters.getVector("FilterPattern", "robot-detection");
        Enumeration e = filterPatterns.elements();
        RobotFilter rf = new RobotFilter();
        while (e.hasMoreElements()) {
            String pattern = (String)e.nextElement();
            Log.note("Installing robot filter for " + pattern);
            RequestProcessor.registerFilter(pattern, rf);
        }
    }

    /**
     * Replicates data from the Web Robots Database (http://info.webcrawler.com/mak/projects/robots/active.html) into a table in the ACS database. 
     * The data is published on the Web as a flat file, whose format is specified in http://info.webcrawler.com/mak/projects/robots/active/schema.txt. 
     * Basically, each non-blank line of the database corresponds to one field (name-value pair) of a record that defines the characteristics of a 
     * registered robot. Each record has a "robot-id" field as a unique identifier. (There are many fields in the schema, but, for now, the only ones 
     * we care about are: robot-id, robot-name, robot-details-url, and robot-useragent.)\n<p>Returns the number of rows replicated. 
     *
     * @return Number of robots processed.
     */
    public static int replicateWebRobotsDB () throws SQLException, ACSException {
        int robotCount = 0;
        String webRobotsDbURL = Parameters.get("WebRobotsDB", "robot-detection");
        if (!Utilities.emptyString(webRobotsDbURL)) {
            Perl5Util re = new Perl5Util();
            String result = null;
            try {
                result = Utilities.getFromURL(webRobotsDbURL);
            } catch (java.io.IOException e) {
                Log.error("RobotFilter: error replicating web robots DB: " + e.toString());
                return 0;
            }
            Vector page = re.split("/" + System.getProperty("line.separator") + "/", result);
            AdSet robotInfo = new AdSet();
            Database db = new Database();
            String priorRobotId = null;
            for (int linenum = 0; linenum < page.size(); linenum++) {
                String line = (String)page.elementAt(linenum);
                // A "robot-id" line delimits a new record, so each
                // time we encounter one, we need to write the prior
                // record (if there is one) into the database. There
                // is only one case in which there will *not* be a prior
                // record, i.e., for the very first record.
                if (re.match("/robot-id: *(.+)/", line)) {
                    String robotId = re.group(1).trim();
                    priorRobotId = robotInfo.get("robot_id");
                    if (!Utilities.emptyString(priorRobotId)) {
                        // As long as there is an actual value for
                        // "robot_useragent", load the record, i.e.,
                        // update it if a record with the same
                        // robot_id already exists or insert it if
                        // one does not.  (There's no point in keeping
                        // info about robots that we can't identify.)
                        String robotUserAgent = robotInfo.get("robot_useragent");
                        String robotName = robotInfo.get("robot_name");
                        String robotDetailsURL = robotInfo.get("robot_details_url");
                        if (!Utilities.emptyString(robotUserAgent)) {
                            if (isRobotInDatabase(priorRobotId)) {
                                Log.note("Updating existing robot: " + priorRobotId);
                                int num = db.dml("update robots " +
                                       "    set robot_name = :robot_name,  " +
                                       "        robot_details_url = :robot_details_url,  " +
                                       "        robot_useragent = :robot_useragent, " +
                                       "        modified_date = sysdate  " +
                                       "    where robot_id = :prior_robot_id", new Object[] {robotName, robotDetailsURL, robotUserAgent, priorRobotId});
                                if (num == 0) {
                                    Log.warn("Failed updating " + robotName + ", row did not exist");
                                }
                            } else {
                                Log.note("Inserting new robot: " + priorRobotId);
                                db.dml("insert into robots " +
                                       "    (robot_id, robot_name, robot_details_url, robot_useragent, insertion_date)  " +
                                       "    values " +
                                       "    (:prior_robot_id, :robot_name, :robot_details_url, :robot_useragent, sysdate)", new Object[] {priorRobotId, robotName, robotDetailsURL, robotUserAgent});
                            }
                            robotCount++;
                        }
                        // Clear out the record so we can start over.
                        robotInfo.clear();
                    }
                    robotInfo.put("robot_id", robotId);
                }
                if (re.match("/robot-name: *(.+)/", line)) {
                    robotInfo.put("robot_name", re.group(1).trim());
                }
                if (re.match("/robot-details-url: *(.+)/", line)) {
                    robotInfo.put("robot_details_url", re.group(1).trim());
                }
                if (re.match("/robot-useragent: *(.+)/", line)) {
                    robotInfo.put("robot_useragent", re.group(1).trim());
                }
            }
            // Don't forget the last record
            priorRobotId = robotInfo.get("robot_id");
            String robotUserAgent = robotInfo.get("robot_useragent");
            String robotName = robotInfo.get("robot_name");
            String robotDetailsURL = robotInfo.get("robot_details_url");
            if (!Utilities.emptyString(robotUserAgent)) {
                if (isRobotInDatabase(priorRobotId)) {
                    Log.note("Updating existing robot: " + priorRobotId);
                    db.dml("update robots " +
                           "    set robot_name = :robot_name,  " +
                           "        robot_details_url = :robot_details_url,  " +
                           "        robot_useragent = :robot_useragent, " +
                           "        modified_date = sysdate  " +
                           "    where robot_id = :prior_robot_id", new Object[] {robotName, robotDetailsURL, robotUserAgent, priorRobotId});
                } else {
                    Log.note("Inserting new robot: " + priorRobotId);
                    db.dml("insert into robots " +
                           "    (robot_id, robot_name, robot_details_url, robot_useragent, insertion_date)  " +
                           "    values " +
                           "    (:prior_robot_id, :robot_name, :robot_details_url, :robot_useragent, sysdate)", new Object[] {priorRobotId, robotName, robotDetailsURL, robotUserAgent});
                }
                robotCount++;
            }
        }
        if (robotCount > 0) {
            buildRobotUserAgentsCache();
        }
        Log.note("Processed " + robotCount + " robots");
        return robotCount;
    }

    /**
     * Returns true if an entry exists in the robots table with the specified robot ID.
     * @param robotId.
     * @return true if robot entry exists.
     */
    public static boolean isRobotInDatabase (String robotId) throws SQLException, ACSException {
        Database db = new Database();
        return !(db.queryToString("select count(*) from robots where robot_id = :robot_id", new Object[] {robotId}).equals("0"));
    }

    /**
     * Caches User-Agent values for known robots.
     */
    private static void buildRobotUserAgentsCache() throws SQLException {
        Database db= new Database();
        ResultSet rs = db.query("select robot_useragent from robots");
        Hashtable newRobotUserAgentCache = new Hashtable();
        while (rs.next()) {
            newRobotUserAgentCache.put(rs.getString("robot_useragent"), "1");
        }
        robotUserAgentCache = newRobotUserAgentCache;
    }

    /**
     * Returns true if the useragent is recognized as a search engine.
     * (checks cache).
     */
    public static boolean isRobot (String useragent) {
        if (robotUserAgentCache == null) {
            try {
                buildRobotUserAgentsCache();
            } catch (SQLException e) {
                Log.error("Error building robot user agents cache: " + e.toString());
            }
        }
        // Using a simple containsKey does not seem to be the right thing to do,
        // based on looking at the data.  However, this matches what the TCL module
        // does.  I'm not sure why it seems to be wrong.  -DEE 10/31/00
        return (robotUserAgentCache != null && robotUserAgentCache.containsKey(useragent));
    }

    /**
     * Will update the robots table if it is empty or if the number of days since it was last updated is greater than the number of days specified by the RefreshIntervalDays configuration parameter in the "robot-detection" section.
     */ 
    public static void updateRobotList() throws SQLException {
        Database db = new Database();
        boolean aborted = false;
        db.beginTransaction();
        try {
            String robotCount = db.queryToString("select count(*) from robots");
            if (robotCount.equals("0")) {
                Log.note("Replicating Web Robots DB, because robots table is empty");
                replicateWebRobotsDB();
            } else {
                int refreshInterval;
                try {
                    refreshInterval = Parameters.getInt("RefreshIntervalDays", "robot-detection", 30);
                } catch (ClassCastException e) {
                    Log.error("Error getting RefreshIntervalDays parameter from config file.  Perhaps type=int is missing from parameter declaration?  Assuming 30.  Error was: " + e.toString());
                    refreshInterval = 30;
                }
                String daysOld = db.queryToString("select trunc(sysdate - max(nvl(modified_date, insertion_date))) as n_days from robots");
                if (Integer.parseInt(daysOld) > refreshInterval) {
                    Log.note("Replicating Web Robots DB, because data in robots table has expired.");
                    replicateWebRobotsDB();
                } else {
                    Log.note("Not replicating Web Robots DB at this time, because data in the robots table has not expired.");
                }
            }
        } catch (Exception e) {
            Log.error ("Error encountered in updateRobotList: " + e.toString());
            db.abortTransaction();
            aborted = true;
        }
        if (!aborted) {
            db.endTransaction();
        }
    }

    /**
     * A filter to redirect any recognized robot to a specified page.
     */
    public int filter(HttpServletRequest req, 
		      HttpServletResponse resp,
		      ServletContext ctx) throws ACSException {
        int result;
        String userAgent = req.getHeader("User-Agent");
        if (isRobot(userAgent)) {
            String robotRedirectURL = Parameters.get("RedirectURL", "robot-detection");
            // Be sure to avoid an infinite loop of redirects (Actually, browsers
            // won't look infinitely; rather, they abort after a URL redirects
            // too many times.)
            // request_url is an attribute that is set at the beginning of the
            // request processor's service method.
            if (!robotRedirectURL.startsWith((String)req.getAttribute("request_url"))) {
                // Requested URL does not start with robot redirect URL (usually a dir)
                Log.note("Robot being bounced by RobotFilter: User-Agent = " + userAgent);
                try {
                    Utilities.sendRedirect(resp, robotRedirectURL);
                } catch (java.io.IOException e) {
                    throw new ACSException (e.getMessage());
                }
                result = FILTER_RETURN;
            } else {
                // We've got a robot, but he is happily in robot heaven.
                result = FILTER_OK;
            }
        } else {
            result = FILTER_OK;
        }
        return result;
    }
}
