package com.arsdigita.acs;

import java.sql.*;
import java.util.*;
import java.io.*;
import com.arsdigita.acs.db.*;

/**
 * Represents a single generic object pulled in from the database.  
 * 
 * @author Bill Schneider (bschneid@arsdigita.com)
 * @version $Version$
 */

public class SingleSelection extends Selection {

    Hashtable props = new Hashtable();

    /** 
     * Generates a single-row selection from the current row in the
     * result set.  Requires that next() has been called at least once
     * on rs.
     */
    public SingleSelection(ResultSet rs) throws SQLException, IOException { 
	super(rs);
	props = super.freeze();
    }

    /**
     * Creates a single-row selection from the currently active row
     * in <code>s</code>.  The returned SingleSelection is "frozen" with
     * the currently-active values in <code>s</code>.
     */

    public SingleSelection(Selection s) throws SQLException, IOException {
	props = s.freeze();
    }

    public String get(String key) throws DataSourceException { 
	Object o = props.get(key);
	if (o != null) 
	    return o.toString();
	else 
	    throw new InvalidKeyException("DataSource does not countain field '" + key + "'");
    }

    public boolean contains(String key) throws DataSourceException {
	return (props.get(key) != null) ;
    }

    public String[] keys () {
	Vector v = new Vector();
	Enumeration e = props.keys(); 
	while (e.hasMoreElements()) {
	    v.addElement((String)e.nextElement());
	}
	String[] retval = new String[v.size()];
	v.copyInto(retval);
	return retval;
    }

    /**
     * Since we are represening a single database row, we never want
     * to (and can't) increment the data source.  Always returns false.
     * @return Always false 
     */
    public boolean next() { 
	return false;
    }

    public int rownum() { 
	return 1;
    }

    /**
     * Returns the contents of a BLOB column in a byte array.
     */
    public byte[] getBLOB(String key, OutputStream os) 
	throws DataSourceException { 
	byte[] blob = null;
	try {
	    blob = (byte[]) props.get(key);
	    if (blob == null) 
		throw new DataSourceException(key + " not found");
	} catch (Exception e) { 
	    throw new DataSourceException(e.getMessage());
	}
	return blob;
    }


    /**
     * Dumps the contents of a BLOB column to an OutputStream.
     * @param key The column name
     * @param os The output stream
     */
    public void dumpBLOB(String key, OutputStream os) 
	throws DataSourceException { 
	try {
	    byte[] blob = (byte[]) props.get(key);
	    if (blob == null) 
		throw new DataSourceException(key + " not found");
	    os.write(blob, 0, blob.length);
	} catch (Exception e) { 
	    throw new DataSourceException(e.getMessage());
	}
    }
}
