package com.arsdigita.acs;

import org.w3c.dom.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import java.io.*;
import java.util.Vector;

public class TldDocGen { 

    /**
     * Simple Java class to convert a JSP tag library descriptor 
     * (TLD) into a human-readable HTML file.
     * 
     * Takes two command line arguments: a path to the TLD file to 
     * process, and the tag library prefix (the part of the tag name
     * that comes before the colon when the tag is used in a JSP page).
     */

    public static String tldPrefix = "ad";

    public static void main(String[] args) throws Exception { 

	if (args.length == 0) {
	    System.err.println("Usage: java com.arsdigita.acs.TldDocGen /path/to/tld-file.tld [tld-prefix]");
	    return;
	}

	if (args.length == 2) { 
	    tldPrefix = args[1];
	}

	Parser saxParser = ParserFactory.makeParser();
	com.sun.xml.tree.XmlDocumentBuilder db = new com.sun.xml.tree.XmlDocumentBuilder();
	saxParser.setDocumentHandler(db);
	java.io.File f = new java.io.File(args[0]);
	saxParser.parse("file:" + f.getAbsolutePath());
	
	Document d = db.getDocument();

	System.out.println(renderHtmlHeader());

	NodeList tagList = d.getElementsByTagName("tag");
	for (int i = 0; i < tagList.getLength(); i++) { 
	    // grab this tag
	    Node thisTag = tagList.item(i);
	    // go through children
	    NodeList nl = thisTag.getChildNodes();
	    // loop over children -- (name, class, info, attributes)

	    String name = null; 
	    String klass = null;
	    String eiKlass = null;
	    String info = null;
	    Vector attrNames = new Vector();
	    Vector attrRequired = new Vector();

	    for (int j = 0; j < nl.getLength(); j++) { 
		Node thisValue = nl.item(j);
		if (thisValue.getNodeName().equals("name")) { 
		    // this kind of sucks--but the CDATA is actually 
		    // a separate node, and we can't do this for
		    // the attributes tag.. ugh
		    name = thisValue.getFirstChild().getNodeValue();
		} else if (thisValue.getNodeName().equals("tagclass")) { 
		    klass = thisValue.getFirstChild().getNodeValue();
		} else if (thisValue.getNodeName().equals("teiclass")) { 
		    eiKlass = thisValue.getFirstChild().getNodeValue();
		} else if (thisValue.getNodeName().equals("info")) { 
		    info = thisValue.getFirstChild().getNodeValue();
		} else if (thisValue.getNodeName().equals("attribute")) { 
		    // loop over attribute subtags
		    // damn this is confusing because of the 
		    // overloading of the word "attribute"
		    NodeList thisAttrAttrs = thisValue.getChildNodes();
		    Boolean required = Boolean.FALSE;
		    String attrName = null;
		    for (int l = 0; l < thisAttrAttrs.getLength(); l++) {
			Node thisAttr = thisAttrAttrs.item(l);
			if (thisAttr.getNodeName().equals("name")) { 
			    attrName = thisAttr.getFirstChild().getNodeValue();
			} else if (thisAttr.getNodeName().equals("required")) { 
			    required = new Boolean(thisAttr.getFirstChild().getNodeValue());
			}
		    }
		    attrNames.addElement(attrName);
		    attrRequired.addElement(required);
		}
	    }

	    // now we have all the properties for a tag
	    // render it as HTML
	    System.out.println(renderHtmlForTag (name, 
						 klass, 
						 eiKlass, 
						 info,
						 attrNames,
						 attrRequired));

	}
	System.out.println(renderHtmlFooter());
    }

    public static String renderHtmlHeader() { 
	return "<html><head><title>Tag Library Definition</title></head>" 
	    + "<body bgcolor=\"white\">\n<h2>Tag Library Definition</h2>\n<hr>";
    }

    public static String renderHtmlFooter() { 
	return "</body></html>";
    }

    /**
     * Renders HTML for a single TLD tag definition.
     */
    public static String renderHtmlForTag ( String name,
					    String klass,
					    String eiKlass,
					    String info,
					    Vector attrNames,
					    Vector attrRequired ) {
	StringBuffer val = new StringBuffer();
	val.append("<h3>" + tldPrefix + ":" + name + "</h3>");
	for (int i = 0; i < attrNames.size(); i++) {
	    if (i == 0) 
		val.append("\nAttributes: \n<ul>\n");
	    val.append("<li>" + attrNames.elementAt(i));
	    if (!((Boolean)attrRequired.elementAt(i)).booleanValue()) { 
		val.append(" [optional]");
	    }
	    val.append("\n");
	}
	val.append("\n</ul>\n<blockquote>\n" + info + "\n</blockquote>\n");
	return val.toString();
    }
}

