package com.arsdigita.acs;

import java.util.Vector;

/**
 * datasource wrapper for a Vector of SingleSelections.  Useful when we have
 * a query from a database that we might want to loop over several times. 
 * This also allows us to use Vectors as sources of ad:loop.
 */

public class VectorDataSource implements DataSource { 

    /** counter */
    private int i = 0;

    /** current pointer */
    private DataSource whereWeAt = null;

    private Vector v;

    /** 
     * creates a new VectorDataSource from a Vector of DataSources.
     */
    public VectorDataSource (Vector v) { 
	this.v = v;
    }

    public boolean next() { 
	if (i >= v.size()) 
	    return false; 
	whereWeAt = (DataSource)v.elementAt(i++);
	return true;
    }
    
    public String get(String key) throws DataSourceException { 
	return whereWeAt.get(key);
    }

    public boolean contains(String key) throws DataSourceException { 
	return whereWeAt.contains(key);
    }

    public String[] keys() throws DataSourceException { 
	return whereWeAt.keys();
    }

    public int rownum() { 
	return i + 1;
    }

    /** 
     * rewinds this datasource to the beginning so we can loop over it 
     * again.
     */
    public void reset() {
	i = 0;
	whereWeAt = null;
    }
}

