package com.arsdigita.acs.bboard;

import com.arsdigita.acs.*;
import com.arsdigita.acs.db.*;
import java.sql.*;
import java.util.Vector;

/**
 * Utility classes and procedural abstractions for bboard.
 * 
 * @author Bill Schneider (bschneid@arsdigita.com)
 * @version $Version$
 */

public class BBoard {

    public static final String getSystemName() { 
	return Parameters.get("SystemName", "bboard", "Discussion Forums");
    }

    public static final String getSystemOwner() { 
	return Parameters.get("SystemOwner", "bboard", "Discussion Forums");
    }

    /** 
     * Increments a base 62 number using 0-9, A-Z, a-z as digits 
     * (A=10, a=36).  
     * Probably not the most efficient algorithm.
     */ 
    public final static String incrBase62(String input) {
	char new_id[] = new char[input.length()];
	boolean carry = false;
	boolean lastCarry = false;
	for (int i = new_id.length - 1; i >= 0; i--) {
	    char c = input.charAt(i);
	    if (i == new_id.length - 1) {
		// first time through?
		if (c == '9')
		    c = 'A';
		else if (c == 'Z') 
		    c = 'a';
		else if (c == 'z') {
		    c = '0';
		    carry = true;
		} else 
		    c++;
	    }
	    if (lastCarry) {
		if (c == '9')
		    c = 'A';
		else if (c == 'Z') 
		    c = 'a';
		else if (c == 'z') {
		    c = '0';
		    carry = true;
		} else c++;
	    } 
	    lastCarry = carry;
	    carry = false;
	    new_id[i] = c;
	}
	return new String(new_id);
    }

    /**
     * gets a message id for a new bboard message, using the msg_id_generator
     * table.
     */
    public static final String getNextId(Database db) {
	// this will grab the exclusive lock on the msg_id_generator table
	// that will keep another copy of this same script from doing anything
	// more than waiting here.
	try {
	    String last_id = db.queryToString("select last_msg_id from msg_id_generator for update of last_msg_id");
	    
	    String next_id = incrBase62(last_id);
	    
	    db.dml("update msg_id_generator set last_msg_id = ?", 
		   new Object[] {next_id});
	    return next_id;
	}
	catch(Exception e) {
	    return null;
	}
    }

    /**
     * Returns a pattern for use in a LIKE expression, when searching
     * for all direct replies to a bboard message.
     */
    public final static String newSortKeyForm(String old_sort_key) {
	if (old_sort_key.indexOf(".") == -1) {
	    // no period found, so old sort key is just "00z3A7", a msg_id
	    //  form for next level is "<msg_id>.<char><char>"
	    return old_sort_key+ ".__";
	} else {
	    // period found, so old sort key is of form "317.CCDDKK"
	    return old_sort_key + "__";
	}
    }

    /**
     * Returns a pattern for use in a LIKE expression, when searching
     * for all direct OR INDIRECT dependent messages to a bboard message.
     */
    public final static String dependentSortKeyForm (String old_sort_key) {
	if (old_sort_key.indexOf(".") == -1) {
	    // no period found, so old sort key is just "00z3A7", a msg_id
	    // form for dependents is "<msg_id>.<zero or more chars>"
	    return old_sort_key + ".%";
	} else {
	    // period found, so old sort key is of form "317.CCDDKK"
	    // we demand at least two chars after the old key ("__")
	    // plus zero or more ("%")
	    return old_sort_key + "__%";
	}
    }
    
    /**
     * Generate a sort key for a new bboard message.
     */
    public final static String newSortKey(String refers_to_key, String last_key) {
	if (Utilities.emptyString(last_key)) {
	    // this is the first message that refers to the previous one, so we
	    // just add ".00" or "00"
	    String new_key = "";
	    if (refers_to_key.indexOf(".") == -1) {
		// no period found, so refers_to_key is just "00007Z", a msg_id
		// (i.e., the thing we're referring to is top level)
		new_key = refers_to_key + ".00";
	    } else {
		// period found, so last_key is of form "00007Z.CCDDKK"
		new_key = refers_to_key + "00";
	    }
	    return new_key;
	} else {
	    //we're not the first response to $refers_to
	    // last key cannot be just a msg id, but must have two chars
	    // at the end (00 through zz)
	    String frontPart = last_key.substring(0, last_key.length()-2);
	    String lastTwo = last_key.substring(last_key.length()-3);
	    return frontPart + incrBase62(lastTwo);
	}
    }

    public static String urlStub() {
	String s = Parameters.get("URL", null);
	s += Parameters.get("ModuleName", "bboard");
	if (!s.endsWith("/")) 
	    s += "/";
	return s;
    }
    
    public static final String[] getModerationPolicyList() {
	return new String[] 
	    { "featured", "moderated", "unmoderated", "private" };
    }

    public static final String getDefaultModerationPolicy() {
	return "featured";
    }
}

