package com.arsdigita.acs.bboard;

import java.util.*;
import com.arsdigita.acs.*;

import org.apache.turbine.util.mail.*;

/**
 * Maintains the list of people about to receive e-mail alerts for a
 * particular bboard posting.  
 */

public final class EmailQueue implements DataSource {

    /** Types of alerts */
    public final static int ALERT_MAINTAINER = 1;
    public final static int ALERT_THREAD = 2;
    public final static int ALERT_INSTANT = 3;
    public final static int ALERT_UNKNOWN = -1;

    // a vector of hashtables
    private Vector v = new Vector();

    // for iteration as a datasource
    private int i = -1;

    public boolean next() { 
	if (++i < v.size())
	    return true;
	else return false;
    }

    public String get(String key) { 
	return (String)((Hashtable)v.elementAt(i)).get(key);
    }

    public int getType() { 
	Integer type = (Integer)((Hashtable)v.elementAt(i)).get("type");
	if (type != null) 
	    return type.intValue();
	else return ALERT_UNKNOWN;
    }

    public boolean contains(String key) {
	return ((Hashtable)v.elementAt(i)).get(key) != null;
    }

    public String[] keys () { 
	Vector v2 = new Vector();
	Hashtable ht = (Hashtable)v.elementAt(i);
	Enumeration e = ht.keys();
	while (e.hasMoreElements()) { 
	    v2.addElement(e.nextElement());
	}
	String[] s = new String[v.size()];
	v2.copyInto(s);
	return s;
    }

    /**    
     * Adds a recipient to this.
     * @param to The e-mail recipient's e-mail address
     * @param from The sender's e-mail address (e.g., bboard@arsdigita.com)
     * @param subject The e-mail subject 
     * @param body 
     * @param poster_email The e-mail address of the bboard msg poster
     */
    public final void add(String to, 
			  String from,
			  String subject,
			  String body,
			  String poster_email) {
	add(ALERT_UNKNOWN, to, from, subject, body, poster_email);
    }

    /**    
     * Adds a recipient to this.
     * @param type The type of email alert
     * @param to The e-mail recipient's e-mail address
     * @param from The sender's e-mail address (e.g., bboard@arsdigita.com)
     * @param subject The e-mail subject 
     * @param body 
     * @param poster_email The e-mail address of the bboard msg poster
     */
    public final void add(int type, 
			  String to, 
			  String from,
			  String subject,
			  String body,
			  String poster_email) {
	// presumably the guy (m/f) doesn't want email about his own posting
	if (to.equals(poster_email)) 
	    return;
	for (int i = 0; i < v.size(); i++) {
	    if (to.equals(((Hashtable)v.elementAt(i)).get("to")))
		// email matched
		return;
	}

	Hashtable ht = new Hashtable();
	ht.put("type", new Integer(type));
	ht.put("to", to);
	ht.put("from", from);
	ht.put("subject", subject);
	ht.put("body", body);
	ht.put("poster_email", poster_email);
	v.addElement(ht);
    }

    /**
     * Send e-mail messages for all queued alerts.
     */
    public void send() throws javax.mail.MessagingException  { 
	for (int i = 0; i<v.size(); i++) { 
	    Hashtable ht = (Hashtable)v.elementAt(i);
	    Email e = new SimpleEmail();
	    e.setFrom((String)ht.get("from"), null);
	    e.addTo((String)ht.get("to"), null);
	    e.setSubject((String)ht.get("subject"));
	    e.setMsg((String)ht.get("body"));
	    e.send();
	}
    }

    public int rownum () { 
	return i + 1;
    }
}  

