package com.arsdigita.acs.bboard;

import com.arsdigita.acs.*;
import com.arsdigita.acs.db.*;
import java.sql.*;

/**
 * Extension of the Selection class specific to bboard messages.  
 * Declares some properties explicitly, to take some code out of JSP 
 * pages; these get methods may be called implicitly by introspection
 * from the ad:sub tag.
 */
 
public class MessagesDataSource extends Selection {

    private String lastCat = "ain't got no category";

    /**
     * Create a new MessagesDataSource out of a SqlStatement.
     */
    public MessagesDataSource(SqlStatement sql, Database db) throws SQLException {
	super(sql, db);
    }

    /**
     * Create a new MessagesDataSource out of a ResultSet.
     */
    public MessagesDataSource(ResultSet rs) throws SQLException {
	super(rs);
    }

    /**
     * gets the message text for display in the browser.
     * if the message is plain text, escape the special HTML characters
     * and insert line breaks so it looks right to the user.
     */
    public String getMessage() throws DataSourceException {
	if ("t".equals(get("html_p"))) 
	    return get("message");
	else return Utilities.textToHtml(get("message"));
    }

    /**
     * gets the associated filename, with all non-standard chars
     * replaced with underscores.
     */
    public String getPrettyFilename() throws DataSourceException {
	com.oroinc.text.perl.Perl5Util re = new com.oroinc.text.perl.Perl5Util();
	return re.substitute("s/[^-_.0-9a-zA-Z]+/_/g", get("client_filename"));
    }


    /**
     * Gets the category, suitable for display (null == "Uncategorized") 
     * if and only if it is different than the category for the previous
     * message (useful in ad:loop)
     */
    public String getCategory() throws DataSourceException { 
	String thisCat = get("category");
	if (thisCat.equals(lastCat)) 
	    return null;
	lastCat = thisCat;
	if (Utilities.emptyString(thisCat)) {
	    thisCat = "Uncategorized";
	} 
	return thisCat;
    }

    /**
     * Gets the interest level.  Returns the string "NULL" if no 
     * interest level is supplied for this message.
     */
    public final String getInterestLevel() throws DataSourceException { 
	String lev = get("interest_level");
	if (Utilities.emptyString(lev))
	    return "NULL";
	else return lev;
    }
}
