package com.arsdigita.acs.bboard;

import com.arsdigita.acs.*;
import com.arsdigita.acs.db.*;
import java.sql.*;
import java.util.*;

/** 
 * Just like a MessagesDataSource, but ensures the <code>next</code>
 * method exactly once; overrides <code>next</code> to always return
 * false.
 */
public class SingleMessageDataSource extends MessagesDataSource {

    java.util.Hashtable props;

    public SingleMessageDataSource(SqlStatement sql, Database db) throws SQLException {
	super(sql, db);
	try {
	    super.next();
	    props = freeze();
	} catch (DataSourceException e) { 
	    throw new SQLException("SingleMessageDataSource: query did not return any rows: " + e.getMessage());
	} catch (java.io.IOException e) { 
	    throw new SQLException("SingleMessageDataSource: IOException found: " + e.getMessage());
	}
    }

    public SingleMessageDataSource(ResultSet rs) throws SQLException, java.io.IOException {
	super(rs);
	props = freeze();
    }

    public String get(String key) throws DataSourceException {
	Object o = props.get(key);
	if (o == null)
	    throw new InvalidKeyException("Message does not contain field '" + key + "'");
	else return(o.toString());
    }

    public void set(String key, Object value) {
	if (value != null) {
	    props.put(key, value);
	} else {
	    props.put(key, "");
	}
    }

    public String[] keys() {
	Vector v = new Vector();
	Enumeration enum = props.keys();
	while (enum.hasMoreElements()) {
	    v.addElement(enum.nextElement());
	}
	String[] retval = new String[v.size()];
	v.copyInto(retval);
	return retval;
    }

    /**
     * Always returns false.
     */
    public boolean next() {
	return false;
    }
}
