package com.arsdigita.acs.bboard;

import com.arsdigita.acs.*;
import com.arsdigita.acs.db.*;
import java.util.*;
import java.sql.*;

/**
 * Class to represent a single bboard topic. Has a special constructor for
 * querying the database, pulling in all the available attributes from 
 * the bboard_topics table.
 */

public class SingleTopicDataSource extends TopicsDataSource {
    private Hashtable props;

    public SingleTopicDataSource(ResultSet rs) throws SQLException, java.io.IOException { 
	super(rs);
	props = this.freeze();
    }

    /** 
     * Initialize this from the database; gets all properties from 
     * bboard_topics where topic_id = <code>topicId</code>.
     */
    public SingleTopicDataSource(Database db, Integer topicId) throws SQLException, java.io.IOException, PageValidationException {
	final String query = "select bboard_topics.*, " + 
	    " u.email as maintainer_email, " + 
	    " u.first_names || ' ' || u.last_name as maintainer_name " +
	    " from users u, bboard_topics " + 
	    " where u.user_id = primary_maintainer_id " + 
	    " and topic_id = ?";
	rs = db.query(query, new Object[] {topicId});
	rsmd = rs.getMetaData();
	if (!rs.next()) {
	    throw new PageValidationException("This topic does not exist.");
	}
	props = this.freeze();
	rs.close();
    }

    /**
     * Returns the Topic datasource object for a given message; pulls in
     * the topic to which that message belongs.
     */
    public SingleTopicDataSource(Database db, String messageId) throws SQLException, java.io.IOException, PageValidationException {
	final String query = "select bboard_topics.*, " + 
	    " u.email as maintainer_email, " + 
	    " u.first_names || ' ' || u.last_name as maintainer_name " +
	    " from users u, bboard_topics, bboard " + 
	    " where u.user_id = primary_maintainer_id " + 
	    " and bboard.topic_id = bboard_topics.topic_id " + 
	    " and msg_id = ?";
	rs = db.query(query, new Object[] {messageId});
	rsmd = rs.getMetaData();
	if (!rs.next()) {
	    throw new PageValidationException("This message does not exist.");
	}
	props = this.freeze();
	rs.close();
    }

    public boolean contains(String key) throws DataSourceException { 
	return(props.containsKey(key));
    }

    public String get(String key) throws DataSourceException {
	if (!props.containsKey(key)) 
	    throw new InvalidKeyException("Topic does not contain field '" + key + "'");
	else {
	    Object o = props.get(key);
	    String s = o.toString();
	    return s;
	}
    }

    public String[] keys() {
	Vector v = new Vector();
	Enumeration e = props.keys();
	while (e.hasMoreElements()) { 
	    v.addElement((String)e.nextElement());
	}
	String[] s = new String[v.size()];
	v.copyInto(s);
	return s;
    }
}
