package com.arsdigita.acs.calendar;

import java.sql.*;
import java.util.*;
import com.oroinc.text.perl.Perl5Util;
import com.arsdigita.acs.*;
import com.arsdigita.acs.db.*;
import javax.servlet.jsp.PageContext;
import com.arsdigita.acs.fs.FileStorageProcs;
import com.arsdigita.acs.portals.PortalsProcs;
import org.apache.turbine.util.Log;

/**
 * This is the first pass at the Java version of the Calendar package.
 * Right now, this is pretty much a straight port.  Near term future plans:
 * <ol>
 * <li>Finish up the few areas that are not finished (some groups/portals stuff, repeating events stuff, attachments)
 * <li>Break the index.jsp page up into several littler pages that include each other as needed
 * <li>Move the hairier SQL into views and PL/SQL
 * <li>Build good Java classes to provide a simple interface for adding items, loading items, etc via Java.
 * </ol>
 * In short, the data model should stay the same.  Everything else will be moved around, and some parts will
 * be totally gutted.
 *
 * Diffs from TCL API:
 * <ul>
 * <li>Terribly named proc my_ns_return has been removed for now.  It was commented out in two places in the TCL code.
 * <li>calendar_validate_boolean removed, should no longer be needed
 * <li>calendar_validate_number removed, no longer needed.
 * </ul>
 *
 * TODO:
 * <ul>
 * <li>Rearchitect.  This was started as a brain-dead port, straight translation of the TCL API.
 * then, I was smart enough to ask Jin if we were to do a brain-dead port or more of a rewrite,
 * and he indicated some degree of rewriting was acceptable to take advantage of Java.  So,
 * I should go back and do that.  In particular, buildWeekMap should be changed to build up 
 * a reasonable data structure that the user can then format as they wish, and 
 * CalendarInfo class should be turned into a real calendar.
 * <li>Move calendarParseOracleError... stuff into an Oracle error class or eliminate entirely
 * <li>Consider using real java.sql.Date instead of strings everywhere
 * <li>Change userId to viewedUserId.  Update and enforce the distinction (it is not presently enforced properly.)
 * </ul>
 */
public class CalendarProcs {
    /**
     * Caches month names for getMonthNames function.  We can't use memoize because it only handles strings.
     */
    private static String[] calMonthNames = null;

    /**
     * This function parses an oracle error message looking for a violation of a null constraint.
     * Needs to be moved somewhere else.
     *
     * @param errorMessage Oracle error message.
     * @return Name of first violating column, or null if no null constraint violation found.
     */
    protected static String parseOracleErrorForNullConstraintViolation (String errorMessage) {
	Perl5Util re = new Perl5Util();
        // TODO: See if this works in Java.
	if (re.match("/cannot insert NULL[^.]*.[^.]*.\"([^\"]*)\"/", errorMessage)) {
	    return re.group(1);
	} else {
	    return null;
	}
    }

    /**
     * Parses Oracle error message looking for a violation of a unique constraint.
     *
     * @param errorMessage Oracle error message.
     * @return Name of unique constraint violated, or null if no unique constraint violation found.
     */
    public static String parseOracleErrorForUniqueConstraintViolation (String errorMessage) {
	Perl5Util re = new Perl5Util();
        // TODO: See if this works in Java.
	if (re.match("/unique constraint \\(([^\\)]*)\\) violated/", errorMessage)) {
	    return re.group(1);
	} else {
	    return null;
	}
    }

    /**
     * Assigns days to days of the week in a given month.
     * get all the sundays then all the mondays, etc
     * and puts <br> tags between the dates
     * @param forDate Date to build map for.
     * @param dayDateTarget base URL to link days to (i.e. baseURL or "").
     * @param dayDateHREFAppend extra info to append to day link URL (i.e. &var="value" or "").  Should be URL encoded and begin with &.
     * @param groupId group Id #, 0 if not viewing a group calendar.
     * @param bool noLinkForCurrentDate flags if "forDate" day should not be linked.
     * @return HTML representing a month; 7 arrays of Strings, where each column contains a day of the week.
     */
    public static String[] buildWeekMap (String forDate, String dayDateTarget, String dayDateHREFAppend, Integer groupId, boolean noLinkForCurrentDate) throws SQLException, java.io.IOException, DataSourceException {
	Database db = new Database();
	ResultSet rs = db.query("select \n" +
                                              "to_char(to_date(:forDate), 'fmMonth') as this_month, \n" +
                                              "to_char(last_day(to_date(:forDate)), 'dd') as last_day, \n" +
                                              "to_char(trunc(to_date(:forDate), 'DD'), 'YYYY') as current_year, \n" +
                                              "to_char(to_date(:forDate), 'MM') as current_month_num, \n" +
                                              "to_char(trunc(to_date(:forDate), 'DD'), 'dd') as current_day, \n" +
                                              "to_char(trunc(to_date(:forDate), 'Month'), 'fmDay') as month_start, \n" +
                                              "to_char(add_months(to_date(:forDate), 1), 'MM') as next_month_month, \n" +
                                              "to_char(add_months(to_date(:forDate), 1), 'YYYY') as next_month_year, \n" +
                                              "to_char(last_day(add_months(to_date(:forDate), -1)), 'dd') as prev_month_last_day, \n" +
                                              "to_char(add_months(to_date(:forDate), -1), 'YYYY') as prev_month_year, \n" +
                                              "to_char(add_months(to_date(:forDate), -1), 'MM') as prev_month_month \n" +
                                "from dual", new Object[] {forDate, forDate, forDate, forDate, forDate, forDate, forDate, forDate, forDate, forDate, forDate});
	rs.next();
	SingleSelection calMonthDates = new SingleSelection(rs); // TODO: Consider freezing this selection.  Maybe better performance?
	//make a week list so that I can translate days into numbers (with lindex)
	String[] weekList = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};
	
	// The calendar code is diverged from here.  The TCL code was awful - it created a list of variable names
	// and then would append to the dereferenced variable name list to do its evil work.  A list of lists
	// or even a list of strings would have been the right approach, so we're doing an array of strings.
	// The vectors will be the dates of the months for their corresponding day/
	String[] weekMap = new String[7];
	weekMap[0] = "<td align=right><font size=-1><b>Su</b>";
	weekMap[1] = "<td align=right><font size=-1><b>Mo</b>";
	weekMap[2] = "<td align=right><font size=-1><b>Tu</b>";
	weekMap[3] = "<td align=right><font size=-1><b>We</b>";
	weekMap[4] = "<td align=right><font size=-1><b>Th</b>";
	weekMap[5] = "<td align=right><font size=-1><b>Fr</b>";
	weekMap[6] = "<td align=right><font size=-1><b>Sa</b>";
	
	// Use the first day of the month and assign 1 to its list.
	// For all the days before that day, pad the first index
	// with days from the last month.
	// Then, fill in all the rest of the lists
	int firstDayIndex, padIndex, prevMonthLastDay;
	String dayDate, dayDateLink;
	for (firstDayIndex = 0; firstDayIndex < weekList.length; firstDayIndex++) {
	    if (weekList[firstDayIndex].equals(calMonthDates.get("month_start"))) {
		break;
	    }
	}
	padIndex = firstDayIndex;
	prevMonthLastDay = Integer.parseInt(calMonthDates.get("prev_month_last_day"));
	while (padIndex > 0) {
	    // pad the lists with days from the last month as necessary
	    padIndex--;
	    
	    dayDate = calMonthDates.get("prev_month_year") + "-" + calMonthDates.get("prev_month_month");
	    if (prevMonthLastDay < 10) {
		dayDate += "0" + prevMonthLastDay;
	    } else {
		dayDate = dayDate + prevMonthLastDay;
	    }
	    dayDateLink = "<a href=\"" + dayDateTarget + "?current_date=" + Utilities.URLEncode(dayDate) + "&group_id=" + groupId + dayDateHREFAppend + "\"><font color=gray>" + prevMonthLastDay + "</font></a>";
	    weekMap[padIndex] += "<br>" + dayDateLink;
	    
	    prevMonthLastDay--;
	}
	int i=1, lastDay=Integer.parseInt(calMonthDates.get("last_day")), dayIndex = firstDayIndex + 1, dayListNum;
	// we want to use 1-based indexing for the day_index so that we
	// can distinguish the first and last day of the week using the
	// mod function.
	while (i <= lastDay) {
	    //fill in the lists
	    
	    //convert day_index back to 0-based indexing for lindex
	    dayListNum = dayIndex - 1;
	    dayDate = calMonthDates.get("current_year") + "-" + calMonthDates.get("current_month_num") + "-";
	    String dateString;
	    if (i < 10) {
		dateString = "0" + i;
	    } else {
		dateString = String.valueOf(i);
	    }
	    dayDate += dateString;
	    dayDateLink = "<a href=\"" + dayDateTarget + "?current_date=" + Utilities.URLEncode(dayDate) + "&group_id=" + groupId + dayDateHREFAppend + "\"><font color=blue>" + i + "</font></a>";
	    
	    if (noLinkForCurrentDate && dateString.equals(calMonthDates.get("current_day"))) {
		// this is the current day, so format it differently
		weekMap[dayListNum] += "<br><b><font color=red>" + i + "</font></b>";
	    } else {
		weekMap[dayListNum] += "<br>" + dayDateLink;
	    }
	    i++;
	    
	    dayIndex = (dayIndex+1) % 7;
	    if (dayIndex == 0) {
		dayIndex = 7;
	    }
	}
	// special case: if day_index is 1, then the last day
	// was saturday since day_index was updated at the end
	// of the while loop.  So, we don't want to fill
	// in the rest of the week
	if (dayIndex == 1) {
	    dayIndex = 8;
	}

	// pad the rest of the weekdays with dates from the following month
	i = 1;
	while (dayIndex <= 7) {
	    dayListNum = dayIndex - 1;
	    
	    dayDate = calMonthDates.get("next_month_year") + "-" + calMonthDates.get("next_month_month") + "-";
	    if (i < 10) {
		dayDate += "0" + i;
	    } else {
		dayDate += i;
	    }
	    dayDateLink = "<a href=\"" + dayDateTarget + "?current_date=" + Utilities.URLEncode(dayDate) + "&group_id=" + groupId + dayDateHREFAppend + "\"><font color=gray>" + i + "</font></a>";
	    
	    weekMap[dayListNum] += "<br>" + dayDateLink;
	    dayIndex++;
	    i++;
	}
	db.close();

	//see if we need to add an extra row to the calendar (to keep all calendars a consistent size)
	int firstSun, extraWeek, numWeeks;
	if (firstDayIndex != 0) {
	    //this is the date of the first sunday
	    firstSun = 8 - firstDayIndex;
	    extraWeek = 1;
	} else {
	    firstSun = 1;
	    extraWeek = 0;
	}
	numWeeks = (int)Math.ceil(((double)(lastDay+1-firstSun))/((double)7)) + extraWeek;
	if (numWeeks < 6) {
	    for (i = 0; i < 7; i++) {
		weekMap[i] += "<br>&nbsp";
	    }
	}
	// close the weekday lists
	for (i = 0; i < 7; i++) {
	    weekMap[i] += "</font></td>\n";
	}

	return weekMap;
    }

    /**
     * @see #calMiniCalendarWidget(String, String, String, int)
     */
    public static String calMiniCalendarWidget (String baseURL) throws SQLException, ACSException, DataSourceException, java.io.IOException {
	return calMiniCalendarWidget (baseURL, "day");
    }

    /**
     * @see #calMiniCalendarWidget(String, String, String, int)
     */
    public static String calMiniCalendarWidget (String baseURL, String currentView) throws SQLException, ACSException, DataSourceException, java.io.IOException {
	return calMiniCalendarWidget (baseURL, currentView, null);
    }

    /**
     * @see #calMiniCalendarWidget(String, String, String, int)
     */
    public static String calMiniCalendarWidget (String baseURL, String currentView, String currentDate) throws SQLException, ACSException, DataSourceException, java.io.IOException {
	return calMiniCalendarWidget (baseURL, currentView, currentDate, new Integer(0));
    }

    /**
     * This proc creates a mini calendar useful for navigating various 
     * calendar views.  This proc will set 2 variables in the base  
     * url's environment: the current view and the current date.  
     *
     * TODO: This proc should really be changed over to a template-based creation.
     * It's especially ugly to do this much HTML in Java.
     *
     * TODO: Consider switching currentDate to Date.  Would help reduce confusing on parameter order.
     *
     * @param baseURL Base URL to which this mini calendar will navigate.
     * @param currentView list, day, week, month, or year.
     * @param currentDate YYYY-MM-DD format.
     * @param groupId Non-zero if viewing group calendar, zero otherwise.
     */
    public static String calMiniCalendarWidget (String baseURL, String currentView, String currentDate, Integer groupId) throws SQLException, ACSException, java.io.IOException, DataSourceException {
	Database db = new Database();
	String viewNavHTML;
	String returnHTML;
	if (currentDate == null) {
	    currentDate = db.queryToString("select to_char(sysdate, 'YYYY-MM-DD') from dual");
	}
	// get the current month, day, and the first day of the month
	ResultSet rs = db.query("select \n" +
				     "trim(to_char(sysdate, 'Month')) || ' ' || to_char(sysdate, 'DD, YYYY') as pretty_today, \n" +
				     "trim(to_char(month.current_month, 'Month')) as this_month, \n" +
				     "to_char(sysdate, 'YYYY-MM-DD') as today, \n" +
				     "to_char(add_months('" + currentDate + "', 12), 'YYYY-MM-DD') as next_year, \n" +
				     "to_char(add_months('" + currentDate + "', -12), 'YYYY-MM-DD') as prev_year, \n" +
				     "to_char(add_months('" + currentDate + "', 1), 'YYYY-MM-DD') as next_month, \n" +
				     "to_char(add_months('" + currentDate + "', -1), 'YYYY-MM-DD') as prev_month, \n" +
				     "to_char(month.current_month, 'MM') as current_month_num, \n" +
				     "to_char(trunc(to_date('" + currentDate + "'), 'DD'), 'dd') as current_day, \n" +
				     "to_char(trunc(to_date('" + currentDate + "'), 'DD'), 'YYYY') as current_year, \n" +
				     "trim(to_char(month.current_month, 'Month')) || ' ' || trim(to_char(month.current_month, 'YYYY')) as pretty_month \n" +
				     "from (select trunc(to_date('" + currentDate + "'), 'Month') \n" +
				     "as current_month \n" +
				     "from dual) month, \n" +
				     "dual");
	rs.next();
	SingleSelection dateSelect = new SingleSelection(rs);
	// TODO: Consider freezing this selection.  Better performance?

	//detect which view I'm in and show the appropriate box
	viewNavHTML = "<tr align=center> \n" +
	    (currentView.equals("list")?"<td BGCOLOR=FFD700>":"<td>") + 
	    "<a href=\"" + baseURL + "?current_view=list&current_date=" + Utilities.URLEncode(currentDate) + "&group_id=" + groupId + "\"> \n" +
	    "<font size=-1 color=blue>List</font></a> \n" +
	    "</td> \n" +
	    (currentView.equals("day")?"<td BGCOLOR=FFD700>":"<td>") +
	    "<a href=\"" + baseURL + "?current_view=day&current_date=" + Utilities.URLEncode(currentDate) + "&group_id=" + groupId + "\"> \n" +
	    "<font size=-1 color=blue>Day</font></a> \n" +
	    "</td> \n" +
	    (currentView.equals("week")?"<td BGCOLOR=FFD700>":"<td>") + 
	    "<a href=\"" + baseURL + "?current_view=week&current_date=" + Utilities.URLEncode(currentDate) + "&group_id=" + groupId + "\"> \n" +
	    "<font size=-1 color=blue>Week</font></a> \n" +
	    "</td> \n" +
	    (currentView.equals("month")?"<td BGCOLOR=FFD700>":"<td>") + 
	    "<a href=\"" + baseURL + "?current_view=month&current_date=" + Utilities.URLEncode(currentDate) + "&group_id=" + groupId + "\"> \n" +
	    "<font size=-1 color=blue>Month</font></a> \n" +
	    "</td> \n" +
	    (currentView.equals("year")?"<td BGCOLOR=FFD700>":"<td>") + 
	    "<a href=\"" + baseURL + "?current_view=year&current_date=" + Utilities.URLEncode(currentDate) + "&group_id=" + groupId + "\"> \n" +
	    "<font size=-1 color=blue>Year</font></a> \n" +
	    "</td> \n" +
	    "</tr>";
	returnHTML = "<table border=1 cellpadding=2 cellspacing=0 width=160>" + viewNavHTML;

	//if this is a month or year view, show the current year in the main bar
	if (currentView.equals("month") || currentView.equals("year")) {
	    // TODO: Make left.gif, right.gif configuration parameters.
	    returnHTML += "<tr><TD NOWRAP ALIGN=CENTER bgcolor=lavender colspan=5> " + 
		"<a href=\"" + baseURL + "?current_view=" + currentView + "&current_date=" + Utilities.URLEncode(dateSelect.get("prev_year")) + "&group_id=" + groupId + "\"> " + 
		"<img border=0 src=\"/graphics/left.gif\"></a> " + 
		"<TT><B>" + dateSelect.get("current_year") + "</B></TT> " + 
		"<a href=\"" + baseURL + "?current_view=" + currentView + "&current_date=" + Utilities.URLEncode(dateSelect.get("next_year")) + "&group_id=" + groupId + "\"> " + 
		"<img border=0 src=\"/graphics/right.gif\"></a> " + 
		"</TD></tr>";
	}
	if (currentView.equals("month")) {
	    //month view
	    // TODO: These should be selected out from Oracle for internationalization.  Some code in templating system
	    // or on First Tuesday does this.
	    String[] monthsList = {"January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"};
	    int[] monthsDays = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
	    int i;
	    
	    returnHTML += "<tr><td colspan=5> " + 
		"<TABLE BGCOLOR=ffffff CELLSPACING=3 CELLPADDING=1 BORDER=0> " + 
		"<tr>";

	    for (i = 0; i < 12; i++) {
		String month = monthsList[i];
		
		//show 3 months in a row
		if ((i % 3 == 0) && (i != 0)) {
		    returnHTML += "</tr><tr>";
		}
	    
		if (month.equals(dateSelect.get("this_month"))) {
		    returnHTML += "<td> " + 
			"<font size=-1 color=red>" + month + "</font> \n" +
			"</td>";
		} else {
		    int newMonthMon = i + 1;
		    String newMonthString = "" + newMonthMon;
		    String newCurrentDay;
		    if (newMonthMon < 10) {
			newMonthString = "0" + newMonthString;
		    } 
		    int lastMonthDay = monthsDays[i];
		    if (Integer.parseInt(dateSelect.get("current_day")) > lastMonthDay) {
			newCurrentDay="" + lastMonthDay;
		    } else {
			newCurrentDay=dateSelect.get("current_day");
		    }
		    String new_month = dateSelect.get("current_year") + "-" + newMonthString + "-" + newCurrentDay;
		    String month_link = "<a href=\"" + baseURL + "?current_view=" + currentView + "&current_date=" + Utilities.URLEncode(new_month) + "&group_id=" + groupId + "\"> \n" +
			"<font size=-1 color=blue>" + month + "</font></a>";
		    returnHTML += "<td>" + month_link + "</td>";
		}
	    }
	    
	    returnHTML += "</tr>";
	} else if (currentView.equals("year")) {
	    //year view
	    int currentYear = Integer.parseInt(dateSelect.get("current_year"));
	    
	    returnHTML += "<tr><td colspan=5> \n" +
		"<TABLE BGCOLOR=ffffff CELLSPACING=3 CELLPADDING=1 BORDER=0> \n" +
		"<tr>";
	
	    //This # came from Oracle, no need to trap for NumberFormatException
	    int i = currentYear - 2;
	    int endYear = currentYear + 2;
	    int count = 0;

	    for (i = currentYear - 2; i <= endYear; i++, count++) {
		if (i == currentYear) {
		    returnHTML += "<td> \n" +
			"<font size=-1 color=red>" + i + "</font> \n" +
			"</td>";
		} else {
		    String newYear = "" + i + "-" + dateSelect.get("current_month_num") + "-" + dateSelect.get("current_day");
		    returnHTML += "<td>\n" +
			"<a href=\"" + baseURL + "?current_view=" + currentView + "&current_date=" + Utilities.URLEncode(newYear) + "&group_id=" + groupId + "\"><font size=-1 color=blue>" + i + "</font></a></td>";
		}
	    }
	
	    returnHTML += "</tr>";
	    
	} else {
	    String[] weekMap;
	    int dayListNum;
	    //list, day, week view
	    returnHTML += "<tr><TD NOWRAP ALIGN=CENTER bgcolor=lavender colspan=5> \n" +
		"<a href=\"" + baseURL + "?current_view=" + currentView + "&current_date=" + Utilities.URLEncode(dateSelect.get("prev_month")) + "&group_id=" + groupId + "\"> \n" +
		"<img border=0 src=\"/graphics/left.gif\"></a> \n" +
		"<TT><B>" + dateSelect.get("pretty_month") + "</B></TT> \n" +
		"<a href=\"" + baseURL + "?current_view=" + currentView + "&current_date=" + Utilities.URLEncode(dateSelect.get("next_month")) + "&group_id=" + groupId + "\"> \n" +
		"<img border=0 src=\"/graphics/right.gif\"></a> \n" +
		"</TD></tr> \n" +
		"<tr><td colspan=5> \n" +
		"<TABLE BGCOLOR=ffffff CELLSPACING=3 CELLPADDING=1 BORDER=0>";
	    weekMap = buildWeekMap(currentDate, baseURL, ("&current_view=" + currentView), groupId, true);
	    returnHTML += "<tr valign=top>";
	    for (int i = 0; i < 7; i++) {
		returnHTML += weekMap[i];
	    }
	    returnHTML += "</tr>";
	}
	String todayURL = baseURL + "?current_view=" + currentView + "&current_date=" + Utilities.URLEncode(dateSelect.get("today")) + "&group_id=" + groupId;
	AdSet formvars = new AdSet();
	formvars.put("current_view", currentView);
	formvars.put("group_id", "" + groupId);
	returnHTML += "<tr><td align=center colspan=7> \n" +
	    "<form method=get action=\"" + baseURL + "\"> \n" +
	    Utilities.exportFormVars(formvars) +
	    "<center> \n" +
	    "<font size=-1> \n" +
	    "<input type=text size=11 name=current_date value=\"" + Utilities.URLEncode(currentDate) + "\"> \n" +
	    "<input type=submit value=\"Go\"><br> \n" +
	    "(YYYY-MM-DD Format) \n" +
	    "</font> \n" +
	    "</center> \n" +
	    "</form> \n" +
	    "</td></tr> \n" +
	    "</table> \n" +
	    "</td></tr> \n" +
	    "<tr><td align=center bgcolor=lavender colspan=5> \n" +
	    "<font size=-1><a href=\"" + todayURL + "\"><font color=blue><b>Today</b></font></a> \n" +
	    "is " + dateSelect.get("pretty_today") + "</font> \n" +
	    "</td></tr> \n" +
	    "</table>";
	db.close();
	return returnHTML;
    }

    /**
     * @see #calMiniMonthCalendar(String, String, int, String).
     */
    public static String calMiniMonthCalendar (String currentYear, String month) throws SQLException, ACSException, DataSourceException, java.io.IOException {
	return calMiniMonthCalendar (currentYear, month, new Integer(0));
    }

    /**
     * @see #calMiniMonthCalendar(String, String, int, String).
     */
    public static String calMiniMonthCalendar (String currentYear, String month, Integer groupId) throws SQLException, ACSException, DataSourceException, java.io.IOException {
	return calMiniMonthCalendar (currentYear, month, groupId, "full");
    }

    /**
     * This proc creates a mini month calendar useful for navigating 
     * on a month calendar.  
     *
     * TODO: This proc should really be changed over to a template-based creation.
     * It's especially ugly to do this much HTML in Java.
     *
     * @param currentYear Year currently being displayed.
     * @param month Month currently being displayed.
     * @param groupId If viewing group calendar.
     * @param viewMode "full" or "modular".
     */
    public static String calMiniMonthCalendar (String currentYear, String month, Integer groupId, String viewMode) throws SQLException, ACSException, DataSourceException, java.io.IOException {
	String[] weekMap;
	String monthLink, dayLink, monthTxt, returnHTML;
	Database db = new Database();
	String calMonthName;

	if (viewMode.equals("modular")) {
	    monthLink = "?current_view=month&view_mode=modular&";
	    dayLink = "&current_view=day&view_mode=modular";
	} else {
	    // view_mode is "full"
	    monthLink = "?current_view=month&view_mode=full&";
	    dayLink = "&current_view=day&view_mode=full";
	}

	//month is the 'MM' form of the current month
	//current_date is 'YYYY' format of current year
     
	monthTxt = currentYear + "-" + month + "-01";

	try {
	    calMonthName = db.queryToString("select to_char(to_date('" + monthTxt + "'), 'fmMonth') as this_month from dual");
	} catch (ACSException e) {
	    // In theory, this query is simple enough that this message can only
	    // happen if Oracle is hosed, so we might as well re-throw as a
	    // SQLException rather than have this method appear to throw a type
	    // of exception it really doesn't throw.
	    throw new SQLException (e.getMessage());
	}
	db.close();
        returnHTML = "<table border=1 cellpadding=1 cellspacing=0> \n" +
	"<tr align=center bgcolor=lavender> \n" +
        "<td colspan=7> \n" +
        "<a href=\"" + monthLink + "group_id=" + groupId + "&current_date=" + monthTxt + "\"> \n" +
        calMonthName + "</a> \n" +
        "</td> \n" +
        "</tr> " + 
        "<tr> \n" +
        "<td> \n" +
        "<TABLE CELLSPACING=3 CELLPADDING=1 BORDER=0 width=100%>";
    
	weekMap = buildWeekMap(monthTxt, "", dayLink, groupId, false);

	returnHTML += "<tr valign=top>";
	for (int i = 0; i < 7; i++) {
	    returnHTML += weekMap[i];
	}
	returnHTML += "</tr></table></td></tr></table>";
	return returnHTML;
    }

    /**
     * Generates the HTML for 12 mini calendar widgets, 3 across and 4 down.
     *
     * @param groupId Just for pass-through.
     * @param viewMode Flag for portals to adjust the display with.  Defaults to "full".
     * @param currentYear The current year.
     * @param currendDate Date to hilite.
     * @return HTML for 12 mini calendar widgets.
     */
  public static String calYearCalendar (Integer groupId, String viewMode, String currentYear, String currentDate) throws SQLException, ACSException, java.io.IOException, DataSourceException {
    StringBuffer returnHTML = new StringBuffer();
    if (Utilities.emptyString(viewMode)) {
      viewMode = "full";
    }
    if (groupId == null) {
      groupId = new Integer(0);
    }

    for (int i = 1; i <= 12; i++) {
      String month;
	if (i < 10) {
	  month = "0" + i;
	} else {
	  month = "" + i;
	}

	if (((i - 1) % 3) == 0) {
	    if (i != 1) {
	      returnHTML.append("</tr>");
	    }
	    returnHTML.append("<tr valign=top>");
	}

	returnHTML.append("<td>");
	returnHTML.append(calMiniMonthCalendar(currentYear, month, groupId, viewMode));
	returnHTML.append("</td>");
    }

    returnHTML.append("</tr>");
    return returnHTML.toString();
  }

  /**************************************************
   *	     BEGIN PROCS FOR CREATING A DAY VIEW
   ****************************************************/

/**
 * Builds HREF string for related URLs.
 *
 * @param title Title (only used if text is empty, then displayed before " - Name").
 * @param name Name.
 * @param url URL.
 * @param text Text of link, displayed before " - Name"
 */
public static String calRelatedURLHtml (String title, String name, String url, String text) {
  StringBuffer result = new StringBuffer("<a href=\"$url\">");
  if (Utilities.emptyString(text)) {
    result.append(title);
  } else {
    result.append(text);
  }
  // This is basically a hack to support the
  // equivalent of the old calendar/monthly view which shows absences.  For
  // absences we stuff the item_map with the group_id of the employees group and
  // the user_id of the user reporting the absence.  That way it appears on both
  // their personal calendar and the employees calendar. The idea is that if the
  // item being displayed has a user_id associated with it, then on the monthly
  // view we want to show "first_names || ' ' || substr(last_name, 1, 1)".  So
  // when user_id is null, you still wind up with " " as the name string.
  if (!Utilities.emptyString(name) && name.length() > 1) {
    result.append(" - ").append(name);
  }
  result.append("</a>");
  return result.toString();
}

/**
 * Generates the HTML for a day view.
 * Should probably be moved into the template.
 */
 public static String calendarDayView (CalendarInfo calendarInfo, String currentDate, Integer loggedInUserId, Integer groupId, String compressDayViewP, Integer beginCalHour, Integer endCalHour) throws SQLException, ACSException{
   return calendarDayView(calendarInfo, currentDate, loggedInUserId, groupId, compressDayViewP, beginCalHour, endCalHour, "modular");
 }

 public static String calendarDayView (CalendarInfo calendarInfo, String currentDate, Integer loggedInUserId, Integer groupId, String compressDayViewP, Integer beginCalHour, Integer endCalHour, String viewMode) throws SQLException, ACSException {
   return calendarDayView(calendarInfo, currentDate, loggedInUserId, groupId, compressDayViewP, beginCalHour, endCalHour, viewMode, "?current_view=day&");
 }

 public static String calendarDayView (CalendarInfo calendarInfo, String currentDate, Integer loggedInUserId, Integer groupId, String compressDayViewP, Integer beginCalHour, Integer endCalHour, String viewMode, String addLink) throws SQLException, ACSException {
   Database db = new Database();
   int numHourRows = Integer.parseInt(db.queryToString("select count(hour) as num_hour_rows from cal_hours"));
   Table dayview = new Table(numHourRows, 1);
   String userOrGroupSQL;

   ResultSet itemRecord;
   if (groupId == null || groupId.intValue() == 0) {
     // user calendar
     // We need to cut-and-paste because the bind variables are different.
     itemRecord = db.query ("select \n" +
      "m.group_id as item_group_id, ug.short_name as item_group_name, ug.short_name as item_group_short_name, \n" +
      "ug.group_type as item_group_type,\n" +
      "combine_category_names(ii.instance_id, ', ') as category_list,"+
      "i.item_id, ii.instance_id, \n" +
      "to_char(start_date, 'fmHH24') as hour, \n" +
      "to_char(end_date, 'fmHH24') as end_hour, \n" +
      "to_char(end_date, 'fmMI') as end_minute, \n" +
      "to_char(start_date, 'YYYY-MM-DD HH24:MI') as start_time, \n" +
      "to_char(start_date, 'fmHH:fmMIam') as pretty_start_time, \n" +
      "to_char(end_date, 'YYYY-MM-DD HH24:MI') as end_time, \n" +
      "to_char(end_date, 'fmHH:fmMIam') as pretty_end_time, \n" +
      "to_char(start_date, 'YYYY-MM-DD') as start_date, to_char(end_date, 'YYYY-MM-DD') as end_date, \n" +
      "decode(m.user_id, NULL, 'f', 't') as user_p, \n" +
      "nvl(ii.title, i.title) as title, \n" +
      "nvl(ii.related_url, i.related_url) as related_url,  \n" +
      "nvl(ii.related_url_text, i.related_url_text) as related_url_text,  \n" +
      "nvl(ii.related_url_p, i.related_url_p) as related_url_p \n" +
      "from cal_items i, cal_item_instances ii,  \n" +
      "     cal_item_map m, user_groups ug \n" +
      "where ug.group_id(+) = m.group_id \n" +
      "  and i.item_id = ii.item_id \n" +
      "  and end_date is not NULL \n" +
      "  and start_date between  \n" +
      "      to_date(:current_date) and \n" +
      "      to_date(:current_date) + (24 - 1/3600)/24 \n" +
      "  and i.item_id = m.item_id \n" +
      "  and (m.user_id = :logged_in_user_id \n" +
      "          or m.group_id in  \n" +
      "             (select ugm.group_id \n" +
      "              from user_group_map ugm, cal_user_update_prefs ugp \n" +
      "              where ugm.user_id = :logged_in_user_id \n" +
      "                and ugp.user_id = ugm.user_id \n" +
      "                and ugm.group_id = ugp.group_id \n" +
      "                and ugp.update_type = 'automatically')  \n" +
      "          ) " +
      "order by start_date", new Object[] {currentDate, currentDate, loggedInUserId, loggedInUserId});
   } else {
     // Group calendar
     itemRecord = db.query ("select \n" +
      "m.group_id as item_group_id, ug.short_name as item_group_name, ug.short_name as item_group_short_name,  \n" +
      "ug.group_type as item_group_type, \n" +
      "combine_category_names(ii.instance_id, ', ') as category_list, \n" +
      "i.item_id, ii.instance_id, \n" +
      "to_char(start_date, 'fmHH24') as hour, \n" +
      "to_char(end_date, 'fmHH24') as end_hour, \n" +
      "to_char(end_date, 'fmMI') as end_minute, \n" +
      "to_char(start_date, 'YYYY-MM-DD HH24:MI') as start_time, \n" +
      "to_char(start_date, 'fmHH:fmMIam') as pretty_start_time, \n" +
      "to_char(end_date, 'YYYY-MM-DD HH24:MI') as end_time, \n" +
      "to_char(end_date, 'fmHH:fmMIam') as pretty_end_time, \n" +
      "to_char(start_date, 'YYYY-MM-DD') as start_date, to_char(end_date, 'YYYY-MM-DD') as end_date, \n" +
      "decode(m.user_id, NULL, 'f', 't') as user_p, \n" +
      "nvl(ii.title, i.title) as title, \n" +
      "nvl(ii.related_url, i.related_url) as related_url,  \n" +
      "nvl(ii.related_url_text, i.related_url_text) as related_url_text,  \n" +
      "nvl(ii.related_url_p, i.related_url_p) as related_url_p \n" +
      "from cal_items i, cal_item_instances ii,  \n" +
      "     cal_item_map m, user_groups ug \n" +
      "where ug.group_id(+) = m.group_id \n" +
      "  and i.item_id = ii.item_id \n" +
      "  and end_date is not NULL \n" +
      "  and start_date between  \n" +
      "      to_date(:current_date) and \n" +
      "      to_date(:current_date) + (24 - 1/3600)/24 \n" +
      "  and i.item_id = m.item_id \n" +
      "  and m.group_id = :group_id \n" +
      "order by start_date", new Object[] {currentDate, currentDate, groupId});
   }
   while (itemRecord.next()) {
     // if there is a calendar item at this time...
     String title=itemRecord.getString("title");
     if (!Utilities.emptyString(title)) {
       int endIndex;
       String startDate = itemRecord.getString("start_date");
       String endDate = itemRecord.getString("end_date");
       if (startDate.equals(endDate)) {
	 int endHour = Integer.parseInt(itemRecord.getString("end_hour"));
	 int endMinute=Integer.parseInt(itemRecord.getString("end_minute"));
	 if ((endMinute > 0) && (endHour < endCalHour.intValue())) {
	   endIndex = endHour + 1;
	 } else {
	   endIndex = endHour;
	 }
       } else {
	 endIndex = 24;
       }
       int hour = Integer.parseInt(itemRecord.getString("hour"));
       //the number of rows this item takes
       int rowSpan = endIndex - hour;
       if (rowSpan < 1) {
	 rowSpan = 1;
	 endIndex++;
       }
       //i is the row, col_index is the col
       int colIndex = 0;
       for (int i = hour; i < endIndex; i++) {
	 int tmpColIndex = dayview.getFirstEmptyCol(i);
	 if (tmpColIndex == -1) {
	   // Add a new column to the table - no room for this entry
	   dayview.addCol();
	   colIndex=dayview.getNumCols()-1;
	   break;
	 }
	 if (tmpColIndex > colIndex) {
	   colIndex = tmpColIndex;
	 }
       }
	
       // Create table cell
       String action = "item-view";
       String itemText;
       if (startDate.equals(endDate)) {
	 itemText = itemRecord.getString("pretty_start_time") + " - " + itemRecord.getString("pretty_end_time");
       } else {
	 itemText = itemRecord.getString("pretty_start_time") + " - " + itemRecord.getString("pretty_end_day") + " " + itemRecord.getString("pretty_end_time");
       }
       if (itemRecord.getString("related_url_p").equals("t")) {
	 itemText += "<a href=\"" + itemRecord.getString("related_url") + "\">";
	 String relatedUrlText = itemRecord.getString("related_url_text");
	 if (Utilities.emptyString(relatedUrlText)) {
	   itemText += title;
	 } else {
	   itemText += relatedUrlText;
	 }
       } else {
	 AdSet vars = new AdSet();
	 vars.put("action", action);
	 vars.put("item_id", itemRecord.getString("item_id"));
	 vars.put("instance_id", itemRecord.getString("instance_id"));
	 vars.put("current_date", currentDate);
	 vars.put("group_id", groupId);
	 vars.put("view_mode_var", calendarInfo.getViewModeVar());
	 itemText += " <a href=\"" + calendarInfo.getDayLink() + Utilities.exportURLVars(vars) + "\">" + title + "</a>";
       }
       // ejc@arsdigita.com 7-27-2000
       // if this is NOT a group portal, show the group name
       // if categories exist, show them in parentheses
       String itemGroupShortName = itemRecord.getString("item_group_short_name");
       String groupHomeLink = "";
       if (groupId != null && groupId.intValue() == 0 && Utilities.emptyString(itemGroupShortName)) {
	 groupHomeLink = getGroupHomeLink (itemRecord.getString("item_group_type"), itemGroupShortName, itemRecord.getString("item_group_name"));
       } 

       String categoryList = itemRecord.getString("category_list");
       // TODO: I think I flubbed this part of the conversion.  Put in some test data and test it out.  Figure out if the list needs special processing or what.  Is it a TCL list?  It looks comma delimited (from the SQL)...
       Vector temp = new Vector();
       if (!Utilities.emptyString(groupHomeLink) || !Utilities.emptyString(categoryList)) {
	 temp.add(groupHomeLink);
	 temp.add(categoryList);
	 itemText += "(" + Utilities.join(temp, ": ") + ")";
       }
       /* Here's the original TCL:
	    if ![empty_string_p $category_list] {
		set item_category_list [list $category_list]
	    } else {
		set item_category_list ""
	    }
	    if {![empty_string_p $group_home_link] || ![empty_string_p $item_category_list]} {
		append item_text "
  		([join "$group_home_link $item_category_list" ": "])
  		"
	    }
       */
       TableCell cell = new TableCell(new java.awt.Point(colIndex, hour), itemText, rowSpan, -1);
       int i = hour;
       while (i < endIndex) {
	 dayview.setCell(colIndex, i, cell);
	 i++;
       }
     }
   }
   itemRecord.close();
 
   String returnHTML = "";
   int numCols = dayview.getNumCols();
   String allDayHTML = "";
   boolean addLinkP = true;
   if (groupId != null && groupId.intValue() > 0) {
     if (calendarInfo.getLoggedInUser().isGroupAdmin(groupId)) {
       addLinkP = false;
     }
   }

   if (addLinkP) {
     allDayHTML += "<tr>\n" +
	"<td valign=top nowrap width=10%> " + 
	"<a href=\"" + calendarInfo.getAddLink() + calendarInfo.getViewModeVar() + "current_date=" + currentDate + "&start_date=" + currentDate + "%2000:00&end_date=&current_view=day&action=item-add\">No Time</a> \n" +
	"</td>";
   } else {
     allDayHTML += "<tr>" + 
	"<td valign=top nowrap width=10%> " + 
	"No Time " + 
	"</td>";
   }

   String allDayItems = "<ul>";
   int count = 0;

   itemRecord = db.query("select " + 
    "m.group_id as item_group_id, ug.short_name as item_group_name, ug.short_name as item_group_short_name, \n" +
    "ug.group_type as item_group_type, \n" +
    "combine_category_names(ii.instance_id, ', ') as category_list, \n" +
    "i.item_id, ii.instance_id, \n" +
    "decode(m.user_id, NULL, 'f', 't') as user_p, \n" +
    "nvl(ii.title, i.title) as title, \n" +
    "upper(nvl(ii.title, i.title)), \n" +
    "nvl(ii.related_url, i.related_url) as related_url,  \n" +
    "nvl(ii.related_url_text, i.related_url_text) as related_url_text,  \n" +
    "nvl(ii.related_url_p, i.related_url_p) as related_url_p \n" +
    "from cal_items i, cal_item_instances ii, cal_item_map m, user_groups ug \n" +
    "where trunc(start_date) =  trunc(to_date(:current_date)) \n" +
    "  and ug.group_id(+) = m.group_id \n" +
    "  and i.item_id = ii.item_id \n" +
    "  and i.item_id = m.item_id \n" +
    "  and end_date is NULL \n" +
    "  and m.group_id = :group_id " + 
    "  order by upper(nvl(ii.title, i.title))", new Object[] {currentDate, groupId});
   while (itemRecord.next()) {
     allDayItems += "<li>";
     if (itemRecord.getString("related_url_p").equals("t")) {
       allDayItems += calRelatedURLHtml(itemRecord.getString("title"), "", itemRecord.getString("related_url"), itemRecord.getString("related_url_text"));
     } else {
       allDayItems += "<a href=\"" + calendarInfo.getDayLink() + calendarInfo.getViewModeVar() + "action=item-view&item_id=" + itemRecord.getString("item_id") + "&instance_id=" + itemRecord.getString("instance_id") + "&current_date=" + currentDate + "&group_id=" + calendarInfo.getGroupId() + "&current_view=day\">" + itemRecord.getString("title") + "</a>";
     }
     String itemGroupShortName = itemRecord.getString("item_group_short_name");
     String groupHomeLink = "";
     if (groupId != null && groupId.intValue() == 0 && !Utilities.emptyString(itemGroupShortName)) {
       groupHomeLink = getGroupHomeLink (itemRecord.getString("item_group_type"), itemGroupShortName, itemRecord.getString("item_group_name"));
     } 
     
     String categoryList = itemRecord.getString("category_list");
     // TODO: I think I flubbed this part of the conversion.  Put in some test data and test it out.  Figure out if the list needs special processing or what.  Is it a TCL list?  It looks comma delimited (from the SQL)...
     Vector temp = new Vector();
     if (!Utilities.emptyString(groupHomeLink) || !Utilities.emptyString(categoryList)) {
       temp.add(groupHomeLink);
       temp.add(categoryList);
       allDayItems += "(" + Utilities.join(temp, ": ") + ")";
     }
       /* Here's the original TCL:
	    if ![empty_string_p $category_list] {
		set item_category_list [list $category_list]
	    } else {
		set item_category_list ""
	    }
	    if {![empty_string_p $group_home_link] || ![empty_string_p $item_category_list]} {
		append item_text "
  		([join "$group_home_link $item_category_list" ": "])
  		"
	    }
       */

     count++;
   }
   itemRecord.close();

   int colspan = dayview.getNumCols();
   if (count > 0) {
     allDayHTML += "<td colspan=" + colspan + ">" + allDayItems + "</ul>";
   } else {
     allDayHTML += "<td bgcolor=\"DCDCDC\" colspan=" + colspan + ">&nbsp;";
   }
   allDayHTML += "</td></tr>";

   if (!compressDayViewP.equals("t") || count > 0) {
     returnHTML += allDayHTML;
   }

     for (int i = 0; i < numHourRows; i++) {

       int filledCellCount = 0;
       String rowHTML = "";
       String bgcolorHTML = "";
       String time;
       if (i < 12) {
	 if (i == 0) {
	   time = "12:00 am";
	 } else {
	   time = "" + i + ":00 am";
	 }
       } else {
	 if (i == 12) {
	   time = "12:00 pm";
	   bgcolorHTML = "bgcolor=\"FFF8DC\"";
	 } else {
	   time = "" + (i-12) + ":00 pm";
	 }
       }

       String fmHour;
       if (i < 10) {
	 fmHour = "0" + i;
       } else {
	 fmHour = "" + i;
       }
       String encodedStartTime = Utilities.URLEncode(currentDate + " " + fmHour + ":00");

       if (addLinkP) {
	 rowHTML += "<tr>" + 
	    "<td valign=top nowrap " + bgcolorHTML + " width=10%>\n" +
	    "<a href=\"" + calendarInfo.addLink + calendarInfo.viewModeVar + "current_date=" + currentDate + "&start_date=" + encodedStartTime + "&action=item-add&current_view=day\">" + 
	    time + "</a></td>";
       } else {
	    rowHTML += "<tr>" + 
	      "<td valign=top nowrap " + bgcolorHTML + " width=10%>" + time + "</td>";
       }

       int j = 0;
       int totalCols = 0;
       int filledIndex;
       while (j < numCols) {
	 TableCell cell = dayview.getCell(j, i);
	 String cellText = cell.getText();

	 // need to check if there is room for this empty cell
	 int filledLeft = dayview.getFilledColsLeft(i, j);
	 int filledCheck = numCols - filledLeft - totalCols;
	 // see if we need to write an empty table cell

	 if (Utilities.emptyString(cellText) && (filledCheck > 0)) {
	   // Cell empty, we may write an empty tag.

	   // find colspan
	   filledIndex = dayview.getNextFilledCol(i, j);
	   if (filledIndex == -1) {
	     colspan = numCols - j;
	   } else {
	     colspan = Math.abs(j - filledIndex);
	   }
	   if (colspan > 0) {
	     rowHTML += "<td colspan=" + colspan + " bgcolor=\"DCDCDC\">&nbsp;</td>";
	     totalCols += colspan;
	   }
	 }

	 // see if we write the contents of this cell
	 java.awt.Point root = cell.getRoot();
	 int rowspan;
	 if (!Utilities.emptyString(cellText)) {
	   filledCellCount++;
	   rowspan = cell.getRowSpan();
	   //see how long to make the colspan for this cell

	   //start at the root row
	   int k = cell.getRoot().y;
	   // go until this cell ends
	   int cellEndRow = k + rowspan;
	   filledIndex = dayview.getNumCols();
	   while (k < cellEndRow) {
	     int tmpFilledIndex = dayview.getNextFilledCol(k, (j + 1));
	     if ((tmpFilledIndex < filledIndex) && (tmpFilledIndex != -1)) {
	       filledIndex = tmpFilledIndex;
	     }
	     k++;
	   }
	   if (filledIndex == -1) {
	     colspan = numCols - j;
	   } else {
	     colspan = filledIndex - j;
	   }
	   // Root cell, write contents...
	   if ((cell.getRoot().x == j) && (cell.getRoot().y == i)) {
	     rowHTML += "<td valign=top rowspan=" + rowspan + "colspan=" + colspan + ">" + 
	       cellText + 
	       "</td>\n";
	   }
	   totalCols = totalCols+colspan;
	 }
	 j++;
       }
       rowHTML += "</tr>";

       if ((filledCellCount > 0) || ((!compressDayViewP.equals("t")) && (i >= beginCalHour.intValue()) && (i <= endCalHour.intValue()))) {
	 returnHTML += rowHTML;
       }

     }

     if (!Utilities.emptyString(returnHTML)) {
       returnHTML = "<table border=1 cellpadding=2 cellspacing=0 width=90%>" + returnHTML + "</table>\n";
     }
     return returnHTML;
   }


 /*###################################################
  #END PROCS FOR CREATING A DAY VIEW
  ###################################################*/

 /** 
  * Generate HTML for calendar week days containing no items.
  */
public static String calendarWeekEmptyDays (CalendarInfo calendarInfo, Vector weekDays, int baseJulianDate, int startJulianDate, int endJulianDate) {
  String result = "";

  int startIndex = startJulianDate - baseJulianDate;
  int endIndex = endJulianDate - baseJulianDate;
  for (int index = startIndex; index <= endIndex; index++) {
    String[] row = (String[])weekDays.elementAt(index);
    String day = row[0];
    String prettyDay = row[1];
    String dayName = row[2];
    result += "<tr>";
    if (day.equals(calendarInfo.getCurrentDate())) {
      result += "<td bgcolor=FFF8DC>";
    } else {
      result += "<td bgcolor=DCDCDC>";
    }

    result += "<b>" + dayName + "</b>, " + 
	"<a href=" + calendarInfo.getDayLink() + calendarInfo.getViewModeVar() + "group_id=" + calendarInfo.getGroupId() + "&current_date=" + day + ">" + prettyDay + "</a> \n" +
	"&nbsp \n" +
	"<font size=-1> \n" +
	"<a href=" + calendarInfo.getAddLink() + calendarInfo.getViewModeVar() + "&current_date=" + day + "&start_date=" + day + "&action=item-add>Add Item</a> \n" +
	"</font> \n" +
	"</td> " + 
	"</tr> \n" +
	"<tr><td> \n" +
	"&nbsp; \n" +
	"</td></tr>";
  }
  return result;
}


/*
# Eventually this will be a standalone proc usable from other modules....
ad_proc -public calendar_add_edit_item { {-item_id "" -new_item_p "t"} \
	start_date, end_date, \
	title, creation_user, {description ""}} {
    Does the actual database work of adding or updating a calendar item.
    Assumes that all inputs have been validated.  
    This is intended to eventually used by other modules that want to add
    calendar items.
} {
    # TODO: What about altering info in cal_item_map on an update?
    db_transaction {
	# If new item, then try to do insert.
	# If the insert fails due to a unique
	# constraint, assume double-click and do an update.
	# Commit if either works.
	set succeeded 0
	set security_error_p "f"
	if {$new_item_p == "t" &&
	    [catch {
		db_dml cal_items_insert "insert into cal_items (item_id, start_date, end_date, title,
		description, creation_user, creation_date)
		values (:item_id, :start_date, :end_date, :title, :description,
		:creation_user, sysdate)"
	    } message] == 0} {
	    # No error, lets update the cal_item_map
	    db_dml cal_item_map_insert "insert into cal_item_map (item_id, user_id, group_id)
		values (:item_id, :user_id, :group_id)"
	    if {[db_resultrows] > 0} {
		set succeeded 1
	    }
	} else {
	    # We had an error on our first insert... if this is not a new item
	    # or it was a unique constraint error, try doing an update instead.
	    if {$new_item_p != "t" ||
	    [calendar_parse_oracle_error_for_unique_constraint_violation $message]} {
		# We do some security checking here -anyone can add a new item, 
		# but only certain people (owners or group admins) can modify
		# already existing items.

		set write_access_p [calendar_item_write_access_p $item_id $group_id]
		if {$write_access_p} {
		    # We've cleared security...

		    # NOTE that the cal_item_map is NOT
		    # presently updated in this case, so we
		    # can't change item ownership here.
		    db_dml cal_items_update "UPDATE cal_items
                               SET start_date=:start_date,
                                   end_date=:end_date,
                                   title=:title,
                                   description=:description,
                                   creation_user=:creation_user,
                                   creation_date=sysdate
                             WHERE item_id=:item_id"
		    if {[db_resultrows] > 0} {
			set succeeded 1
		    }
		} else {
		    set security_error_p "t"
		}
	    }
	}
	# I think the db_transaction API should take care of this aborting
	# for me, but it doesn't seem to yet.  Have e-mailed Jon Salz about it,
	# may be able to remove this check for abort if he agrees.
	if { $succeeded == 0} {
	    ns_log Notice "Calling db_abort_transaction"
	    db_abort_transaction
	}
    }
    db_release_unused_handles

    if {$succeeded} {
	return $item_id
    } else {
	return 0
    }
}

ad_proc calendar_days_of_week {day_nums {day_abbrevs "S M T W T F S"}} {
    set html ""
    set count 0
    foreach abbrev $day_abbrevs {
	set checked ""
	if {[lsearch $day_nums $count] != -1} {
	    set checked " checked"
	}
	append html "<input name=\"days_of_week\" type=checkbox value=$count$checked>&nbsp;[lindex $day_abbrevs $count]&nbsp;&nbsp;"
	incr count
    }
    return $html
}

# For each index, substitute corresponding value in replacements
ad_proc translate_numeric_list {indices replacements {joiner ", "}} {
    foreach index $indices {
	lappend result [lindex $replacements $index]
    }
    return [join $result $joiner]
}


*/

    /**
     * Gets id of calendar's file-storage folder for specified user.
     * folder with name equal to FileAttachmentsFolder from parameters file will
     * be created in the user's personal tree if it doesn't already exist.
     *
     * @param userId User Id.
     * @return file_id of the folder in the file-storage where calendar items are kept
     *         for the specified user.
     */
    public static Integer userCalendarFolderId (Integer userId) throws SQLException, ACSException {
	return calendarFolderId (userId, new Integer(0));
    }

    /**
     * Gets id of calendar's file-storage folder for specified group.
     * folder with name equal to FileAttachmentsFolder from parameters file and 
     * owned by userId will be created if it doesn't already exist.
     *
     * @param userId User Id.
     * @param groupId Group Id (will lookup user calendar if null or 0).
     * @return file_id of the folder in the file-storage where calendar items are kept
     *         for the specified group.
     */
    public static Integer calendarFolderId (Integer userId, Integer groupId) throws SQLException, ACSException {
	String attachmentsFolderName = Parameters.get("FileAttachmentsFolder", "calendar", "CalendarAttachments");
	Database db = new Database();
	String folderId;
	if (groupId == null || groupId.intValue() == 0) {
	    folderId = db.queryToString("select file_id " + 
					"from fs_files " + 
					"where file_title=:attachments_folder_name " + 
					"and owner_id=:user_id\n" +
					"and group_id IS NULL", new Object[] {attachmentsFolderName, userId}, "");
	} else {
	    folderId = db.queryToString("select file_id " + 
					"from fs_files \n" +
					"where file_title=:attachments_folder_name \n" +
					"and group_id=:group_id", new Object[] {attachmentsFolderName, groupId}, "");
	}
    
	if (! folderId.equals("")) {
	    return new Integer(folderId);
	}

	//folder doesn't exist and must be created
	String fileSeqId = db.queryToString("select fs_file_id_seq.nextval from dual");
	String versionSeqId = db.queryToString("select fs_version_id_seq.nextval from dual");

        db.beginTransaction();
	try {
	    if (groupId == null || groupId.intValue() == 0) {
		db.dml("insert into fs_files " + 
		       "(file_id, file_title, sort_key, depth, folder_p, owner_id) \n" +
		       "values \n" +
		       "(:file_seq_id, :attachments_folder_name, 10000, 0, 't', :user_id)", new Object[] {fileSeqId, attachmentsFolderName, userId});
	    } else {
		db.dml("insert into fs_files " + 
		       "(file_id, file_title, sort_key, depth, folder_p, owner_id, group_id) \n" +
		       "values \n" +
		       "(:file_seq_id, :attachments_folder_name, 10000, 0, 't', :user_id, :group_id)", new Object[] {fileSeqId, attachmentsFolderName, userId, groupId});
	    }

	    // now we want to insert a "dummy" version so that we can also create the permission
	    // records
	    db.dml("insert into fs_versions \n" +
		   "(version_id, file_id, creation_date, author_id) \n" +
		   "values \n" +
		   "(:version_seq_id, :file_seq_id, sysdate, :user_id)", new Object[] {versionSeqId, fileSeqId, userId});
	} catch (SQLException e) {
	    // Intercept long enough to abort & end transaction.
            db.abortTransaction();
	    throw e;
	} finally {
            db.endTransaction();
	}

	FileStorageProcs.orderFiles(db);
	db.close();
    
	return new Integer(fileSeqId);
    }
 
    /**
     * Initializes calinfo variable and populates it with information used on calendar pages.    
     * @param request Servlet request object, used to short circuit all this evaluation and just return the previously calculated calendarInfo object (if it exists).
     * @param viewMode "full" or "modular": view page with the minicalendar for navigation (full) or view page in isolation.
     */
    public static CalendarInfo initializeCalendarParameters (javax.servlet.http.HttpServletRequest request, Integer viewedUserId, UserInfo loggedInUser, Integer groupId, String currentView, String currentDate, String compressDayViewP, String viewMode, String action) throws SQLException, ACSException {
	// TODO: Require loggedinuser not null?  Work out details of when and where we are dependent on a login.

	// check if initialization was already done
	if (request.getAttribute("calendarInfo") != null) {
	    return ((CalendarInfo)request.getAttribute("calendarInfo"));
	}
	CalendarInfo calinfo = new CalendarInfo();
	//if (apm_package_enabled_p("portals") && portal_page_p()) {
	String groupIdFromPortals = PortalsProcs.portalsParam("group_id");
	String defaultCompressDayViewP;
	if (!Utilities.emptyString(groupIdFromPortals)) { 
	    defaultCompressDayViewP = "t";
	} else {
	    defaultCompressDayViewP = "f";
	}
	//} else {
	//groupIdFromPortals = "";
	//defaultCompressDayViewP = "f";
	//}

	if (groupId == null) {
	    if (Utilities.emptyString(groupIdFromPortals)) {
		groupId = new Integer(0);
	    } else {
		groupId = new Integer(groupIdFromPortals);
	    }
	}
	if (viewMode == null) {
	    viewMode = "modular";
	}
	if (action == null) {
	    action = "view";
	}
	if (currentView == null) {
	    currentView = "";
	}

	Database db = (Database)request.getAttribute("db");
	if (db == null) {
	    db = new Database();
	}

	String prettyCurrentDate;
	if (currentDate == null || currentDate.equals("")) {
	    ResultSet rs = db.query("select to_char(trunc(sysdate), 'YYYY-MM-DD') as current_date, to_char(sysdate, 'Month DD, YYYY') as pretty_current_date from dual");
	    rs.next();
	    currentDate = rs.getString("current_date");
	    prettyCurrentDate = rs.getString("pretty_current_date");
	    rs.close();
	} else {
	    prettyCurrentDate = (String)MemoizeCache.lookup("calPrettyCurrentDate " + currentDate);
	    if (prettyCurrentDate == null) {
		prettyCurrentDate = calPrettyCurrentDate(currentDate, db);
		MemoizeCache.cache("calPrettyCurrentDate " + currentDate, prettyCurrentDate);
	    }
	}

	calinfo.viewedUserId = viewedUserId;
	calinfo.loggedInUser = loggedInUser;

	calinfo.currentDate = currentDate;
	calinfo.prettyCurrentDate = prettyCurrentDate;
	if (groupId.intValue()==0) {
	    calinfo.type = "user";
	    calinfo.groupName = "";
	    calinfo.prettyName = "";
	    calinfo.groupMemberP = false;
	    calinfo.groupAdminP = false;
	} else {
	    calinfo.type = "group";
	    calinfo.groupName = db.queryToString("select short_name from user_groups where group_id=:group_id", new Object[] {groupId});
	    calinfo.prettyName = db.queryToString("select group_name from user_groups where group_id=:group_id", new Object[] {groupId});
	    calinfo.groupMemberP = (loggedInUser != null)?(loggedInUser.isGroupMember(groupId)):false;
	    calinfo.groupAdminP = (loggedInUser != null)?(loggedInUser.isGroupAdmin(groupId) || calinfo.loggedInUser.isSiteWideAdmin()):false;
	}
	calinfo.groupId = groupId;

	if (loggedInUser != null) {
	    ResultSet userPrefs=db.query("select nvl(default_view, 'day') as default_view, \n" +
					 "nvl(compress_day_view_p, :default_compress_day_view_p) as default_compress_day_view_p, \n" +
					 "new_items_age \n" +
					 "from cal_user_prefs \n" +
					 "where user_id = :user_id", new Object[] {defaultCompressDayViewP, calinfo.loggedInUser.getUserId()});
	    userPrefs.next();
	    if (currentView == null || currentView.equals("")) {
		currentView=userPrefs.getString("default_view");
	    }
	    if (compressDayViewP == null) {
		compressDayViewP = userPrefs.getString("default_compress_day_view_p");
	    }
	    calinfo.newItemsAge = userPrefs.getString("new_items_age");
	    userPrefs.close();
	} else {
	    if (currentView == null || currentView.equals("")) {
		currentView="day";
	    }
	    if (compressDayViewP == null) {
		compressDayViewP = defaultCompressDayViewP;
	    }
	    // TODO: Default for new items age?
	}

	calinfo.currentView=currentView;
	calinfo.compressDayViewP = compressDayViewP;

	//  we should get rid of this update; it's performance killer. we should never heve dml statements on frequently visites page
	//    Tarik Alatovic, 07/20/2000
	//    
	//    db_dml "update cal_user_prefs
	//    set default_view = '$current_view'
	//    where user_id = $logged_in_user_id"

	if (groupId.intValue() != 0) {
	    calinfo.title = "Calendar for " + calinfo.prettyName;
	} else {
	    calinfo.title = "Calendar";
	    if (calinfo.loggedInUser != null) {
		calinfo.title += " for " + calinfo.loggedInUser.getFullName();
	    }
	}
	calinfo.viewMode = viewMode;
	calinfo.viewModeVar = "view_mode=" + viewMode + "&";
	calinfo.action = action;
    
	// To where should the add link go?
	if (viewMode.equals("full")) {
	    calinfo.addLink = "?current_date=" + currentDate + "&";
	} else {
	    //TODO: This /calendar should be configurable?
	    calinfo.addLink = "/calendar/?current_date=" + currentDate + "&";
	}

	if (groupId.intValue() > 0) {
	    calinfo.addLink += "scope=group&group_id=" + groupId + "&";
	}

	String currentViewLink;
	if (viewMode.equals("full")) {
	    currentViewLink = "?current_view=" + currentView + "&view_mode=full&";
	} else {
	    currentViewLink = "?view_mode=modular&";
	}
	calinfo.currentViewLink = currentViewLink;

	String dayLink;
	if (viewMode.equals("full")) {
	    dayLink = "/calendar/?current_view=day&";
	} else {
	    dayLink = "/calendar/?current_view=day&view_mode=modular&";
	}
	calinfo.dayLink=dayLink;
      Log.note("returning calinfo: " + calinfo);
	return calinfo;
    }

    /**
     * Returns view title bar (a small table with title & left & right linked arrows). 
     * Arrows link to different current dates depending on the present view mode.
     *
     * @param request Used for CalendarInfo object and request's db.
     * @return Title bar.
     */
    public static String calendarViewTitle (javax.servlet.http.HttpServletRequest request) throws SQLException, ACSException {
	CalendarInfo calinfo = (CalendarInfo)request.getAttribute("calendarInfo");
	if (calinfo == null) {
            throw new ACSException ("calendarViewTitle invoked without first calling initializeCalendarParameters");
	}
	String compressLink = "";
	Database db = (Database)request.getAttribute("db");
	if (db == null) {
	    Log.warn("PageContext DB variable was null, creating new Database object");
	    db = new Database();
	}
	String prettyCurrentItem, previousItem, nextItem;
	if (calinfo.currentView.equals("day")) {
	    compressLink = "&compress_day_view_p=" + calinfo.compressDayViewP;
	    ResultSet daySelect = db.query("select \n" +
					   "to_char(to_date('" + calinfo.currentDate + "'), 'fmDay, fmMonth fmDD, YYYY') as pretty_current_item, \n" +
					   "to_char((to_date('" + calinfo.currentDate + "') - 1), 'YYYY-MM-DD') as previous_item, \n" +
					   "to_char((to_date('" + calinfo.currentDate + "') + 1), 'YYYY-MM-DD') as next_item \n" +
					   "from dual");
	    daySelect.next();
	    prettyCurrentItem = daySelect.getString("pretty_current_item");
	    previousItem = daySelect.getString("previous_item");
	    nextItem = daySelect.getString("next_item");
	    daySelect.close();
	} else if (calinfo.currentView.equals("week")) {
	    ResultSet weekSelect = db.query("select \n" +
					    "to_char(next_day(to_date('" + calinfo.currentDate + "') - 7, 'SUNDAY'), 'Month fmDD, YYYY') || ' - ' || to_char(next_day(to_date('" + calinfo.currentDate + "'), 'SATURDAY'), 'Month fmDD, YYYY') as pretty_current_item, \n" +
					    "to_char((to_date('" + calinfo.currentDate + "') - 7), 'YYYY-MM-DD') as previous_item, \n" +
					    "to_char((to_date('" + calinfo.currentDate + "') + 7), 'YYYY-MM-DD') as next_item \n" +
					    "from dual");
	    weekSelect.next();
	    prettyCurrentItem = weekSelect.getString("pretty_current_item");
	    previousItem = weekSelect.getString("previous_item");
	    nextItem = weekSelect.getString("next_item");	    
	    weekSelect.close();
	} else if (calinfo.currentView.equals("month")) {
	    ResultSet monthSelect = db.query("select " + 
					     "to_char(to_date('" + calinfo.currentDate + "'), 'fmMonth') || ' ' || to_char(to_date('" + calinfo.currentDate + "'), 'YYYY') as pretty_current_item, \n" +
					     "to_char(trunc(add_months(to_date('" + calinfo.currentDate + "'), 1)), 'YYYY-MM-DD') as next_item, \n" +
					     "to_char(trunc(add_months(to_date('" + calinfo.currentDate + "'), -1)), 'YYYY-MM-DD') as previous_item \n" +
					     "from dual");
	    monthSelect.next();
	    prettyCurrentItem = monthSelect.getString("pretty_current_item");
	    previousItem = monthSelect.getString("previous_item");
	    nextItem = monthSelect.getString("next_item");
	    monthSelect.close();
	} else if (calinfo.currentView.equals("year")) { 
	    ResultSet yearSelect = db.query("select \n" +
					    "to_char(to_date('" + calinfo.currentDate + "'), 'YYYY') as pretty_current_item, \n" +
					    "to_char(add_months(to_date('" + calinfo.currentDate + "'), -12), 'YYYY-MM-DD') as previous_item, \n" +
					    "to_char(add_months(to_date('" + calinfo.currentDate + "'), 12), 'YYYY-MM-DD') as next_item \n" +
					    "from dual");
	    yearSelect.next();
	    prettyCurrentItem = yearSelect.getString("pretty_current_item");
	    previousItem = yearSelect.getString("previous_item");
	    nextItem = yearSelect.getString("next_item");
	    yearSelect.close();
	} else {
	    throw new ACSException("calendarViewTitle: Invalid view '" + calinfo.currentView + "'");
	}
    
        return ("<table cellpadding=3 cellspacing=0 border=0 width=90%> \n" +
		"<tr bgcolor=lavender> \n" +
		"<td align=center> \n" +
		"<a href=" + calinfo.currentViewLink + "group_id=" + calinfo.groupId + "&current_view=" + calinfo.currentView + "&current_date=" + previousItem + compressLink + ">" + 
		"<img border=0 src=/graphics/left.gif></a> \n" +
		"<FONT face=\"Arial,Helvetica\" SIZE=-1> \n" +
		"<B>" + prettyCurrentItem + "</B> \n" +
		"</FONT> \n" +
		"<a href=" + calinfo.currentViewLink + "group_id=" + calinfo.groupId + "&current_view=" + calinfo.currentView + "&current_date=" + nextItem + compressLink + "> \n" +
		"<img border=0 src=/graphics/right.gif></a> \n" +
		"</td> \n" +
		"</tr> \n" +
		"</table>");
	}

    /**
     * Yet another pretty date function.
     * @param current_date Date in Oracle YYYY-MM-DD format.
     * @return Date in 'Month DD, YYYY' format.
     */
    public static String calPrettyCurrentDate (String current_date, Database db) throws SQLException, ACSException {
         return db.queryToString("select to_char(to_date(:current_date), 'Month DD, YYYY') from dual", new Object[] {current_date});
    }

    /**
     *     Displays calendar items for a particular day of the month for user or group.
     */
    public static String displayMonthItem (CalendarInfo calendarInfo, String startDate) throws SQLException {
	Database db = new Database();
	StringBuffer dayItemsHTML = new StringBuffer("<table border=0 width=100%>");
	ResultSet calendarDayItems = db.query("select i.item_id, ii.instance_id, combine_category_names(ii.instance_id, ', ') as category_list, \n" +
	   " ug2.group_id as item_group_id, ug2.short_name as item_group_name, \n" +
           "ug2.short_name as item_group_short_name, ug2.group_type as item_group_type, \n" +
 	   "decode(m.group_id, NULL, 'f', 't') as group_p, \n" +
	   "to_char(start_date, 'YYYY-MM-DD HH24:MI') as start_time, \n" +
	   "to_char(start_date, 'fmHH:fmMIam') as pretty_start_time, \n" +
	   "to_char(start_date, 'Month, DD YYYY') as pretty_start_day, \n" +
	   "to_char(end_date, 'YYYY-MM-DD HH24:MI') as end_time, \n" +
	   "to_char(end_date, 'fmHH:fmMIam') as pretty_end_time, \n" +
	   "to_char(end_date, 'Month, DD YYYY') as pretty_end_day, \n" +
	   "nvl(ii.title, i.title) as title, \n" +
           "ug.short_name(+) as group_name \n" +
	   "from cal_items i, cal_item_instances ii, cal_item_map m, user_groups ug, user_groups ug2 \n" +
	   "where m.group_id = ug2.group_id(+) \n" +
           "  and i.item_id = ii.item_id \n" +
           "and to_date(:start_date) = trunc(start_date) \n" +
           "and ((:group_id = 0 \n" +
	   "      and i.item_id = m.item_id  \n" +
 	   "      and (m.user_id = :user_id \n" +
	   "      or m.group_id in  \n" +
	   "      (select ugm.group_id \n" +
	   "       from user_group_map ugm, cal_user_update_prefs ugp \n" +
	   "       where ugm.user_id = :user_id \n" +
	   "       and ugp.user_id = ugm.user_id \n" +
	   "	  and ugm.group_id = ugp.group_id \n" +
	   "	  and ugp.update_type = 'automatically') \n" +
	   "      ) \n" +
	   "     ) \n" +
	   "     or \n" +
	   "    (i.item_id = m.item_id \n" +
	   "     and m.group_id = :group_id) \n" +
	   "    ) \n" +
           "and m.group_id=ug.group_id(+) \n" +
	   "order by start_date", new Object[] {startDate, calendarInfo.getGroupId(), calendarInfo.getLoggedInUserIdOr0(), calendarInfo.getLoggedInUserIdOr0(), calendarInfo.getGroupId()});
	int count = 0;
	while (calendarDayItems.next()) {
	    count++;
	    dayItemsHTML.append("<tr>" + 
	                        "<td>\n" +
	                        "<font size=-1>");
	    if (!Utilities.emptyString(calendarDayItems.getString("end_time"))) {
		dayItemsHTML.append("<b>" + calendarDayItems.getString("pretty_start_time") + "</b>");
	    } else {
		// append day_items_html "<li>"
	    }
	
	    if ( calendarDayItems.getString("group_p").equals("f") || calendarInfo.getType().equals("group") ) {
		dayItemsHTML.append("<a href=" + calendarInfo.getAddLink() + calendarInfo.getViewModeVar() + "action=item-view&item_id=" + calendarDayItems.getString("item_id") + "&instance_id=" + calendarDayItems.getString("instance_id") + "&current_view=" + calendarInfo.getCurrentView() + ">" + calendarDayItems.getString("title") + "</a>" + 
		                    "</font>" + 
		                    "</td>" + 
		                    "</tr>");
	    } else {
		dayItemsHTML.append("<a href=[ad_calendar add_link][ad_calendar view_mode_var]action=item-view&item_id=$item_id&instance_id=$instance_id&current_view=[ad_calendar current_view]>$title (for $group_name)</a>" + 
	           "</font>" + 
	           "</td>" + 
	           "</tr>");
	    } 
	}
	if (count == 0) {
	    return "";
	} else {
	    dayItemsHTML.append("</table>");
	    return dayItemsHTML.toString();
	}
    }

    /** 
    * Generate HTML for calendar month days containing no items.
    */
    public static String calendarMonthEmptyDays (CalendarInfo calendarInfo, String month, Vector monthList, int baseJulianDate, int startJulianDate, int endJulianDate) {
	StringBuffer result = new StringBuffer();
	int startIndex = startJulianDate - baseJulianDate;
	int endIndex = endJulianDate - baseJulianDate;
	for (int index = startIndex; index <= endIndex; index++) {
	    String[] row = (String[])monthList.elementAt(index);
	    String theDate = row[0];
	    String theMonth = row[1];
	    String prettyDate = row[2];
	    String weekday = row[3];
	    if (weekday.equals("Sunday")) {
		result.append("<tr>\n");
	    }
	    if (theDate.equals(calendarInfo.getCurrentDate())) { 
		result.append("<td width=\"14%\" valign=top bgcolor=FFF8DC>");
	    } else if (!theMonth.equals(month)) { 
		result.append("<td width=\"14%\" valign=top bgcolor=DCDCDC>");
	    } else if (!theDate.equals(calendarInfo.getCurrentDate())) { 
		result.append("<td width=\"14%\" valign=top>");
	    }
	    result.append("<a href=" + calendarInfo.getDayLink() + "current_date=" + theDate + "&group_id=" + calendarInfo.getGroupId() + ">" + prettyDate + "</a></td>\n");
	    if (weekday.equals("Saturday")) {
		result.append("</tr>\n");
	    }
	}
	return result.toString();
    }

  /**
   *     This clause is used to control display of group vs. user items for all the views.  This defines it in one central location. 
   *     Note that this isn't actually used everywhere; not sure why (-Eison)
   */
  private static String getUserGroupSQL (Integer userId, Integer groupId) {
    return "((" + groupId + " = 0 \n" +
      "and i.item_id = m.item_id \n" +
      "and (m.user_id = " + userId +
      "     or m.group_id in  \n" +
      "     (select ugm.group_id \n" +
      "      from user_group_map ugm, cal_user_update_prefs ugp \n" +
      "      where ugm.user_id = " + userId +
      "      and ugp.user_id = ugm.user_id \n" +
      "      and ugm.group_id = ugp.group_id \n" +
      "      and ugp.update_type = 'automatically') \n" +
      "      ) \n" +
      "    ) \n" +
      " or (i.item_id = m.item_id \n" +
      "     and m.group_id = " + groupId + ") \n" +
      ") ";
  }

    /**
     * Displays list of calendar items for the next upcoming_items_days and in to the past for past_items_day.  Displays the first upcoming_items_length number of items.
     */
  public static String displayUpcoming (CalendarInfo calendarInfo, Integer upcomingItemsDays, Integer pastItemsDays, String upcomingItemsLength, String orderby) throws SQLException {
      Database db = new Database();
      boolean selectDays = false;
      boolean selectLength = false;
      if (Utilities.emptyString(orderby)) {
	orderby = "orderby_category";
      }
      
      Vector prefsForSelect = new Vector();
      if (upcomingItemsDays == null || upcomingItemsDays.intValue() == 0) {
	selectDays = true;
	prefsForSelect.add("upcoming_items_days");
      }
      if (Utilities.emptyString(upcomingItemsLength)) {
	selectLength = true;
	prefsForSelect.add("upcomingItemsLength");
      }
      if (prefsForSelect.size() > 0) {
	ResultSet rs = db.query("select " + Utilities.join(prefsForSelect, ",") + 
		 "from cal_user_prefs \n" +
		 "where user_id = [ad_calendar logged_in_user_id]");
	rs.next();
	if (selectDays) {
	  upcomingItemsDays = new Integer(rs.getString("upcoming_items_days"));
	}
	if (selectLength) {
	  upcomingItemsLength = rs.getString("upcoming_items_length");
	}
	rs.close();
      }

      String lengthWhere ="";
      if (!Utilities.emptyString(upcomingItemsLength)) {
	lengthWhere = "where rownum < = " + upcomingItemsLength;
      }

      ResultSet upcomingDateSelect = db.query("select \n" +
					      " to_char((to_date(:current_date) + :upcoming_items_days), 'Month DD, YYYY') as upcoming_end_pretty_date, \n" +
					      " to_char((to_date(:current_date) - :past_items_days), 'Month DD, YYYY') as upcoming_start_pretty_date \n" +
					      " from dual", new Object[] {calendarInfo.getCurrentDate(), upcomingItemsDays, calendarInfo.getCurrentDate(), pastItemsDays});
      upcomingDateSelect.next();
      String upcomingEndPrettyDate = upcomingDateSelect.getString("upcoming_end_pretty_date");
      String upcomingStartPrettyDate = upcomingDateSelect.getString("upcoming_start_pretty_date");
      upcomingDateSelect.close();

      // TODO: Write ad_table and use this table_def.
      /*set table_def {
	{day_start_day {Day of Week} {} c}
	{pretty_start_day {Date} {start_date $order} c}
	{pretty_start_time {Start Time} {orderby_start_time $order} {<td align=center>[
	if [empty_string_p $pretty_end_time] {
	    return "No Time"
	} else {
	    return $pretty_start_time
	}]</td>} }
	{pretty_end_time {End Time} {orderby_end_time $order} {} c}
	{item_group_name {Group Name} {} \
		{<td>[cal_group_home_link $item_group_type $item_group_short_name $item_group_name]</td>} }
	{title {Title} {} {<td>[
	if {$related_url_p == "t"} {
	    return "[cal_related_url_html $title "" $related_url $related_url_text]"
	} else {
	    return "<a href=[ad_calendar add_link]action=item-view&item_id=$item_id&instance_id=$instance_id&current_view=[ad_calendar current_view]>$title</a>"
	}]</td>} }
	{orderby_category {Categories} {} c}
	}*/

      ResultSet calUpcomingItems = db.query("select * from (select i.item_id, ii.instance_id, \n" +
          "ug.group_id as item_group_id, ug.short_name as item_group_name, ug.short_name as item_group_short_name, ug.group_type as item_group_type, \n" +
          "combine_category_names(ii.instance_id, '<BR>') as item_categories, \n" +
          "category as orderby_category, \n" +
          "decode(m.user_id, NULL, 'f', 't') as user_p, \n" +
          "start_date, \n" +
          "to_char(start_date, 'MM/DD/YY') as pretty_start_day, \n" +
	  "to_char(start_date, 'Day') as day_start_day, \n" +
          "to_char(end_date, 'fmMonth fmDD, YYYY') as pretty_end_day, \n" +
          "to_char(end_date, 'Day') as day_end_day, \n" +
          "to_char(start_date, 'fmHH:fmMIam') as pretty_start_time, \n" +
          "to_char(end_date, 'fmHH:fmMIam') as pretty_end_time, \n" +
          "to_char(start_date, 'amHH:MI') as orderby_start_time, \n" +
          "to_char(end_date, 'amHH:MI') as orderby_end_time, \n" +
	  "nvl(ii.title, i.title) as title, m.user_id, \n" +
          "nvl(ii.related_url, i.related_url) as related_url,  \n" +
          "nvl(ii.related_url_text, i.related_url_text) as related_url_text,  \n" +
          "nvl(ii.related_url_p, i.related_url_p) as related_url_p \n" +
          "from cal_items i, cal_item_instances ii, cal_item_map m,  \n" +
          "     cal_user_prefs u, user_groups ug, \n" +
          "     cal_categories cc, cal_category_map ccm \n" +
	  "where ug.group_id(+) = m.group_id \n" +
          "  and u.user_id = :loggedInUserId \n" +
          "  and i.item_id = ii.item_id   \n" +
	  "  and u.list_upcoming_p = 't' \n" +
  	  "  and i.item_id = m.item_id \n" +
	  "  and start_date between  \n" +
          "        to_date(:current_date) and \n" +
 	  "	  to_date(:current_date) + :upcoming_items_days \n" +
	  "   and \n" + getUserGroupSQL(calendarInfo.getViewedUserId(), calendarInfo.getGroupId()) +
          "   and ii.instance_id = ccm.instance_id(+) \n" +
          "   and ccm.category_id = cc.category_id(+)) \n" +
              lengthWhere + /*[ad_order_by_from_sort_spec $orderby $table_def]*/ " order by orderby_start_time",
	new Object[] { calendarInfo.getLoggedInUserIdOr0(), calendarInfo.getCurrentDate(), calendarInfo.getCurrentDate(), upcomingItemsDays } );

    
    /* set rowcode {}
    set remove_column [list]
    switch [ad_sort_primary_key $orderby] {
	orderby_category {
	    lappend remove_column orderby_category
	    set rowcode {
		[if {![ad_table_same orderby_category]} {
		    ad_table_span [switch $orderby_category {
			"" {subst {<b>No Category</b>}}
			default {subst {<b>Category: $orderby_category</b>}}
		    }] "bgcolor=cccccc"
		}]
		}
	    }
	}


	#
	# generate the columns we really want to see given the list of removes
	#
	
	set all_columns {}
	foreach column $table_def {
	    lappend all_columns [lindex $column 0]
	}
	set col [list]
	foreach acol $all_columns { 
	    if {[lsearch -exact $remove_column $acol] < 0} { 
		lappend col $acol
	    }
	}

	set upcoming_items_html [ad_table -Taudit {instance_id} -Tcolumns $col -Tpre_row_code $rowcode -Torderby $orderby -Tmissing_text "" cal_upcoming_items  $sql_query $table_def]*/

	int count = 0;
	StringBuffer upcomingItemsHTML = new StringBuffer();

	while (calUpcomingItems.next()) {
	    count++;
	    upcomingItemsHTML.append("<tr>");
	    upcomingItemsHTML.append("<td>");
	    upcomingItemsHTML.append(calUpcomingItems.getString("day_start_day"));
	    upcomingItemsHTML.append("</td><td>");
	    upcomingItemsHTML.append(calUpcomingItems.getString("pretty_start_day"));
	    upcomingItemsHTML.append("</td><td align=center>");
	    if (Utilities.emptyString(calUpcomingItems.getString("pretty_end_time"))) {
		upcomingItemsHTML.append("No Time");
	    } else {
		upcomingItemsHTML.append(calUpcomingItems.getString("pretty_start_time"));
	    }
	    upcomingItemsHTML.append("</td><td>");
	    upcomingItemsHTML.append(calUpcomingItems.getString("pretty_end_time"));
	    upcomingItemsHTML.append("</td><td>");
	    upcomingItemsHTML.append(getGroupHomeLink(calUpcomingItems.getString("item_group_type"), calUpcomingItems.getString("item_group_short_name"), calUpcomingItems.getString("item_group_name")));
	    upcomingItemsHTML.append("</td><td>");
	    if (calUpcomingItems.getString("related_url_p").equals("t")) {
		upcomingItemsHTML.append(calRelatedURLHtml(calUpcomingItems.getString("title"), "", calUpcomingItems.getString("related_url"), calUpcomingItems.getString("related_url_text")));
	    } else {
		upcomingItemsHTML.append("<a href=" + calendarInfo.getAddLink() + "action=item-view&item_id=" + calUpcomingItems.getString("item_id") + "&instance_id=" + calUpcomingItems.getString("instance_id") + "&current_view=" + calendarInfo.getCurrentView() + ">" + calUpcomingItems.getString("title") + "</a>");
	    }
	    upcomingItemsHTML.append("</td>");
	    upcomingItemsHTML.append("</tr>");
	}
	calUpcomingItems.close();

	String pageContent = "";
	if (count == 0) {
	    pageContent += "<ul><li><em>No upcoming items.</em></ul>";
	} else {
	    pageContent += "<b>Upcoming Items: " + upcomingStartPrettyDate;
	    if ( upcomingItemsDays.intValue() > 1 ) {
		pageContent += " - " + upcomingEndPrettyDate;
	    }
	
	    pageContent += "</b> \n" +
	                   "<blockquote> \n" +
	                   "<table>" + upcomingItemsHTML + "</table>\n" +
	                   "</blockquote> \n" +
	                   "<br>";
	}
	return pageContent;
    }
    

    public static String displayNew (CalendarInfo calendarInfo, Integer newItemsAge) throws SQLException {
	return displayNew(calendarInfo, newItemsAge, null);
    }

    /**
     *     Display items added in the last new_items_age days.
     *     upcoming_items_length  and past_items_days were dropped because it was not used.  orderBy is currently ignored.
     */
    public static String displayNew (CalendarInfo calendarInfo, Integer newItemsAge,  String orderBy) throws SQLException {

      Database db = new Database();

	if (Utilities.emptyString(orderBy)) {
	    orderBy = "orderby_category";
	}
    
	/* if n_past_days exists, use it.
	if {[apm_package_enabled_p "education"]} {
	    set new_items_age [edu_get_n_past_days]
	} else {
	    set new_items_age [ad_calendar new_items_age]
	}*/
	    
	int numNewItems = 0;

	//TODO: Use this def with ad_table
	/*set table_def {
	{day_start_day {Day of Week} {} c}
	{pretty_start_day {Date} {start_date $order} c}
	{pretty_start_time {Start Time} {orderby_start_time $order} {<td align=center>[
	if [empty_string_p $pretty_end_time] {
	    return "No Time"
	} else {
	    return $pretty_start_time
	}]</td>} }
	{pretty_end_time {End Time} {orderby_end_time $order} c}
	{item_group_name {Group Name} {} \
		{<td>[cal_group_home_link $item_group_type $item_group_short_name $item_group_name]</td>} }
	{title {Title} {} {<td>[
	if {$related_url_p == "t"} {
	    return "[cal_related_url_html $title "" $related_url $related_url_text]"
	} else {
	    return "<a href=[ad_calendar add_link]action=item-view&item_id=$item_id&instance_id=$instance_id&current_view=[ad_calendar current_view]>$title</a>"
	}]</td>} }
	{orderby_category {Categories} {} c}
	}*/

	ResultSet calNewItems = db.query("select i.item_id, ii.instance_id, \n" +
					 "ug.group_id as item_group_id, ug.short_name as item_group_name, ug.short_name as item_group_short_name, ug.group_type as item_group_type, \n" +
					 "combine_category_names(ii.instance_id, '<BR>') as item_categories, \n" +
					 "category as orderby_category, \n" +
					 "decode(m.user_id, NULL, 'f', 't') as user_p, \n" +
					 "start_date, \n" +
					 "to_char(start_date, 'MM/DD/YY') as pretty_start_day, \n" +
					 "to_char(start_date, 'Day') as day_start_day, \n" +
					 "to_char(end_date, 'fmMonth fmDD, YYYY') as pretty_end_day, \n" +
					 "to_char(end_date, 'Day') as day_end_day, \n" +
					 "to_char(start_date, 'fmHH:fmMIam') as pretty_start_time, \n" +
					 "to_char(end_date, 'fmHH:fmMIam') as pretty_end_time, \n" +
					 "to_char(start_date, 'amHH:MI') as orderby_start_time, \n" +
					 "to_char(end_date, 'amHH:MI') as orderby_end_time, \n" +
					 "nvl(ii.title, i.title) as title, \n" +
					 "m.user_id, \n" +
					 "nvl(ii.related_url, i.related_url) as related_url,  \n" +
					 "nvl(ii.related_url_text, i.related_url_text) as related_url_text,  \n" +
					 "nvl(ii.related_url_p, i.related_url_p) as related_url_p \n" +
					 "from cal_items i, cal_item_instances ii, cal_item_map m, cal_user_prefs u, \n" +
					 "     user_groups ug, \n" +
					 "     cal_categories cc, cal_category_map ccm \n" +
					 "where ug.group_id(+) = m.group_id \n" +
					 "  and u.user_id = :logged_in_user_id \n" +
					 "  and i.item_id = ii.item_id \n" +
					 "and u.list_new_p = 't' \n" +
					 "and i.item_id = m.item_id \n" +
					 "and i.creation_date > sysdate - :new_items_age \n" +
					 "and " + getUserGroupSQL(calendarInfo.getViewedUserId(), calendarInfo.getGroupId()) +
					 "and ii.instance_id=ccm.instance_id(+) \n" +
					 "and cc.category_id(+)=ccm.category_id \n" +
					 /*[ad_order_by_from_sort_spec $orderby $table_def]*/ "order by orderby_start_time",
					 new Object[] { calendarInfo.getViewedUserId(), newItemsAge } );

	/*set rowcode {}
    set remove_column [list]
    switch [ad_sort_primary_key $orderby] {
	orderby_category {
	    lappend remove_column orderby_category
	    set rowcode {
		[if {![ad_table_same orderby_category]} {
		    ad_table_span [switch $orderby_category {
			"" {subst {<b>No Category</b>}}
			default {subst {<b>Category: $orderby_category</b>}}
		    }] "bgcolor=cccccc"
		}]
		}
	    }
	}


	#
	# generate the columns we really want to see given the list of removes
	#
	
	set all_columns {}
	foreach column $table_def {
	    lappend all_columns [lindex $column 0]
	}
	set col [list]
	foreach acol $all_columns { 
	    if {[lsearch -exact $remove_column $acol] < 0} { 
		lappend col $acol
	    }
	}

	set new_items_html [ad_table -Taudit {instance_id} -Tcolumns $col -Tpre_row_code $rowcode -Torderby $orderby -Tmissing_text "" cal_new_items $sql_query $table_def]*/

	int count = 0;
	StringBuffer newItemsHTML = new StringBuffer();
	while (calNewItems.next()) {
	    count++;
	    newItemsHTML.append("<tr>");
	    newItemsHTML.append("<td>");
	    newItemsHTML.append(calNewItems.getString("day_start_day"));
	    newItemsHTML.append("</td><td>");
	    newItemsHTML.append(calNewItems.getString("pretty_start_day"));
	    newItemsHTML.append("</td><td align=center>");
	    if (Utilities.emptyString(calNewItems.getString("pretty_end_time"))) {
		newItemsHTML.append("No Time");
	    } else {
		newItemsHTML.append(calNewItems.getString("pretty_start_time"));
	    }
	    newItemsHTML.append("</td><td>");
	    newItemsHTML.append(calNewItems.getString("pretty_end_time"));
	    newItemsHTML.append("</td><td>");
	    newItemsHTML.append(getGroupHomeLink(calNewItems.getString("item_group_type"), calNewItems.getString("item_group_short_name"), calNewItems.getString("item_group_name")));
	    newItemsHTML.append("</td><td>");
	    if (calNewItems.getString("related_url_p").equals("t")) {
		newItemsHTML.append(calRelatedURLHtml(calNewItems.getString("title"), "", calNewItems.getString("related_url"), calNewItems.getString("related_url_text")));
	    } else {
		newItemsHTML.append("<a href=" + calendarInfo.getAddLink() + "action=item-view&item_id=" + calNewItems.getString("item_id") + "&instance_id=" + calNewItems.getString("instance_id") + "&current_view=" + calendarInfo.getCurrentView() + ">" + calNewItems.getString("title") + "</a>");
	    }
	    newItemsHTML.append("</td>");
	    newItemsHTML.append("</tr>");
	}

	String newItemsHeader;
	if (count == 0) {
	    newItemsHeader = "No items";
	} else {
	    newItemsHeader = "Items";
	}
	newItemsHeader += " added in the ";
	if (newItemsAge.intValue() == 1) {
	    newItemsHeader += "last day";
	} else {
	    newItemsHeader += "last " + newItemsAge + " days";
	}

	String pageContent = "";
	if (count == 0) {
	    pageContent += "<ul><li><em>" + newItemsHeader + "</em></ul>";
	} else {
	    pageContent += "<b>" + newItemsHeader + "</b>" + 
		"<blockquote> \n"  + 
		"<table>" + newItemsHTML + "</table> \n" +
		"</blockquote>";
	}
	/*
    # We should put in another slider, but for now just do nothing.
    if {[apm_package_enabled_p "education"]} {
	append page_content "
	<blockquote>[edu_new_items_slider]</blockquote>
	"
	}*/
    return pageContent;
    }

  /**
   * Possibly generate an HTML link on group_name to the preferred home for
   * the group.  If portals are enabled, link to group portal, otherwise 
   * don't link.
   */
  private static String getGroupHomeLink (String groupType, String shortName, String groupName) {
      if (Utilities.emptyString(shortName)) {
	  return "";
      }
      // TODO: Support portals
      /*if [apm_package_enabled_p "portals"] {
	set group_type_url_root [portals_get_url_root $group_type]	 
	return "<a href=\"/[ad_urlencode $group_type_url_root]/[ad_urlencode $short_name]/\">$group_name</a>"
      } else {*/
	  return groupName;
      /* } */
   }

   public static boolean isWriteAccessAllowed (UserInfo user, Integer itemId) throws SQLException, ACSException {
       return isWriteAccessAllowed(user, itemId, null);
   }

   /**
    * Replaces calendar_item_write_access_p.
    * Checks write access for the given user on the given item.
    */
   public static boolean isWriteAccessAllowed (UserInfo user, Integer itemId, Integer groupId) throws SQLException, ACSException {
       Database db = new Database();
       String writePrivilege = db.queryToString("select 1 " +
                                      "from cal_user_write_privilege " +
				      "where user_id = :user_id " +
				      "and item_id = :item_id ", new Object[] {user.getUserId(), itemId}, "0");
       if (groupId != null && groupId.intValue() != 0 && writePrivilege != "1") {
	   return user.isGroupAdmin(groupId);
       } else {
	   return (writePrivilege.equals("1"));
       }
    }

    /**
     * Returns a string w/ leading zeros trimmed.
     * Used for compatibility with TCL versions; TCL
     * interprets leading zeros as octal.
     */
    public static String trimLeadingZeros (String str) {
	// trim leading zeros
	while (str.startsWith("0")) {
	    // This is inefficient for lots of leading zeros, but we're not dealing with lots.
	    // This will be quicker than making a new regexp matcher.
	    str = str.substring(1);
	}
	if (Utilities.emptyString(str)) {
	    // ... but not all of the zeros
	    str = "0";
	}
	return str;
    }


    /**
     * Parses the passed in string into a list of year, month, day, hour, minutes, seconds.
     * Assumes AD standard "YYYY-MM-DD HH24:MI:SS", allows specifying single-digit values
     * and leaving off information to the right side of the string (i.e. you can leave off
     * the seconds).
     * Values default to 1 for MM/DD and 0 for HH/MI/SS if not found in string.
     * Leading zeroes will be stripped to stay compatible w/ TCL version (TCL interprets these in Octal).
     * NOTE that no validation is done - it's entirely possible to enter an invalid date such
     * as Feb 31.  Oracle will check for this sort of stupidity on insert and throw an error
     * if it happens.  This is a parser, not a validator.
     */
    public static int[] parseDateTimeString (String dateTimeString) {
        // Setup defaults
        int[] result = new int[6];
        result[0] = 0; // year
        result[1] = 1; // month
        result[2] = 1; // day
        result[3] = 0; // hours
        result[4] = 0; // minutes
        result[5] = 0; // seconds
        
        // Split into year/month/day/hours/minutes/seconds
        // This ugly regexp uses
        // non-reporting parens to group stuff that may not be included on the right hand side.  It's ugly, but
        // basically it lets the user terminate the string early.
        Perl5Util re = new Perl5Util();
        if (dateTimeString == null || ! re.match("/^\\s*([0-9][0-9][0-9][0-9])(?:-([0-1]?[0-9])(?:-([0-3]?[0-9])(?:\\s*([0-2]?[0-9])(?::([0-5]?[0-9])(?::([0-5]?[0-9]))?)?)?)?)?\\s*$/", dateTimeString)) { 
            Log.error("CalendarProcs.parseDateTimeString: Regexp Fails pulling apart default time '" + dateTimeString + "' - (looking for YYYY-MM-DD HH24:MI:SS, with a bit of flexibility for single digits and leaving information off the end)");
            // We could fall back on Oracle here and throw this at their date parser... or we could give up.
            // For now, we're giving up, to limit the scope creep in this proc.
            return result;
        }
        // NOTE that re.groups() returns # of matches PLUS ONE (the "0" match for the whole string), so we begin counting at 1 and set i-1 (since we don't care about the 0th match)
        // So we will be going from 1 to 6, inclusive, and cutting off at 7.
        int i;
        for (i = 1; i < re.groups(); i++) {
            String tmp = re.group(i);
            if (tmp != null) {
                result[i-1] = Integer.parseInt(tmp);
            }
        }
        return result;
    }

    /** 
     * Simple stupid utility macro for building up a sequence of selects in a string.
     */
    public static void utilAddToSQLBuffer(StringBuffer buf, String str) {
        if (buf.length() == 0) {
            // Nothing yet, start with a select
            buf.append("select " + str);
        } else {
            // Some SQL exists, append another column
            buf.append(", " + str);
        }
    }

    public static String getWidgetDatetime (String name, String value, String granularity) throws SQLException, ACSException {
        return getWidgetDatetime (name, value, granularity, false);
    }

    public static String getWidgetDatetime (String name, String value, String granularity, boolean useAmPm) throws SQLException, ACSException {
        return getWidgetDatetime (name, value, granularity, useAmPm, true);
    }

    public static String getWidgetDatetime (String name, String value, String granularity, boolean useAmPm, boolean showDate) throws SQLException, ACSException {
        return getWidgetDatetime (name, value, granularity, useAmPm, showDate, "&nbsp;");
    }

    /**
     * Yet another date/time widget!  You'd think we'd have a super-duper canonical version somewhere in the ACS.
     *  Based on First Tuesday's ad_form_widget_datetime (/web/ftdev/tcl/template/ad-form-widget.tcl
     *  on dev0103-001 as of 6/29/00) which is the templating system's standard
     *  datetime but with granularity control.
     *  granularity can be "months" "days" "hours" "halves" "quarters" "fives" "minutes" or "seconds"
     *  You cannot ask for granularity less fine than months (presently)
     *  Presently no support for timezones - all times are just taken and spewed directly, do
     *  any TZ conversion on the other side of this proc.
     *  Parameters other than granularity are obvious.
     *  End result is a text string containing HTML widgets with names
     *  "$name.year", "$name.month", "$name.day", "$name.hours", "$name.minutes", "$name.seconds",
     *  and "$name.ampm".  These will be numeric ("ampm" is 0 for am, 1 for pm)
     *  All HTML widgets will always be output *except* if show_date is 0; they will be hidden with if not needed
     *  to satisfy the current granularity level.
     *  Values default to 1 for MM/DD and 0 for HH/MI/SS/AM if not found in input
     *  string or if below granularity threshold.  This means
     *  you can pass the results straight to a stupid processor and it can readily
     *  convert to the proper Oracle date string just by stringing everything together.
     *
     * TODO: This really begs to be its own little class.
     */
    public static String getWidgetDatetime (String name, String value, String granularity, boolean useAmPm, boolean showDate, String dateTimeSep) throws SQLException, ACSException {
        int toPrecision;
        if (Utilities.emptyString(granularity)) {
            toPrecision = 15;
        } else if (granularity.equals("months")) {
            toPrecision = 40000;
        } else if (granularity.equals("days")) {
            toPrecision = 1440;
        } else if (granularity.equals("hours")) {
            toPrecision = 60;
        } else if (granularity.equals("halves")) {
            toPrecision = 30;
        } else if (granularity.equals("quarters")) {
            toPrecision = 15;
        } else if (granularity.equals("fives")) {
            toPrecision = 5;
        } else if (granularity.equals("minutes")) {
            toPrecision = 1;
        } else if (granularity.equals("seconds")) {
            toPrecision = 0;
        } else {
            toPrecision = 15;
        }
        
        boolean showDay = toPrecision < 1441;
        boolean showHours = toPrecision < 61;
        boolean showMinutes = toPrecision < 60;
        boolean showSeconds = toPrecision < 1;
        
        if (toPrecision == 0) {
            toPrecision = 1;
        }
        
        if (Utilities.emptyString(value)) {
            // Shouldn't happen, but might as well default to something nice if it does.
            Log.warn("CalendarProcs.getWidgetDatetime called with a null value for '" + name + "'; this is most likely a programmer error somewhere.  Returning current time.");
            Database db = new Database();
            value = db.queryToString("select to_char(sysdate, 'YYYY-MM-DD HH24:MI:SS') from dual");
            db.close();
        }
        
        // Should do some timezone conversion here, take a look at FirstTuesday
        int[] parsedDate = CalendarProcs.parseDateTimeString(value);
        int year = parsedDate[0];
        int month = parsedDate[1];
        int day = parsedDate[2];
        int hours = parsedDate[3];
        int minutes = parsedDate[4];
        int seconds = parsedDate[5];
        
        // Kludge to get minutes rounded.  Should make general-purpose for the other values too...
        if (toPrecision < 60) {
            minutes = roundToPrecision(minutes, toPrecision);
        }


       StringBuffer input = new StringBuffer();
        
        if (showDate) {
            input.append( getWidgetMonth(name + ".month", month));
            input.append( getWidgetMaybeRange(showDay, name + ".day", 1, 31, String.valueOf(day), 1, false, String.valueOf(1)));
            input.append( "<input name=\"" + name + ".year\" size=5 maxlength=4 value=\"" + year + "\"> " + dateTimeSep);
        }
        if (useAmPm) {
            if ( hours > 12 ) {
                input.append( getWidgetMaybeRange(showHours, name + ".hours", 1, 12, String.valueOf((hours - 12)), 1, false));
            } else if (hours == 0) {
                input.append( getWidgetMaybeRange(showHours, name + ".hours", 1, 12, String.valueOf(12), 1, false));
            } else {
                input.append( getWidgetMaybeRange(showHours, name + ".hours", 1, 12, String.valueOf(hours), 1, false));
            }
        } else {
            input.append( getWidgetMaybeRange(showHours, name + ".hours", 0, 23, String.valueOf(hours), 1, true));
        }
        if (showMinutes) { 
            input.append( ":" );
        }
        input.append( getWidgetMaybeRange(showMinutes, name + ".minutes", 0, 59, String.valueOf(minutes), toPrecision, true));
        if (showSeconds) { 
            input.append( ":" );
        }
        input.append( getWidgetMaybeRange(showSeconds, name + ".seconds", 0, 59, String.valueOf(seconds), 1, true));
        
        if (useAmPm) {
            String amSelected, pmSelected;
            if (hours < 12 || ! showHours) {
                amSelected = " selected";
                pmSelected = "";
            } else {
                amSelected = "";
                pmSelected = " selected";
            }
            input.append( "<select name=\"" + name + ".ampm\">\n" +
                          "<option value=\"0\"" + amSelected + ">AM\n" +
                          "<option value=\"1\"" + pmSelected + ">PM\n" +
                          "</select>");
        } else {
            input.append( Utilities.exportFormVar(name + ".ampm", "AM"));
        }
        // First Tuesday's widget displayed a hidden field w/ the timezone in it here.  We
        // may want to do something similar some day.
        
        return input.toString();
        
    }

    /**
     * Rounds the given number to the given precision.
     * i.e. <tt>calendar_round_to_precision 44 5</tt> will round to the nearest
     * 5 and return 45, while <tt>calendar_round_to_precision 32.678 .1</tt> will round
     * to 32.7.
     */
    public static int roundToPrecision (int number, int precision) {
        // We multiply by 1.0 to make sure we're in floating point for the divide.
        // This way we get a round instead of a truncate.
        return (int)(precision * Math.round(((float)number) / (float)precision));
    }

    /**
     *  Builds a picklist of months of the year.  Taken from First Tuesday's ad_widget_month.
     *  Presently hard-wired to just use English - more time should be spent to figure out
     *  ad_locale.
     */
    public static String getWidgetMonth (String name, int defVal) throws SQLException, ACSException {

        // Modified by akatwala to allow for I18N

        // set month_names [ad_util_month_names_by_locale [ad_locale user language]]
        // Removing locale selection to avoid dependency on publishing system.
        // set month_names [calendar_util_month_names_by_locale en]
        // Removing month_names_by_locale use because 'en' wasn't standard.

        String[] monthNames = getMonthNames();
        StringBuffer input = new StringBuffer("<select name=\"" + name + "\">\n");

        for (int i = 1; i <= 12; i++) {
            String monthName = monthNames[i - 1];
            String selected;
            if (i == defVal) {
                selected = " selected";
            } else {
                selected = "";
            }
            input.append("<option" + selected + " value=" + i + ">" + monthName + "\n");
        }
        input.append("</select>\n");
        
        return input.toString();
    }

    /**
     * Non-internationalized version of calendar_util_month_names_by_locale.
     * Hopefully this will work on more systems than the internationalized version 
     * (which didn't seem to be configured right on many systems)
     * Returns a list of month names from Oracle.
     */
    public static String[] getMonthNames() throws SQLException, ACSException {
        // Can't use memoize because it only handles strings and you can't cast to a String[].
        if (calMonthNames == null) {
            calMonthNames = new String[12];
            // 1-based for month #s
            // Keeping this evil loop would be utterly unforgiveable if it wasn't cached and only done once.  As is, it's still a bad idea...
            Database db = new Database();
            for (int i = 1; i <= 12; i++) {
                calMonthNames[i-1] = db.queryToString("select to_char(to_date('1999" + getLeadingZeros(i, 2) + "01','YYYYMMDD'), 'Month') as month_name from dual");
            }
            db.close();
            
            Log.note("CalendarProcs.getMonthNames initialized months: " + calMonthNames[0] + "-" + calMonthNames[11]);
        }
        return calMonthNames;
    }

    public static String getWidgetMaybeRange (boolean askForValue, String name, int startRange, int endRange, String defaultValue) {
        return getWidgetMaybeRange(askForValue, name, startRange, endRange, defaultValue, 1);
    }

    public static String getWidgetMaybeRange (boolean askForValue, String name, int startRange, int endRange, String defaultValue, int interval) {
        return getWidgetMaybeRange(askForValue, name, startRange, endRange, defaultValue, interval, false);
    }

    public static String getWidgetMaybeRange (boolean askForValue, String name, int startRange, int endRange, String defaultValue, int interval, boolean withLeadingZeros) {
        return getWidgetMaybeRange(askForValue, name, startRange, endRange, defaultValue, interval, withLeadingZeros, "00");
    }    

    /**
     * Taken from templating system.
     * Returns form numeric range, or hidden 00 if ask_for_value is false.
     */
    public static String getWidgetMaybeRange (boolean askForValue, String name, int startRange, int endRange, String defaultValue, int interval, boolean withLeadingZeros, String hiddenValue) {
        if (!askForValue) {
            // Note that this flattens to zero for hidden fields, and it may be better to use default_value for some things.
            if (withLeadingZeros) {
                return Utilities.exportFormVar (name, hiddenValue);
            } else {
                return Utilities.exportFormVar (name, trimLeadingZeros(hiddenValue));
            }
        }
        return getWidgetNumericRange(name, startRange, endRange, defaultValue, interval, withLeadingZeros);
    }

    public static String getWidgetNumericRange ( String name, int begin, int end ) {
        return getWidgetNumericRange(name, begin, end, "");
    }

    public static String getWidgetNumericRange ( String name, int begin, int end, String defVal ) {
        return getWidgetNumericRange(name, begin, end, defVal, 1);
    }

    public static String getWidgetNumericRange ( String name, int begin, int end, String defVal, int interval ) {
        return getWidgetNumericRange(name, begin, end, defVal, interval, false);
    }

    /**
     * Taken (unmodified, for once!) from First Tuesday's templating system
     */
    public static String getWidgetNumericRange ( String name, int begin, int end, String defVal, int interval, boolean withLeadingZeros ) {
        StringBuffer input = new StringBuffer();
        input.append("<select name=\"" + name + "\">\n");
        int defval = Integer.parseInt(defVal);
        String selected;
        for ( int i = begin; i <= end; i += interval) {
            if ( i == defval ) {
                selected = " selected";
            } else {
                selected = "";
            }
            if (withLeadingZeros) {
                input.append("<option" + selected + ">" + getLeadingZeros(i, 2) + "\n");
            } else {
                input.append("<option" + selected + ">" + i + "\n");
            }
        }
        input.append("</select>");
        return input.toString();
    }

    /**
    * Adds leading zeros to an int to give it the desired number of digits.
    */
    public static String getLeadingZeros (int number, int nDesiredDigits) {
        return getLeadingZeros(new Integer(number), nDesiredDigits);
    }

    /**
    * Adds leading zeros to a String to give it the desired number of digits.
    */
    public static String getLeadingZeros (Integer number, int nDesiredDigits)  {
        StringBuffer value = new StringBuffer();
        nDesiredDigits -= number.toString().length();
        while (nDesiredDigits > 0) {
            value.append("0");
            nDesiredDigits--;
        }
        value.append(number.toString());
        return value.toString();
    }


    public static String formatDateTimeString (String value, String granularity) throws SQLException, ACSException {
        return formatDateTimeString (value, granularity, false);
    }

    public static String formatDateTimeString (String value, String granularity, boolean useAmPm) throws SQLException, ACSException {
        return formatDateTimeString (value, granularity, useAmPm, true);
    }

    public static String formatDateTimeString (String value, String granularity, boolean useAmPm, boolean showDate) throws SQLException, ACSException {
        return formatDateTimeString (value, granularity, useAmPm, showDate, "&nbsp;");
    }

    /**
    * Formats date/time
    * See description of getWidgetDatetime
    * @see #getWidgetDatetime
    */
    public static String formatDateTimeString (String value, String granularity, boolean useAmPm, boolean showDate, String dateTimeSep) throws SQLException, ACSException {

        int toPrecision;
        if (Utilities.emptyString(granularity)) {
            toPrecision = 15;
        } else if (granularity.equals("months")) {
            toPrecision = 40000;
        } else if (granularity.equals("days")) {
            toPrecision = 1440;
        } else if (granularity.equals("hours")) {
            toPrecision = 60;
        } else if (granularity.equals("halves")) {
            toPrecision = 30;
        } else if (granularity.equals("quarters")) {
            toPrecision = 15;
        } else if (granularity.equals("fives")) {
            toPrecision = 5;
        } else if (granularity.equals("minutes")) {
            toPrecision = 1;
        } else if (granularity.equals("seconds")) {
            toPrecision = 0;
        } else {
            toPrecision = 15;
        }

        boolean showDay = toPrecision < 1441;
        boolean showHours = toPrecision < 61;
        boolean showMinutes = toPrecision < 60;
        boolean showSeconds = toPrecision < 1;
        
        if (toPrecision == 0) {
            toPrecision = 1;
        }
        
        if (Utilities.emptyString(value)) {
            // Shouldn't happen, but might as well default to something nice if it does.
            Database db = new Database();
            value = db.queryToString("select to_char(sysdate, 'YYYY-MM-DD HH24:MI:SS') from dual");
            db.close();
        }
        
        // Should do some timezone conversion here, take a look at FirstTuesday
        int[] parsedDate = CalendarProcs.parseDateTimeString(value);
        int year = parsedDate[0];
        int month = parsedDate[1];
        int day = parsedDate[2];
        int hours = parsedDate[3];
        int minutes = parsedDate[4];
        int seconds = parsedDate[5];
        
        // Kludge to get minutes rounded.  Should make general-purpose for the other values too...
        if (toPrecision < 60) {
            minutes = roundToPrecision(minutes, toPrecision);
        }

        String ampm = null;
        if (useAmPm) {
            if (hours < 12) {
                ampm = " am";
            } else {
                ampm = " pm";
            }
            if (hours > 12) {
                hours = (hours - 12);
            } else if (hours == 0) {
                hours = 12;
            } else {
                // hours stays the same
            }
        } 

       StringBuffer date = new StringBuffer();
        
        if (showDate) {
            date.append(String.valueOf(getLeadingZeros(year, 4))).append("-").append(getLeadingZeros(month, 2));
            if (showDay) {
                date.append("-").append(getLeadingZeros(day, 2));
            }
            if (showHours) {
                date.append(dateTimeSep).append(getLeadingZeros(hours, 2));
            }
            if (showMinutes) {
                date.append(":").append(getLeadingZeros(minutes, 2));
            }
            if (showSeconds) {
                date.append(":").append(getLeadingZeros(seconds, 2));
            }
        } else {
            if (showHours) {
                date.append(getLeadingZeros(hours, 2));
            }
            if (showMinutes) {
                date.append(":").append(getLeadingZeros(minutes, 2));
            }
            if (showSeconds) {
                date.append(":").append(getLeadingZeros(seconds, 2));
            }
        }

        if (ampm != null) {
            date.append(ampm);
        }

        return date.toString();
    }

    public static String getCategoriesWidget () throws SQLException, ACSException {
        return getCategoriesWidget ("public", null);
    }

    public static String getCategoriesWidget (String scope, Integer id) throws SQLException, ACSException {
        return getCategoriesWidget (scope, id, null);
    }

    public static String getCategoriesWidget (String scope, Integer id, Vector defaultCategoryIdList) throws SQLException, ACSException {
        return getCategoriesWidget (scope, id, defaultCategoryIdList, "");
    }

    public static String getCategoriesWidget (String scope, Integer id, Vector defaultCategoryIdList, String defaultReturn) throws SQLException, ACSException {
        return getCategoriesWidget (scope, id, defaultCategoryIdList, defaultReturn, 2);
    }

    public static String getCategoriesWidget (String scope, Integer id, Vector defaultCategoryIdList, String defaultReturn, int selectSize) throws SQLException, ACSException {
        return getCategoriesWidget (scope, id, defaultCategoryIdList, defaultReturn, selectSize, "category_id_list");
    }

    public static String getCategoriesWidget (String scope, Integer id, Vector defaultCategoryIdList, String defaultReturn, int selectSize, String varName) throws SQLException, ACSException {
        String idClause;
        SqlStatement category = new SqlStatement();
        Log.note("categoryList=" + defaultCategoryIdList);
        if ( scope == null || scope.equalsIgnoreCase("public")) {
            idClause = "";
        } else {
            if (! scope.equalsIgnoreCase("user") && !scope.equalsIgnoreCase("group")) {
                // Shouldn't happen, but want to be safe with our dynamically generated SQL
                throw new ACSException ("Invalid scope: " + scope);
            }
            idClause = "and " + scope + "_id = :id ";
            category.bind("id", id);
        }
        StringBuffer options = new StringBuffer("");
        int nCategories = 0;
        category.setSql("select category_id, category " +
                            "from cal_categories " +
                            "where enabled_p = 't' " +
                            "and scope = :scope " +
                            idClause + 
                            "order by category");
        category.bind("scope", scope);
        Database db = new Database();
        ResultSet rs = db.query(category);
        while (rs.next()) {
            nCategories++;
            String currentcategory = rs.getString("category");
            String category_id = rs.getString("category_id");
            if (defaultCategoryIdList != null && defaultCategoryIdList.contains(category_id)) {
                options.append("<option value=\"").append(category_id).append("\" SELECTED>").append(currentcategory).append("</option>");
            } else {
                options.append("<option value=\"").append(category_id).append("\">").append(currentcategory).append("</option>");
            }
        }
        rs.close();
        db.close();

        if (nCategories == 0) {
            return defaultReturn;
        } else {
            return "<select multiple size=" + selectSize + " name=" + varName + ">" + options.toString() + "</select>";
        }
    }

    public static String transformDatetime ( AdSet date, boolean truncate) throws ACSException {
        return transformDatetime(date, truncate, null);
    }

    /**
     * Stolen and slightly modified from templating system
     * fallbackDate is a quick hack added to let end_date pull some of its data from start_date if date wasn't
     * displayed.
     * Throws an error if year, month, or day are not specified (could default to current_day,
     * but that would be dangerous if it did so silently since the user wouldn't know what was happening).
     * Different from the TCL version in that it does not return to_date... because that would require returning
     * a string + bind variables and would be more confusing.
     * 
     * TODO: This should be a class.
     *
     * @date An AdSet containing year, month, day, hours, minutes, seconds, ampm (0/1).
     * @fallbackDate An AdSet containing year, month, day, hours, minutes, seconds, ampm (0/1).
     * @truncate True if time should be ignored and replaced with 00:00.
     * @return A string representing this date.  This string has validated that all of the fields are integers, and
     *         is safe to pass to Oracle, but no further validation has been performed.
     */
    public static String transformDatetime ( AdSet date, boolean truncate, AdSet fallbackDate ) throws ACSException {
        SqlStatement dateSql;
        String year = null;
        String month = null;
        String day = null;
        String hours;
        String minutes;
        String seconds;
        int ampm;
        // We use Integer.valueOf as a numeric validation, for safety.
        // The end result of this function *should* still be entered
        // into Oracle as a bind variable, but you never know when
        // someone will forget to so we might as well validate that
        // we're only dealing with numbers.
        if (date.get("year") != null) {
            year = getLeadingZeros(Integer.valueOf(date.get("year")), 4);
        }
        if (date.get("month") != null) {
            month = getLeadingZeros(Integer.valueOf(date.get("month")), 2);
        }
        if (date.get("day") != null) {
            day = getLeadingZeros(Integer.valueOf(date.get("day")), 2);
        }
        if (year == null || year.equals("0000") && fallbackDate != null) {
            if (fallbackDate.get("year") != null) {
                year = getLeadingZeros(Integer.valueOf(fallbackDate.get("year")), 4);
            }
        }
        if (month == null || month.equals("00") && fallbackDate != null) {
            if (fallbackDate.get("month") != null) {
                month = getLeadingZeros(Integer.valueOf(fallbackDate.get("month")), 2);
            }
        }
        if (day == null || day.equals("00") && fallbackDate != null) {
            if (fallbackDate.get("day") != null) {
                day = getLeadingZeros(Integer.valueOf(fallbackDate.get("day")), 2);
            }
        }
        if (year.equals("0000") || month.equals("00") | day.equals("00")) {
            throw new ACSException ("Year, month, and day must all be specified and not equal to 0.");
        }

        if (truncate) {
            hours = "0";
            minutes = "0";
            seconds = "0";
            ampm = 0;
        } else {
            if (Utilities.emptyString(date.get("hours")) && fallbackDate != null) {
                hours = getLeadingZeros(Integer.valueOf(fallbackDate.get("hours")), 2);
            } else {
                hours = getLeadingZeros(Integer.valueOf(date.get("hours")), 2);
            }
            if (Utilities.emptyString(date.get("minutes")) && fallbackDate != null) {
                minutes = getLeadingZeros(Integer.valueOf(fallbackDate.get("minutes")), 2);
            } else {
                minutes = getLeadingZeros(Integer.valueOf(date.get("minutes")), 2);
            }
            if (Utilities.emptyString(date.get("seconds")) && fallbackDate != null) {
                seconds = getLeadingZeros(Integer.valueOf(fallbackDate.get("seconds")), 2);
            } else {
                seconds = getLeadingZeros(Integer.valueOf(date.get("seconds")), 2);
            }
            if (Utilities.emptyString(date.get("ampm")) && fallbackDate != null) {
                ampm = Integer.parseInt(fallbackDate.get("ampm"));
            } else {
                ampm = Integer.parseInt(date.get("ampm"));
            }
            if (ampm == 0) {
                if (hours.equals("12")) {
                    // if this is am and hours are 12, we really mean 00: midnight.
                    hours = "00";
                }
            } else if (ampm == 1) {
                if (!hours.equals("12")) {
                    // 12 pm is 12 hours, but 1 pm is 13.
                    hours = String.valueOf(Integer.parseInt(hours) + 12);
                }
            } else {
                throw new ACSException ("Invalid am/pm value '" + ampm + "', should be 0 or 1.");
            }
        }
        return (year + "-" + month + "-" + day + " " + hours + ":" + minutes + ":" + seconds);
    }


    /**
     * same as insertItem, defaults editable_p to true.
     */
    public static Integer insertItem (Integer item_id, Integer instance_id, String title, SqlStatement startDate, SqlStatement endDate, Integer user_id, Integer group_id, String description, Integer creation_user_id) throws SQLException, ACSException {
        Integer result = insertItem (item_id, instance_id, title, startDate, endDate, user_id, group_id, description, creation_user_id, null);
        return (result);
    }

    /**
    Inserts a new calendar item into the appropriate calendar.
    Does not do validation of arguments.

    @param -item_id and @param -instance_id support passing in an item_id 
    and instance_id for double-click protection.  If @param -item_id 
    isn't passed in, then both will be generated. Specifying @param -item_id
    without @param -instance_id is an error.

    @param -title 

    @param -start_date_sql and @param -end_date_sql are used to directly 
    specify the start_date and end_date to be inserted into the DB.  These
    parameters override the following parameters.

    These parameters are used to calculate start_date and end_date based on a 
    combination of date, time and duration:
    @param -date assumes 'YYYY-MM-DD'.
    @param -start_time and @param -end_time assume 'HH24:MI'.  If 
    @param -start_time is not specified, then a No Time item will be created
    and the value of @param -end_time will be ignored.
    If @param -end_time is not specified, then the value of @param -duration will
    be used.

    At a bare minimum, either @param -start_date_sql or @param -date must be 
    specified.

    @param -user_id indicates a personal item.
    @param -group_id indicates a group item.
    Specifying neither or both is currently undefined but will not cause an
    error.

    @param -description 
    @param -html_p indicates whether or not description has html tags;
    default is 'f'

    @param -related_url if not specified, then no link will appear in the 
    item details. 
    @param -related_url_text if not specified, then the value of -related_url
    will be displayed as the link text.
    @param -related_url_p if 'f' then @param title will be displayed for the item
    on the various views as a link to details about the items.  
    @param related_url and @param related_url_text will be used to display a 
    link on the details page.
    if 't' then @param related_url and @param related_url_text will be used to 
    form the link that appears in the various views.
    The default is 'f'.

    @param -creation_user is required.  Generally it will be the logged in user.

    @param -on_what_id and @param -on_which_table throws an error if one but 
    not both are specified.  If both are specified, then a row will be added
    to the mapping table.

    @param -editable_p indicates whether or not the item can be edited by 
    anyone or only deleted.  

    Throws error if insert fails.
    Returns instance_id (from whence item_id can be derived) on success.
    */
    public static Integer insertItem (Integer item_id, Integer instance_id, String title, SqlStatement startDate, SqlStatement endDate, Integer user_id, Integer group_id, String description, Integer creation_user_id, Boolean editable_p) throws SQLException, ACSException {
/*    -item_id
    -instance_id
    -title
    -start_date_sql
    -end_date_sql
    -date
    -start_time
    -end_time
    {-duration 60}
    -user_id
    -group_id
    -description
    -html_p
    -related_url
    -related_url_text
    -related_url_p
    -creation_user:required
    -on_what_id
    -on_which_table
    -editable_p*/

    // Do argument interpretation in TCL, then dispatch to appropriate
    // PL/SQL function.
    
    // default to null: title user_id group_id description html_p related_url related_url_text related_url_p editable_p endDate
        String html_p = null;
        String related_url = null;
        String related_url_text = null;
        String related_url_p = null;
        String editableP;
        if (editable_p == null || editable_p.booleanValue()) {
            editableP = "t";
        } else {
            editableP = "f";
        }

        Database db=  new Database();

    // Figure out start_date, end_date
    String dateFormat = "'YYYY-MM-DD HH24:MI'";

    // TODO: Add support for date/time (instead of startDate, endDate) parameters later
    /*
    if ![exists_and_not_null start_date_sql] {
	if [exists_and_not_null date] {
	    if {[exists_and_not_null start_time]} {
		set start_date_sql "to_date('$date $start_time', $date_format)"

		if [exists_and_not_null end_time] {
		    set end_date_sql "to_date('$date $end_time', $date_format)"
		} else {
		    # Oracle date arithmetic uses fractional days
		    set end_date_sql "$start_date_sql + $duration / (24 * 60)"
		}
	    } else {
		set start_date_sql "to_date('$date')"
		set end_date_sql "NULL"
	    }
	} else {
	    error "Must specify either -date or -start_date_sql."
	}
    }*/

        // TODO: add this, too...
    /*# on_what_id/on_which_table
    if {[exists_and_not_null on_what_id] && [exists_and_not_null on_which_table]} {
	if {[exists_and_not_null item_id]} {
	    if ![exists_and_not_null instance_id] {
		error "Must specify instance_id if item_id is specified"
	    }
	    # Have to do the steps separately
	    db_transaction {
		db_dml cal_insert_item_with_ids "BEGIN
		cal_insert_item_with_ids(:item_id, :instance_id,
		$start_date_sql, $end_date_sql,
		:title, :user_id, :group_id, :description, :html_p,
		:related_url, :related_url_text, :related_url_p,
		:creation_user, :editable_p);
		END;
		"
		db_dml cal_insert_db_row_map "
		INSERT INTO cal_instance_db_row_map
		(instance_id, on_which_table, on_what_id)
		VALUES
		(:instance_id, upper(trim(:on_which_table)), :on_what_id)
		"
	    }
	} else {
	    set instance_id [db_exec_plsql cal_ins_item_on_id_table "BEGIN
	    :1 := cal_ins_item_on_id_table(:on_which_table, :on_what_id, 
	    $start_date_sql, $end_date_sql,
	    :title, :user_id, :group_id, :description, :html_p, 
	    :related_url, :related_url_text, :related_url_p,
	    :creation_user, :editable_p);
	    END;
	    "]
	}
    } elseif {[exists_and_not_null on_what_id] || [exists_and_not_null on_which_table]} {
	# Only one specified, throw error.
	error "Only one of on_what_id/on_which_table specified."
    } else {*/

        // Not on_what_id/on_which_table...
	if (item_id != null) {
            if (instance_id == null) {
                throw new ACSException ("Must specify instance_id if item_id is specified.");
            }
            SqlStatement insert = new SqlStatement();
            insert.append("BEGIN \n" +
                          "cal_insert_item_with_ids(:item_id, :instance_id, ").append(startDate).append(", ").append(endDate);
            insert.append(", :title, :user_id, :group_id, :description, :html_p, :related_url, :related_url_text, :related_url_p, :creation_user_id, :editable_p); \n" +
                          "END;");
            insert.bind("item_id", item_id);
            insert.bind("instance_id", instance_id);
            insert.bind("title", title);
            insert.bind("user_id", user_id);
            insert.bind("group_id", group_id);
            insert.bind("description", description);
            insert.bind("html_p", html_p);
            insert.bind("related_url", related_url);
            insert.bind("related_url_text", related_url_text);
            insert.bind("related_url_p", related_url_p);
            insert.bind("creation_user_id", creation_user_id);
            insert.bind("editable_p", editableP);
            db.dml(insert);
	} else {
            //TODO: Fix this code, it's broken right now due to bindFixup not being able to handle :1 :=.
	    //set instance_id [db_exec_plsql cal_insert_item 
            SqlStatement insert = new SqlStatement ();
            insert.append("BEGIN \n" +
                       "? := cal_insert_item(").append(startDate).append(", ").append(endDate).append(", \n" +
                       ":title, :user_id, :group_id, :description, :html_p, \n" +
                       ":related_url, :related_url_text, :related_url_p, \n" +
                       ":creation_user_id, :editable_p); \n" +
                       "END;");
            insert.bind("title", title);
            insert.bind("user_id", user_id);
            insert.bind("group_id", group_id);
            insert.bind("description", description);
            insert.bind("html_p", html_p);
            insert.bind("related_url", related_url);
            insert.bind("related_url_text", related_url_text);
            insert.bind("related_url_p", related_url_p);
            insert.bind("creation_user_id", creation_user_id);
            insert.bind("editable_p", editableP);
            Object retval = db.plsql(insert , java.sql.Types.NUMERIC);
            // TODO: Find more efficient conversion
            instance_id = new Integer(retval.toString());
	}
        db.close();
        return instance_id;
    }
} // End class CalendarProcs
