package com.arsdigita.acs.calendar;

import com.arsdigita.acs.calendar.TableCell;
import java.awt.Point;
import com.arsdigita.acs.Utilities;
import java.util.Vector;

public class Table {
  int numCols; // i, 1-based
  int numRows; // j, 1-based
  Vector cols;

  public Table () {
    this(1, 1);
  }

  public Table (int numRows, int numCols) {
    if (numRows < 1) {
      numRows = 1;
    }
    if (numCols < 1) {
      numCols = 1;
    }
    this.numRows = numRows;
    // numcols is set by repeated addCol calls.

    cols = new Vector(numCols);
    for (int i = 0; i < numCols; i++) {
      addCol();
    }    
  }

  /**
   * Returns item at col,row (0-based).  Does not validate bounds in advance.
   */
public TableCell getCell(int col, int row) {
  TableCell[] column = (TableCell[])(cols.elementAt(col));
  return column[row];
}

public void setCell(int col, int row, TableCell cell) {
  TableCell[] column = (TableCell[])cols.elementAt(col);
  column[row] = cell;
}

public void addCol () {
  int i = numCols;
  TableCell[] cells = new TableCell[numRows];
  cols.addElement(cells);
  for (int j = 0; j < numRows; j++) {
    TableCell current = new TableCell(new Point(i, j), "", 1, 1);
    cells[j] = current;
  }
  numCols++;
}

public int getNumCols() {
  return numCols;
}

public int getNumRows() {
  return numRows;
}

  /**
   * returns the index of the first empty column in a row
   * returns -1 if there is no empty column.
   */
  public int getFirstEmptyCol (int row) {
    int i = 0;
    while (i < numCols) {
      if (Utilities.emptyString(getCell(i, row).getText())) {
	return i;
      }
      i++;
    }
    return -1;
  }

  /**
   * Returns the index of the first full column in a row
   * Returns -1 if there are no further empty columns.
   */
public int getNextFilledCol (int row) {
  return getNextFilledCol(row, 0);
}

  /**
   * Returns the index of the next full column in a row
   * Returns -1 if there are no further empty columns.
   */
public int getNextFilledCol (int row, int col) {
  int i = col;
  while (i < numCols) {
    if (!Utilities.emptyString(getCell(i, row).getText())) {
      return i;
    }
    i++;
  }
  return -1;
}

  /**
   * returns how many columns are left to write in this row, starting with col
   */
public int getFilledColsLeft (int row) {
  return getFilledColsLeft(row, 0);
}

public int getFilledColsLeft (int row, int col) {
  int i = col;
  int returnNum = 0;
  while (i < numCols) {
    if (!Utilities.emptyString(getCell(i, row).getText())) {
      returnNum++;
    }
    i++;
  }
  return returnNum;
}

public String toString () {
  StringBuffer returnHTML = new StringBuffer("<table border=1>");
  for (int i = 0; i < numRows; i++) {
    for (int j = 0; j < numCols; j++) {
      // I'm not sure why j and i are backwards here, but that's the way the TCL was written...
      returnHTML.append("<td nowrap><b>(").append(j).append(i).append("</b>").append(getCell(j, i)).append("</td>");
    }
    returnHTML.append("</tr>");
  }
  returnHTML.append("</table>");
  return returnHTML.toString();
}

}
