package com.arsdigita.acs.chat;

import java.util.Vector;

/**
 * Class that displays a list of links that denote actions that can
 * be taken from the current page.  
 * <p>
 * Elements are added to the action list in order; each element consists
 * of a path to be used as a relative URL link, and the title to be displayed
 * as a link to that path.  When displayed, the actions are separated by a
 * vertical bar.
 *
 * @author Luke Pond (luke@arsdigita.com)
 * @version $Version$
 */
public class ActionList {
    private Vector paths;
    private Vector titles;

    /** 
     * Creates a new action list.
     */
    public ActionList() {
	paths = new Vector();
	titles = new Vector();
    }
 
    /**
     * Adds a new element to the action list.
     * @param path Relative URL to appear in <code>href</code>
     * @param title Title for this page
     */
    public void add(String path, String title) {
	paths.addElement(path);
	titles.addElement(title);
    }

    /** 
     * Returns HTML representation of this action list.
     * @return HTML action list
     */      
    public String getHTML() {
	StringBuffer sb = new StringBuffer();
	for (int i=0; i<paths.size(); i++) {
	    String path = (String)paths.elementAt(i);
	    String title = (String)titles.elementAt(i);
	    sb.append("<a href=");
	    sb.append(path);
	    sb.append(">");
	    sb.append(title);
	    sb.append("</a>");
	    if (i != paths.size() - 1) {
		sb.append(" | ");
	    }
	}
	return sb.toString();
    }

    /** 
     * Returns HTML representation of this action list.
     * @return HTML action list
     */      
    public String toString() {
	return getHTML();
    }
}
