package com.arsdigita.acs.chat;

import com.arsdigita.acs.*;
import com.arsdigita.acs.db.*;
import java.sql.*;

/**
 * This DataSource returns the first N messages from the results
 * of the query.  N is determined by looking in the parameters file
 * under NShortMessages, NMediumMessages, or NLongMessages.
 */
public class ChatDataSource extends Selection {

    private int counter = 0;
    private int nMessages;
    private String nRows;
    private boolean canDisplayMore = false;

    public ChatDataSource(SqlStatement sql, Database db, String nRows) throws SQLException {
	super(sql, db);
	initRowLimit(nRows);
    }

    public ChatDataSource(ResultSet rs, String nRows) throws SQLException {
	super(rs);
	initRowLimit(nRows);
    }

    private void initRowLimit(String nRows) {
	this.nRows = nRows;
	String param;

	if (nRows.equals("short")) {
	    param = Parameters.get("NShortMessages", "chat", "25");
	} else if (nRows.equals("medium")) {
	    param = Parameters.get("NMediumMessages", "chat", "50");
	} else if (nRows.equals("long")) {
	    param = Parameters.get("NLongMessages", "chat", "75");
	} else {
	    throw new IllegalArgumentException("nRows must be one of short, medium, or long");
	}
	nMessages = new Integer(param).intValue();
    }

    public boolean next() throws DataSourceException {
	if (counter >= nMessages) {
	    try {
		rs.close();
	    } catch (SQLException e) {
		throw new DataSourceException(e.getMessage());
	    }
	    canDisplayMore = !(nRows.equals("long"));
	    return false;
	} else {
	    counter++;
	    return super.next();
	}	
    }

    public boolean isSystemNote() {
	try {
	    return rs.getString("system_note_p").equals("t");
	} catch (Exception e) {
	    return false;
	}
    }

    public String getFirstNames() {
	try {
	    return rs.getString("first_names");
	} catch (Exception e) {
	    return null;
	}
    }

    public String getLastName() {
	try {
	    return rs.getString("last_name");
	} catch (Exception e) {
	    return null;
	}
    }

    public String getTime() {
	try {
	    return rs.getString("time");
	} catch (Exception e) {
	    return null;
	}
    }

    public String getFilteredMsg() {
	try {
	    // NYI: set filtered_msg [link_urls [ns_quotehtml $filtered_msg]]
	    return Utilities.quoteHtml(rs.getString("filtered_msg"));
	} catch (Exception e) {
	    return null;
	}
    }

    public String getCreationUserId() {
	try {
	    return rs.getString("creation_user");
	} catch (Exception e) {
	    return null;
	}
    }

    /**
     * @return true if there are more messages than requested
     */
    public boolean canDisplayMore() {
	return canDisplayMore;
    }
}
