package com.arsdigita.acs.fs;

import javax.servlet.http.*;
import javax.servlet.*;
import java.sql.*;

import com.arsdigita.acs.*;
import com.arsdigita.acs.db.*;

/**
 * Serves as a registered procedure for path
 * /file-storage/download/<id>/pretty-name.  This is useful because sometimes
 * we can't count on the browser recognizing the Content-Disposition header
 * for deciding what to do with a HTTP response.
 */
public class Download extends Filter {

    static {
	// register ourselves
	RequestProcessor.registerFilter
	    ("/file-storage/download/", 
	     new com.arsdigita.acs.fs.Download());
    }

    private static final String query = 
	"select version_content, file_type" + 
	" from 	fs_versions" +
	" where  version_id=?";

    public int filter (HttpServletRequest req, 
		       HttpServletResponse resp, 
		       ServletContext ctx) 
	throws ACSException {

	try {
	    UserInfo user = Security.getVerifiedUserInfo(req,resp);
	    if (user == null) {
		// already returned a redirect, so stop
		return FILTER_RETURN;
	    }
	    Integer userId = user.getUserId();
	    String stringToFind = "file-storage/download/";
	    String requestPath = req.getRequestURI();
	    String restOfPath = requestPath.substring(requestPath.indexOf(stringToFind) + stringToFind.length());
	    String id = restOfPath.substring(0, restOfPath.indexOf("/"));
	    String prettyName = restOfPath.substring(restOfPath.indexOf("/") + 1);
	    Integer version_id = new Integer(id);
	    if (!FileStorageProcs.checkRead(userId,version_id)) {
		// probably want to redirect to a standard permission-denied
		// page
		return FILTER_BREAK;
	    }
	    java.io.OutputStream os = resp.getOutputStream();
	    resp.setContentType("application/other");
	    Database db = new Database();
	    ResultSet rs = db.query(query, new Object[] {version_id});
	    rs.next();
	    db.dumpBLOB(rs,"version_content",os);
	    db.close();
	    os.close();
	    return FILTER_RETURN;
	}
	catch (java.io.IOException e) {
	    throw new ACSException(e.getMessage());
	}
	catch (SQLException e) {
	    throw new ACSException(e.getMessage());
	}
    }
}
	

	    
	

	
	
