package com.arsdigita.acs.fs;

import java.sql.*;
import java.util.*;
import com.oroinc.text.perl.Perl5Util;
import com.arsdigita.acs.*;
import com.arsdigita.acs.db.Database;

public class FileStorageProcs {
    public static String privateIndividualOption() {
	return "<option value=\"private_individual\">Your personal document tree</option>";
    }

    public static String publicIndividualOption(Integer userId, String userName) {
	return "<option value=\"user_id " + userId + "\">" + userName + "'s document tree</option>";
    }

    public static String privateGroupOption(Integer groupId, String groupName) {
	return "<option value=\"private_group " + groupId + "\">" + groupName + " group document tree</option>";
    }

    public static String publicGroupOption(Integer groupId, String groupName) {
	return "<option value=\"public_group " + groupId + "\">" + groupName + " group document tree</option>";
    }
    
    public static String publicOption() {
	return "<option value=\"all_public\">All publically accessible files</option>";
    }

    public static String sharedOption() {
	if (Parameters.getBoolean("PublicDocumentTreeP", "fs")) {
	    String systemName = Parameters.get("SystemName", "acs");
	    return "<option value=public_tree>" + systemName + " shared document tree</option>";
	} else {
	    return "";
	}
    }

    public static String headerRowForFiles(String title, boolean authorP) {
	int colspan = 5;
	if (authorP) {
	    colspan++;
	}

	String font = "<nobr>" + Parameters.get("FileInfoDisplayFontTag", "fs");
	String headerColor = Parameters.get("HeaderColor", "fs");
	String columnNameHeaderRow = "";
	if (title != null && !title.equals("")) {
	    columnNameHeaderRow += "<tr><td colspan=" + colspan + " bgcolor=#666666>" + 
		font + "&nbsp;<font color=white>" + title + "</td></tr>";
	}
	columnNameHeaderRow += "<tr><td bgcolor=" + headerColor + ">" + font + " &nbsp; Name</td>\n" +
	    "<td bgcolor=" + headerColor + " align=left>" + font + "&nbsp; Action &nbsp;</td>";
	if (authorP) {
	    columnNameHeaderRow += "<td bgcolor=" + headerColor + " align=left>" + font + " &nbsp; Author &nbsp;</td>";
	}
	columnNameHeaderRow += "<td bgcolor=" + headerColor + " align=right>" + font + " &nbsp; Size &nbsp;</td>\n" +
	    "<td bgcolor=" + headerColor + ">" + font + " &nbsp; Type &nbsp;</td>\n" +
	    "<td bgcolor=" + headerColor + ">" + font + " &nbsp; Modified &nbsp;</td>\n" +
	    "</tr>";
	return columnNameHeaderRow;
    }

    public static String rowForOneFile(ResultSet rs, String exportURLVars) {
	return rowForOneFile(rs, true, exportURLVars, false, null, null);
    }

    public static String rowForOneFile(ResultSet rs, boolean linksP, String exportURLVars, boolean authorP, Integer ownerId, String ownerName) {
	try {
	    String font =  "<nobr><font face=arial,helvetica size=-1>";
	    String headerColor = Parameters.get("HeaderColor", "fs");
	    String row = "";
	    String spacerGIF;
	    String graphicsDir = Parameters.get("GraphicsDir", "fs");
	
	    if (exportURLVars == null) {
		exportURLVars = "";
	    } else {
		exportURLVars = "?" + exportURLVars;
	    }

	    int n_pixels_in = 0;
	    try {
		n_pixels_in = rs.getInt("n_pixels_in");
	    } catch(SQLException sqle) {
		// might not be in result set
	    }

	    if (n_pixels_in == 0) {
		spacerGIF = "";
	    } else {
		spacerGIF = "<img src=\"" + graphicsDir + "/spacer.gif\" width=" + n_pixels_in + " height=1>";
	    }

	    if (rs.getString("folder_p").equals("t")) {
		row += "<tr><td valign=top>&nbsp; " + spacerGIF + " " + font;
		if (linksP) {
		    row += "<a href=\"one-folder" + exportURLVars + "\">";
		}

		row += "<img border=0 src=\"" + graphicsDir + "/ftv2folderopen.gif\" align=top>";
		if (linksP) {
		    row += "</a> <a href=\"one-folder" + exportURLVars + "\">";
		}

		row += rs.getString("file_title");
		if (linksP) {
		    row += "</a>";
		}

		row += "</td>\n<td align=left>&nbsp;</td>";

		if (authorP) {
		    row += "<td align=left>&nbsp;</td>";
		}

		row += "<td align=right>&nbsp;</td>\n" 
		    + "<td>" + font + " &nbsp; File Folder &nbsp;</td>\n" 
		    + "<td>&nbsp;</td>"
		    + "</tr>\n";
	    } else if (Utilities.emptyString(rs.getString("n_kbytes"))) {
		String fileAnchor = "<a href=\"one-file" + exportURLVars + "\">";
		row += "<tr><td valign=top>&nbsp; " + spacerGIF + font
		    + fileAnchor
		    + "<img border=0 src=\"" + graphicsDir + "/ftv2doc.gif\" align=top></a>\n"
		    + fileAnchor + rs.getString("file_title") + "</a>&nbsp;</td>"
		    + "<td align=left>" + font +"(<a href=\"" + rs.getString("url") + "\">go</a>)</td>";
	    
		if (authorP) {
		    row += "<td align=left>&nbsp;</td>";
		}

		row += "<td align=right>&nbsp;</td>\n"
		    + "<td>" + font + " &nbsp; URL &nbsp;</td>\n"
		    + "<td>" + font + " &nbsp; " + rs.getString("creation_date") + " &nbsp;</td>\n"
		    + "</tr>\n";
	    } else {
		Perl5Util re = new Perl5Util();
		String clientFileName = rs.getString("client_file_name");
		if (re.match("/\\\\([^\\\\]+)/", clientFileName)) {
		    clientFileName = re.group(1);
		}
		String prettyFileName = re.substitute("s/[^-_.0-9a-zA-Z]+/_/g", clientFileName);
		String fileAnchor = "<a href=\"one-file" + exportURLVars + "\">";

		row += "<tr><td valign=top>&nbsp; " + spacerGIF + font
		    + fileAnchor
		    + "<img border=0 src=\"" + graphicsDir + "/ftv2doc.gif\" align=top></a>"
		    + fileAnchor + rs.getString("file_title") + "</a>&nbsp;</td>"
		    + "<td align=left>" + font + "(<a href=\"download?version_id=" + rs.getString("version_id") + "\">download</a>)</td>";

		if (authorP) {
		    row += "<td><a href=\"/shared/community-member?user_id=" + ownerId + "\">" + ownerName + "</a>&nbsp;</td>";
		}

		String sizeString;
		int nKbytes = rs.getInt("n_kbytes");
		if (nKbytes == 0) {
		    sizeString = "0 bytes";
		} else {
		    sizeString = nKbytes + " KB";
		}

		row += "<td align=right>" + font + " &nbsp; " + sizeString + "&nbsp;</td>\n"
		    + "<td>" + font + " &nbsp; " + rs.getString("file_type") + " &nbsp;</td>\n"
		    + "<td>" + font + " &nbsp; " + rs.getString("creation_date") + " &nbsp;</td>\n"
		    + "</tr>\n";
	    }
	    return row;
	} catch (SQLException e) {
	    return "Oops: " + e.getMessage();
	}
    }

    public static String fsFolderDefSelection (Database db, Integer user_id, Integer group_id, boolean public_p, Integer file_id, Integer folder_default) {
	try {
	// Write out the SELECT box that allows the user to move a file to
	// another folder, or - if folder_default is provided - create a new
	// folder.
	
	// Get the current location of the file (ie parent_id).
	Integer current_parent_id = null;
	String children_clause = null;
	String sql_query = null;
	String file_options = "";
	
	if (file_id != null) {
	    String current_parent_id_str = db.queryToString("select parent_id from fs_files where file_id=?", new Object[] {file_id});
	    if (current_parent_id_str != null) {
		current_parent_id = new Integer(current_parent_id_str);
	    } 

	    // We don't want to display any folders which are children
	    // of the selected file, so use this clause to block them.
	    children_clause = "and fs_files_tree.file_id not in " +
		"(select file_id from fs_files " +
		"connect by prior file_id = parent_id " +
		"start with file_id = " + file_id + ")";
	} else {
	    current_parent_id = null;
	    children_clause = "";
	}

	String top_level = "";
	
	if (group_id != null && !public_p) {
	    sql_query = "select file_title as folder, fs_files_tree.file_id as new_parent, lpad('x',the_level,'x') as spaces " +
		"from   fs_files_tree " +
		"where  folder_p='t' " +
		"and    (public_p <> 't' or public_p is null) " +
		"and    group_id = " + group_id + 
		"and    deleted_p = 'f' " + children_clause;
	    String group_name = db.queryToString("select group_name from user_groups where group_id=?", new Object[] {group_id});
	    top_level = group_name + " group tree";
	} else if (public_p) {
	    sql_query = "select file_title as folder, fs_files_tree.file_id as new_parent, lpad('x',the_level,'x') as spaces " +
		"from   fs_files_tree " +
		"where folder_p='t' " +
		"and owner_id=" + user_id +
		" and public_p = 't' " +
		"and group_id is null " +
		"and deleted_p = 'f' " + children_clause;
	    top_level = "Shared user tree";
	} else  {
	    sql_query = "select file_title as folder, fs_files_tree.file_id as new_parent, lpad('x',the_level,'x') as spaces " +
		"from   fs_files_tree " +
		"where folder_p='t' " + 
		"and owner_id=" + user_id + 
		"and public_p = 'f' " +
		"and group_id is null " +
		"and deleted_p = 'f'" + children_clause;
	    top_level =  "Private user tree";
	
	}
    
	if (current_parent_id != null && folder_default == null) {
	    file_options = "<option value=\"\" selected> " + top_level + " </option>\n";
	} else {
	    file_options = "<option value=\"\" > " + top_level + " </option>\n";
	}
	int folder_count = 0;
	Perl5Util re = new Perl5Util();
	ResultSet rs = db.query(sql_query);
	while(rs.next()) {
	    String spaces = re.substitute("s/x/&nbsp; &nbsp;/g", rs.getString("spaces"));
	    Integer new_parent = new Integer(rs.getInt("new_parent"));
	    String folder = rs.getString("folder");
	    if (file_id == null || !file_id.equals(new_parent)) {
		folder_count ++;
		if ((current_parent_id != null && current_parent_id.equals(new_parent)) || (folder_default != null && folder_default.equals(new_parent))) {
		    file_options += "<option value=" + new_parent + " selected>" + spaces + folder + "</option>\n";
		} else {
		    file_options += "<option value=" + new_parent + " > " + spaces + folder + "</option>\n";
		}
	    }
	}
	
	int size_count;
	if (folder_count > 8) { 
	    size_count = 8;
	} else {
	    size_count = folder_count + 1;
	}
	
	String file_options_list  = "<select size=" + size_count + " name=parent_id>" + file_options + " </select>";
	
	return file_options_list;
	} catch(SQLException sqle) {
	    return "Oops: " + sqle.getMessage();
	}
	catch(ACSException acse) {
	    return "Oops: " + acse.getMessage();
	}
    }

    public static boolean checkRead(Integer userId, Integer version_id) 
    throws ACSException {
	try {
	    Database db = new Database();
	    Integer count = new Integer(db.queryToString("select count(fs_files.file_id) from fs_files, fs_versions_latest where fs_files.file_id = fs_versions_latest.file_id and fs_files.owner_id = ? and version_id = ?", new Object[] {userId, version_id}));
	    db.close();
	    if (count.intValue()>0)
		// user is owner
		return true;
	    // user is not owner, so call general permissions
	    else
		return GeneralPermissions.userHasPermission(userId, "read", version_id != null ? version_id.toString() : null, "FS_VERSIONS");
	}
	catch (java.sql.SQLException se) {
	    throw new ACSException(se.getMessage());
	}
    }

    public static boolean checkWrite(UserInfo user, Integer version_id) 
    throws ACSException, SQLException {
	    Database db = new Database();

	    String file_id = db.queryToString("select file_id from fs_versions_latest where version_id = ?", new Object[] {version_id});
	    

	    Integer count = new Integer(db.queryToString("select count(fs_files.file_id) from fs_files where fs_files.file_id = ? and fs_files.owner_id = ?", new Object[] {file_id, user.getUserId()}));
	    if (count.intValue()>0) 
		// user is owner
		return true;
	    else {
		// see if the file is a folder 
		ResultSet rs = db.query("select folder_p, group_id " +
					"from fs_files " +
					"where fs_files.file_id = ?", 
					new Object[] {file_id});
		rs.next();
		String group_id = rs.getString("group_id");
		if ("t".equals(rs.getString("folder_p")) && !Utilities.emptyString(group_id)) {
		    return user.isGroupMember(group_id);
		}
		return GeneralPermissions.userHasPermission(user.getUserId(), "write", version_id.toString(), "FS_VERSIONS");
	    }
    }

    public static void orderFiles(Database db) throws SQLException {
	String order_query = "select file_id, the_level from fs_files_tree";
	ResultSet rs = db.query(order_query);
	Vector list = new Vector();
	int sort_key = 1;
	while (rs.next()) {
	    Integer id = new Integer(rs.getInt(1));
	    Integer depth = new Integer(rs.getInt(2));
	    list.addElement(new Integer[] {id, depth});
	}
	for (int i = 0; i<list.size(); i++) {
	    Integer[] id_depth = (Integer[]) list.elementAt(i);
	    db.dml("update fs_files set sort_key=?, depth=? where file_id=?", new Object[] {new Integer(sort_key), id_depth[1], id_depth[0]});
	    sort_key++;
	}
    }

    public static Integer latestVersion(Database db, Integer file_id) throws SQLException {
	ResultSet rs = db.query("select version_id from fs_versions_latest where file_id = ?", new Object[] {file_id});
	if (rs.next()) {
	    return new Integer(rs.getInt(1));
	} else {
	    return null;
	}
    }

    public static String guessSource(Database db, boolean public_p, Integer owner_id, Integer group_id, UserInfo local_user) throws ACSException, SQLException {
	// Given some information about a file, tries to guess in which 
	// subtree the file belongs. Mainly used by one-file.tcl.
	
	if (!public_p) {
	    if (group_id != null) {
		if (local_user.isGroupMember(group_id)) {
		    return "private_group";
		} else {
		    return "public_group";
		}
	    } else if (owner_id != null) {
		if (owner_id.equals(local_user.getUserId())) {
		    return "private_individual";
		} else {
		    return "public_individual";
		}
	    }
	}
	return "shared";
    }

    public static String datePicture() {
	return Parameters.get("DatePicture", "fs", "mm/dd/yyyy hh24:mi");
    }

}
