package com.arsdigita.acs.fs;

import com.arsdigita.acs.*;
import com.arsdigita.acs.db.*;
import java.util.*;
import java.sql.*;

public class FolderBox {
    // Return option list for group folders that the given
    // user can view.

    private Database db;
    private Vector groupOptions;
    private Vector privateOptions;
    private Vector allOptions;

    private Vector groupFolders(Integer userId) throws SQLException {
	Vector options = new Vector();
	
	ResultSet rs = db.query("select group_id, group_name "
				+ "from user_groups "
				+ "where ad_group_member_p(?, group_id) = 't'",
				new Object[] { userId });

	Vector groupIds = new Vector();

	while (rs.next()) {
	    String groupId = rs.getString("group_id");
	    groupIds.addElement(groupId);
	    options.addElement("<option value=\"private_group " + groupId + "\">"
			       + rs.getString("group_name") + " group document tree</option>");
	}

	
	String groupClause = "";
	if (options.size() > 0) {
	    groupClause = " and ug.group_id not in (" + Utilities.join(groupIds, ",") + ")";
	}

	rs = db.query("select unique ug.group_id, ug.group_name "
		 + "from user_groups ug, fs_files fsf, fs_versions_latest fsvl "
		 + "where fsf.file_id = fsvl.file_id "
		 + "and fsf.group_id = ug.group_id "
		 + "and fsf.public_p = 't' "
		 + "and fsf.deleted_p = 'f'"
		 + groupClause);
	while (rs.next()) {
	    options.addElement("<option value=\"public_group " + rs.getString("group_id") + "\">"
			       + rs.getString("group_name") + " group public document tree</option>");
	}
	return options;
    }

    private Vector privateFolders(Integer userId) throws SQLException {
	Vector options = new Vector();
	ResultSet rs = db.query(
	    "select unique u.user_id as folder_user_id, "
	    + "  u.first_names || ' ' || u.last_name as user_name "
	    + "from users u, fs_files fsf, fs_versions_latest fsvl "
	    + "where fsf.file_id = fsvl.file_id "
	    + "  and not fsf.owner_id = ? "
	    + "  and fsf.owner_id = u.user_id "
	    + "  and fsf.group_id is null "
	    + "  and (fsf.public_p = 'f' or fsf.public_p is null) "
	    + "  and ad_general_permissions.user_has_row_permission_p (?, 'read', fsvl.version_id, 'FS_VERSIONS') = 't'", new Object[] { userId, userId });
	
	while (rs.next()) {
	    options.addElement("<option value=\"user_id " +  rs.getString("folder_user_id") + "\">"
			       + rs.getString("user_name") + "'s document tree</option>");
	}
	return options;
    }
    
    public FolderBox(Integer userId, String topmostOption, Database db) throws SQLException {
	this.db = db;

	groupOptions = groupFolders(userId);
	privateOptions = privateFolders(userId);
	allOptions = new Vector();
	allOptions.addElement(topmostOption);
	allOptions.addElement(FileStorageProcs.privateIndividualOption());
	if (FileStorageProcs.sharedOption().length() > 0) {
	    allOptions.addElement(FileStorageProcs.sharedOption());
	}
	allOptions.addElement(FileStorageProcs.publicOption());
	for (int i = 0; i < groupOptions.size(); i++) {
	    allOptions.addElement(groupOptions.elementAt(i));
	}
	for (int i = 0; i < privateOptions.size(); i++) {
	    allOptions.addElement(privateOptions.elementAt(i));
	}
	Utilities.removeDups(allOptions);
    }

    public String toString() {
	return "<form action=\"group\">\n"
	    + Parameters.get("FileInfoDisplayFontTag", "fs")
	    + "Go to <select name=\"group_id\">\n"
	    + Utilities.join(allOptions, "\n") + "\n"
	    + "</select>\n"
	    + "<input type=submit value=go>\n"
	    + "</form>";
    }
}
