package com.arsdigita.acs.fs;

import com.arsdigita.acs.db.*;
import com.arsdigita.acs.html.*;
import com.arsdigita.acs.*;
import java.sql.*;
import java.util.*;
import javax.servlet.jsp.PageContext;

/**
 * Represents a single file in the file storage module.
 *
 * @author Bill Schneider
 * @version $Version$
 *
 */

public class StoredFile {
    private String title;
    private String parentTitle;
    private boolean isFolder;
    private Integer parentId;
    private boolean isPublic;
    private Integer ownerId;
    private Integer groupId;
    private boolean deleted;
    private String url;
    private String ownerName;
    private int versionCount;
    private Integer fileId;
    private String treeName; 

    public String getTitle () { return title; }
    public String getParentTitle () { return parentTitle; }
    public boolean getIsFolder () { return isFolder; }
    public Integer getParentId () { return parentId; }
    public boolean getIsPublic () { return isPublic; }
    public Integer getOwnerId() { return ownerId; }
    public Integer getGroupId() { return groupId; }
    public Integer getFileId() { return fileId; }
    public boolean getDeleted() { return deleted; }
    public String getUrl() { return url; }
    public String getOwnerName() { return ownerName; }
    public int getVersionCount() { return versionCount; }
    public String getTreeName() { return treeName; }

    public int getOlderVersions(Database db) 
	throws SQLException, ACSException { 
	return new Integer(
	   db.queryToString("select count(*) from fs_versions"
			    + " where file_id=? " 
			    + " and not superseded_by_id is null", 
			    new Object[] {fileId})).intValue();
    }

    private static final String onWhichTable = "FS_VERSIONS";

    private static final String fileInfoQuery = 
	"select count ( fsv.version_id ) as n_versions, "
	+ " fsf1.file_title,"
	+ " fsf2.file_title as parent_title,"
	+ " fsf1.folder_p,"
	+ " fsf1.parent_id,"
	+ " fsf1.public_p, "
	+ " fsf1.owner_id,"
	+ " fsf1.group_id,"
	+ " fsf1.deleted_p,"
	+ " fsvl.url,"
	+ " u.first_names || ' ' || u.last_name as owner_name"
	+ " from   fs_files fsf1,"
	+ "        fs_files fsf2,"
	+ "        fs_versions fsv,"
	+ "        fs_versions_latest fsvl,"
	+ "        users u"
	+ " where  fsf1.file_id = ?"
	+ " and    fsf1.parent_id = fsf2.file_id (+)"
	+ " and    fsf1.file_id = fsvl.file_id"
	+ " and    ad_general_permissions.user_has_row_permission_p (?, 'read', fsvl.version_id, ?) = 't'"
	+ " and    fsf1.owner_id = u.user_id"
	+ " group by fsf1.file_title,"
	+ "          fsf2.file_title,"
	+ "          fsf1.folder_p,"
	+ "          fsf1.parent_id,"
	+ "          fsf1.public_p,"
	+ "  	     fsf1.owner_id,"
	+ "          fsf1.group_id,"
	+ "          fsf1.deleted_p,"
	+ "          fsvl.url,"
	+ "          u.first_names,"
	+ "          u.last_name";

    public static final String allVersionsQuery =     
	"select fsv.version_id," 
	+ " fsv.version_description," 
	+ " fsv.client_file_name," 
	+ " round (fsv.n_bytes / 1024) as n_kbytes," 
	+ " n_bytes," 
	+ " u.first_names || ' ' || u.last_name as creator_name," 
	+ " to_char ( fsv.creation_date, '" + FileStorageProcs.datePicture() + "' ) as pretty_creation_date," 
	+ " nvl (fsv.file_type, upper (fsv.file_extension) || ' File') as file_type," 
	+ " fsv.author_id," 
	+ " decode (ad_general_permissions.user_has_row_permission_p (?, 'read', fsv.version_id, 'FS_VERSIONS'), 't', 1, 0) as read_p," 
	+ " decode (ad_general_permissions.user_has_row_permission_p (?, 'write', fsv.version_id, 'FS_VERSIONS'), 't', 1, 0) as write_p," 
	+ " decode (ad_general_permissions.user_has_row_permission_p (?, 'administer', fsv.version_id, 'FS_VERSIONS'), 't', 1, 0) as administer_p" 
	+ " from   fs_versions fsv," 
	+ "        users u" 
	+ " where  fsv.file_id = ?" 
	+ " and    fsv.author_id = u.user_id" 
	+ " order by fsv.creation_date desc";

    public static final String oneVersionQuery =     
	"select fsv.version_id," 
	+ " fsv.version_description," 
	+ " fsv.client_file_name," 
	+ " round (fsv.n_bytes / 1024) as n_kbytes," 
	+ " n_bytes," 
	+ " u.first_names || ' ' || u.last_name as creator_name," 
	+ " to_char ( fsv.creation_date, '" + FileStorageProcs.datePicture() + "' ) as pretty_creation_date," 
	+ " nvl (fsv.file_type, upper (fsv.file_extension) || ' File') as file_type," 
	+ " fsv.author_id," 
	+ " decode (ad_general_permissions.user_has_row_permission_p (?, 'read', fsv.version_id, 'FS_VERSIONS'), 't', 1, 0) as read_p," 
	+ " decode (ad_general_permissions.user_has_row_permission_p (?, 'write', fsv.version_id, 'FS_VERSIONS'), 't', 1, 0) as write_p," 
	+ " decode (ad_general_permissions.user_has_row_permission_p (?, 'administer', fsv.version_id, 'FS_VERSIONS'), 't', 1, 0) as administer_p" 
	+ " from   fs_versions_latest fsv," 
	+ "        users u" 
	+ " where  fsv.file_id = ?" 
	+ " and    fsv.author_id = u.user_id" 
	+ " order by fsv.creation_date desc";

    public void select(Database db, Integer fileId, Integer userId) 
    throws SQLException
    {
	this.fileId  = fileId;
	ResultSet rs = db.query(fileInfoQuery, 
				new Object[] {fileId, userId, onWhichTable});
	rs.next();
	deleted = "t".equals(rs.getString("deleted_p"));
	isFolder = "t".equals(rs.getString("folder_p"));
	isPublic = "t".equals (rs.getString("public_p"));
	title = rs.getString("file_title");
	parentTitle = rs.getString("parent_title");
	url = rs.getString("url");
	ownerName = rs.getString("owner_name");
	groupId = new Integer(rs.getInt("group_id"));
	if (groupId.equals(new Integer(0))) 
	    groupId = null;
	ownerId = new Integer(rs.getInt("owner_id"));
	versionCount = rs.getInt("n_versions");     
    }

    /**
     * Returns all versions of a file visible to a given user.
     */
    public StoredFileVersion[] getAllVersions(Database db, 
					      Integer userId) 
    throws SQLException {
	Vector v = new Vector();
	ResultSet rs = db.query(allVersionsQuery, 
				new Object[] {userId, userId, userId, fileId});
	
	while (rs.next()) {
	    v.addElement(new StoredFileVersion(rs));
	}
	StoredFileVersion[] ret = new StoredFileVersion[v.size()];
	v.copyInto(ret);
	return ret;
    }

    /**
     * Returns the latest version of a file visible to a given user.
     */
    public StoredFileVersion getLatestVersion(Database db, 
					      Integer userId) 
	throws SQLException {

	ResultSet rs = db.query(allVersionsQuery, 
				new Object[] {userId, userId, userId, fileId});
	if (rs.next()) {
	    return new StoredFileVersion(rs);
	} else {
	    return null;
	}
    }

    public ContextBar getNavbar(PageContext pageContext, Database db, String source) throws SQLException, ACSException {
	ContextBar navbar = new ContextBar(pageContext);
	if (source.equals("private_individual")) {
	    treeName = "Your personal document tree";
	    navbar.add("", Parameters.get("SystemName", "fs"));
	    navbar.add("private-one-person", treeName);
	    navbar.add("One File");
	} else if (source.equals("private_group")) {
	    String group_name = db.queryToString("select group_name from user_groups where group_id=?", new Object[] {this.getGroupId()});
	    treeName = group_name + " group document tree";
	    navbar.add("",Parameters.get("SystemName", "fs"));
	    navbar.add("private-one-group?group_id=" + this.getGroupId(), treeName);
	} else if (source.equals("public_individual")) {
	    treeName = this.getOwnerName() + "'s document tree";
	    navbar.add("", Parameters.get("SystemName", "fs"));
	    navbar.add("all-public", "Publically accessible files");
	    navbar.add("public-one-person?user_id=" + this.ownerId, treeName);
	    navbar.add("One File");
	} else if (source.equals("public_group")) {
	    String group_name = db.queryToString("select group_name from user_groups where group_id=?", new Object[] {this.getGroupId()});
	    treeName = group_name + " group public document tree";
	    navbar.add("", Parameters.get("SystemName", "fs"));
	    navbar.add("all-public", "Publically accessible files");
	    navbar.add("public-one-group?group_id=" + this.getGroupId(), treeName);
	    navbar.add("One File");
	} else {
	    treeName = "Shared " + Parameters.get("SystemName", "acs") + " document tree";
	    navbar.add("", Parameters.get("SystemName","fs"));
	    navbar.add("One File");
	}
	return navbar;
    }
}



