package com.arsdigita.acs.generalcomments;

import com.arsdigita.acs.*;
import com.arsdigita.acs.db.Database;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.ServletRequest;

/**
 * Utility classes and procedural abstractions for general-comments.
 * 
 * @author teadams@arsdigita.com
 * @author rory@arsdigita.com (Java port)
 * @version $Id: GeneralComments.java,v 1.4 2000/10/18 18:42:00 bschneid Exp $
 */

public class GeneralComments {

    public static final int MODIFIED_LONG = 0;
    public static final int MODIFIED_TIME = 1;
    public static final int MODIFIED_DATE = 2;

    /** Helper procedure (for commentsSummarySorted in particular), to format one comment w/ appropriate attachment link.
     *  @param file_type the MIME type
     *  @param original_width This should be -1 if none was provided.
     *  @param original_height This should be -1 if none was provided.
     *  @param comment_html_p true if the content is HTML formatted
     * 
     *  @see $WEB_ROOT/general-comments/view-one.jsp for an example of its use.
     *  @see GeneralComments.commentsSummarySorted
     */
    public static final String formatGeneralComment(HttpServletRequest request, int comment_id, String client_file_name,
						    String file_type, int original_width, int original_height,
						    String caption, String content, boolean comment_html_p) {
	return formatGeneralComment(request, comment_id, client_file_name, file_type, original_width,
				    original_height, caption, content, comment_html_p, null);
    }
    
    
    /** Helper procedure (for commentsSummarySorted in particular), to format one comment w/ appropriate attachment link.
     *  @param file_type the MIME type
     *  @param original_width This should be -1 if none was provided.
     *  @param original_height This should be -1 if none was provided.
     *  @param comment_html_p true if the content is HTML formatted
     *  @param one_line optional, defaults to null (if other signature is used)
     * 
     *  @see $WEB_ROOT/general-comments/view-one.jsp for an example of its use.
     *  @see GeneralComments.commentsSummarySorted
     */
    public static final String formatGeneralComment(HttpServletRequest request, int comment_id, String client_file_name,
						    String file_type, int original_width, int original_height,
						    String caption, String content, boolean comment_html_p,
						    String one_line) {
	// all this HTML in Java is going to get messy but oh well.
	String return_string = "";
	String return_url = request.getRequestURI() + "?" + Utilities.exportURLVars(new AdSet((ServletRequest)request));
	                           //"[ns_conn url]?[export_ns_set_vars url]"

	if ( !client_file_name.equals("") ) {
	    // We have an attachment.
	    if ( file_type.toLowerCase().indexOf("image/") != -1 ) {
		// It was an image.
		if ( original_width != -1 
		     && original_width < Parameters.getInt("InlineImageMaxWidth","general-comments",512) ) {
		    // It's narrow enough to display inline.
		    return_string += "<center><img src=\"/general-comments/attachment/" + comment_id + "/";
		    return_string += client_file_name + "\" width=" + original_width + " height=" + original_height;
		    return_string += "><p><i>" + caption + "</i></center><br>\n";
		    return_string += comment_html_p?content:Utilities.textToHtml(content);
		    return_string += "\n";
		} else {
		    // Send to an image display page.
		    return_string += comment_html_p?content:Utilities.textToHtml(content);
		    return_string += "\n<br><i>Image: <a href=\"/general-comments/image-attachment?";
		    return_string += Utilities.exportURLVars(request,"comment_id return_url") + "\">" + client_file_name + "</a></i>";
		}
	    } else {
		// Send to raw file download.
		return_string += comment_html_p ? content : Utilities.textToHtml(content);
		return_string += "\n<br><i>Attachment: <a href=\"/general-comments/attachment/" + comment_id + "/";
		return_string += client_file_name + "\">" + client_file_name + "</a></i>";
	    }
	} else {
	    // No attachment
	    if (one_line != null)
		return_string += "<h4>" + one_line + "</h4>";
	    return_string += comment_html_p?content:Utilities.textToHtml(content);
	    return_string += "\n";
	}
	return return_string;
    }


    /** Given an item id and table name, this will return the SQL query
     *  that returns all comments for that item. This is to abstract away
     *  the SQL from people wanting to include a list of comments for an item on a page.
     *  @see $WEB_ROOT/news/item.jsp for an example of its use
     */
    public final static String getOneItemQueryString(int onWhatId, String onWhichTable) {
	String query = "";
	//<ad:db_query id="get_comments" obj="allComments" type="com.arsdigita.acs.generalcomments.GeneralCommentsDataSource">
	query += "select general_comments.comment_id, content, comment_date,\n";
	query += "               first_names || ' ' || last_name as commenter_name,\n";
	query += "               users.user_id as comment_user_id,\n";
	query += "               html_p as comment_html_p,\n";
	query += "               client_file_name,\n";
	query += "               file_type,\n";
	query += "               original_width,\n";
	query += "               original_height,\n";
	query += "               caption,\n";
	query += "               one_line,\n";
	query += "               to_char(modified_date, 'Month DD, YYYY HH:MI AM') as pretty_modified_long,\n";
	query += "               to_char(modified_date, 'MM/DD/YY HH24:MI') as pretty_modified_time,\n";
	query += "               to_char(modified_date, 'MM/DD/YY') as pretty_modified_date\n";
	query += "	from general_comments, users\n";
	query += "	where on_what_id = " + onWhatId + "\n";
	query += "	and on_which_table = '" + onWhichTable + "'\n";
	query += "	and general_comments.approved_p = 't'\n";
	query += "	and general_comments.user_id = users.user_id\n";
	query += "	order by comment_date asc\n";
	return query;
    }

    public final static void commentList(javax.servlet.jsp.PageContext pc,
					 String onWhichTable,
					 Integer onWhatId,
					 String item,
					 String module,
					 String return_url) 
	throws javax.servlet.ServletException, java.io.IOException
    { 
	pc.getOut().flush();
	pc.include("/general-comments/comment-list.tpl?on_which_table=" + 
		   Utilities.URLEncode(onWhichTable) + 
		   "&on_what_id=" + onWhatId + 
		   "&item=" + Utilities.URLEncode(item) + 
		   "&module=" + Utilities.URLEncode(module) + 
		   "&return_url=" + Utilities.URLEncode(return_url));
    }
}
