package com.arsdigita.acs.portals;

import com.arsdigita.acs.*;
import com.arsdigita.acs.db.*;
import java.sql.*;

public class PortalsDataSource extends Selection {

  public PortalsDataSource(SqlStatement sql, Database db) throws SQLException {
    super(sql, db);
  }
  
  public PortalsDataSource(ResultSet rs) throws SQLException {
    super(rs);
  }
  
  /**
   * Translates group_name to lower case and replaces each space with a dash.
   * Encodes everything else using Utilities.URLEncode
   * Needed to encode group_name as a nice url
   *
   */
  public String getURL() throws DataSourceException {
    return Utilities.URLEncode(get("group_name").toLowerCase().replace(' ','-'));
  }
  
  /**
   * Returns the PortalExtension parameter
   */
  public static String getPortalExtension() {
    return Parameters.get("PortalExtension","portals",".ptl");
  }

  /**
   * Returns the AllowUserLevelPortals parameter
   */
  public static boolean allowUserLevelPortals() {
    return Parameters.getBoolean("AllowUserLevelPortals", "portals");
  }

  /**
   * Formats the table name for display in the portals (limiting to 32 chars and in upper case)
   * 
   * @return string representing the table name
   */
  public String getTableName( ) throws DataSourceException { 
    String tableName = new String();
    tableName = get("table_name").toUpperCase();
    if (tableName.length() <= 32) {
      return tableName;
    }
    return tableName.substring(0,31);
  }

}



