package com.arsdigita.acs.portals;

import java.sql.*;
import java.util.*;
import java.io.IOException;
import com.arsdigita.acs.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;
import com.arsdigita.acs.html.*;


import com.arsdigita.acs.db.Database;
// import com.oroinc.text.perl.Perl5Util;

public class PortalsProcs {
  
  /**
   * Returns the portals parameter for name. Wrapper since we use so many parameters
   *
   * @param name the name of the parameter
   * @param default any default value to assign
   */
  public static String portalsParam ( String name, String defaultValue ) {
    if (defaultValue != null) {
      return Parameters.get(name,"portals",defaultValue);
    } 
    return Parameters.get(name,"portals");
  }

  /**
   * Same as portalsParam(name, null)
   */
  public static String portalsParam ( String name ) {
    return portalsParam(name,null);
  }

		    
  /**
   * Generates the admin header for /portals/admin

   *
   * @param title Title of the current page
   */
  public static String adminHeader (String title) {
    String html = "<html><head><title>" + title + "</title></head>\n";
    html += portalsParam("BodyTag");
    html += "<h2>" + title + "</h2>";
    return html;
  }

  /**
   * Generates a standard context bar to use for portal admin pages
   * 
   * @param finalP If true, doesn't make the the section linkable
   */
  public static ContextBar adminContextBar ( PageContext pc, boolean finalP ) {
    ContextBar navbar = new ContextBar(pc);
    if (finalP) {
      navbar.add(Parameters.get("SystemName","portals","Portals") + " Administration");
    } else {
      navbar.add("/admin/portals/", "Administration");
    }
    return navbar;
  }

  /**
   * Same as adminContextBar(false)
   */
  public static ContextBar adminContextBar ( PageContext pc ) {
    return adminContextBar(pc, false);
  }

  /**
   * Same as adminContextBar(false) + one more link
   */
  public static ContextBar adminContextBar ( PageContext pc, String sectionTitle) {
    ContextBar navbar = adminContextBar(pc, false);
    navbar.add(sectionTitle);
    return navbar;
  }

  /**
   * Returns the group name associated with groupId. 
   *
   * @param groupId The group whose name we wants
   * @return The name of the group (or empty string if the group doesn't exist)
   */
  public static String groupName ( String groupId ) throws SQLException, ACSException {
    Database db = new Database();
    String groupName = db.queryToString("select ug.group_name from user_groups ug where ug.group_id = ?", new Object[] {groupId}, "");
    db.close();
    return groupName;
  }


  /**
   * Casts argument to string and returns with calll to groupName(String ...)
   */
  public static String groupName ( Integer groupId ) throws SQLException, ACSException {
    return groupName( groupId.toString() );
  }



  /**
   * Returns the group_id of the portals Super Administrators group
   *
   * @return group id of Super Administrators
   */
  public static String getSuperAdminGroupId ( ) throws SQLException, ACSException {
    Database db = new Database();
    String groupId = db.queryToString("select ug.group_id from user_groups ug where ug.group_name = 'Super Administrators' and ug.group_type = 'portal_group'", "");
    db.close();
    return groupId;
  }


  /** 
   * Same as checkAdministratorMaybeRedirect(req, resp, groupId.toString(), null);
   */

  public static UserInfo checkAdministratorMaybeRedirect ( HttpServletRequest req, 
							   HttpServletResponse resp, 
							   Integer groupId ) throws SQLException, IOException, ACSException {
      if (groupId == null)
	return checkAdministratorMaybeRedirect( req, resp, null, null );
      return checkAdministratorMaybeRedirect( req, resp, groupId.toString(), null );
  }

    /**
     * Redirects to user registration unless the user is a portal
     * administrator (role of "administrator" in the group "Super
     * Administrators" of type "portal_group")
     *
     * @param userId User id of the person we are checking. If "0", we redirect.
     * @param groupId optional argument. If specified, user must also be an administrator of that group. 
     * @param redirectLocation Where to redirect (defaults to /register)
     * 
     * @return UserInfo object of the logged in user (or null if not logged in)
     */
    public static UserInfo checkAdministratorMaybeRedirect ( HttpServletRequest req, 
							     HttpServletResponse resp, 
							     String groupId, 
							     String redirectLocation ) throws SQLException, IOException, ACSException {
      UserInfo user = Security.getVerifiedUserInfo(req, resp);
      if (user == null) {
	// user was redirected to register
	return null;
      }
      Integer userId = user.getUserId();

      // Now we're logged in... Let's build up the sql query we need
      // set up the where clause - a blank groupId results in a more restrictive group check
      String groupRestriction = "";
      Object[] bind;
      
      if ( Utilities.emptyString(groupId) ) {
	groupRestriction += "group_name=  'Super Administrators'";
	bind = new Object[] {userId};
	if ( redirectLocation == null ) {
	  redirectLocation = "/admin/portals/";
	}
      } else {
	groupRestriction += "(map.group_id = :groupId or group_name=  'Super Administrators')";
	bind = new Object[] {userId, groupId};
	if ( redirectLocation == null ) {
	  redirectLocation = "/portals/admin/manage-portal?group_id=" + groupId;
	}
      }
      
      String query = "select count(ug.group_id)" +
                     "  from user_group_map map, user_groups ug" +
                     " where map.user_id = :userId" +
                     "   and map.group_id = ug.group_id" +
                     "   and ug.group_type = 'portal_group'" +
	             "   and role='administrator'" + 
	             "   and " + groupRestriction;

      Database db = new Database();
      String count = db.queryToString(query, bind);
      db.close();
      
      if (count.equals("0")) {
	Utilities.sendRedirect(resp, redirectLocation);
	return null;
      }
      return user;
    }
  

  /**
   * Same as checkAdministratorMaybeRedirect(req, resp, null, null)
   *
   */
  public static UserInfo checkAdministratorMaybeRedirect ( HttpServletRequest req, 
							   HttpServletResponse resp ) throws SQLException, IOException, ACSException {
    return checkAdministratorMaybeRedirect(req, resp, null, null);
    
  }


  /**
   * Returns the subset of portals parameters associated with the display of each portlet
   * 
   * @return AdSet of all the parameters associated with the display of each portlet
   */
  public static AdSet getDisplayInfo () {
    AdSet displayInfo = new AdSet();

    displayInfo.put("SystemName", portalsParam("SystemName"));
    displayInfo.put("BodyTag", portalsParam("BodyTag"));
    displayInfo.put("FontTag", portalsParam("FontTag"));
    displayInfo.put("BeginTable", portalsParam("BeginTable"));
    displayInfo.put("EndTable", portalsParam("EndTable"));
    displayInfo.put("HeaderTD", portalsParam("HeaderTD"));
    displayInfo.put("HeaderBGColor", portalsParam("HeaderBGColor"));
    displayInfo.put("SubHeaderTD", portalsParam("SubHeaderTD"));
    displayInfo.put("SubHeaderBGColor", portalsParam("SubHeaderBGColor"));
    displayInfo.put("NormalTD", portalsParam("NormalTD"));

    return displayInfo;
  }


  public static String adminFormatOnePortalPage ( String currentPage, 
						  String maxPage,
						  String groupName,
						  String pageName,
						  String leftSelect,
						  String rightSelect,
						  String extraOptions
						  ) {
    String rightLink;
    if ( currentPage.equals(maxPage) ) {
      rightLink = "<td align=right>(a new page if needed)</td>";
    } else {
      rightLink = "<td align=right>(<a target=_new href=/portals/" + groupName.toLowerCase() + "-" + currentPage + PortalsProcs.portalsParam("PortalExtension") + ">current version</a>)</td>";
    }
    return "\n" + 
      "<table width=100% bgcolor=#0000 border=0 cellpadding=0 cellspacing=1>\n" + 
      " <tr>\n" + 
      "  <td>\n" + 
      "   <table bgcolor=#cccccc cellspacing=1 cellpadding=4 width=100% border=0>\n" + 
      "    <tr>\n" + 
      "     <td colspan=2 bgcolor=#cccccc>\n" + 
      "      <table width=100% border=0 cellpadding=0 cellspacing=0>\n" + 
      "       <tr>\n" + 
      "        <td>\n" + 
      "         Page #"+currentPage+" - Titled: <font face=arial,helvetica size=-1>\n" + 
      "         <input name=page_name"+currentPage+" type=text size=30 value=\""+pageName+"\">\n" + 
      "        </td>\n" + 
      "        <td>" +
                rightLink +
      "        </td>\n" + 
      "       </tr>\n" + 
      "      </table>\n" + 
      "     </td>\n" + 
      "    </tr>\n" + 
      "    <tr>\n" + 
      "     <td bgcolor=#dddddd valign=top align=center>\n" + 
      "      <table border=0 cellpadding=1 cellspacing=0>\n" + 
      "       <tr>\n" + 
      "        <td>\n" + 
      "         <table cellpadding=4>\n" + 
      "          <tr>\n" + 
      "           <td>\n" + 
      "            <a href=\"\" onClick=\"return spawnWindow('edit','left',"+currentPage+")\"><img src=../pics/edit.gif width=18 height=15 border=0 alt=Edit></a>\n" + 
      "           </td>\n" + 
      "          </tr>\n" + 
      "          <tr>\n" + 
      "           <td>\n" + 
      "            <a href=\"\" onClick=\"return Delete('left',"+currentPage+")\"><img src=../pics/x.gif width=18 height=15 border=0 alt=Delete></a>\n" + 
      "           </td>\n" + 
      "          </tr>\n" + 
      "         </table>\n" + 
      "        </td>\n" + 
      "        <td><font face=courier size=-1>\n" + 
      "         <select name=\"left"+currentPage+"\" size=6>" + leftSelect + " " + extraOptions + "</select>\n" + 
      "        </td>\n" + 
      "        <td>\n" + 
      "         <table cellpadding=4>\n" + 
      "          <tr>\n" + 
      "           <td>\n" + 
      "            <a href=\"\" onClick=\"return moveTable('up','left',"+currentPage+")\"><img src=../pics/up.gif width=18 height=15 border=0 alt=\"Up\"></a>\n" + 
      "           </td>\n" + 
      "          </tr>\n" + 
      "          <tr>\n" + 
      "           <td>\n" + 
      "            <a href=\"\" onClick=\"return slide('left',"+currentPage+")\"><img src=../pics/right.gif width=18 height=15 border=0 alt=\"Right\" hspace=10></a>\n" + 
      "           </td>\n" + 
      "          </tr>\n" + 
      "          <tr>\n" + 
      "           <td>\n" + 
      "            <a href=\"\" onClick=\"return moveTable('down','left',"+currentPage+")\"><img src=../pics/down.gif width=18 height=15 border=0 alt=Down></a>\n" + 
      "           </td>\n" + 
      "          </tr>\n" + 
      "         </table>\n" + 
      "        </td>\n" + 
      "       </tr>\n" + 
      "      </table>\n" + 
      "     </td>\n" + 
      "     <td bgcolor=#dddddd valign=top align=center width=50%>\n" + 
      "      <table border=0 cellpadding=1 cellspacing=0>\n" + 
      "       <tr>\n" + 
      "        <td>\n" + 
      "         <table cellpadding=4>\n" + 
      "          <tr>\n" + 
      "           <td align=right>\n" + 
      "            <a href=\"\" onClick=\"return moveTable('up','right',"+currentPage+")\"><img src=../pics/up.gif width=18 height=15 border=0 alt=\"Up\"></a>\n" + 
      "           </td>\n" + 
      "          </tr>\n" + 
      "          <tr>\n" + 
      "           <td>\n" + 
      "            <a href=\"\" onClick=\"return slide('right',"+currentPage+")\"><img src=../pics/left.gif width=18 height=15 border=0 alt=\"Left\" hspace=10></a>\n" + 
      "           </td>\n" + 
      "          </tr>\n" + 
      "          <tr>\n" + 
      "           <td align=right>\n" + 
      "            <a href=\"\" onClick=\"return moveTable('down','right',"+currentPage+")\"><img src=../pics/down.gif alt=Down width=18 height=15 border=0></a>\n" + 
      "           </td>\n" + 
      "          </tr>\n" + 
      "         </table>\n" + 
      "        </td>\n" + 
      "        <td><font face=courier size=-1>\n" + 
      "         <select name=\"right"+currentPage+"\" size=6>" + rightSelect + " " + extraOptions + "</select>\n" + 
      "        </td>\n" + 
      "        <td>\n" + 
      "         <table cellpadding=4>\n" + 
      "          <tr>\n" + 
      "           <td>\n" + 
      "            <a href=\"\" onClick=\"return spawnWindow('edit','right',"+currentPage+")\"><img src=../pics/edit.gif width=18 height=15 border=0 alt=Edit></a>\n" + 
      "           </td>\n" + 
      "          </tr>\n" + 
      "          <tr>\n" + 
      "           <td>\n" + 
      "            <a href=\"\" onClick=\"return Delete('right',"+currentPage+")\"><img src=../pics/x.gif width=18 height=15 border=0 alt=Delete></a>\n" + 
      "           </td>\n" + 
      "          </tr>\n" + 
      "         </table>\n" + 
      "        </td>\n" + 
      "       </tr>\n" + 
      "      </table>\n" + 
      "     </td>\n" + 
      "    </tr>\n" + 
      "   </table>\n" + 
      "  </td>\n" + 
      " </tr>\n" + 
      "</table>\n" + 
      "<br>\n";
  }


}
