package com.arsdigita.acs.spam;

import com.arsdigita.acs.*;
import com.arsdigita.acs.db.*;
import java.sql.*;
import java.util.Vector;
import com.oroinc.text.perl.*;
import java.io.*;
import org.apache.turbine.util.Log;

/**
 * Utility classes and procedural abstractions for spam.
 * @author Phong Nguyen (phong@arsdigita.com)
 * @version $Version$
 */
public class Spam {

    private static String email_sending_enabled_p = "1";
    private static String daemon_enabled_p = "0";
    private static String daemon_sending_p = "0";
    private static String use_bulkmail_p = "0";

    /**
     * Returns the state of the spam daemon.
     */
    public static String getDaemonEnabledP () {
	return daemon_enabled_p;
    }
    
    /** 
     * Returns the state of sending spam.
     */
    public static String getEmailSendingEnabledP () {
	return email_sending_enabled_p;
    }
    
    /** 
     * Returns the state of using bulkmail.
     */
    public static String getUseBulkmailP () {
	return use_bulkmail_p;
    }
    
    /**
     * Enable/disable the sending of spam.
     */
    public static void setEmailSending(String val) {
	email_sending_enabled_p = val;
    }
   
    /**
     * Enable/disable the spam daemon.
     */
    public static void setDaemonActive(String val) {
	daemon_enabled_p = val;
    }

    /**
     * Enable/disable use of bulk mailer for sending of spam.
     */
    public static void setUseBulkMailP(String val) {
	use_bulkmail_p = val;
    }
    
    /**
     * Calls on the spam daemon to send out all spam.
     */
    public static String sendScheduledSpamMessages() 
	throws SQLException, ACSException, FileNotFoundException, IOException {

	Log.note("running scheduled spam sending daemon");
	
	if ( email_sending_enabled_p.equals("0") ) {
	    Log.note("spam daemon is disabled via the spam_email_sending_enabled_p flag");
	    return "";
	}
	
	if ( daemon_sending_p.equals("1") ) {
	    Log.note("send_scheduled_spam_messages: in another thread, a spam daemon process is busy running, returning");
	    return "";
	}
	
      daemon_sending_p = "1";
      
      /*
       	We loop pulling spams out of the spam_history table that
	are ready to send. Warning: there is a race condition here -
	if another thread tries to scan the queue simultaneously, then
	there is a danger of two threads trying to send the same spam.
	In production, make sure that the only one who calls this function
	is a single scheduled proc, to avoid this condition.
	
	It is also possible to rewrite this as an "update" query to do an atomic
	grab of the top item on the queue and move it to "sending" state
	simultaneously. That would fix the race condition. [hqm]
      */
      
      String more_spam_p = "1";
      
      // check for files deposited by users as specified in the daily_spam_files table
      Log.note("++++++++ about to run checkSpamDropZone ++++++");
      if ( daemon_enabled_p.equals("1") ) {
	  checkSpamDropZone();
      }
      Log.note("++++++++ finished running checkSpamDropZone +++++");

      // get a database connection
      Database db = new Database();
      
      Log.note("++++++++ starting WHILE loop in sendScheduledSpamMessages ++++++++");
      while ( more_spam_p.equals("1") ) {
	  
	  Log.note("send_scheduled_spam_messages: checking spam_history queue");
	  
	  /* Pick the spam which is furthest past deadline of all
		 spams which have current or past-due deadlines,and
		 which have not already been processed. */

	  String sql_query = "" +
		  "SELECT spam_id, " +
		  "       creation_user, " +
		  "       from_address, " +
		  "       body_plain, " +
		  "       body_html, " +
		  "       body_aol, " +
		  "       title, " +
		  "       user_class_query, " +
		  "       user_class_description, " +
		  "       send_date, " +
		  "       status, " +
		  "       last_user_id_sent, " +
		  "       template_p " +
		  "FROM spam_history " +
		  "WHERE spam_id = (select min(spam_id) " +
		  "                 from spam_history " +
		  "                 where send_date <= sysdate and " +
		  "                      (status = 'unsent' OR status = 'interrupted')) ";
	      
	  ResultSet rs = db.query(sql_query);
	  if (rs.next()) {
	      more_spam_p = "1";
	  } else {
	      more_spam_p = "0";
	  }
	  
	  if (more_spam_p.equals("1")) {

	      // create an Adset to hold all values
	      AdSet spam_row_set = new AdSet();
	      spam_row_set.put("spam_id", rs.getString("spam_id"));
	      spam_row_set.put("creation_user", rs.getString("creation_user"));
	      spam_row_set.put("from_address", rs.getString("from_address"));
	      spam_row_set.put("body_plain", rs.getString("body_plain"));
	      spam_row_set.put("body_html", rs.getString("body_html"));
	      spam_row_set.put("body_aol", rs.getString("body_aol"));
	      spam_row_set.put("title", rs.getString("title"));
	      spam_row_set.put("user_class_query", rs.getString("user_class_query"));
	      spam_row_set.put("user_class_description", rs.getString("user_class_description"));
	      spam_row_set.put("send_date", rs.getString("send_date"));
	      spam_row_set.put("status", rs.getString("status"));
	      spam_row_set.put("last_user_id_sent", rs.getString("last_user_id_sent"));
	      spam_row_set.put("template_p", rs.getString("template_p"));

	      String spam_id = spam_row_set.get("spam_id");
	      String user_class_description = spam_row_set.get("user_class_description");
	      
	      String sql_update = "" +
		  "update spam_history " +
		  "set status = 'sending', " +
		  "    begin_send_time = sysdate  " +
		  "where spam_id = :spam_id ";
	      db.dml(sql_update, new Object[] {spam_id});

	      Log.note("spam_daemon: sending spam_id " + spam_id + " '" + user_class_description + "'");
	      
	      sendSpamMessage(spam_row_set);

	  }
      } //while
      Log.note("+++++++++ finished WHILE loop in sendScheduledSpamMessages");
      
      daemon_sending_p = "0";
      return "";
    }
    
    /**
     * Extract the msg bodies and target recipients list query from the spam msg, and
     * send email to the recipients.
     * @param spam_row_set Contains a spam row from spam_history.
     */
    public static void sendSpamMessage(AdSet spam_row_set) throws SQLException {
    
	Log.note("++++++ executing sendSpamMessage +++++");

	// these are the variables passed in the AdSet
	String spam_id = spam_row_set.get("spam_id");
	String creation_user = spam_row_set.get("creation_user");
	String from_address = spam_row_set.get("from_address");
	String body_plain = spam_row_set.get("body_plain");
	String body_html = spam_row_set.get("body_html");
	String body_aol = spam_row_set.get("body_aol");
	String title = spam_row_set.get("title");
	String user_class_query = spam_row_set.get("user_class_query");
	String user_class_description = spam_row_set.get("user_class_description");
	String send_date = spam_row_set.get("send_date");
	String status = spam_row_set.get("status");
	String last_user_id_sent = spam_row_set.get("last_user_id_sent");
	String template_p = spam_row_set.get("template_p");

	Log.note("send_spam_message: got spam_id=" + spam_id + ", from_address=" + from_address + ", title=" + title);

	if (Utilities.emptyString(user_class_description)) {
	    user_class_description = "ACS mailing list unnamed";
	}

	// These hold any extra mail headers we want to send with each message.
	// Headers for HTML mail
	AdSet msg_html_headers = new AdSet();
	msg_html_headers.put("Mime-Version", "1.0");
	msg_html_headers.put("Content-Type", "multipart/alternative; boundary=\"" + spamMimeBoundary() + "\"");
	msg_html_headers.put("X-User-Class-List", user_class_description);
	    
	// Headers for AOL mail
	AdSet msg_aol_headers = new AdSet();
	msg_aol_headers.put("Content-type", "text/html; charset=\"iso-8859-1\"");

	// Headers for plain text mail
	AdSet msg_plain_headers = new AdSet();

	// For each spam, get the list of interested users, and send mail.
	// standard query but make sure that we don't send to dont_spam_me_p or
	// deleted_p users
	if (Utilities.emptyString(last_user_id_sent)) {
	    last_user_id_sent = "0";
	}

	// Note how we isolate the user_class_query in a subquery. This is
	// potentially inefficient, but safe, and maybe the Oracle optimizer
	// can do something smart with it.
	String sql_query = "" +
	    "SELECT c.user_id, " +
	    "       uspam.email, " +
	    "       up.email_type, " +
	    "       uspam.first_names, " +
	    "       uspam.last_name " +
            " FROM (" + user_class_query + ") c, users_spammable uspam, users_preferences up " +
	    "WHERE up.user_id = uspam.user_id " +
	    "  AND uspam.user_id = c.user_id " +
	    "  AND uspam.user_id > " + last_user_id_sent + " " +
	    "ORDER BY c.user_id";

	Log.note("SEND_SPAM_MESSAGE:  1) query = " + sql_query);

	
	// bulkmail is not supported
	// db_with_handle db {
	//   if {$use_bulkmail_p == 1} {
	//     set bulkmail_id [bulkmail_begin $creation_user "spam_id $spam_id"]
	//   } else {
	//   set bulkmail_id ""
	//   }
	// }
	String bulkmail_id = "0";
	
	// get a database handle
	Database db = new Database();
	
	// query sets user_id for each interested user

	// more accurate flow rate if we set this as the start time, because above query takes
	// a couple minutes sometimes
	db.dml("update spam_history set begin_send_time = sysdate where spam_id = :spam_id", new Object[] {spam_id});

	int send_count = 0;
	
	// Send the spam to the recipients
	send_count = sendMsgToRecipients(sql_query, spam_id, from_address, body_plain, body_html, body_aol, msg_plain_headers, msg_html_headers, msg_aol_headers, template_p, title, bulkmail_id);

	int logging_frequency = checkpointLoggingFrequency();
	
	// bulkmail is not supported
	// close out the bulkmail run if needed, add up the last group of users sent
	// if {$use_bulkmail_p == 1} {
        //   bulkmail_end $bulkmail_id
	// }

	int temp_n_sent = send_count % logging_frequency;
	db.dml("update spam_history set n_sent = n_sent + " + temp_n_sent + " where spam_id = :spam_id", new Object[] {spam_id});

	db.dml("update spam_history set status = 'sent', finish_send_time = sysdate where spam_id = :spam_id", new Object[] {spam_id});

	Log.note("moving spam_id " + spam_id + " to \"sent\" state");

	Log.note("++++++ finished sendSpamMessage +++++");
    }

    /**
     * Returns html code fragment for time input
     * @param column The base name for the input variables in a form.
     */
    public static String timeWidget(String column) throws SQLException, ACSException {
	
	String output = "";
	
	// get the current time
	String sql_query = "select to_char(sysdate, 'AM') as ampm, to_char(sysdate, 'HH12:MI:SS') as time from dual";
	Database db = new Database();
	ResultSet rs = db.query(sql_query);    
	rs.next();
	String time = rs.getString("time");
	String ampm = rs.getString("ampm");
	
	// produce the time input field
	output = "<input name=" + column + ".time value=" + time + " type=text size=9>&nbsp;";
	
	// produce the am or pm select
	output += "<select name=" + column + ".ampm>";
	
	if (ampm.equals("AM")) {
	    output += "<option value=AM selected> AM";
	    output += "<option value=PM> PM";
	} else {
	    output += "<option value=AM> AM";
	    output += "<option value=PM selected> PM";
	}
	output += "</select>";
	
	return output;
    }

    /**
     * Return html code fragment for date input
     * @param column The base name for the input variables in a form.
     */
    public static String dateWidget(String column) throws SQLException, ACSException {
      
	String output = "";
	String month_option[] = new String[] {"January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"};
	String month_value[] = new String[] {"01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12"};
	
	// get the current date
	String sql_query = "select to_char(sysdate, 'MM') as month, to_char(sysdate, 'DD') as day, to_char(sysdate, 'YYYY') as year from dual";
	Database db = new Database();
	ResultSet rs = db.query(sql_query);
	rs.next();
	String month = rs.getString("month");
	String day = rs.getString("day");
	String year = rs.getString("year");
	
	// add the month select menu
	output += "<select name=" + column + ".month>";
	output += htmlSelectValueOptions(month_option, month_value, month);
	output += "</select>&nbsp;";
      
	// add the day input
	output += "<input name=" + column + ".day value=\"" + day + "\" type=text size=3 maxlength=2>&nbsp;";
	
	// add the year input
	output += "<input name=" + column + ".year value=\"" + year + "\" type=text size=5 maxlength=4>";
	
	return output;
    }
    
    /**
     * Generate html option tags with values for an html selection widget. If
     * select_option is passed and there exists a value for it in the values
     * list, this option will be marked as selected.
     * @param sql The sql query used to obtain values
     * @param option The column from sql to use as the value of the option.
     * @param value The column from sql to use as the name of the option.
     * @param selected_value The value to mark as selected.
     */
    public static String htmlSelectValueOptions (String sql, 
						 String option, 
						 String value, 
						 String selected_value) 
	throws SQLException, ACSException {
	
	String output = "";
	Database db = new Database();
	ResultSet rs = db.query(sql); 	
	while ( rs.next() ) {
	    if ( selected_value.equals(rs.getString(value)) ) {
		output += "<option value=\"" + rs.getString(value) + "\" selected>";
		output += rs.getString(option) + "</option>\n";
	    } else {
		output += "<option value=\"" + rs.getString(value) + "\">";
		output += rs.getString(option) + "</option>\n";
	    }
	}
	return output;
    }

    /**
     * Generate html option tags with values for an html selection widget. If
     * select_option is passed and there exists a value for it in the values
     * list, this option will be marked as selected.
     * @param option An array containing the values for each option.
     * @param value An array contaiing the names for each option.
     * @param selected_value The value to mark as selected.
     */
    public static String htmlSelectValueOptions (String [] option, 
					  String [] value, 
					  String selected_value) {
	
	String output = "";
	for (int i = 0; i < value.length; i++) {
	    if ( selected_value.equals(value[i]) ) {
		output += "<option value=\"" + value[i] + "\" selected>";
		output += option[i] + "</option>\n";
	    } else {
		output += "<option value=\"" + value[i] + "\">";
		output += option[i] + "</option>\n";
	    }
	}
	return output;
    }


    /**
     * Takes date values from an AdSet and checks that they are valid.
     * @param date An AdSet containing "year", "month", and "day" fields.
     * @returns The combined date string or "0" if invalid.
     */
    public static String verifyDate(AdSet date) throws SQLException, ACSException {
	String output = "0";
	// let the database do the conversion for us
	String temp_date = date.get("year") + "-" + date.get("month") + "-" + 
	    date.get("day") + " " + date.get("time") + ":" + date.get("ampm");
	String sql_query = "select to_char(to_date('" + temp_date + 
	    "', 'YYYY-MM-DD HH12:MI:SS:AM'), 'YYYY-MM-DD HH24:MI:SS') from dual";
	Database db = new Database();
	output = db.queryToString(sql_query);
	return output;
    }  
    
    /**
     * Returns html code fragment for an option list of integer values
     * @param begin The begin number for option list.
     * @param end The end number for the option list.
     * @param selected_value the value to mark as selected.
     */
    public static String integerOptionList (int begin, int end, String selected_value) {
	String output = "";
	for (int i=begin; i<=end; i++) {
	    if (selected_value.equals("" + i)) {
		output += "<option selected value=\"" + i + "\">" + i + "</option>\n";
	    } else {
		output += "<option value=\"" + i + "\">" + i + "</option>\n";
	    }
	}
	return output;
    }
    
    
    /**
     * Insert a message to be sent by the spam daemon at a scheduled time send_date.
     * @param spam_id Spam_id of the message.
     * @param template_p Is this message in template format?
     * @param from_address Address to send from.
     * @param title Title of the spam.
     * @param body_plain Message body for plain messages.
     * @param body_html Message body for html messages.
     * @param body_aol Message body for aol messages.
     * @param target_users_description Description for the group of users to send spam to.
     * @param target_users_query Query that returns the group of users to spam.
     * @param send_date A string of the form "YYYY-MM-DD HH24:MI:SS" or else an empty string.
     * @param creation_user User_id of creator.
     * @param status Status of spam state.
     * @return Return "0" if no errors, else returns the error message.
     */
    public static String postNewSpamMessage (String spam_id, String template_p, 
					     String from_address, String title, 
					     String body_plain, String body_html, 
					     String body_aol, String target_users_description, 
					     String target_users_query, String send_date, 
					     String creation_user, String status) 
	throws DataSourceException, ACSException {
	
	// validate and initialize arguments
	Vector missing_args = new Vector();
	if (Utilities.emptyString(template_p)) {
	    template_p = "f";
	}
	if (Utilities.emptyString(status)) {
	    status = "unsent";
	}
	if ( !status.equals("hold") && Utilities.emptyString(body_plain) ) {
	    missing_args.addElement("body_plain");
	}
	if ( Utilities.emptyString(creation_user) ) {
	    missing_args.addElement("creation_user");
	}
	if ( missing_args.size() > 0 ) {
	    throw new ACSException("missing " + missing_args.size() + "arg(s): " + Utilities.join(missing_args, ", "));
	}
	
	String sql_send_date = send_date;
	if (Utilities.emptyString(sql_send_date)) {
	    sql_send_date = "NULL";
	}
	
	try {
	    Database db = new Database();
	    String sql_query = "" +
		"insert into spam_history " +
		"(spam_id, template_p, from_address, title, " +
		" body_plain, body_html, body_aol, " +
		" user_class_description, user_class_query, " +
		" send_date, creation_date, creation_user, " +
		" status, creation_ip_address, pathname) " +
		"values " +
		"(?, ?,	?, ?, ?, ?, ?, ?, ?," +
		" nvl(to_date(?, \'YYYY-MM-DD HH24:MI:SS\'), sysdate)," +
		" sysdate," +
		" ?, ?," +
		" \'0.0.0.0\', null)";
	    Object[] bind = new Object[] {spam_id, template_p, from_address, title,
					  body_plain, body_html, body_aol,
					  target_users_description, target_users_query, 
					  sql_send_date, creation_user, status};
	    db.dml(sql_query, bind);
	} catch (SQLException sqle) {
	    return sqle.getMessage();
	}
	return "0";
    }
    
    /**
     * Returns the list of parameter var names used to define a user class.
     */
    public static Vector userClassParameters () {
	Vector v = new Vector();
	v.addElement("category_id");
	v.addElement("country_code");
	v.addElement("usps_abbrev");
	v.addElement("intranet_user_p");
	v.addElement("group_id");
	v.addElement("last_name_starts_with");
	v.addElement("email_starts_with");
	v.addElement("expensive");
	v.addElement("user_state");
	v.addElement("sex");
	v.addElement("age_above_years");
	v.addElement("age_below_years");
	v.addElement("registration_during_month");
	v.addElement("registration_before_days");
	v.addElement("registration_after_days");
	v.addElement("registration_after_date");
	v.addElement("last_login_before_days");
	v.addElement("last_login_after_days");
	v.addElement("last_login_equals_days");
	v.addElement("number_visits_below");
	v.addElement("number_visits_above");
	v.addElement("user_class_id");
	v.addElement("sql_post_select");
	v.addElement("crm_state");
	v.addElement("curriculum_elements_completed");
	return v;
    }
        
    /**
     * Takes an AdSet of key/value pairs and produces a human-readable 
     * description of the class of users specified.
     */
    public static String userClassDescription (AdSet setid) throws ACSException, SQLException {
	
	try {
	    // Initialize variables
	    Database db = new Database();
	    Vector ucp = Spam.userClassParameters();
	    Vector clauses = new Vector();
	    String pretty_description = "";
	    String cur_elem = "";
	    String cur_value = "";
	    String temp = "";
	    
	    for (int i=0; i < ucp.size(); i++ ) {
		cur_elem = (String) ucp.elementAt(i);
		cur_value = setid.get(cur_elem);
		if (!Utilities.emptyString(cur_value)) {
		    
		    if (cur_elem.equals("category_id")) {
			temp = db.queryToString("select category from categories where category_id = :category_id", new Object[] {cur_value});
			clauses.addElement("said they were interested in " + temp);
			
		    } else if (cur_elem.equals("country_code")) {
			temp = db.queryToString("select country_name from country_codes where iso = :country_code", new Object[] {cur_value});
			clauses.addElement("told us that they live in " + temp);
			
		    } else if (cur_elem.equals("usps_abbrev")) {
			temp = db.queryToString("select state_name from states where usps_abbrev = :usps_abbrev", new Object[] {cur_value});
			clauses.addElement("told us that they live in " + temp);
			
		    } else if (cur_elem.equals("intranet_user_p")) {
			clauses.addElement("are an employee");
			
		    } else if (cur_elem.equals("group_id")) {
			temp = db.queryToString("select group_name from user_groups where group_id = :group_id", new Object[] {cur_value});
			clauses.addElement("are a member of " + temp);
			
		    } else if (cur_elem.equals("last_name_starts_with")) {
			clauses.addElement("have a last name starting with " + cur_value);
			
		    } else if (cur_elem.equals("email_starts_with")) {
			clauses.addElement("have an email address starting with " + cur_value);
			
		    } else if (cur_elem.equals("expensive")) {
			clauses.addElement("have accumulated unpaid carges of more than " + Parameters.get("ExpensiveThreshold","member-vlue"));
			
		    } else if (cur_elem.equals("user_state")) {
			clauses.addElement("have user state of " + cur_value);
			
		    } else if (cur_elem.equals("sex")) {
			clauses.addElement("are " + cur_value);
			
		    } else if (cur_elem.equals("age_above_years")) {
			clauses.addElement("is older than " + cur_value);
			
		    } else if (cur_elem.equals("age_below_years")) {
			clauses.addElement("is younger than " + cur_value);
			
		    } else if (cur_elem.equals("registration_during_month")) {
			temp = db.queryToString("select to_char(:registration_during_month,'YYYYMM'),'fmMonth YYYY') from dual", new Object[] {cur_value});
			clauses.addElement("registered during " + temp);
			
		    } else if (cur_elem.equals("registration_before_days")) {
			clauses.addElement("registered over " + cur_value + " days ago");
			
		    } else if (cur_elem.equals("registration_after_days")) {
			clauses.addElement("registerd in the last " + cur_value + " days");
			
		    } else if (cur_elem.equals("registration_after_date")) {
			clauses.addElement("registerd on or after " + cur_value);
			
		    } else if (cur_elem.equals("last_login_before_days")) {
			clauses.addElement("have not visited the site in " + cur_value + " days");
			
		    } else if (cur_elem.equals("last_login_after_days")) {
			clauses.addElement("have not visited the site in " + cur_value + " days");
			
		    } else if (cur_elem.equals("last_login_equals_days")) {
			if (cur_value.equals("1")) {
			    clauses.addElement("visited the site exactly 1 day ago");
			} else {
			    clauses.addElement("visited the site exactly " + cur_value + "days ago");
			}
			
		    } else if (cur_elem.equals("number_of_visits_below")) {
			clauses.addElement("have visited less than " + cur_value + " times");
			
		    } else if (cur_elem.equals("number_of_visits_above")) {
			clauses.addElement("have visited more than " + cur_value + " times");
			
		    } else if (cur_elem.equals("user_class_id")) {
			temp = db.queryToString("select name from user_classes where user_class_id = :user_class_id", new Object[] {cur_value});
			clauses.addElement("are in the user class " + temp);
			
		    } else if (cur_elem.equals("sql_post_select")) {
			clauses.addElement("are returned by \"<i>select users(*) from " + cur_value);
			
		    } else if (cur_elem.equals("crm_state")) {
			clauses.addElement("are in the customer state \"" + cur_value + "\"");
			
		    } else if (cur_elem.equals("curriculum_elements_completed")) {
			if (cur_value.equals("1")) {
			    clauses.addElement("who have completed exactly " + cur_value + "element");
			} else {
			    clauses.addElement("who have completed exactly " + cur_value + "elements");
			}
		    }
		    
		}
	    }
	    
	    if ( !Utilities.emptyString(setid.get("combind_method")) && setid.get("combinb_method").equals("or") ) {
		pretty_description = Utilities.join(clauses, " or ");
	    } else {
		pretty_description = Utilities.join(clauses, " and ");
	    }
	    
	    return pretty_description;
	    
	} catch (SQLException sqle) {
	    throw new ACSException(sqle.getMessage());
	}
	
    }
    
    /**
     * Takes an AdSet of key/value pairs and produces a query for 
     * the class of users specified (one user per row returned).
     * @param set_id An AdSet containing all the parameters of the user class.
     * @return Returns an sql query string that returns the users to spam.
     */
    public static String userClassQuery (AdSet setid) throws SQLException, ACSException {
	String final_query = "";
	Vector select_list = new Vector();
	Database db = new Database();
	
	if ( setid.get("count_only_p") != null && 
	     setid.get("count_only_p").equals("1") ) {
	    select_list.addElement("count(users.user_id)");
	} else {
	    // get the columns from the users table
	    String sql_query = "select lower(column_name) as column_name from user_tab_columns where table_name='USERS'";
	    ResultSet rs = db.query(sql_query);
	    while (rs.next()) {
		select_list.addElement("users." + rs.getString("column_name"));
	    }
	}
	if ( setid.get("user_class_id") != null && !setid.get("user_class_id").equals("") ) {
	    String sql_post_select = db.queryToString("select sql_post_select from user_classes where user_class_id = :user_class_id", new Object[] {setid.get("user_class_id")});
	    return "select " + Utilities.join(select_list, ",\n    ") + " \n" + sql_post_select;
	}
	return final_query;
    }
    
    /**
     * Remove any pathname metachars from filename,
     * allow only a very clean set of characters
     * @param filename The filename to sanitize.
     */
    public static String sanitizeFilename (String filename) {
	//TO DO: sanitizeFilename doesn't handle spaces in directories
	Perl5Util re = new Perl5Util();      
	String new_filename;
	if (re.match("@([^//\\\\]+)$@", filename)) {
	    new_filename = re.group(1);
	} else {
	    new_filename = filename;
	}
	return new_filename;
    }
    
    /**
     * Returns the absolute path to the spam file.
     * @param filename The name of the file.
     * @return Spam dropzone directory with filename appended to the end.
     */
    public static String fileLocation(String filename) {
	return Parameters.get("DailySpamDirectory", "spam", "") + "/" + filename;
    }

    /**
     * Returns the file from the dropzone as a string.
     * @param filename The name of the file.
     * @return The contents of the file from the dropzone as a string.
     */
    public static String getSpamFromFilesystem(String filename) throws FileNotFoundException, IOException {
	String fname = fileLocation(filename);	
	return readFileAsString(fname);
    }
    
    /**
     * Look for a set of files with names specified from the daily_spam_files table.
     */
    public static void checkSpamDropZone() 
	throws SQLException, ACSException, FileNotFoundException, IOException {

	/*
	  Look for a set of files with names specfied from the daily_spam_files table
	  
	  For example, file_prefix=daily looks for the file "daily[-MM-DD-YYYY]" as the indicator
	  that the spam is ready to go. The date is optional. If there is no dated file,
	  the plain file prefix name will be used at the content source file.
	  
	  It will also look for aux files "daily-html[-MM-DD-YYYY]" and "daily-aol[-MM-DD-YYYY]"
	  and if they exist, their content will be sent to users with those email type preferences.
	  
	  If no AOL file is found, but there is an HTML file, the HTML file's content will be used for the AOL
	  content.
	  
	  Bug: There is thus a race condition, if the
	  content file is only partially written when the daemon probes for it.
	  A solution would be to use another dummy file whose existence indicates the
	  other spam files are complete and ready. Make sure to at least write the
	  HTML and AOL files first, to close the race condition window a little.
	  
	  Note: if you omit the date from a filename, the same file will be picked up and
	  sent once per day. This usually is not what you want.
	*/
	
	// get a database handle
	Database db = new Database();

	// create Perl5Util object for regexps
	Perl5Util re = new Perl5Util();

	String date = db.queryToString("select to_char(sysdate,'MM-DD-YYYY') from dual");
	String pretty_date = db.queryToString("select to_char(sysdate,'MM/DD/YYYY') from dual");

	// outer loop - iterate over daily_spam_files table looking for descriptors of
	// files in the drop zone.

	String sql_query = "select file_prefix, from_address, subject, target_user_class_id, template_p, user_class_description, period, day_of_week, day_of_month, day_of_year from daily_spam_files";

	ResultSet rs = db.query(sql_query);
	
	while (rs.next()) {
	    /*
	      depending if the period is daily, weekly, or monthly, compute the date
	      by which it needs to be sent.
	      
	      If there exists a spam for this file-prefix within
	      the "last_sent_the window" condition defined below, then
	      is is too early to send a new periodic spam of that type.
	    */
	    String subject = rs.getString("subject");
	    String period = rs.getString("period");
	    String file_prefix = rs.getString("file_prefix");
	    String last_sent_window = "";
	    String send_date_condition = "";
	    if (period.equals("daily")) {
		last_sent_window = " (trunc(sysdate) - trunc(send_date)) < 1 ";
		send_date_condition = " 1 = 1 ";
	    } else if (period.equals("weekly")) {
		// +++ This needs to be internationalized somehow - it shouldn't
                // depend on english day names
		String day_of_week = rs.getString("day_of_week");
                if (Utilities.emptyString(day_of_week)) {
                    day_of_week = "1";
                }
                last_sent_window = "(sysdate - send_date) < 3 ";
                send_date_condition = " trunc(sysdate-next_day(sysdate-7,'MONDAY')) = ( " + day_of_week + " - 1) ";
	    } else if (period.equals("monthly")) {
		String day_of_month = rs.getString("day_of_month");
		if (Utilities.emptyString(day_of_month)) {
		    day_of_month = "1";
		}
                last_sent_window = " trunc(sysdate, 'MM') - trunc(send_date,'MM') = 0";
		send_date_condition = " trunc(sysdate - trunc(sysdate,'MM')) = ( " + day_of_month + " - 1) ";
	    } else if (period.equals("yearly")) {
		String day_of_year = rs.getString("day_of_year");
		if (Utilities.emptyString(day_of_year)) {
                    day_of_year = "1";
		}
                last_sent_window = "(sysdate - (trunc(send_date,'YYYY'))) > ( " + day_of_year + " - 1))";
                send_date_condition = "trunc(sysdate - trunc(sysdate,'YYYY')) = ( " + day_of_year + "- 1)";
	    } else {
		last_sent_window = " (sysdate - send_date) < 1 ";
		send_date_condition = "1 = 1";
	    }
	    
	    // Check if a spam has already been queued for this file prefix and today's date or period
	    String already_queued = db.queryToString("select count(spam_id) from spam_history where pathname = :file_prefix and " + last_sent_window, new Object[] {rs.getString("file_prefix")});

	    if (!already_queued.equals("0")) {
		continue;
	    } else {
		// check if the date today matches the send_date condition
		String date_ok = db.queryToString("select count(1) from dual where " + send_date_condition);
		
		if (!date_ok.equals("1")) {
		    continue;
		}
		
		// See if file prefix with date exists. If so, then look for other
		// files with date appended.
		
		String dropfile_path = fileLocation(rs.getString("file_prefix"));

		Log.note("checking for dropfile with and without date suffix: [set dropfile_path]-[set date]");
				
		File temp_file_1 = new File(dropfile_path + "-" + date);
		String date_suffix = "";
		if (temp_file_1.canRead()) {
		    date_suffix = "-" + date;
		}

		File temp_file_2 = new File(dropfile_path + date_suffix);
		if (!temp_file_2.canRead()) {
		    Log.note("Daily spam for \"" + file_prefix + "\" could not be sent - no dropfile found at " + dropfile_path + "[" + date_suffix + "]");
		} else {
		    
		    // get the contents of files, and insert into queue
		    // Check for file name which includes date
		    String dropfile_path_plain = dropfile_path + date_suffix;
		    String dropfile_path_html = dropfile_path + "-html" + date_suffix;
		    String dropfile_path_aol = dropfile_path + "-aol" + date_suffix;

		    String file_plain = readFileAsString(dropfile_path_plain);
		    String file_html = readFileAsString(dropfile_path_html);
		    String file_aol = readFileAsString(dropfile_path_aol);

		    // Set other spam params
		    String from_address = rs.getString("from_address");
		    if (Utilities.emptyString(rs.getString("from_address"))) {
			from_address = systemDefaultFromAddress();
		    }
		    
		    // replace all %%DATE%% occurences in subject
		    String subject_x = re.substitute("s/%%DATE%%/" + pretty_date + "/g", subject);
		    		    
		    String system_user = "1";

		    // generate SQL query from user_class_id
		    String target_user_class_id = rs.getString("target_user_class_id");
		    AdSet set_args = new AdSet();
		    set_args.put("user_class_id", target_user_class_id);

		    String query = userClassQuery(set_args);
		    
		    /* TO DO: handle 3 clobs in checkDropZone
		    db_dml queue_new_periodic_spam "
                     insert into spam_history
                        (spam_id,
                         template_p,
                         from_address,
                         title,
                         body_plain,
                         body_html,
                         body_aol,
                         user_class_description,
                         user_class_query,
                         send_date,
                         creation_date,
                         creation_user,
                         status,
                         creation_ip_address,
                         pathname)
                      values
                        (spam_id_sequence.nextval,
                        :template_p,
                        :from_address,
                        :subject_x,
                        empty_clob(),
                        empty_clob(),
                        empty_clob(),
                        :user_class_description,
                        :query,
                        sysdate,
                        sysdate,
                        :system_user,
                        'unsent',
                        '0.0.0.0',
                        :file_prefix)
                     returning body_plain, body_html, body_aol into :1, :2, :3
                   " -clobs [list $file_plain $file_html $file_aol]
		   Log.note("Daily spam queued for \"" + file_prefix + "\" from " + dropfile_path_plain);
		    */
		} 
	    } 
	} 
    }
    
    /**
     * Really, this should make sure to generate a boundary string that
     * does not appear in the content.
     */
    public static String spamMimeBoundary() {
	return "__NextPart_000_00A0_01BF7A3C.09877D80";
    }

    /**
     * Checkpoint the last user_id message was sent to, every n msgs
     */
    public static int checkpointLoggingFrequency() {
	return 20;
    }

    /** 
     * Sends out a spam for spam_id.
     */
    public static int sendMsgToRecipients(String recipients_query, String spam_id, 
					  String from_address, String body_plain, 
					  String body_html, String body_aol, 
					  AdSet msg_plain_headers, AdSet msg_html_headers, 
					  AdSet msg_aol_headers, String template_p, 
					  String title, String bulkmail_id) 
	throws SQLException {
	      
	Log.note("++++++++++ executing sendMsgToRecipioents ++++++++++");

	// create a Pertl5Util for regexps
	Perl5Util re = new Perl5Util();

	// how often we update the n_sent field of the spam record
	int logging_frequency = checkpointLoggingFrequency();

	// Get site-wide removal blurb
	String txt_removal_blurb = removalBlurb("", "txt");
	String html_removal_blurb = removalBlurb("", "htm");
	String aol_removal_blurb = removalBlurb("", "aol");
	
	// prepare the full message
	// TO DO: regsub /r with "" in body_html
	//body_html = re.substitute("s/\\r/ /g", body_html);
	//body_plain = re.substitute("s/\\r/ /g", body_plain);
	String body_plain_full = body_plain + "\n\n" + txt_removal_blurb;
	String full_html_msg = body_html + html_removal_blurb;
	String qp_body_html = encodeQuotedPrintable(body_html);
	String qp_body_plain = encodeQuotedPrintable(body_plain_full); 

	// The SQL which generates the list of target users may be badly formed
	// (perhaps it was created by hand). So we need to put a catch here and
	// skip over this spam if it is causing an error.
	// Mark the spam as 'error' to keep it from being retried forever.
	
	int send_count = 0;
	
	// get a database handle
	Database db = new Database();
	
	ResultSet rs = db.query(recipients_query);

	try {
	    while (rs.next()) {
		
		Log.note("========== (sendMsgToRecipient) inside while loop");
		
		String user_id = rs.getString("user_id");
		String email = rs.getString("email");
		String email_type = rs.getString("email_type");
		String first_names = rs.getString("first_names");
		String last_name = rs.getString("last_name");
		String message_body = "";
		String message_body_template = "";
		
		if ( email_type == null ) {
		    email_type = "text/plain";
		}
		
		if (email_sending_enabled_p.equals("0")) {
		    Log.note("** spam disabled:  spam to " + user_id + ", " + email + ", \"" + title + "\"");
		    break;
		}
		
		// If the HTML or AOL spam message text is null, revert to sending plain text to everyone
		if (email_type.equals("")) {
		    email_type = "text/plain";
		} else if (email_type.equals("text/html") && Utilities.emptyString(body_html)) {
		    email_type = "text/plain";
		} else if (email_type.equals("text/aol-html") && Utilities.emptyString(body_aol)) {
		    email_type = "text/plain";
		}
		
		AdSet extra_headers = new AdSet();
		if (email_type.equals("text/html")) {
		    extra_headers = msg_html_headers;
		    message_body_template = createMultiPartHtmlMessageBody(qp_body_plain, qp_body_html);
		} else if (email_type.equals("text/aol-html")) {
		    extra_headers = msg_aol_headers;
		    message_body_template = body_aol + "\n" + aol_removal_blurb;
		} else {
		    extra_headers = msg_plain_headers;
		    message_body_template = body_plain_full;
		}
		
		/* TO DO: in sendMsgToRecipients, check out global ad_sec_user_id
		   if {[string match $template_p "t"]} {
  		     if [catch {
		       global ad_sec_user_id
		       set message_body [subst $message_body_template]
		       set title [subst $title]
		     } errmsg] {
		     Log.error("Tcl evaluator error in subst call for spam_id " + spam_id + ": " + errmsg + "\nAborting the sending of this spam.");
		     break
		     }
		   } else {
		     set message_body $message_body_template
		   }
		   
		*/
		if (template_p.equals("t")) {
		    message_body = message_body_template;
		}

		// replace all %%EMAIL%% occurences with recipient's email
		message_body_template = re.substitute("s/%%EMAIL%%/" + email + "/g", message_body);

		if (!Utilities.emptyString(first_names)) {
		    // replace all %%FIRST_NAMES%% occurences with recipient's first name
		    message_body_template = re.substitute("s/%%FIRST_NAMES%%/" + first_names + "/g", message_body_template);
		}

		if (!Utilities.emptyString(last_name)) {
		    // replace all %%LAST_NAME%% occurences with recipient's last name 
		    message_body_template = re.substitute("s/%%LAST_NAME%%/" + last_name + "/g", message_body_template);
		}
		
		message_body = message_body_template;
		
		Log.note("Sending spam to " + email + ", \"" + title + "\"");
		if (email_sending_enabled_p.equals("0")) {
		    Log.note("** spam disabled:  spam to " + user_id + ", " + email + ", \"" + title + "\"");
		    daemon_sending_p = "0";
		    break;
		} else {
		    if (use_bulkmail_p.equals("1")) {
		        // bulkmail is not supported
			// The sender address is overwritten automatically by the bulkmailer
			// bulkmail_send $bulkmail_id $user_id $email $from_address $title $message_body {} $extra_headers
		    } else {
			Log.note("========== (sendMsgToRecipient) sendMail to " + email);
			Utilities.sendMail(email, from_address, title, message_body);
		    }
		}
		
		// we log this every N sends, to keep the overhead down
		send_count += 1;
		if (send_count % logging_frequency == 0) {
		    db.dml("UPDATE spam_history SET last_user_id_sent = :user_id, n_sent = n_sent + :logging_frequency WHERE spam_id = :spam_id", new Object[] {user_id, "" + logging_frequency, spam_id});
		}
		
	    } //while
	    
	} catch (SQLException sqle) {
	    Log.error("Tcl evaluator/db-error when executing user_class_query in send_scheduled_spam_messages: sql=" + recipients_query + "\\errmsg=" + sqle.getMessage() + "\nAborting the sending of spam_id " + spam_id + ".");
	    
	    db.dml("update spam_history set status = 'error' where spam_id = :spam_id", new Object[] {spam_id});
	} catch (Exception e) {
	    
	}
	
	Log.note("++++++++++ finished sendMsgToRecipient ++++++++++");
	
	return send_count;
    } 
  
    /**
     * Replaces/removes bad characters from msg
     * @param msg The message to encode
     */
    public static String encodeQuotedPrintable(String msg) {
      // not suported at this time.
      return msg;
    }
  
    /**
     * Gets the standard site-wide or subsection-specific 
     * removal blurb, for attaching to outgoing email.  
     * txt=plain, htm=html mail, aol = aol mail
     * @param subsection
     * @param filetype
     */
    public static String removalBlurb(String subsection, String filetype) {	
        // not supported at this time.
	return "";
    }
    
    /**
     * Reads the file and returns the contents as a string
     * @param filename The file to read
     * @return The contents of the file as a string.
     */
    public static String readFileAsString(String filename) throws FileNotFoundException, IOException {
	String file_contents = "";
	File filein = new File(filename);
	if (filein.canRead()) {
	    BufferedReader buffered_file = new BufferedReader (new FileReader(filein));
	    while (buffered_file.ready()) {
		file_contents += buffered_file.readLine() + "\n";
	    }
	}
	return file_contents;
    }
    
    /**
     * Returns the default from address for spam messages set in the parameters file
     */
    public static String systemDefaultFromAddress() {
	return Parameters.get("SpamRobotFromAddress", "spam", "email-notification@arsdigita.com");
    }
    
    /** 
     * Creates a multipart/alternative MIME message with a plain text part and HTML part
     * The two parts should be passed in encoded as quoted-printable text.
     * @param qp_body_plain Alternative plain message
     * @param qp_body_html Alternative html message
     */
    public static String createMultiPartHtmlMessageBody(String qp_body_plain, String qp_body_html) {
	String body = "--" + spamMimeBoundary() + "\n";
	body += "Content-Type: text/plain\n";
	body += "Content-Disposition: inline\n";
	body += "Content-Transfer-Encoding: quoted-printable\n";
	body += "\n";
	body += qp_body_plain + "\n";
	body += "\n";
	body += "--" + spamMimeBoundary() + "\n";
	body += "Content-Type: text/html\n";
	body += "Content-Transfer-Encoding: quoted-printable\n";
	body += "Content-Disposition: inline\n";
	body += "\n";
	body += qp_body_html + "\n";
	body += "--[spam_mime_boundary]--\n";
	return body;
    }
}

