package com.arsdigita.acs.staticcontent;

import javax.servlet.*;
import javax.servlet.jsp.*;
import javax.servlet.http.*;
import org.apache.turbine.util.Log;
import com.arsdigita.acs.*;
import com.arsdigita.acs.db.Database;
import com.arsdigita.acs.db.SqlStatement;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.ServletRequest;
import java.io.*;
import java.lang.InterruptedException;
import java.sql.ResultSet;
import java.sql.SQLException;
import com.oroinc.text.perl.Perl5Util;

/**
 * Utility classes and procedural abstractions for static-content module.
 * 
 * @author rory@arsdigita.com (Java port)
 * @version $Id: StaticContent.java,v 1.7 2000/10/27 21:08:03 jbank Exp $
 */

public class StaticContent {

    public static String getDocRoot(PageContext pc) {
        return getDocRoot(pc.getServletContext());
    }

    public static String getDocRoot(ServletContext sc) {
        String path = Parameters.get("DocRoot","staticcontent");
        if (path.endsWith("/")) {
            path = path.substring(0, path.length()-1);
        }
        File pathfile = new File(path);
        if (!pathfile.canRead() && sc != null) {
            /* Try to get it via the pageContext */
            return sc.getRealPath("/");
        } else {
            return path;
        }
    }

    public static Perl5Util re = new Perl5Util();

    public static String walkTree(PageContext pc, File dir, AdSet seen_already_cache,
				String include_pattern, String[] exclude_patterns,
				boolean debugging_p)
	throws IllegalArgumentException, ACSException, SQLException, IOException {
	//include_pattern defaults to ".*"
	//exclude_patterns is optional

	if ( !dir.isDirectory() )
	    throw new IllegalArgumentException
		("Directory specified is in fact not a directory at all... or so we think.");

	if ( Utilities.emptyString(include_pattern) )
	    include_pattern = ".*";

	String page_content = "";

	// dont think I need to do this:
	// do this so that pwd works (so that we can avoid infinite loops)
	// cd $dir;

	if ( seen_already_cache.contains(dir.toString()) ) {
	    // already exists
	    if ( debugging_p ) {
		page_content += "walk_tree: skipping directory " + dir + " (already seen)<br>";
	    }
	    return page_content;
	}

	// mark this directory as having been seen
	seen_already_cache.put(dir.toString(),"1");
                 
	if ( debugging_p ) {
	    page_content += "walk_tree: checking out directory " + dir + "<br>\n";
	}

	File[] files = dir.listFiles();
	for (int i = 0; i < files.length; i += 1) { // [glob -nocomplain "$dir/*"] {
	    if ( files[i].canRead() ) {
                boolean excluded_p = false;
                for (int j = 0; j < exclude_patterns.length; j += 1) {
                    if ( re.match("m|"+exclude_patterns[j]+"|",files[i].toString() ) ) {
                        excluded_p  = true;
                        break;
                    }
                }
		if ( files[i].isDirectory() ) {
		    // let's just assume that this is true for now
		    // if [ad_running_on_wimpy_machine_p] {
		    // we sleep for one second in order to not trash Web service
                    if (false) {
                        try {
                            Thread.sleep(1000);
                        } catch (InterruptedException e) {
                            throw new ACSException("Something weird happened. Interrupted while trying to slepp for one second.");
                        }
                    }
                    if (!excluded_p) {
                        page_content += walkTree(pc, files[i],
                                                 seen_already_cache,
                                                 include_pattern,
                                                 exclude_patterns,
                                                 debugging_p);
                    }
                } else if ( files[i].isFile() ) {
                    // this means that this is a "normal file"... I that means it's not a symlink
                    boolean match_p = re.match("m|"+include_pattern+"|",files[i].toString());
                    if ( match_p && !excluded_p ) {
                        page_content += ad_check_file_for_sync(pc, files[i]);
                    }
                }
            }
	}
	return page_content;
    }

    public static String ad_check_file_for_sync(PageContext pc, File f)
	throws SQLException, IOException, FileNotFoundException {
	Database db = new Database();
	String page_content = "";
	String  url_stub = f.getPath().substring(getDocRoot(pc).length()-1);
	//[string length [ns_info pageroot]] [expr [string length $f] -1]];
	page_content +=  "<li> " + url_stub + "\n";

	// let's get the file into a string.
	FileInputStream fis = new FileInputStream(f);
	StringWriter sw = new StringWriter();
	int c = fis.read();
	while ( c != -1 ) {
	    sw.write(c);
	    c = fis.read();
	}
	fis.close();
	String content = sw.toString();
	
	// we don't want to mess with zero length files, esp. since they 
	// cause the Oracle driver to cough up an error
	if ( content.length() == 0 ) {
	    page_content +=  " ... is zero length; not touching database.";
	    return page_content;
	}
	String page_title = grep_for_title(content,url_stub);
	page_content += "(" + Utilities.quoteHtml(page_title) + ")\n";

	// here we basically do a db_string returning the value into an int
	SqlStatement get_count_sql = new SqlStatement
	    ("select count(*) as n_rows_already " +
	     " from static_pages " +
	     " where url_stub = :url_stub");
	get_count_sql.bind(url_stub);
	ResultSet get_count_rs = db.query(get_count_sql);
	int n_rows_already;
	if ( get_count_rs.next() ) {
	    n_rows_already = get_count_rs.getInt("n_rows_already");
	} else {
	    throw new SQLException("StaticContent.java: Query was not a statement returning rows:\n"+
				   get_count_sql.getSql() );
	}

	if ( n_rows_already == 0 ) {
	    page_content += "... not in database.  Preparing to stuff...";
	    SqlStatement insert_page_sql = new SqlStatement
		("begin\ninsert into static_pages" +
		 "   ( page_id," +
		 "     url_stub," +
		 "     page_title," +
		 "     page_body" +
		 "   ) values (" +
		 "     page_id_sequence.nextval," +
		 "     :url_stub," +
		 "     :page_title," +
		 "     empty_clob()" +
		 "   ) returning page_body into :1;" +
                 "\nend;");
	    insert_page_sql.bind(url_stub);
	    insert_page_sql.bind(page_title);
	    db.clobDml(insert_page_sql,content);
	    page_content += "done!";
	} else {
	    page_content += "... already in database.";

	    // remove this later: need update to title?
	    if ( re.match("m|.adp$|",url_stub) ) {
		String existing_title;
		SqlStatement page_title_sql = new SqlStatement
		    ("select page_title " +
		     " from static_pages " +
		     " where url_stub = :url_stub");
		page_title_sql.bind(url_stub);
		ResultSet page_title_rs = db.query(page_title_sql);
		if ( page_title_rs.next() ) {
		    existing_title = page_title_rs.getString("page_title");
		    if ( !page_title.equals(existing_title) ) {
			SqlStatement update_title_sql = new SqlStatement
			    ( "update static_pages " +
			      " set page_title= :page_title " +
			      " where url_stub = :url_stub" );
			update_title_sql.bind(page_title);
			update_title_sql.bind(url_stub);
			db.dml(update_title_sql);
			page_content += " (updated page title)";
		    }
		}
	    }
	}
	return page_content;
    }
 

    /** grepts for whatever is in between <title></title>. probably should add a few other cases here later.
     */
    public static String grep_for_title(String content, String url_stub) {
	String[] patterns = {"<title>(.+)</title>"};

	// possible patterns to deal with later (regexp's in Java is quite the pain)
	//  "<asj_header[^>]*>([^<]+)</asj_header>",
	//  "set page_title \\\"([^\\\"]+)\\\"" };

	for (int i = 0; i < patterns.length; i += 1) {
	    // do a case-insensitive match
	    if ( re.match("m|"+patterns[i]+"|i",content) ) {
		return re.group(1);
	    }
	}
                 
	return  "untitled document at " + url_stub;
    }

    public static boolean sendAuthorComment(String comment_type, String action) {
        String parameter = "";
        if (action.equals("add")) {
            if (comment_type.equals("unanswered_question")) {
                parameter = "EmailNewUnansweredQuestion";
            } else if (comment_type.equals("alternative_perspective")) {
                parameter = "EmailNewAlternativePerspective";
            } else if (comment_type.equals("rating")) {
                parameter = "EmailNewRating";
            } else {
                return false;
            }
        } else {
            if (comment_type.equals("unanswered_question")) {
                parameter = "EmailNewUnansweredQuestion";
            } else if (comment_type.equals("alternative_perspective")) {
                parameter = "EmailNewAlternativePerspective";
            } else if (comment_type.equals("rating")) {
                parameter = "EmailNewRating";
            } else {
                return false;
            }
        }
        return Parameters.getBoolean(parameter, "comments", true);
    }


    public static void linkNotify(String url, String newStatus) {
        try {
	/* wasnt able to get this to work yet */
	Database db = new Database();
	SqlStatement sql = new SqlStatement
	    ( "select l.page_id, l.url, l.link_title, u.email, sp.url_stub, sp.page_title" +
	      "  from links l, static_pages sp, users u" +
	      " where l.url = :url" +
	      "   and l.contact_p = 't'" +
	      "   and u.user_id = l.user_id" +
	      "   and l.page_id = sp.page_id" );
        sql.bind(url);

	String body = "The BooHoo automated link management system is unable to reach" +
	    url +
	    "\nIts new status is " + newStatus.toUpperCase() + ".  Links go from 'live' to 'coma'" +
	    "to 'dead' and then are removed.  Promotion in status occurs if the link is" +
	    "unreachable one night.  A link that becomes reachable goes back to live immediately.\n" +
	    "Here are the pages that will no longer have " + url + "as a related link:\n\n";

	Selection mySelection = new Selection(sql,db);
	if ( !mySelection.next() ) {
            throw new SQLException("No rows found");
        }
        body += "   " + Parameters.get("URL", "") + mySelection.get("url_stub") + " (" + mySelection.get("page_title") + ")\n";
	
	body +=
	    "\n\nIf you have moved servers or something and the URL is no longer valid," +
	    "then just come back to the pages above and add the new URL.  The old" +
	    "dead one will get weeded out within a day or two.\n";

	// we don't want errors killing us
        String email_to = mySelection.get("email");
        String email_from = Parameters.get("SystemOwner","staticcontent");
        String subject = url + " is unreachable";

	try {
            Utilities.sendMail(email_from, email_to, subject, body);
	} catch (Exception e) {
	    Log.note("Unable to compose or send message '" + subject + "' to " + email_to);
	}
        } catch (Exception e) {
	    Log.note("LinkNotify exception: " + e.getMessage());
        }
    }

    /** Some junk for testing:
	public static void main (String[] arg) {
	if ( re.match("m|\\.html?$|",arg[0]) )
	    System.out.println("true");
	else
	    System.out.println("false");
    }
    */
}
