package com.arsdigita.acs.staticcontent;

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.sql.ResultSet;
import org.apache.turbine.util.Log;
import com.oroinc.text.perl.Perl5Util;
import com.arsdigita.acs.db.Database;
import com.arsdigita.acs.db.SqlStatement;
import com.arsdigita.acs.*;

/* Note that this works as either a filter,  or an extension handler.
 * Currently, it is just registered as an extension handler, which should
 * work fine. */
public class StaticHtmlFilter extends Filter {

    static {
        StaticHtmlFilter shf = new StaticHtmlFilter();
	RequestProcessor.registerHandler("htm", shf);
	RequestProcessor.registerHandler("html", shf);
    }


    // This definition should probably go in the parameters file
    public static String ad_dnd_tag = "<!--DND-->";
    // This is a list of patterns for pages that should not have the comments stuff put in. 
    private static String naked_html_patterns[] = {"/doc/*",
					     "/admin/*",
					     "/pages/*",
					     "/ct/*",
					     Parameters.get("GlobalURLStub","","/global")+"/*" };

    public int filter(HttpServletRequest r, HttpServletResponse res, ServletContext c) throws ACSException {
        StaticHtmlFilterHelper shfh = new StaticHtmlFilterHelper(r, res, c);
        return shfh.filter();
    }

    private static Perl5Util re = new Perl5Util();

    private class StaticHtmlFilterHelper {
        private HttpServletRequest req;
        private HttpServletResponse resp;
        private ServletContext ctx;
        StaticHtmlFilterHelper(HttpServletRequest r, HttpServletResponse res, ServletContext c) {
            req = r;
            resp = res;
            ctx = c;
        }


        //     The procedure that actually serves all the HTML pages on an ACS.
        //  	  It looks first to see if the file is in one of the naked_html
        //        directories.  If so, it simply returns the raw bytes.  It then looks
        //        to see if the ad_dnd_tag ("do not disturb") comment pattern is
        //        present.  Again, if so, it simply returns.  Otherwise, the procedure
        //  	   tries to add comments and related links.  If the database is busy, it
        //        will simply add links to comments and related links.
        public int filter() throws ACSException {
            Log.note("ad_server_html_pages: serving");

            String url_stub = req.getRequestURI();
            String qualified_file_name = StaticContent.getDocRoot(ctx) + url_stub;

            Log.note("StaticHtmlFilter: " + url_stub + " qualified_file_name: " + qualified_file_name);
            File requested_file = new File(qualified_file_name);
            if (!requested_file.exists()) {
                /* File not found, let other stuff handle this. */
                return FILTER_OK;
            }

            String mime_type = ctx.getMimeType(requested_file.toString());

            // OK, it exists so now let's try to get the file into a string
            String whole_page = Utilities.fileToString(requested_file,
                                                       "StaticHtmlFilter: we tried to read the request file and got this error:\n");
            for (int i = 0; i < naked_html_patterns.length; i += 1) {
                if (Utilities.stringGlobMatch(naked_html_patterns[i], url_stub)) {
                    // if we are supposed to exclude it, just stream the page back.
                    Utilities.writeStringToResponse(resp, whole_page, resp.SC_OK, mime_type);
                    return FILTER_RETURN;
                }
            }

            // sometimes we don't want comments to come up for a given page
            if ( whole_page.indexOf(ad_dnd_tag) != -1 ) {
                return FILTER_OK;
            }

            if ( re.match("m|(.*)</body>(.*)|s",whole_page) ) {
                // there was a "close body" tag, let's try to insert a comment
                // link at least
                // before we do anything else, let's stream out what we can
                String pre_body = re.group(1);
                String post_body = re.group(2);
                //FIXME: Curriculum module isn't around, so can't do this yet.
                //StaticContent.static_add_curriculum_bar_if_necessary(pre_body);
                whole_page = pre_body; 
                CommentLinkInfo cl_info = get_comments_and_links(url_stub,post_body);
                String comment_link_options_fragment = format_comments_and_links(cl_info);

                // now decide what to do with the comments and links we've queried from the database
                whole_page += comment_link_options_fragment + "\n\n</body>" + post_body;
                Utilities.writeStringToResponse(resp, whole_page, resp.SC_OK, mime_type);
                return FILTER_RETURN;
            } else {
                // couldn't find a </body> tag
                Utilities.writeStringToResponse(resp, whole_page, resp.SC_OK, mime_type);
                return FILTER_RETURN;
            }
        }

        // helper proc for coming back with options, info, etc. 
        /**
           Returns a CommentLinkInfo of comment_bytes link_bytes options_list comment_option
           link_option or the empty string if this page isn't
           registered in the database 
        **/
        public CommentLinkInfo get_comments_and_links (String url_stub, String post_body) 
            throws ACSException {

            String comment_bytes = new String();
            String link_bytes = new String();
            AdSet options_list = new AdSet();
            String comment_option = new String();
            String link_option = new String();

            UserInfo user;
            try {
                user = Security.getVerifiedUserInfo(req, resp, false);
            } catch (IOException e) {
                throw new ACSException("StaticHtmlFilter: error reading user's cookie.");
            }

            Database db;
            try {
                db = new Database();
                SqlStatement html_id_sql = new SqlStatement
                    ( "select page_id," +
                      "       accept_comments_p," +
                      "       accept_links_p," +
                      "       inline_comments_p," +
                      "       inline_links_p" +
                      "  from static_pages" +
                      " where url_stub = :url_stub" );
                html_id_sql.bind(url_stub);
                ResultSet html_id_rs = db.query(html_id_sql);
                if ( !html_id_rs.next() ) {
                    // this page isn't registered in the database so we can't
                    // accept comments on it or anything
                    Log.note("Someone grabbed " + url_stub + " but we weren't able to offer a comment link because this page isn't registered in the db.");

                    return null;
                } else {

                    if ( html_id_rs.getString("inline_comments_p").equals("t") ) {
                        // so we display comments in-line
                        SqlStatement all_comments_sql = new SqlStatement
                            ( "select comments.comment_id," +
                              "       comments.page_id," +
                              "       comments.user_id as poster_user_id," +
                              "       users.first_names || ' ' || users.last_name as user_name," +
                              "       message," + 
                              "       posting_time," +
                              "       html_p," +
                              "       client_file_name," +
                              "       file_type," +
                              "       original_width," +
                              "       original_height," +
                              "       caption" +
                              "  from static_pages sp," +
                              "       comments_not_deleted comments," +
                              "       users" +
                              " where sp.page_id = comments.page_id" +
                              "   and comments.user_id = users.user_id" +
                              "   and comments.page_id = :page_id" +
                              "   and comments.comment_type = 'alternative_perspective'" +
                              " order by posting_time" );
                        all_comments_sql.bind(html_id_rs.getString("page_id"));
                        ResultSet all_comments_rs = db.query(all_comments_sql);

                        while ( all_comments_rs.next() ) {
                            String content = Database.getCLOB(all_comments_rs, "message");
                            comment_bytes += "<blockquote>";

                            comment_bytes += format_static_comment( req,
                                                                    all_comments_rs.getInt("comment_id"),
                                                                    all_comments_rs.getString("client_file_name"),
                                                                    all_comments_rs.getString("file_type"),
                                                                    all_comments_rs.getInt("original_width"),
                                                                    all_comments_rs.getInt("original_height"),
                                                                    all_comments_rs.getString("caption"),
                                                                    content,
                                                                    all_comments_rs.getBoolean("html_p") );

                            if ( user.getUserId().intValue() == all_comments_rs.getInt("poster_user_id") ) {
                                // the user wrote the message, so let him/her edit it
                                comment_bytes +=
                                    " &nbsp; [ <A HREF=\"/comments/persistent-edit?comment_id=" +
                                    all_comments_rs.getString("comment_id") +
                                    "\">edit your comment</a> ]\n";
                            }


                            comment_bytes += "<br><br>\n" +
                                "-- <A HREF=\"/shared/community-member?user_id=" + 
                                all_comments_rs.getString("poster_user_id") + "\">" +
                                all_comments_rs.getString("user_name") + "</a>" +
                                ", " + all_comments_rs.getString("posting_time") +
                                //FIXME: [util_AnsiDatetoPrettyDate $posting_time]" +
                                "</blockquote>\n";
                        }
                    }

                    if ( html_id_rs.getString("accept_comments_p").equals("t") &&
                         html_id_rs.getString("inline_comments_p").equals("t") ) {
                        // we only display the option if we're inlining comments; 
                        // we assume that if the comments aren't in line but are legal
                        // then the publisher has an explicit link 
                        comment_option = "<a href=\"/comments/add?page_id=" +
                            html_id_rs.getString("page_id") +
                            "\">Add a comment</a>";
                        options_list.put("comment_option",comment_option);
                    }

                    ////
                    // links
                    ////
                    if ( html_id_rs.getString("inline_links_p").equals("t") ) { //*

                        SqlStatement all_links_sql = new SqlStatement
                            ("select links.page_id," +
                             "       links.user_id as poster_user_id," +
                             "       users.first_names || ' ' || users.last_name as user_name," +
                             "       links.link_title," +
                             "       links.link_description," +
                             "       links.url as url" +
                             "  from static_pages sp," +
                             "       links," +
                             "       users" +
                             " where sp.page_id = links.page_id" +
                             "   and users.user_id = links.user_id" +
                             "   and links.page_id = :page_id" +
                             "   and status = 'live'" +
                             " order by posting_time" );
                        all_links_sql.bind(html_id_rs.getString("page_id"));
                        ResultSet all_links_rs = db.query(all_links_sql);
                        while ( all_links_rs.next() ) {
                            link_bytes += "<li><a href=\"" + all_links_rs.getString("url") + "\">" +
                                all_links_rs.getString("link_title") + "</a>- " + all_links_rs.getString("link_description");
		
                            if ( user.getUserId().intValue() == all_links_rs.getInt("poster_user_id") ) {
                                // the user added, so let him/her edit it
                                link_bytes += "&nbsp;&nbsp;(<A HREF=\"/links/edit?page_id=" + html_id_rs.getString("page_id") +
                                    "&url=" + Utilities.URLEncode(all_links_rs.getString("url")) + "\">edit/delete</a>)";
                            } else {
                                // the user did not add it, link to the community_member page
                                link_bytes += "&nbsp;&nbsp;  " +
                                    "<font size=-1>(contributed by <A HREF=\"/shared/community-member?user_id=" +
                                    all_links_rs.getString("poster_user_id") + "\">" + all_links_rs.getString("user_name") +
                                    "</a>)</font>";
                            }
                            link_bytes += "\n<p>\n";
                        } 
                    } 

                    if ( html_id_rs.getString("accept_links_p").equals("t") &&
                         html_id_rs.getString("inline_links_p").equals("t") ) {
                        // we only display the option if we're inlining links; 
                        // we assume that if the links aren't in line but are legal
                        // then the publisher has an explicit link 
                        link_option = "<a href=\"/links/add?page_id=" + html_id_rs.getString("page_id") + "\">" + 
                            "Add a link</a>";
                        options_list.put("link_option",link_option);
				     
                    } else {
                        link_option = "";
                    }
                }
            } catch (Exception e) {
                Log.note("Received an exception in get comments_and_links: " + Utilities.exceptionToString(e));
            }

            return new CommentLinkInfo(comment_bytes, link_bytes, options_list, comment_option, link_option);
        }



        // helper proc for formatting comments, links, etc.
        /** 
         * Takes list of comment_bytes link_bytes options_list comment_option
         * link_option and produces HTML fragment to stick at bottom of page.
         **/
        public String format_comments_and_links (CommentLinkInfo cl_info) 
            throws ACSException {

            if ( cl_info == null )
                return "";

            String centered_options = "";

            if ( Utilities.emptyString(cl_info.comment_bytes) && Utilities.emptyString(cl_info.link_bytes) ) {
                // no comments nor links
                if ( cl_info.options_list.size() > 0 ) {
                    centered_options = "<center>" + cl_info.options_list.join(" | ") + "</center>";
                }
                return centered_options;
            } else if ( !Utilities.emptyString(cl_info.comment_bytes) && Utilities.emptyString(cl_info.link_bytes) ) {
                // there are comments but no links
                return "<center><h3>Reader's Comments</h3></center>" +
                    cl_info.comment_bytes +
                    "<center>" + cl_info.options_list.join(" | ") + "</center>";
            } else if ( Utilities.emptyString(cl_info.comment_bytes) && !Utilities.emptyString(cl_info.link_bytes) ) {
                // links but no comments 
                return "<center><h3>Related Links</h3></center>" +
                    "<ul>" + cl_info.link_bytes + "</ul>" +
                    "<center>" + cl_info.options_list.join(" | ") + "</center>";
            } else {
                // comments and links 
                return "<center><h3>Reader's Comments</h3></center>" +
                    cl_info.comment_bytes +
                    "<center>" +
                    cl_info.comment_option +
                    "</center>" +
                    "<center><h3>Related Links</h3></center>" +
                    "<ul>" + cl_info.link_bytes + "</ul>" +
                    "<center>" +
                    cl_info.link_option +
                    "</center>";
            }
        }

        /* Simple class to encapsulate comment and link information. */
        private class CommentLinkInfo {
            public String comment_bytes;
            public String link_bytes;
            public AdSet options_list;
            public String comment_option;
            public String link_option;

            public CommentLinkInfo() {};

            public CommentLinkInfo(String c, String l, AdSet o, String co, String lo) {
                comment_bytes = c;
                link_bytes = l;
                options_list = o;
                comment_option = co;
                link_option = lo;
            }

            public String join(String delim) {
                if ( delim == null )
                    delim = ", ";
                return comment_bytes + delim + link_bytes + delim +
                    options_list + delim + comment_option + delim + link_option;
            }


        }
    }
    /**
       Helper procedure for formatting 'alternative_perspective' comments on static
       pages, which presents inline images or attachment links as appropriate.
       Taken from a similar procedure in ad-general-comments.tcl.
    */
    static public String format_static_comment(HttpServletRequest req, String comment_id, String client_file_name,
                                               String file_type, String original_width, String original_height,
                                               String caption, String content, boolean comment_html_p)
        throws ACSException {
        int original_width_i = -1;
        int original_height_i = -1;
        if (!Utilities.emptyString(original_width)) {
            original_width_i = Integer.parseInt(original_width);
        }
        if (!Utilities.emptyString(original_height)) {
            original_height_i = Integer.parseInt(original_height);
        }
        return format_static_comment(req, Integer.parseInt(comment_id), client_file_name, file_type,
                                     original_width_i, original_height_i, caption, content, comment_html_p);
    }

    static public String format_static_comment(HttpServletRequest req, int comment_id, String client_file_name,
                                               String file_type, int original_width, int original_height,
                                               String caption, String content, boolean comment_html_p)
        throws ACSException {
        String return_string = new String();
        String return_url = req.getRequestURI();

        if ( !Utilities.emptyString(client_file_name) ) {
            // We have an attachment.
            if ( re.match("m|image/*|i",file_type.toLowerCase()) ) {
                // It was an image.
                if ( original_width != -1 &&
                     original_width < Parameters.getInt("InlineImageMaxWidth","comments",512) ) {
                    // It's narrow enough to display inline.
                    return_string += "<center>" +
                        "<img src=\"/comments/attachment-view.jsp?comment_id=" + comment_id + "&with_page_p=f\"" +
                        "width=" + original_width + " height=" + original_height + ">" + 
                        "<p><i>" + caption + "</i></center><br>\n";
                    return_string += comment_html_p?content:Utilities.textToHtml(content);
                } else {
                    // Send to an image display page.
                    AdSet a;
                    try {
                        a = new AdSet(new Object[][] { {"comment_id",new Integer(comment_id)},
                                                       {"return_url",return_url},
                                                       {"with_page_p","true"} } );
                    } catch (Exception e) {
                        throw new ACSException ("StaticHtmlFilter: error instantiating AdSet. Bad args.");
                    }
                    return_string += comment_html_p?content:Utilities.textToHtml(content);
                    return_string += "<br><i>Image: " +
                        "<a href=\"/comments/attachment-view?" + Utilities.exportURLVars(a) + "\">" +
                        client_file_name + "</a></i>";
                }
            } else {
                // Send to raw file download.
                AdSet a;
                try {
                    a = new AdSet(new Object[][] { {"comment_id",new Integer(comment_id)},
                                                   {"with_page_p","false"} } );
                } catch (Exception e) {
                    throw new ACSException ("StaticHtmlFilter: error instantiating AdSet. Bad args.");
                }
                return_string += comment_html_p?content:Utilities.textToHtml(content);
                return_string += "<br><i>Attachment:" +
                    "<a href=\"/comments/attachment-view?" + Utilities.exportURLVars(a) + "\">" +
                    client_file_name +
                    "</a></i>";
            }
        } else {
            // No attachment
            return_string += comment_html_p?content:Utilities.textToHtml(content); 
        }
        return return_string;
    }

}
