package com.arsdigita.acs.tags;

import com.arsdigita.acs.*;
import com.arsdigita.acs.db.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;
import java.sql.*;
import java.io.*;


public class ForeachTag
    extends BodyTagSupport
    implements BodyTag
{
    private SqlStatement sql;
    private String row_name;
    private ResultSet rs;
    private int num_rows;

    public int doStartTag() throws JspException {
	num_rows = 0;

	if (sql == null) {
	    throw new JspTagException("db_foreach must have a valid query associated with it.");
	}

	PageTag page = (PageTag) findAncestorWithClass(this, com.arsdigita.acs.tags.PageTag.class);
	if (page == null) {
	    throw new JspTagException("db_foreach must be enclosed within a page tag");
	}
	Database db = page.getDatabase();
	try {
	    rs = db.query(sql);
	    if (rs.next()) {
		num_rows++;
		pageContext.setAttribute(row_name, rs);
		return EVAL_BODY_TAG;
	    } else {
		return SKIP_BODY;
	    }
	} catch (SQLException e) {
	    throw new JspTagException("db_foreach encountered database error: " + e.getMessage());
	}
    }
		

    public int doAfterBody() throws JspException {
	BodyContent bc = getBodyContent();
	try {
	    bc.writeOut(getPreviousOut());
	    bc.clearBody();

	    if (rs.next()) {
		num_rows++;
		return EVAL_BODY_TAG;
	    } else {
		return SKIP_BODY;
	    }
	} catch (IOException e) {
	    throw new JspTagException("db_foreach encountered IOException while attempting to evaluate body: " + e.getMessage());
	} catch (SQLException e) {
	    throw new JspTagException("db_foreach encountered SQL Exception while attempting to move result set forward: " + e.getMessage());
	}
    }
    
    public int doEndTag() throws JspException {
	try {
	    rs.close();
	    if (num_rows > 0) {
		pageContext.setAttribute("db_num_rows", 
					 new Integer(num_rows));
	    }
	} catch (SQLException e) { }

	return EVAL_PAGE;
    }

    public void setIn(String s) {
	sql = (SqlStatement) pageContext.getAttribute(s);
    }

    public void setRow(String s) {
	row_name = s;
    }
}
