package com.arsdigita.acs.tags;

import com.arsdigita.acs.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;
import java.sql.*;
import java.io.*;
import java.util.*;


public class LoopTag
    extends BodyTagSupport
    implements BodyTag, DataSourceScopeTag
{
    private DataSource ds;
    private int num_rows;
    private String joinText = null;

    public int doStartTag() throws JspException {
	num_rows = 0;

	if (ds == null) {
	    return SKIP_BODY;
	}

	try {
	    if (ds.next()) {
		num_rows++;
		return EVAL_BODY_TAG;
	    } else {
		return SKIP_BODY;
	    }
	} catch (DataSourceException e) {
	    throw new JspTagException("loop encountered datasource error: " + e.getMessage());
	}
    }

    public int doAfterBody() throws JspException {
	BodyContent bc = getBodyContent();
	try {
	    bc.writeOut(getPreviousOut());
	    bc.clearBody();
	    if (ds.next()) {
		num_rows++;
		if (joinText != null) {
		    bc.print(joinText);
		}
		return EVAL_BODY_TAG;
	    } else {
		return SKIP_BODY;
	    }
	} catch (DataSourceException e) {
	    throw new JspTagException("loop encountered IOException while attempting to evaluate body: " + e.getMessage());
	}
	catch (IOException e) {
	    throw new JspTagException("loop encountered IOException while attempting to evaluate body: " + e.getMessage());
	} 
    }
    
    public int doEndTag() throws JspException {
	pageContext.setAttribute("loop_num_rows", 
				 new Integer(num_rows));

	return EVAL_PAGE;
    }

    public void setJoin(String joinText) {
	this.joinText = joinText;
    }

    public void setDs(String dsName) {
	this.ds = (DataSource)pageContext.getAttribute(dsName);
    }

    DataSource getDs() { 
	return this.ds;
    }
    
    public Enumeration getDataSources() {
	return new Enumeration() {
		private boolean firstTime = true;

		public boolean hasMoreElements() {
		    return firstTime;
		}

		public Object nextElement() {
		    if (firstTime) {
			firstTime = false;
			return ds;
		    } else {
			throw new NoSuchElementException();
		    }
		}
	    };
    }
}
