package com.arsdigita.acs.tags;

import com.arsdigita.acs.*;
import javax.servlet.jsp.tagext.*;
import javax.servlet.jsp.*;
import javax.servlet.http.*;

/**
 * A tag wrapper for MemoizeCache.  Writes cache lookup to response.
 * If not found, evaluates tag body and caches it.
 */

public class MemoizeTag extends BodyTagSupport implements BodyTag { 
    String key = null;
    String val = null;
    Integer duration = null;

    /** 
     * At start of tag, try looking up key and see if we have a valid
     * mapping for it.  If we do, write it out to the response.  Otherwise
     * evaluate the body, and save it in the tag.
     */
    public int doStartTag() throws JspException {
	HttpServletRequest req = (HttpServletRequest)pageContext.getRequest();

	if (key == null) { 
	    // if no key provided, we allow a key
	    // to be specified as a Request attribute.  this is 
	    // because you can't say <ad:memoize key="<%= myCacheKey %>">.
	    key = (String)req.getAttribute("MemoizeTagKey");
	}

	if (key == null) {
	    // use the current request as the default key: /page?foo=bar
	    key = req.getRequestURI();
	    String queryStr = req.getQueryString();
	    if (!Utilities.emptyString(queryStr)) {
		key += "?" + queryStr;
	    }
	}
	val = (String)MemoizeCache.lookup(key);

	if (val != null) {
	    return SKIP_BODY;
	} else {
	    return EVAL_BODY_TAG;
	}
    }

    public int doAfterBody() throws JspException { 
	String val = getBodyContent().getString();
	if (duration == null) {
	    MemoizeCache.cache(key, val);
	} else {
	    MemoizeCache.cache(key, val, duration.intValue());
	}
	this.val = val;
	return SKIP_BODY;
    }
	
    public int doEndTag() throws JspException {
	try {
	    pageContext.getOut().print(val);
	} catch (Exception e) { 
	    throw new JspException(e.getMessage());
	}
	return EVAL_PAGE;
    }

    public void setKey(String k) { 
	key = k;
    }

    public void setDuration(String nSeconds) {
	duration = new Integer(nSeconds);
    }
}

