package com.arsdigita.acs.tags;

import com.arsdigita.acs.*;
import com.arsdigita.acs.db.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;
import java.sql.*;
import java.io.*;

/**
 * Tag class for ad:db_1row.
 * Tag body contains a SQL statement.  At end of tag, performs the query
 * and stuffs the specified pageContext attribute with the result row.
 */

public class OneRowTag 
    extends SqlTag
    implements BodyTag

{
    private String obj;
    private String type;

    public int doAfterBody() throws JspException {
	sql.setSql(getBodyContent().getString());
	return SKIP_BODY;
    }
    
    public int doEndTag() throws JspException {
	sql.bindFixup();
	String id = getId();
	
	if (obj != null) {
	    try { 
		Database db = (Database)pageContext.getAttribute("db");
		Selection sel;
		ResultSet rs = db.query(sql);
		if (!rs.next()) {
		    throw new JspTagException("db_1row: query did not return a row");
		}

		if (type == null) { 
		    // default to actual type of SingleSelection
		    // note that we have to declare the apparent type as
		    // Selection because most of the SingleFooDataSource's
		    // extend Selection, not SingleSelection! (sometimes
		    // you really need multiple inheritance... sigh)
		    sel = new SingleSelection(rs);
		} else {
		    java.lang.reflect.Constructor cstr = 
			Class.forName(type).getConstructor(new Class[] {
			    java.sql.ResultSet.class
			});
		    sel = (Selection)cstr.newInstance(new Object[] {rs});
		}

		// throw exception if more than one row returned
		if (rs.next()) {
		    throw new JspTagException("db_1row: query returned more than one row");
		}

		// cover all our bases
		pageContext.setAttribute(obj, sel);
		pageContext.getRequest().setAttribute(obj, sel);
	    }
	    catch (Exception e) { 
		throw new JspException(e.toString() + ": " + e.getMessage());
	    }
	}
	return EVAL_PAGE;
    }

    public void setObj(String s) {
	obj = s;
    } 

    public void setType(String s) { 
	type = s;
    }
}
