package com.arsdigita.acs.tags;

import java.util.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;
import javax.servlet.http.*;
import com.arsdigita.acs.db.Database;
import com.arsdigita.acs.*;
import org.apache.turbine.util.Log;

public class PageContractTag
    extends TagSupport
    implements Tag
{
    private String vars;
    private Vector complaints = new Vector();

    public void setVars(String s) {
	vars = s;
    }

    public void addComplaint(String s) {
	complaints.addElement(s);
    }

    public int doStartTag() throws JspException {
	complaints.clear();

	String id = getId();
	if (id != null) {
	    pageContext.setAttribute(getId(), this);
	}

	// Get the names of the variables to create.
	Vector varDescs = PageContractTagExtraInfo.parseVars(vars);

	Enumeration e = varDescs.elements();

	// set up AdSet with parameters
	AdSet nameValuePairs = (AdSet) pageContext.getAttribute("form");
	while (e.hasMoreElements()) {
	    ContractVarDesc varDesc = (ContractVarDesc) e.nextElement();
	    
	    // throw an exception if both date and multiple -- multiple
	    // dates don't make sense because we can't count on correlating
	    // years, months, etc. (dependent on what order getParameterNames
	    // returns in)

	    if (varDesc.getBaseType().equals("java.sql.Date") 
		&& varDesc.isMultiple()) {
		throw new JspTagException("ad:page_contract: cannot supply both 'date' and 'multiple' options for a page variable");
	    }

	    if (varDesc.isArray()) {
		// Get all the keys that begin with "arrayName.".
		String arrayName = varDesc.getName();

		AdSet s = new AdSet();

		// Search through all the parameters looking for ones
		// that begin with the arrayName. Build up an AdSet
		// of all matching key/value pairs.
		Enumeration paramNames = nameValuePairs.keysAsEnumeration();
		while (paramNames.hasMoreElements()) {
		    String param = (String) paramNames.nextElement();
		    if (param.startsWith(arrayName + ".")) {
			if (varDesc.getBaseType().equals("java.sql.Date")) { 
			    // we're getting a date out:
			    // form vars are "$arrayName.key_month", etc.
			    String key = 
				param.substring(arrayName.length() + 1,
						param.lastIndexOf("_"));
			    java.sql.Date dateValue = 
				getDateFromForm(arrayName + "." + key);
			    if (dateValue != null) 
				s.put(key, dateValue);
			} else {
			    String key = 
				param.substring(arrayName.length() + 1);
			    String[] values = nameValuePairs.getAll(param);
			    for (int i = 0; i < values.length; i++) {
				s.put(key, values[i]);
			    }
			}
		    }
		}

		// Handle optional flag for array here.
		if (s.size() == 0 && !varDesc.isOptional()) {
		    addComplaint("required parameter " + varDesc.getName() + " not supplied");
		    return EVAL_BODY_INCLUDE;
		}
		
		// Validate values and store in another AdSet.
		AdSet finalResult = new AdSet();
		Enumeration keys = s.keysAsEnumeration();
		while (keys.hasMoreElements()) {
		    String key = (String) keys.nextElement();
		    String[] values = s.getAll(key);
		    Object validatedValue = validate(values, varDesc);
		    if (validatedValue != null) {
			if (validatedValue.getClass().isArray()) {
			    // Insert each element individually.
			    Object[] validatedArray = (Object[]) validatedValue;
			    for (int i = 0; i < validatedArray.length; i++) { 
				finalResult.put(key, validatedArray[i]);
			    }
			} else {
			    finalResult.put(key, validatedValue);
			}
		    }
		}

		pageContext.setAttribute(varDesc.getName(), finalResult);
	    } else if (varDesc.getBaseType().equals("java.sql.Date")) { 
		java.sql.Date dateValue = getDateFromForm(varDesc.getName());
		if (dateValue != null) 
		    pageContext.setAttribute(varDesc.getName(), dateValue);
	    } else {
		// "array" flag not specified.
		
		// Get value of this parameter from request.
		String[] values = nameValuePairs.getAll(varDesc.getName());
		
		// Validate and parse the value according to flags.
		Object validatedValue = validate(values, varDesc);
		
		// Record the value in the page context.
		if (validatedValue != null) {
		    pageContext.setAttribute(varDesc.getName(), validatedValue);
		}
	    }
	}
	return EVAL_BODY_INCLUDE;
    }

    /**
     * Reads out a date from HTML form variables.
     * @param formName The name of the form variable.
     * @return the SQL date formed by the form variables "$formName_month",
     * "$formName_year", and "$formName_day".
     */
    private java.sql.Date getDateFromForm(String formName) { 
 	AdSet nameValuePairs = (AdSet) pageContext.getAttribute("form");

	String year, month, day;

	// check for null 
	if ((year = nameValuePairs.get(formName + "_year")) == null ||
	    (month = nameValuePairs.get(formName + "_month")) == null ||
	    (day = nameValuePairs.get(formName + "_day")) == null) {
	    // if it's optional, it'll be validated later; if not, 
	    // PageValidationException will be thrown during validation
	    return null;
	}

	// WRS 9/18
	// have to manipulate dates here, since we need to
	// get out three vars instead of one
	Calendar calendar = Calendar.getInstance();
	calendar.set(Integer.parseInt(year),
		     Integer.parseInt(month),
		     Integer.parseInt(day));
	java.sql.Date dateValue = 
	    new java.sql.Date(calendar.getTime().getTime());	
	return dateValue;
    }
	
	    
    private Object validate(String[] values, ContractVarDesc varDesc) {
	Object[] retValues = values;
	String varName = varDesc.getName();
	if (values == null) {
	    // Not specified; let's see if it's optional.
	    if (! varDesc.isOptional()) {
		addComplaint("required parameter " + varName + " not supplied");
		return null;
	    }

            // This uglyness has been trashed by DEE 10/2/00.  If something is null, it should be null,
            // not an array containing null!
            // Note that this could break some code that was expecting an array containing null.
	    // values = new String[1];
	    // values[0] = null;
            return null;
	}

	String type = varDesc.getBaseType();
	if (type.equals("Integer")) {
	    Integer[] intValues = new Integer[values.length];
	    for (int i = 0; i < values.length; i++) {
		try {
		    if (values[i] != null && values[i].length() > 0)
			intValues[i] = new Integer(values[i]);
		    else 
			intValues[i] = null;
		} catch (NumberFormatException e) {
		    addComplaint("integer parameter " + varName + " supplied as non-integer '" + values[i] + "'");
		    return null;
		}
	    }
	    retValues = intValues;
	} else if (type.equals("Boolean")) {  // -- rory@arsdigita.com 9/13/2000
	    Boolean[] boolValues = new Boolean[values.length];
	    for (int i = 0; i < values.length; i += 1) {
		if (values[i] != null && values[i].length() > 0)
		    // The Boolean constructor does not throw FormatException like Integer.
		    // It returns false for all values except "true", so we need to check by hand  
		    // for any stuff the programmer would have inteded to be true 
		    if ( values[i].toLowerCase().equals("true") || values[i].toLowerCase().equals("t") ||
			 values[i].toLowerCase().equals("yes") || values[i].toLowerCase().equals("y") ||
			 values[i].equals("1") || values[i].toLowerCase().equals("exists") )
			boolValues[i] = new Boolean(true);
		    else 
			boolValues[i] = new Boolean(false);
		else
		    boolValues[i] = null;
	    }
	    retValues = boolValues;
	}

	if (varDesc.isMultiple()) {
	    return retValues;
	} else {
	    return retValues[0];
	}
    }

    public int doEndTag() throws JspException {
	if (complaints.size() > 0) {
	    // There were validation errors. Report them
	    // by throwing an exception.
	    
	    // but first free up the database handle
	    Database db = (Database)pageContext.getAttribute("db");
	    try {
		db.close();
	    } catch(Exception e) { }
	    throw new PageValidationException(complaints);
	}
	return EVAL_PAGE;
    }
}
