package com.arsdigita.acs.tags;

import java.util.*;
import javax.servlet.jsp.tagext.*;
import com.oroinc.text.perl.Perl5Util;
import com.oroinc.text.regex.*;

public class PageContractTagExtraInfo 
    extends TagExtraInfo {

    private static Perl5Util re = new Perl5Util();

    // Returns Vector of three element arrays:
    // { name, type, Vector of flags }
    // last argument can be null if no flags.
    static Vector parseVars(String vars) {
	Vector varDescs = new Vector();

	PatternMatcherInput input = new PatternMatcherInput(vars);
	while (re.match("/\\b([^:\\s]+):?(\\S*)\\b/", input)) {
	    MatchResult result = re.getMatch();
	    String var = result.group(1);
	    String flagString = result.group(2);
	    ContractVarDesc varDesc = new ContractVarDesc(var);

	    // Figure out what type to make this attribute available as.
	    varDesc.setBaseType("String"); // the default

	    if (flagString != null) {
		// There were some flags; process them.
		Vector flags = re.split("/,/", flagString);
		
		Enumeration e = flags.elements();
		while (e.hasMoreElements()) {
		    String flag = (String) e.nextElement();
		    if (flag.equals("integer")) {
			varDesc.setBaseType("Integer");
		    } else if (flag.equals("boolean")) {
			varDesc.setBaseType("Boolean");
		    } else if (flag.equals("multiple")) {
			varDesc.setMultiple(true);
		    } else if (flag.equals("array")) {
			varDesc.setArray(true);
		    } else if (flag.equals("optional")) {
			varDesc.setOptional(true);
		    } else if (flag.equals("file")) {
			// uploaded files: add a new ".tmpfile" 
			// description
			ContractVarDesc varDesc2 = new ContractVarDesc(var + "_tmpfile");
			varDesc2.setBaseType("String");
			varDesc2.setOptional(true);
			varDescs.addElement(varDesc2);
		    } else if (flag.equals("date")) {  // WRS 9/18/00
			varDesc.setBaseType("java.sql.Date");
		    }
		}
	    }
	    varDescs.addElement(varDesc);
	}
	return varDescs;
    }

    public VariableInfo[] getVariableInfo(TagData data) {
	Vector retval = new Vector();

	// Add implicit page contract variable if id has been specified.
	String id = data.getAttributeString("id");
	if (id != null) {
	    retval.addElement(new VariableInfo(id, "com.arsdigita.acs.tags.PageContractTag", true, VariableInfo.NESTED));
	}

	// Get variable information.
	String vars = data.getAttributeString("vars");
	Vector varDescs = parseVars(vars);

	Enumeration e = varDescs.elements();
	while (e.hasMoreElements()) {
	    ContractVarDesc varDesc = (ContractVarDesc) e.nextElement();
	    retval.addElement(new VariableInfo(varDesc.getName(), varDesc.getType(), true, VariableInfo.AT_BEGIN));
	}

	// Huh, this doesn't work, ClassCastException.
	// return (VariableInfo[]) retval.toArray();
	int len = retval.size();
	VariableInfo[] varinfo = new VariableInfo[len];
	for (int i = 0; i < len; i++) {
	    varinfo[i] = (VariableInfo) retval.elementAt(i);
	}
	return varinfo;
    }
}
