package com.arsdigita.acs.tags;

import java.util.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;
import com.arsdigita.acs.*;

public class SubTag
    extends TagSupport
    implements Tag {

    private String var;
    private String dsName;
    private boolean encode = false;
    private boolean quotehtml = false;
    
    public int doStartTag() {
	// should be empty anyway
	return SKIP_BODY;
    }

    public int doEndTag() throws JspException {
	if (dsName != null) {
	    // Explicit datasource set.
	    DataSource ds = (DataSource) pageContext.getAttribute(dsName);
	    try {
		pageContext.getOut().print(get(ds, var));
		return EVAL_PAGE;
	    } catch (Exception e) {
		throw new JspTagException(e.getMessage());
	    }
	}

	// first find the enclosing DataSourceScopeTag
	Tag from = this;
	boolean firstTime = true;
	DataSourceScopeTag scope;
	
	while ((scope = (DataSourceScopeTag) findAncestorWithClass(from, DataSourceScopeTag.class)) != null) {
	    from = (Tag)scope;
	    firstTime = false;
	    
	    // Loop through its available data sources.
	    Enumeration sources = scope.getDataSources();
	    while (sources.hasMoreElements()) {
		DataSource ds = (DataSource) sources.nextElement();
		try {
		    pageContext.getOut().print(get(ds, var));
		    return EVAL_PAGE;
		} catch (InvalidKeyException e) {
		    continue;
		} catch (Exception e) {
		    throw new JspTagException(e.toString() + " " + e.getMessage());
		}
	    }
	}

	// at this point, we haven't found any enclosing DataSourceScopeTags
	// or haven't found the var we're looking for inside them;
	// check the request object and the pageContext object

	try {
	    Object obj = pageContext.getAttribute(var);
	    if (obj != null) {
		if (encode) {
		    obj = Utilities.URLEncode(obj.toString());
		}
                if (quotehtml) {
		    obj = Utilities.textToHtml(obj.toString());
                }
		pageContext.getOut().print(obj);
		return EVAL_PAGE;
	    }
	    obj = pageContext.getRequest().getAttribute(var);
	    if (obj != null) { 
		if (encode) {
		    obj = Utilities.URLEncode(obj.toString());
		}
                if (quotehtml) {
		    obj = Utilities.textToHtml(obj.toString());
                }
		pageContext.getOut().print(obj);
		return EVAL_PAGE;
	    }
	} catch (Exception e) { 
	    throw new JspTagException (e.getMessage());
	}
	
	throw new JspTagException("Requested key '" + var + "' not found.  ad:sub must be contained within a scoping tag (ad:loop, ad:scope), or requested key must be an attribute in the request or pageContext object");
	
    }

    // converts property_name to getPropertyName
    private static String convertName(String prop) {
	StringBuffer sb = new StringBuffer("get");
	com.oroinc.text.perl.Perl5Util re = new	com.oroinc.text.perl.Perl5Util();
	Vector v = re.split ("/_/", prop);
	for (int i = 0; i < v.size(); i++) { 
	    String word = (String)v.elementAt(i);
	    char capFirst = Character.toUpperCase(word.charAt(0));
	    sb.append(capFirst);
	    sb.append(word.substring(1, word.length()));
	}
	return sb.toString();
    }

    public void setVar(String s) {
	var = s;
    }

    public void setDs(String s) { 
	dsName = s;
    }

    public static void main(String args[]) {
	System.out.println(convertName("this_is_a_test"));
    }

    // Retrieve var from ds, using first introspection then DataSource.get().
    private String get(DataSource ds, String var) throws DataSourceException {
	String s = null;

	// Try to introspect for the value that we're looking for
	try {
	    Object o = ds.getClass().getMethod(convertName(var),null).invoke(ds, null);
	    if (o != null) {
		s = o.toString();
	    }
	} catch (NoSuchMethodException ex) {
	    s = ds.get(var);
	} catch (IllegalAccessException ex) {
	    s = ds.get(var);
	} catch (java.lang.reflect.InvocationTargetException ex) {
	    s = ds.get(var);
	}
        if (encode) {
            s = Utilities.URLEncode(s);
        }
        if (quotehtml) {
            s = Utilities.quoteHtml(s);
        }

	return s;
    }

    public void setEncode(String s) { 
	encode = new Boolean(s).booleanValue();
    }

    public void setQuotehtml(String s) { 
	quotehtml = new Boolean(s).booleanValue();
    }
}
	
	
