package com.arsdigita.acs.ug;

import java.sql.*;
import java.util.*;
import com.arsdigita.acs.*;
import com.arsdigita.acs.db.*;

/**
 * Represents a series of user groups pulled in from the database.
 *
 * @version $Version$
 */

public class GroupDataSource extends Selection {

    public GroupDataSource (SqlStatement s, Database db) throws SQLException {
	super (s, db);
    }

    public GroupDataSource (ResultSet rs) throws SQLException { 
	super (rs);
    }

    public GroupDataSource() { }

    public Integer getGroupId() { 
	try {
	    return new Integer(get("group_id")); 
	} catch (Exception e) {
	    return null;
	}
    }

    public String getGroupType () { 
	try {
	    return get("group_type"); 
	} catch (Exception e) {
	    return null;
	}
    }

    public String getGroupName () { 
	// we may frequently have a group_id and need a group_name,
	// so we special-case a "get group name" query
	try {
	    String s = get("group_name"); 
	    if (s == null && db != null) {
		s = db.queryToString("select group_name from user_groups where group_id = ?", new Object[] {getGroupId()});
	    }
	    return s;
	} catch (Exception e) {
	    return null;
	}
    }
    
    public String getShortName () { 
	try {
	    return get("short_name"); 
	} catch (Exception e) {
	    return null;
	}
    }

    public Integer getParentGroupId() {
	try {
	    String s = get("parent_group_id");
	    return s == null? null: new Integer(s);
	} catch (Exception e) {
	    return null;
	}
    }

    /** 
     * Returns a list of supplemental fields and values for this group.
     */
    public SingleSelection  getSupplementalGroupInfo(Database db) 
	throws SQLException, ACSException, java.io.IOException { 

	SingleGroupTypeDataSource gt = new SingleGroupTypeDataSource(getGroupType());
	String helperTableName = gt.getHelperTableName(db);
	final String columnQuery = "select column_name " + 
	    "from user_group_type_fields " +
	    "where group_type = :group_type " + 
	    "order by sort_key";
	Vector v = db.queryToList(columnQuery , new Object[] {getGroupType()});

	if (v.size() > 0) {
	    ResultSet rs = db.query("select " + Utilities.join(v, ", ") + 
				    " from " + helperTableName + 
				    " where group_id = ?",
				    new Object[] {getGroupId()});
	    rs.next();
	    return new SingleSelection(rs);
	} else 
	    return null;
    }

    /** 
     * Returns the additional data collected about users in this
     * group.  
     * @return A MemberField datasource object
     */
    public MemberField getMemberFields(Database db) throws SQLException {
	final String query = "select field_name, field_type, " +
	    "sort_key " +
	    "from user_group_member_fields " +
	    "where group_id = ? " +
	    "order by sort_key";
	SqlStatement ss = new SqlStatement();
	ss.setSql(query);
	ss.addBindValue(getGroupId());
	return new MemberField(ss, db);
    }

    /**
     * Returns the number of additional fields collected about users
     * in this group.
     * @return the number of additional fields collected
     */
    public int getMemberFieldCount(Database db) 
	throws SQLException, ACSException {
	final String query = "select count(*) from user_group_member_fields" +
	    " where group_id = ? ";
	return Integer.parseInt(db.queryToString(query, new Object[] {getGroupId()}));
    }

}
