package com.arsdigita.acs.ug;

import java.sql.*;
import java.util.Vector;
import com.arsdigita.acs.*;
import com.arsdigita.acs.db.*;

/** 
 * Represents a user group type. (e.g., administration, intranet, events)
 * 
 * @version $Version$
 */

public class GroupTypeDataSource extends Selection {

    public GroupTypeDataSource () { } 

    public GroupTypeDataSource (SqlStatement s, Database db) throws SQLException {
	super(s, db);
    }

    public GroupTypeDataSource (ResultSet rs) throws SQLException {
	super(rs);
    }

    public String getGroupType() {
	try {
	    return get("group_type"); 
	} catch (Exception e) {
	    return null;
	}
    } 

    public String getPrettyName() { 
	try {
	    return get("pretty_name");
	} catch (Exception e) {
	    return null;
	}
    }

    public String getPrettyPlural() { 
	try {
	    return get("pretty_plural");
	} catch (Exception e) {
	    return null;
	}
    }

    public String getApprovalPolicy() { 
	try {
	    return get("approval_policy"); 
	} catch (Exception e) {
	    return null;
	}
    }

    public String getDefaultNewMemberPolicy() { 
	try {
	    return get("default_new_member_policy"); 
	} catch (Exception e) {
	    return null;
	}
    }

    public String getGroupModuleAdministration() { 
	try {
	    return get("group_module_administration");
	} catch (Exception e) {
	    return null;
	}
    }

    public String getHasVirtualDirectory() { 
	try {
	    return get("has_virtual_directory_p"); 
	} catch (Exception e) {
	    return null;
	}
    }

    public String getGroupTypeAdminDirectory() { 
	try {
	    return get("group_type_admin_directory"); 
	} catch (Exception e) {
	    return null;
	}
    }

    public String getGroupPublicDirectory() { 
	try {
	    return get("group_public_directory"); 
	} catch (Exception e) {
	    return null;
	}
    }
    
    public String getGroupAdminDirectory() { 
	try {
	    return get("group_admin_directory"); 
	} catch (Exception e) {
	    return null;
	}
    }

    /** 
     * @return Pretty description of the group_module_administration
     * field
     */
    public String getPrettyGroupModuleAdministration() { 
	String groupModuleAdministration = getGroupModuleAdministration();
	if (groupModuleAdministration.equals("full")) {
	    return "Complete";
	} else if (groupModuleAdministration.equals("enabling")) {
	    return "Enabling/Disabling";
	} else if (groupModuleAdministration.equals("none")) {
	    return "None";
	} else 
	    return "undefined";
    }

    public static String generateHelperTableName(String s) {
	return s.trim() + "_info";
    }

    /**
     * Returns the helper table name for this group type, creating it
     * if it doesn't exist.
     */
    public String getHelperTableName(Database db) throws SQLException,
    ACSException{ 
	String helperTable = generateHelperTableName(getGroupType());
	// see if table already exists
	String result = db.queryToString("select count(*) from user_tables"
					 + " where table_name = upper(?)",
					 new Object[] {helperTable});
	if ("0".equals(result)) {
	    // if it doesn't, create it.
	    db.dml("create table " + helperTable + "(group_id integer)");
	}
	return helperTable;
    }

    /** 
     * Gets a list of supplemental fields stored for each group 
     * in a particular group type.
     */
    public GroupField getGroupFields(Database db) throws SQLException {
	final String query = "select column_name, pretty_name, " + 
	    "column_type, column_actual_type, column_extra, sort_key " + 
	    "from user_group_type_fields " + 
	    "where group_type = ? " + 
	    "order by sort_key";
	SqlStatement ss = new SqlStatement();
	ss.setSql(query);
	ss.addBindValue(getGroupType());
	return new GroupField(ss, db);
    }

    /** 
     * Gets a list of data stored for each member belonging to a group 
     * of this type.
     */
    public MemberField getMemberFields(Database db) throws SQLException {
	final String query = "select field_name, field_type " +
	    "from user_group_type_member_fields " +
	    "where group_type = ? " +
	    "order by sort_key";
	
	SqlStatement ss = new SqlStatement();
	ss.setSql(query);
	ss.addBindValue(getGroupType());

	return new MemberField(ss, db);
    }
}
    

