package com.arsdigita.acs.ug;

import java.sql.*;
import java.util.*;
import com.arsdigita.acs.*;
import com.arsdigita.acs.db.*;

/** 
 * Represents a single user group.
 * 
 * @version $Version$
 */

public class SingleGroupDataSource extends GroupDataSource {

    Hashtable props; 

    public SingleGroupDataSource (ResultSet rs) throws SQLException {
	super(rs);
	try {
	    props = freeze();
	} catch(java.io.IOException e) { 
	    props = new Hashtable();
	}
    }

    public SingleGroupDataSource (Integer groupId) {
	props = new Hashtable();
	props.put ("group_id", groupId);
    }

    /**
     * Given the module and submodule of an administration group, looks up
     *    the group_id.  The returned selection only has one key, group_id.
     * @exception Throws ACSException if there is no group.
     * @param module Which module to look up
     * @param submodule An optional submodule to look up.
     */
    public SingleGroupDataSource (String module, String submodule) throws
    ACSException { 
	final String queryBase = 
	    "select group_id from administration_info where module = ? ";
	String s = null;
	try {
	    Database db = new Database();
	    if (Utilities.emptyString(submodule)) {
		s = db.queryToString(queryBase + " and submodule is null",
				     new Object[] {module});
	    } else {
		s = db.queryToString(queryBase + " and submodule = ?" ,
				     new Object[] {module, submodule});
	    }
	    db.close();
	    props = new Hashtable();
	    props.put("group_id", s);
	} 
	catch (SQLException e) { 
	    throw new ACSException(e.getMessage());
	}
    }

    /**
     * Same as SingleGroupDataSource(module, null).
     */
    public SingleGroupDataSource (String module) throws ACSException { 
	this(module, null);
    }

    public String get(String key) throws DataSourceException {
	Object o = props.get(key);
	if (o == null)
	    throw new InvalidKeyException("Message does not contain field '" + key + "'");
	else 
	    return o.toString();
    }

    public boolean next() {
	return false;
    }

    public String[] keys() {
	Enumeration e = props.keys();
	Vector v = new Vector();
	while (e.hasMoreElements()) 
	    v.addElement(e.nextElement());
	String[] retval = new String[v.size()];
	v.copyInto(retval);
	return retval;
    }

    public boolean contains(String key) { 
	return (props.get(key) != null);
    }
}
    



