package com.arsdigita.acs.ug;

import com.arsdigita.acs.*;
import com.arsdigita.acs.db.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.util.Vector;
import java.sql.*;

import org.apache.turbine.util.Log;

/**
 * Registered filter to set up variables in request attributes for 
 * user-group pages.  Its static initializer registers the filter 
 * to intercept the /groups URL, and to also /group-type where group-type
 * is a group type that has a virtual directory.
 */

public class UserGroupFilter extends Filter {

    static {
    // register ourselves
    RequestProcessor.registerFilter
        ("/groups", 
         new com.arsdigita.acs.ug.UserGroupFilter());
    // register filters for group types
    try {
        Database db = new Database();
        Vector v = db.queryToList("select group_type from user_group_types where has_virtual_directory_p='t'", null);
        db.close();
        for (int i = 0; i < v.size(); i++) { 
        RequestProcessor.registerFilter
            ("/" + v.elementAt(i), 
             new com.arsdigita.acs.ug.UserGroupFilter());
        }
    } catch (SQLException sqle) { 
        Log.error("UserGroupFilter: caught SQLException on initializion: " + sqle.getMessage());
    }
    }

    /**
     * Processes an incoming URL under /groups or /(group-type) and sets 
     * appropriate request attributes for the requested URL.
     * <p>
     * After the filter has run successfully, the following request
     * attributes are set:
     * <ul>
     * <li>group_id
     * <li>group_name
     * <li>group_type_pretty_plural
     * <li>group_type
     * <li>group_type_pretty_name
     * <li>group_type_url_p: true if the URL included a group type virtual
     * directory
     * <li>groups_public_dir: the path to the groups public directory root.
     * <li>groups_admin_dir: the path to the groups admin directory root.
     * <li>group_public_dir: the actual path to where the public files for
     *  a group are to be served from.
     * <li>group_admin_dir:  the actual path to where the admin files for
     *  a group are to be served from.
     * <li>group_public_root_url: the root URL for public group pages
     *  for all groups of this group type.
     * <li>group_admin_root_url: the root URL for admin group pages
     *  for all groups of this group type.
     * </ul>
     */
    public int filter (HttpServletRequest req, 
               HttpServletResponse resp,
               ServletContext ctx) 
    throws ACSException {

    try {

        req.setAttribute("already_filtered", "yes");
        
        String urlStub = req.getRequestURI();
        Vector urlList = UserGroups.parseURL(urlStub);
        int urlListLength = urlList.size();

        if (urlListLength < 1) { 
            // this should never happen, so return an error to indicate that something went wrong
            Log.error("/tcl/user-group-defs.tcl: function ug_serve_group_pages: got url_list_length less than 1");
            throw new ACSException("Error in serving group pages. If the problem persists, please contact the system administrator.");
        }
        
        String groups_public_dir = "/groups";
        String groups_admin_dir = "/groups/admin";
        String group_public_dir = "/groups/group";
        String group_admin_dir = "/groups/admin/group";
        String group_public_root_url = UserGroups.getUrl();
        String group_admin_root_url = UserGroups.getAdminUrl();
        String group_type_pretty_plural = "";
        String group_type = "";
        String group_type_pretty_name = "";
        Boolean group_type_url_p;
        Vector group_context_bar_list = new Vector();

        String urlFirstElement = (String)urlList.elementAt(0);
        
        Database db = new Database();
        ResultSet rs;

        if (urlFirstElement.equals(Parameters.get("GroupsDirectory", "ug", "groups"))) {
            // we are serving /groups directory, so set the appropriate 
            // directories where files are located
        
            // (already set defaults above)
        
            // this flag indicates that page was accessed through 
            // generic /groups pages as opposed to /$group_type pages
            group_type_url_p = new Boolean(false);
        
        } else {
            // let's check if we are dealing with one of the group type
            // virtual directories
            rs = db.query
                ("select group_type, " + 
                 "pretty_name as group_type_pretty_name, " + 
                 "pretty_plural as group_type_pretty_plural, " + 
                 "group_type_public_directory, group_type_admin_directory, " +
                 "group_public_directory, group_admin_directory " + 
                 "from user_group_types " + 
                 "where has_virtual_directory_p='t' " + 
                 " and group_type=:url_first_element",
                 new Object[] {urlFirstElement});
            if (!rs.next()) { 
                // this should never happen since this procedure is called only with url's 
                // registered with ad_register_proc to handle group type directories
                Log.error("/tcl/user-group-defs.tcl: function ug_serve_group_pages: first element in url_list is not parameter GroupsDirectory, module ug");
                throw new ACSException("Error in serving group pages. If the problem persists, please contact the system administrator.");
            }

            group_type = urlFirstElement;
        
            if (rs.getString("group_type_public_directory") != null) {
                groups_public_dir = rs.getString("group_type_public_directory");
            }
            if (rs.getString("group_type_admin_directory") != null) {
                groups_admin_dir = rs.getString("group_type_admin_directory");
            }
            if (rs.getString("group_type_pretty_plural") != null) {
                group_type_pretty_plural = rs.getString("group_type_pretty_plural");
            }
            if (rs.getString("group_public_directory") != null) {
                group_public_dir = rs.getString("group_public_directory");
            }
            if (rs.getString("group_admin_directory") != null) {
                group_admin_dir = rs.getString("group_admin_directory");
            }
            group_public_root_url = "/" + Utilities.URLEncode(rs.getString("group_type"));
            group_admin_root_url = "/" + Utilities.URLEncode(rs.getString("group_type")) + "/" + Parameters.get("GroupsAdminDirectory", "ug", "admin");
            // this flag indicates that page was accessed through 
            // $group_type pages as opposed to generic /groups pages
            group_type_url_p = new Boolean(true);
        }
        
        if (urlListLength==1) { 
            // this means that url is /groups or /$group_type
            // then just redirect url to the directory /groups/ or 
            // /$group_type/

            // System.out.println("GroupFilter A redirecting to: " + urlStub + "/");
            db.close();
            resp.sendRedirect(urlStub + "/");
            return FILTER_RETURN;
        }

        req.setAttribute("scope", "group");
        req.setAttribute("groups_public_dir", groups_public_dir);
        req.setAttribute("group_type_url_p", group_type_url_p);
        req.setAttribute("group_type", group_type);
        req.setAttribute("group_type_pretty_name", group_type_pretty_name);
        req.setAttribute("group_type_pretty_plural", group_type_pretty_plural);
        req.setAttribute("group_public_root_url", group_public_root_url);
        req.setAttribute("group_admin_root_url", group_admin_root_url);
        req.setAttribute("group_context_bar_list", group_context_bar_list);
        
        String url_second_element = (String)urlList.elementAt(1);
        if (url_second_element.equals("/")) { 
            // this is groups public directory, so serve the appropriate
            // groups index page
            db.close();
            RequestDispatcher rd = req.getRequestDispatcher(groups_public_dir + "/index");
            // System.out.println("GroupFilter B forwarding to: " + groups_public_dir + "/index");
            rd.forward(req, resp);
            return FILTER_RETURN;
        }
        
        if (url_second_element.equals(Parameters.get("GroupsAdminDirectory", "ug", "admin"))) {
            // these are /groups/admin pages
            if (urlListLength == 2) {
                // System.out.println("GroupFilter C redirecting to: " + urlStub + "/");
                db.close();
                resp.sendRedirect(urlStub + "/");
                return FILTER_RETURN;
            }

            if (urlListLength < 3) {
                // this should never happen, so return an error to indicate that something went wrong
                db.close();
                Log.error("UserGroupFilter: got urlListLength less than 3 " + urlStub);
                throw new WarningException("Error", "Error in serving group pages.  If the problem persists, please contact the system administrator.");
            }

            String url_third_element = (String)urlList.elementAt(2);

            if (url_third_element.equals("/")) {
                // this is /groups/admin/ so serve the groups admin index page
                db.close();
                RequestDispatcher rd = req.getRequestDispatcher(groups_admin_dir + "/index");
                // System.out.println("GroupFilter D forwarding to: " + groups_admin_dir + "/index");
                rd.forward(req, resp);
                return FILTER_RETURN;
            }

            group_context_bar_list.addElement(new String[] { group_admin_root_url + "/", "Groups Admin" });

            rs = db.query("select group_id, short_name, admin_email, group_name " +
                                    "from user_groups " +
                                    "where short_name=:url_third_element", 
                                    new Object [] { url_third_element });
            if (!rs.next()) {
                // it might be a file; let the request processor handle it 
                db.close();
                RequestDispatcher rd = req.getRequestDispatcher(groups_admin_dir + "/" + url_third_element);
                // System.out.println("GroupFilter E forwarding to: " + groups_admin_dir + "/" + url_third_element);
                rd.forward(req, resp);
                return FILTER_RETURN;
            }

            String group_id = rs.getString("group_id");
            String short_name = rs.getString("short_name");
            String admin_email = rs.getString("admin_email");
            String group_name = rs.getString("group_name");
            
            req.setAttribute("group_id", group_id);
            req.setAttribute("group_short_name", short_name);
            req.setAttribute("group_name", group_name);
            req.setAttribute("group_admin_email", admin_email);
            req.setAttribute("group_public_url", group_public_root_url + "/" + Utilities.URLEncode(short_name));
            req.setAttribute("group_admin_url", group_admin_root_url + "/" + Utilities.URLEncode(short_name));


            if (urlListLength == 3) {
                // this means that url is $group_admin_root_url/short_name
                // so we just redirect url to the directory $group_admin_root_url/short_name/
                // System.out.println("GroupFilter F redirecting to: " + urlStub + "/");
                db.close();
                resp.sendRedirect(urlStub + "/");
                return FILTER_RETURN;
            }

            String url_fourth_element = (String)urlList.elementAt(3);
            if (url_fourth_element.equals("/")) {
                // this is /groups/admin/short_name/ so serve the admin index page of that group
                RequestDispatcher rd = req.getRequestDispatcher(group_admin_dir + "/index");
                // System.out.println("GroupFilter G forwarding to: " + groups_admin_dir + "/index");
                db.close();
                rd.forward(req, resp);
                return FILTER_RETURN;
            }
    
            // append groups listing page link to the context bar list
            group_context_bar_list.addElement(new String[] { 
                group_admin_root_url + "/" + Utilities.URLEncode(short_name) + "/", "One Group Admin" });

            // in this query, notice that we only have content section administration for 
            // sections of section_type admin, system and custom
            rs = db.query("select section_type, section_key, module_key " +
                          "from content_sections " +
                          "where scope='group' and group_id=:group_id " +
                          "and (section_type='admin' or section_type='system' or section_type='custom') " +
                          "and section_key=:url_fourth_element",
                          new Object[] { group_id, url_fourth_element });
            
            if (!rs.next()) {
                // it might be a file; let the request processor handle it 
                db.close();
                RequestDispatcher rd = req.getRequestDispatcher(group_admin_dir + "/" + url_fourth_element);
                // System.out.println("GroupFilter H forwarding to: " + group_admin_dir + "/" + url_fourth_element);
                rd.forward(req, resp);
                return FILTER_RETURN;
            }
    
            if (urlListLength == 4) {
                // this means that url is $group_admin_root_url/short_name/section_key
                // so we just redirect url to the directory $group_admin_root_url/short_name/section_key/
                // System.out.println("GroupFilter I redirecting to: " + urlStub + "/");
                db.close();
                resp.sendRedirect(urlStub + "/");
                return FILTER_RETURN;
            }

            String url_fifth_element = (String)urlList.elementAt(4);
    
            String section_type = rs.getString("section_type");
            String section_key = rs.getString("section_key");
            String module_key = rs.getString("module_key");
            
            String admin_url;
            if (section_type.equals("admin") || section_type.equals("system")) {
                admin_url = db.queryToString("select admin_directory from acs_modules where module_key=:module_key",
                                             new Object[] { module_key }, "");
            } else {
                admin_url = db.queryToString("select admin_directory from acs_modules where module_key='custom-sections'", "");
            }
                        
            if (url_fifth_element.equals("/")) {
                admin_url += "/index";
            } else {
                admin_url += "/" + url_fifth_element;
            }
                
            // serve a page in /groups/admin/short_name/section_key/ 
            db.close();
            RequestDispatcher rd = req.getRequestDispatcher(admin_url);
            // System.out.println("GroupFilter J forwarding to: " + admin_url);
            rd.forward(req, resp);
            return FILTER_RETURN;

            // END OF CODE SERVING ADMIN PAGES
        }

        // append group listing page link to the context bar
        group_context_bar_list.addElement(new String[] { 
            group_public_root_url + "/", (group_type_url_p.booleanValue() ? group_type_pretty_plural : "Groups") });

        // at this point, check to see if the URL is of the form
        // /group|$group_type/$short_name
        
        rs = db.query
            ("select group_id, short_name, group_name, admin_email " + 
             "from user_groups where short_name=:url_second_element", 
             new Object[] {url_second_element});

        if (!rs.next()) { 
            // it might be a file in the /group directory - let the request processor handle it.
            db.close();
            // System.out.println("GroupFilter K allowing request to pass through for " + urlStub);
            return FILTER_OK;
        } 
        String group_id = rs.getString("group_id");
        String short_name = rs.getString("short_name");
        String group_name = rs.getString("group_name");
        String admin_email = rs.getString("admin_email");

        req.setAttribute("group_id", group_id);
        req.setAttribute("group_short_name", short_name);
        req.setAttribute("group_name", group_name);
        req.setAttribute("group_admin_email", admin_email);
        req.setAttribute("group_public_url", group_public_root_url + "/" + Utilities.URLEncode(short_name));
        req.setAttribute("group_admin_url", group_admin_root_url + "/" + Utilities.URLEncode(short_name));

        if (urlListLength == 2) {
            // this means that url is $group_public_root_url/short_name
            // so we redirect url to the directory $group_public_root_url/short_name/
            // System.out.println("GroupFilter L redirecting to: " + urlStub + "/");
            resp.sendRedirect(urlStub  + "/");
            return FILTER_RETURN;
        }

        String url_third_element = (String)urlList.elementAt(2);
    
        if (url_third_element.equals("/")) {
            // this is /groups/short_name/ so serve the index page of that group
            db.close();
            // System.out.println("GroupFilter M forwarding to: " + group_public_dir + "/index");
            RequestDispatcher rd = req.getRequestDispatcher(group_public_dir + "/index");
            rd.forward(req,resp);
            return FILTER_RETURN;
        } 
        
        // append group index page link to the context bar list
        group_context_bar_list.addElement(new String[] { 
            group_public_root_url + "/" + Utilities.URLEncode(short_name) + "/", 
                (group_type_url_p.booleanValue() ? "One " + group_type_pretty_name : "One Group") });

        rs = db.query
            ("select section_id, section_type, section_pretty_name, section_key, " +
             "section_url_stub, module_key, requires_registration_p, visibility " +
             "from content_sections " +
             "where scope = 'group' and group_id = :group_id " +
             "and section_key = :url_third_element " +
             "and section_type != 'admin' " +
             "and enabled_p = 't'",
             new Object[] { group_id, url_third_element });       

        if (!rs.next()) {
            // it might be a file; let the request processor handle it 
            db.close();
            RequestDispatcher rd = req.getRequestDispatcher(group_public_dir + "/" + url_third_element);
            // System.out.println("GroupFilter N forwarding to: " + group_public_dir + "/" + url_third_element);
            rd.forward(req, resp);
            return FILTER_RETURN;
        }
        
        String section_id = rs.getString("section_id");
        String section_type = rs.getString("section_type");
        String section_pretty_name = rs.getString("section_pretty_name");
        String section_key = rs.getString("section_key");
        String section_url_stub = rs.getString("section_url_stub");
        String module_key = rs.getString("module_key");
        String requires_registration_p = rs.getString("requires_registration_p");
        String visibility = rs.getString("visibility");

        // if section is custom or static, let's see if the user is allowed to see that section 
        // this is determined using requires_registration_p and visibility
        if (section_type.equals("static") || section_type.equals("custom")) {
            // NYI: authorize might throw a ForbiddenException: make sure this gets handled by errors.jsp
            new Scope(req, resp).scopeAuthorize("none", (visibility.equals("private") ? "group_member" : "registered"), "none");
        }

        if (urlListLength == 3) {
            // this means that url is $group_public_root_url/short_name/section_key
            // so we redirect url to the directory $group_public_root_url/short_name/section_key/
            // System.out.println("GroupFilter O redirecting to: " + urlStub + "/");
            resp.sendRedirect(urlStub  + "/");
            return FILTER_RETURN;
        }

        String url_fourth_element = (String)urlList.elementAt(3);

        // figure out the navigation bars 

        Vector group_navbar_list = new Vector();

        rs = db.query("select cs.section_key as to_section_key, " +
                                "cs.section_pretty_name as to_section_pretty_name " +
                                "from content_section_links csl, content_sections cs " +
                                "where csl.from_section_id=:section_id " +
                                "and csl.to_section_id=cs.section_id " +
                                "order by cs.section_key", 
                                new Object[] { section_id });

        while (rs.next()) {
            group_navbar_list.addElement(new String[] { group_public_root_url + "/" + 
                                         Utilities.URLEncode(short_name) + "/" +
                                         Utilities.URLEncode(rs.getString("to_section_key")) + "/",
                                         rs.getString("to_section_pretty_name") });
        }                                         
        req.setAttribute("group_navbar_list", group_navbar_list);
    
        if (section_type.equals("static")) {
            // no matter what follows the section name in the url, a static section should just
            // function as a shortcut to a static page.
            db.close();
            RequestDispatcher rd = req.getRequestDispatcher(section_url_stub);
            // System.out.println("GroupFilter P forwarding to: " + section_url_stub);
            rd.forward(req, resp);
            return FILTER_RETURN;
        }

        if (section_type.equals("custom")) {
            module_key = "custom-sections";
        }
        String public_dir = db.queryToString("select public_directory from acs_modules where module_key=:module_key", 
                                             new Object[] { module_key }, "");

        if (url_fourth_element.equals("/")) {
            // this is /groups/short_name/section_key/
            db.close();
            RequestDispatcher rd = req.getRequestDispatcher(public_dir + "/index");
            // System.out.println("GroupFilter Q forwarding to: " + public_dir + "/index");
            rd.forward(req, resp);
            return FILTER_RETURN;
        }

        if (section_type.equals("system")) {
            // forward this request to a group-scoped module page
            db.close();
            RequestDispatcher rd = req.getRequestDispatcher(public_dir + "/" + url_fourth_element);
            // System.out.println("GroupFilter R forwarding to: " + public_dir + "/" + url_fourth_element);
            rd.forward(req, resp);
            return FILTER_RETURN;
        }

        // only "custom" section type is left at this point

        // append group section index page link to the context bar list            
        group_context_bar_list.addElement(new String[] {group_public_root_url + "/" + 
                                          Utilities.URLEncode(short_name) + "/" +
                                          Utilities.URLEncode(section_key) + "/",
                                          section_pretty_name });

        rs = db.query("select content_file_id, file_type " +
                                "from content_files " +
                                "where section_id=:section_id " +
                                "and file_name= :url_fourth_element",
                                new Object[] { section_id, url_fourth_element });

        if (!rs.next()) {
            // it might be a file; let the request processor handle it.
            db.close();
            RequestDispatcher rd = req.getRequestDispatcher(public_dir + "/" + url_fourth_element);
            // System.out.println("GroupFilter S forwarding to: " + public_dir + "/" + url_fourth_element);
            rd.forward(req, resp);
            return FILTER_RETURN;
        }

        String file_type = rs.getString("file_type");

        String customUrl = "/custom-sections/file/" + 
            (file_type.equals("text/html") ? "index.tcl" : "get-binary-file.tcl");

        db.close();
        RequestDispatcher rd = req.getRequestDispatcher(customUrl);
        // System.out.println("GroupFilter T forwarding to: " + customUrl);
        rd.forward(req, resp);
        return FILTER_RETURN;
    } catch (Exception e) { 
        throw new ACSException(e.toString() + ": " + e.getMessage());
    }
    }
}
