package com.arsdigita.acs.ug;

import java.sql.*;
import java.util.Vector;
import com.arsdigita.acs.*;
import com.arsdigita.acs.db.Database;

public class UserGroups {
    private static com.oroinc.text.perl.Perl5Util re = new com.oroinc.text.perl.Perl5Util();

    public final static String getGroupName (Database db, Integer groupId) { 
	// we may frequently have a group_id and need a group_name,
	// so we special-case a "get group name" query
	try {
	    return db.queryToString("select group_name from user_groups where group_id = ?", new Object[] {groupId});
	} catch (Exception e) {
	    return null;
	}
    }

    /**
     * returns groups url directory with leading slash.
     * (parameter: GroupsDirectory, module ug)
     */
    public final static String getUrl() { 
	return "/" + Parameters.get("GroupsDirectory", "ug", "groups");
    }

    /**
     * returns groups admin url directory with leading slash.
     * (parameters: GroupsDirectory, GroupsAdminDirectory)
     */
    public final static String getAdminUrl () {
	return "/" + Parameters.get("GroupsDirectory", "ug", "groups") + "/" + Parameters.get("GroupsAdminDirectory", "ug", "admin");
    }

    /**
     * This procedure takes url in the form /A/B/C and returns a Vector
     * whose members are A, B and C. if the last element of this tcl list
     * is /, then this / will be added as the last element in the list 
     * (e.g. /A/B/C/ will have elements A, B, C and /). if url_string is 
     * empty, procedure will return an empty list
     */
    public final static Vector parseURL(String s) { 
	boolean trailingSlash = false;
	if (s.endsWith("/")) trailingSlash = true;
	Vector v = re.split("@/@", s);
	if (((String)v.elementAt(0)).equals("")) {
	    v.removeElementAt(0);
	}
	if (trailingSlash) {
	    v.addElement("/");
	    if (((String)v.elementAt(v.size()-2)).equals("")) { 
		v.removeElementAt(v.size()-2);
	    }
	}
	return v;
    }

    /**
     * Given url list as described in ug_parse_url this procedure puts back
     * the url from the list. thus, if list contains elements A, B and C, 
     * this procedure will return A/B/C. if list contains elements A, B, C 
     * and / than this procedure will return A/B/C/.
     */
    public final static String urlFromList(Vector v) { 
	boolean trailingSlash = false;
	if (((String)v.elementAt(v.size()-1)).equals("/")) { 
	    trailingSlash = true;
	    v.removeElementAt(v.size()-1);
	}
	String s = Utilities.join(v, "/");
	if (trailingSlash) 
	    s += "/";
	return s;
    }


    /** 
     * adds an admin group to the database given a name, a short name (key),
     * the module/submodule it administers, and the URL for this group's
     * admin page.
     */
    public final static void adminGroupAdd(Database db,
					   String name, 
					   String shortname, 
					   String module,
					   String submodule,
					   String url) throws SQLException {
	db.plsql("begin administration_group_add(:name, :short_name, :module, :submodule, :multi_role_p, :url); end;", 
	       new Object[] {name, shortname, module, submodule, "f", url});
    }

    /** 
     * adds an admin group to the database.  
     */
    public final static void adminGroupAdd(Database db,
					   String name, 
					   String module,
					   String submodule,
					   String url) throws SQLException {
	adminGroupAdd(db, name, name, module, submodule, url);
    }

    /** 
     * Returns a group_id for an admin group from the database, given
     * a module/submodule.
     */
    public final static String getAdminGroupId(Database db,
					       String module,
					       String submodule)
	throws SQLException {
	try {
	    return db.queryToString("select group_id from administration_info where module = ? and submodule = ?", new Object[] {module, submodule});
	} catch (ACSException  e) { 
	    return "";
	}
    }

    /** 
     * adds a user to an admin group in a certain role.
     */
    public final static void adminGroupAddUser(Database db,
					       Object user_id,
					       String role,
					       String module,
					       String submodule,
					       String mapping_id,
					       String mapping_ip)
	throws SQLException, ACSException {

	// TO DO: ERROR CHECKING!!

	// first get out the group id
	String group_id;
	if (!Utilities.emptyString(submodule)) { 
	    group_id = db.queryToString("select group_id from administration_info where module=? and submodule=?", new Object[] {module, submodule});
	} else {
	    group_id = db.queryToString("select group_id from admin_info where module=? and submodule is null", new Object[] {module});
	}

	db.dml("insert into user_group_map (group_id, user_id, role, mapping_user, mapping_ip_address) select :group_id, :user_id, :role, :mapping_user, :mapping_ip_address from dual where not exists (select 1 from user_group_map where user_id = ? and group_id = ?)",
	       new Object[] {group_id, user_id, role, mapping_id, mapping_ip, user_id, group_id});
    }
}


	    
