package org.apache.turbine.services.template;

/*
 * Copyright (c) 1997-1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */
 
// Java Core Classes
import java.util.*;
import java.io.File;
import javax.servlet.*;

// Turbine Utility Classes
import org.apache.turbine.util.*;
import org.apache.turbine.services.*;
import org.apache.turbine.services.resources.TurbineResources;
import org.apache.turbine.modules.*;


/**
 * This service provides a method for mapping templates to their appropriate
 * Screens or Navigations.  It also allows templates to define a 
 * layout/navigations/screen modularization within the template structure.
 * It also performs caching if turned on in the properties file.
 *
 * @author <a href="mailto:john.mcnally@clearink.com">John McNally</a>
 * @author <a href="mailto:mbryson@mont.mindspring.com">Dave Bryson</a>
 * @version $Revision: 1.1 $
 */
public class TurbineTemplateService extends TurbineBaseService 
    implements TemplateService
{
    /** The hashtable used to cache Screen names*/
    private Hashtable screenCache = null; 

    /** The hashtable used to cache Navigation names */
    private Hashtable navCache = null; 

    /** The hashtable used to cache layout template names*/
    private Hashtable layoutCache = null; 
    
    /** flag set if cache is to be used. */
    private boolean useCache = false;
    
    /** default extension */
    private String extension = null;
    
    /** 
        The absolute paths where the appropriate template engine
        will be searching for templates
    */
    private String templateRoot = null;
  
    /**
     * Called the first time the Service is used.
     */
    public void init(ServletConfig info)
    {
        if ( getInit() )
        {
            return;
        }
        
        try
        {
            org.apache.turbine.util.Log.note ( "TemplateService init()....starting!");
            initTemplate(info);
            setInit(true);
            org.apache.turbine.util.Log.note ( "TemplateService init()....finished!");
        }
        catch (Exception e)
        {
            org.apache.turbine.util.Log.error ( "Cannot initialize TurbineTemplateService!" );
            org.apache.turbine.util.Log.error (e);
        }
    }
 
    /**
    * 
    */
    private void initTemplate(ServletConfig info) throws Exception
    {
/*     
       if (info == null || info.getServletConfig() == null)
       {
           String message = "The TemplateService requires an initialization" +
               " object \nthat was unavailable when the Service was requested.\n" +
               " This can happen when an outside resource is using a service \n" +
               " that requires Turbine to be initialized and it was not at the time.\n" +
               " this service was first requested.";
           throw new Exception(message);
       }
       else
       {
*/
        ServletContext context = info.getServletContext();
        
        useCache = TurbineResources.getBoolean("modules.cache", true);
        Properties props = getProperties();
        
        if (useCache)
        {
            int layoutSize = Integer
                .parseInt(props.getProperty("layout.cache.size", "2"));
            int navigationSize = Integer
                .parseInt(props.getProperty("navigation.cache.size", "10"));
            int screenSize = Integer
                .parseInt(props.getProperty("screen.cache.size", "50"));
            layoutCache = new Hashtable( (int)(1.25*layoutSize) + 1);
            navCache = new Hashtable( (int)(1.25*navigationSize) + 1);
            screenCache = new Hashtable( (int)(1.25*screenSize) + 1);
        }
           
        templateRoot = props.getProperty("template.path", "/WEB_INF/templates");

        // if getRealPath is working, transform paths to be webapp root relative
        if(context.getRealPath("/") != null)
        {
            String pathSep = System.getProperty("path.separator");
            StringTokenizer st = new StringTokenizer(templateRoot,pathSep);
            StringBuffer buf = new StringBuffer();
            while(st.hasMoreTokens())
            {
                buf.append(context.getRealPath(st.nextToken()));
                if(st.hasMoreTokens())
                {
                    buf.append(pathSep);
                }
            }
            templateRoot = buf.toString();
        }
        
        // store the converted path in service properties for 
        // Turbine based providers 
        props.put("template.path", templateRoot);

        // the extension that is added to layout templates (e.g.)
        extension = props.getProperty("default.extension", "html");        
       
   }
        

    /**
     *  Adds the object into the hashtable.
     */
    private void addToCache ( String key, Object value, Hashtable c )
    {
        if (useCache && value != null)
        {
            c.put(key, value);
        }
    }
    
    /**
     * locate and return the name of a layout template
     * @param String the name of the template.
     * @return String the layout template path
     */
    public String getLayoutTemplateName(String name) throws Exception
    {
        String layoutName = null;
        
        if (  useCache && layoutCache.containsKey(name) )
        {
            layoutName = (String)layoutCache.get(name);
        }
        else
        {
            String[] names = parseScreenTemplate(name);
            layoutName = names[1];
            addToCache( name, names[0], screenCache );
            addToCache( name, names[1], layoutCache );
        }
        return layoutName;
    }    

    /**
     * locate and return the name of a Navigation module
     * @param String the name of the template.
     * @return String the Navigation name
     */
    public String getNavigationName(String name) throws Exception
    {
        String nav_name = null;
        
        if (  useCache && navCache.containsKey(name) )
        {
            nav_name = (String)navCache.get(name);
        }
        else
        {
            nav_name = parseNavigationTemplate(name);
            addToCache( name, nav_name, navCache );
        }
        return nav_name;
    }
    
    /**
     * locate and return the name of a Screen module
     * @param String the name of the template.
     * @return String the Screen name
     */
    public String getScreenName(String name) throws Exception
    {
        String screenName = null;
        
        if (  useCache && screenCache.containsKey(name) )
        {
            screenName = (String)screenCache.get(name);
        }
        else
        {
            String[] names = parseScreenTemplate(name);
            screenName = names[0];
            addToCache( name, names[0], screenCache );
            addToCache( name, names[1], layoutCache ); 
        }
        return screenName;
    }
    
    /**
     * get the default extension given in the properties file
     * @return String the extension
     */
    public String getDefaultExtension()
    {
        return extension;
    }

     /**
     * This method takes the template parameter and parses it, so that relevant 
     * Screen/Layout-template information can be extracted.
     * Possible layout paths and Screen packages placed in a TemplateHolder
     * @param String the template name
     * @return TemplateHolder
     */
    private String[] parseScreenTemplate( String template )
    {
        StringTokenizer st = new StringTokenizer(template, "/");
        Vector tokens = new Vector(st.countTokens());
        while(st.hasMoreTokens())
        {
            String token = st.nextToken();
            if (!token.equals(""))
            {
                tokens.addElement(token);
            }
        }
        String fileName = (String)tokens.lastElement();
        tokens.setSize(tokens.size()-1);
        int dot = fileName.lastIndexOf('.');
        String className = null;
        if (dot>0)
        {
            className = fileName.substring(0, dot);
        }
        else
        {
            className = fileName;
        }
        String firstChar = String.valueOf(className.charAt(0));
        firstChar = firstChar.toUpperCase();
        className = firstChar + className.substring(1);

        String[] paths = new String[tokens.size() + 2];
        String[] pkgs = new String[tokens.size() + 2];
        int arrayIndex = 0;
        for (int i=tokens.size(); i>=0;  i--)
        {
            StringBuffer path = new StringBuffer();
            StringBuffer pkg = new StringBuffer();
            for (int j=0; j<i; j++)
            {
                path.append("/").append((String)tokens.elementAt(j));
                pkg.append((String)tokens.elementAt(j)).append('.');
            }
            if ( i == tokens.size() )
            {
                StringBuffer distinctPath = new StringBuffer(path.toString());
                StringBuffer distinctPkg = new StringBuffer(pkg.toString());
                paths[arrayIndex] = distinctPath.append('/').append(fileName).toString();
                pkgs[arrayIndex] = distinctPkg.append(className).toString();
                arrayIndex++;
            }
            paths[arrayIndex] = path.append("/default." + extension).toString();
            pkgs[arrayIndex] = pkg.append("Default").toString();
            arrayIndex++;
        }
        
        
        String[] holder = new String[2];
        holder[0] = getScreenName(pkgs);
        holder[1] = getLayoutTemplateName(paths);

        return holder;
    }



    /**
     * Parse the template name out to a package path to locate the 
     * Navigation module<br>
     * This is different than the Screen/Layout parser in that it only looks
     * for packages.<br>
     * Note: If caching is enabled, this is only performed once for each unique template.
     * @param String the template name (i.e folder/headernav.wm)
     * @return String the name of the Navigation module to use for the template
     */
    private String parseNavigationTemplate( String template )
    {
        StringTokenizer st = new StringTokenizer(template, "/");
        Vector tokens = new Vector(st.countTokens());
        while(st.hasMoreTokens())
        {
            String token = st.nextToken();
            if (!token.equals(""))
            {
                tokens.addElement(token);
            }
        }
        String fileName = (String)tokens.lastElement();
        tokens.setSize(tokens.size()-1);
        int dot = fileName.lastIndexOf('.');
        String className = null;
        if (dot>0)
        {
            className = fileName.substring(0, dot);
        }
        else
        {
            className = fileName;
        }
        String firstChar = String.valueOf(className.charAt(0));
        firstChar = firstChar.toUpperCase();
        className = firstChar + className.substring(1);

        String[] pkgs = new String[tokens.size() + 2];
        int arrayIndex = 0;
        for (int i=tokens.size(); i>=0;  i--)
        {
            StringBuffer pkg = new StringBuffer();
            for (int j=0; j<i; j++)
            {
                pkg.append((String)tokens.elementAt(j)).append('.');
            }
            if ( i == tokens.size() )
            {
                StringBuffer distinctPkg = new StringBuffer(pkg.toString());
                pkgs[arrayIndex] = distinctPkg.append(className).toString();
                arrayIndex++;
            }
            pkgs[arrayIndex] = pkg.append("Default").toString();
            arrayIndex++;
        }
        return getNavigationName( pkgs);
    }

    /**
     * Extract possible layouts paths.
     * @param String[] possible paths to search
     * @return String name of the layout template 
     */
    private String getLayoutTemplateName(String[] possiblePaths)
    {
        for (int i=0; i<possiblePaths.length; i++)
        {
            if (new File(templateRoot, "layouts" + possiblePaths[i]).exists())
            {
                return possiblePaths[i];
            }
        }
        return getProperties()
            .getProperty("default.layout.template", "/default." + extension);
    }

    /**
     * Extract a possible Screen from the packages.
     * @param String[] possible paths to search
     * @return String name of the Screen class to use 
     */      
    private String getScreenName( String[] possibleScreens)
    {
        for (int i=0; i<possibleScreens.length; i++)
        {
            try
            {
                ScreenLoader.getInstance().getInstance(possibleScreens[i]);
                return possibleScreens[i];
            }
            catch (Exception e)
            {
                // Log.error ("Error in WebMacroSitePage: ", e);
            }
        }
        return getProperties().getProperty("default.screen", null);
    }


    /**
     * Seaches for the Navigation class that may match the 
     * name of the Navigation template.
     * @param String[] possible navigation packages
     * @return String the name of the Navigation class to use.
     */
    private String getNavigationName( String[] possibleNavigations)
    {
        for (int i=0; i<possibleNavigations.length; i++)
        {
            try
            {
                NavigationLoader.getInstance().getInstance(possibleNavigations[i]);
                return possibleNavigations[i];
            }
            catch (Exception e) {}
        }
        return getProperties().getProperty("default.navigation", null);
    }
    
}