package org.apache.turbine.tools;

/*
 * Copyright (c) 1997-1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

import java.io.*;
import org.apache.turbine.util.db.map.TableMap;
import org.apache.turbine.util.db.map.ColumnMap;
import org.apache.turbine.util.db.map.DatabaseMap;
import org.apache.turbine.util.db.map.MapBuilder;

//XML Stuff
import org.w3c.dom.*;
import org.apache.xml.serialize.*;

// Ant Stuff
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;

/** Generate a XML description from a DatabaseMap
 * <p>This class is a base class for generating a XML description
 * for a database structure.
 * The source DatabaseMap is givent through the abstract
 * getDatabaseMap() method, which should be implemented in
 * descendent.
 * This class can be used to generate XML from existant Databases for
 * reingenieering.
 * <p>This is a Ant build task that has 1 parameter: xmlfile.
 *
 * @author Regis Koenig <a href="mailto:rkoenig@chez.com">Regis Koenig</a>
 */
public abstract class BaseXmlGeneratorTask
{
    /* the output file */
    private String xmlfile;

    /** current DatabaseMap */
    private DatabaseMap dbmap;
    /** current TableMap */
    private TableMap tmap;
    /** current columnMap */
    private ColumnMap cmap;

    // elements
    private static final String COLUMN = "column";
    private static final String TABLE = "table";
    private static final String DATABASE = "database";

    // atrributes global
    private static final String NAME = "name";
    // attributes database
    private static final String CONNECTION_POOL = "connectionPool";
    // attributes column
    private static final String PRIMARY_KEY = "primaryKey";
    private static final String TYPE = "type";
    private static final String NOTNULL = "notNull";
    private static final String FOREIGN_TABLE_NAME = "foreignPKTable";
    private static final String FOREIGN_COLUMN_NAME = "foreignPKColumn";

    // attributes column value
    private static final String PK_AUTOINCREMENT = "autoIncrement";
    private static final String PK_NONE = "none";
    private static final String PK_SEQUENCE = "sequence";
    private static final String PK_IDBROKERTABLE = "idBroker";

    // types value
    private static final String TYPE_INTEGER = "integer";
    private static final String TYPE_DATE = "date";
    private static final String TYPE_HASHTABLE = "hashtable";
    private static final String TYPE_STRING = "string";

   
    /** Get the source DatabaseMap
     * <p>This method must be implemented in
     * descendents ans should return the source
     * DabaseMap for sql generation
     */
    public abstract DatabaseMap getDatabaseMap()
        throws Exception;

    /** Ant execute method
     */
    public void execute()
        throws BuildException
    {
        try
        {
            dbmap = getDatabaseMap();
            Document doc = new org.apache.xerces.dom.DocumentImpl();
            createDom(dbmap,doc);
            OutputStream out = new FileOutputStream(getXmlfile());
            SerializerFactory sf = SerializerFactory.getSerializerFactory(Method.XML);
            OutputFormat format =  new OutputFormat();
            format.setIndenting(true);
            Serializer ser = sf.makeSerializer(out,format);;
            ser.asDOMSerializer().serialize(doc);
        }
        catch (Exception e)
        {
            e.printStackTrace();
            throw new BuildException(e.toString());
        }
    }

    /** Build the document
     */
    public void createDom(DatabaseMap dbmap, Document doc)
    {
        Element root = doc.createElement(DATABASE);
        doc.appendChild(root);
        TableMap[] tMaps = dbmap.getTables();
        for (int i=0; i<tMaps.length; i++ )
        {
            tmap = tMaps[i];
            writeTable( tmap, root, doc );
        }
    }

    public void writeTable(TableMap tmap, Element dbel, Document doc)
    {
        Element root = doc.createElement(TABLE);
        dbel.appendChild(root);
        root.setAttribute(NAME, tmap.getName());
        ColumnMap[] cMaps = tmap.getColumns();
        for( int i=0; i< cMaps.length; i++)
        {
            cmap = cMaps[i];
            writeColumn(cmap, root, doc );
        }
    }

    public void writeColumn(ColumnMap cmap, Element tel, Document doc)
    {
        Element root = doc.createElement(COLUMN);
        tel.appendChild(root);

        root.setAttribute(NAME, cmap.getColumnName());
        root.setAttribute(TYPE, getType(cmap));
        String type;
        

        if( cmap.isNotNull() )
            root.setAttribute(NOTNULL, "true");

        if( cmap.isForeignKey() )
        {
            root.setAttribute(FOREIGN_TABLE_NAME, cmap.getRelatedTableName());
            root.setAttribute(FOREIGN_COLUMN_NAME, cmap.getRelatedColumnName());
        }
        else if( cmap.isPrimaryKey() )
        {
            String method = PK_NONE;
            String pkm = tmap.getPrimaryKeyMethod();
            if( pkm.equals(  TableMap.AUTOINCREMENT ) )
                method = PK_AUTOINCREMENT;
            else if (pkm.equals( TableMap.SEQUENCE ) )
                method = PK_SEQUENCE;
            else if (pkm.equals( TableMap.IDBROKERTABLE ) )
                method = PK_IDBROKERTABLE;
            root.setAttribute(PRIMARY_KEY,method);
        }
        
    }

    private String getType(ColumnMap cmap)
    {
        Object obj = cmap.getType();
        if( obj instanceof Integer )
            return TYPE_INTEGER;
        else if( obj instanceof java.util.Date )
            return TYPE_DATE;
        else if( obj instanceof java.util.Hashtable )
            return TYPE_HASHTABLE;
        else
            return TYPE_STRING;
    }


    
    /**
       * Get the value of xmlfile.
       * @return Value of xmlfile.
       */
    public String getXmlfile() {return xmlfile;}
    
    /**
       * Set the value of xmlfile.
       * @param v  Value to assign to xmlfile.
       */
    public void setXmlfile(String  v) {this.xmlfile = v;}
    
}
        
