package org.apache.turbine.tools.torque;

/*
 * Copyright (c) 1997-1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

import java.io.*;
import java.util.*;
import java.lang.*;

import org.apache.turbine.util.db.map.TableMap;
import org.apache.turbine.util.db.map.ColumnMap;
import org.apache.turbine.util.db.map.DatabaseMap;
import org.apache.turbine.util.db.map.MapBuilder;

// Ant Stuff
//import org.apache.tools.ant.BuildException;
//import org.apache.tools.ant.Task;

import java.util.*;
import org.apache.turbine.util.db.map.DatabaseMap;

/**
 * This class generates a Peer-based object model from an XML
 * database schema. It does so by first creating a
 * DatabaseMap from the XML database schmea and subsequently
 * uses the DatabaseMap to generate the required classes
 * for a Peer-based object model for a Turbine project.
 *
 *  @author <a href="mailto:jvanzyl@periapt.com">Jason van Zyl</a>
 *  @version $Id: XMLSchemaToObjectModel.java,v 1.1 2000/08/30 21:08:46 bschneid Exp $
 * 
 * I just combined this with the BaseOMGenerator.
 * 
 *  This class is what is used to generate MapBuilders from a DatabaseMap.
 *  <p>It should be extended in order to get the DatabaseMap object
 *  which will be used for generation.
 *  <p>One have to implement the <code>getDatabaseMap()</code> method
 *  in order to give the code to the generator the right databasemap.
 *  Then, this class goes over
 *  that DatabaseMap and outputs the MapBuilder files.
 *  <p>This descendents of this class are used as a task in Ant to generate the map.
 *
 *  @author <a href="mailto:jon@collab.net">Jon S. Stevens</a>
 *  @author <a href="mailto:mbryson@mont.mindspring.com">Dave Bryson</a>
 *  @author <a href="mailto:leon@opticode.co.za">Leon Messerschmidt</a>
 *  @author <a href="mailto:gjritter@codespin.org">Greg Ritter</a>
 */
public class XMLSchemaToObjectModel
{
    
    /** the database map */
    private DatabaseMap dbMap = null;
    /** used to put data into the map */
    private static String string = new String("");
    /** used to put data into the map */
    private static Integer integer = new Integer(0);
    /** used to put data into the map */
    private static java.util.Date date = new Date();

    // Properties set in the XML file
    /** The directory where the java files will be created */
    private static String rootdir = "";
    /** The SQL file to use */
    private static String sqlfile = "";
    /** The package name */
    private static String packageName = "";
    /** The class extended */
    private static String extend = "";
    /** The class prefix used */
    private static String prefix = "";
    /** The class suffix used */
    private static String suffix = "";
    /** The Mapname */
    private static String mapname = "";
    /** The License header */
    private static String licenseFile = "";
    
    /** torque properties (torque.props) */
    protected Properties props;
    /** torque macros (torque.macros) */
    protected Properties macros;
    /** Name of XML database schema to use */
    protected String xmlSchema;
    /** Name of SQL schema to use */
    protected String sqlSchema;

    /** default constructor */
    public XMLSchemaToObjectModel()
    {
        super();
    }

    /**
      * Method invoked to produce the object model
      * for the give XML database schema.
      */
    public void execute()
    {
        setProperties();
    
        try
        {
            dbMap = getDatabaseMap();
            createDatabaseMapClasses(dbMap);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    public void executeSQL()
    {
        setProperties();
        
        try
        {
            dbMap = SQLToDatabaseMap.generateMap(mapname, new FileReader(sqlSchema));
            createDatabaseMapClasses(dbMap);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    /** Set properties for the generation of the OM classes */
    public void setProperties()
    {
        xmlSchema = props.getProperty("xmlSchema");
        sqlSchema = props.getProperty("sqlSchema");
        setRootdir(props.getProperty("outputDir") + "/");
        setLicenseFile(null);
        setExtend(props.getProperty("extend"));
        setMapname(props.getProperty("mapname"));
        setSuffix(props.getProperty("suffix"));
        setPackage(props.getProperty("package"));
    }

    /**
      * Set the torque properties file.
      */
    public void setProperties(Properties props)
    {
        this.props = props;
    }

    /**
      * Set the torque macros file.
      */
    public void setMacros(Properties macros)
    {
        this.macros = macros;
    }

    /** Create the DatabaseMap from the XML file
        */
    public DatabaseMap getDatabaseMap() throws Exception
    {
        XMLSchemaToDatabaseMap mapper =
                new XMLSchemaToDatabaseMap(xmlSchema);
        return mapper.getDatabaseMap(getMapname());
    }

    /**
        * For Testing only
        */
    public static void main(String[] args)
    {
        XMLSchemaToObjectModel mapper = new XMLSchemaToObjectModel();
    }

    /** This method should be implemented to provide
     * the code generator with the right DataBaseMap
     * @returns the DatabaseMap
     */
    //public abstract DatabaseMap getDatabaseMap() throws Exception;

    /**
     * Get the value of mapname.
     * @return Value of mapname.
     */
    public String getMapname()
    {
        return mapname;
    }

    /**
       * Set the value of mapname.
       * @param v  Value to assign to mapname.
       */
    public void setMapname(String v)
    {
        this.mapname = v;
    }

    /**
       * Get the value of suffix.
       * @return Value of suffix.
       */
    public String getSuffix()
    {
        return suffix;
    }

    /**
       * Set the value of suffix.
       * @param v  Value to assign to suffix.
       */
    public void setSuffix(String v)
    {
        this.suffix = v;
    }

    /**
       * Get the value of prefix.
       * @return Value of prefix.
       */
    public String getPrefix()
    {
        return prefix;
    }

    /**
       * Set the value of prefix.
       * @param v  Value to assign to prefix.
       */
    public void setPrefix(String v)
    {
        this.prefix = v;
    }

    /**
       * Get the value of extends.
       * @return Value of extends.
       */
    public String getExtend()
    {
        String extend = (this.extend != null ? this.extend.trim() : "");
        if (extend.equals(""))
        {
            return " implements MapBuilder";
        }
        else
        {
            return " extends " + extend;
        }
    }

    /**
       * Set the value of extends.
       * @param v  Value to assign to extends.
       */
    public void setExtend(String v)
    {
        this.extend = v;
    }

    /**
       * Get the value of package.
       * @return Value of package.
       */
    public String getPackage()
    {
        return packageName;
    }

    /**
       * Set the value of package.
       * @param v  Value to assign to package.
       */
    public void setPackage(String v)
    {
        this.packageName = v;
    }

    /**
       * Get the value of rootdir.
       * @return Value of rootdir.
       */
    public String getRootdir()
    {
        return rootdir;
    }

    /**
       * Set the value of rootdir (which should supply a trailing slash).
       * @param v  Value to assign to rootdir.
       */
    public void setRootdir(String v)
    {
        this.rootdir = v;
    }

    /**
       * Get the value of licenseFile.
       * @return Value of licenseFile.
       */
    public String getLicenseFile()
    {
        return licenseFile;
    }

    /**
       * Set the value of licenseFile.
       * @param v  Value to assign to licenseFile.
       */
    public void setLicenseFile(String v)
    {
        this.licenseFile = v;
    }

    /**
          creates the database map classes. not thread safe at all.
      */
    public void createDatabaseMapClasses(DatabaseMap dm) throws Exception
    {

        TableMap[] tMaps = dbMap.getTables();
        System.out.println("createData " + tMaps.length);
        for (int i = 0; i < tMaps.length; i++)
        {
            TableMap tmap = tMaps[i];
            String pth = rootdir + getPackageAsPath();
            System.out.println("Parsed path..." + pth);

            // Need to parse apart path and make each directory recurisivly

            // Make top dir
            new File (pth).mkdirs();
            // Make map dir
            new File (pth + "map").mkdirs();
            // Make peer dir
            new File (pth + "peer").mkdirs();
            writeMaps(tmap);
            writePeers(tmap);
            writeDOs(tmap);
        }

    }

    /**
          give me just the column name out of a
          TABLE.COLUMN type of string
      */
    private String getColumnName(String fullname)
    {
        return fullname.substring(fullname.indexOf(".") + 1);
    }
    /**
          returns the type of object foo. if it doesn't know
          what it is, then it makes it "string"
      */
    private String getType(Object foo)
    {
        String type = "string";
        if (foo instanceof String)
        {
            type = "string";
        }
        else if (foo instanceof Integer)
        {
            type = "integer";
        }
        else if (foo instanceof Date)
        {
            type = "date";
        }
        return type;
    }

    /**
          returns the type of object foo. if it doesn't know
          what it is, then it makes it "string"
      */
    private String getVillageType(Object foo)
    {
        String type = "string";
        if (foo instanceof String)
        {
            type = "asString()";
        }
        else if (foo instanceof Integer)
        {
            type = "asInt()";
        }
        else if (foo instanceof Date)
        {
            type = "asDate()";
        }
        return type;
    }

    /**
       * Get the Java Type
       */
    private String getJavaType(Object foo)
    {
        String type = "string";
        if (foo instanceof String)
        {
            type = "String";
        }
        else if (foo instanceof Integer)
        {
            type = "int";
        }
        else if (foo instanceof Date)
        {
            type = "Date";
        }
        return type;
    }
    /**
          tell me if i have PREFIX in my string
      */
    private boolean hasPrefix (String data)
    {
        return (data.indexOf(getPrefix()) != -1);
    }
    /**
          removes the PREFIX
      */
    private String removePrefix (String data)
    {
        return data.substring(getPrefix().length());
    }
    /**
          removes the PREFIX
          removes the underscores and makes firstlettercaps

          SCARAB_FOO_BAR becomes FooBar
      */
    private String removeUnderScores (String data)
    {
        String tmp = null;
        StringBuffer out = new StringBuffer();
        if (hasPrefix(data))
        {
            tmp = removePrefix(data);
        }
        else
        {
            tmp = data;
        }

        StringTokenizer st = new StringTokenizer(tmp, "_");
        while (st.hasMoreTokens())
        {
            String element = (String) st.nextElement();
            out.append (firstLetterCaps(element));
        }
        return out.toString();
    }
    /**
          makes the first letter caps and the rest lowercase
      */
    private String firstLetterCaps (String data)
    {
        String firstLetter = data.substring(0, 1).toUpperCase();
        String restLetters = data.substring(1).toLowerCase();
        return firstLetter + restLetters;
    }
    /**
       * Turn the package name into an OS-specific directory name.
       *
       * @return The package name as a path name.
       */
    private String getPackageAsPath()
    {
        return getPackage().replace('.', File.separator.charAt(0)) +
                File.separator;
    }

    /**
       * Write all the classes for Databasemaps
       */
    private void writeMaps(TableMap tmap)
    {
        PrintWriter outFile = null;
        try
        {
            String realTableName = tmap.getName();
            String shortTableName = removeUnderScores(realTableName);
            String className = shortTableName + suffix;

            String fullpath = rootdir + getPackageAsPath() + "map" +
                File.separator + className + ".java";

            System.out.println ("Generating " + fullpath + "...");

            outFile = new PrintWriter(new FileWriter(fullpath),
                true);
            outFile.println("package " + getPackage() + ".map;");
            outFile.println("");
            writeLicense(outFile);
            outFile.println("");
            outFile.println("// JDK classes");
            outFile.println("import java.util.Date;");
            outFile.println("");
            outFile.println("// Turbine classes");
            outFile.println("import org.apache.turbine.util.db.map.MapBuilder;");
            outFile.println("import org.apache.turbine.util.db.map.DatabaseMap;");
            outFile.println("import org.apache.turbine.util.db.map.TableMap;");
            outFile.println("import org.apache.turbine.util.db.pool.DBBroker;");
            outFile.println("");
            outFile.println (
                        "/** This class was autogenerated by XMLSchemaToObjectModel on: " +
                        new Date().toString() + " */");
            outFile.println("public class " + className +
                getExtend());
            outFile.println("{");
            outFile.println("    /** the name of this class */");
            outFile.println(
                        "    public static final String CLASS_NAME = \"" +
                        getPackage() + ".map." + className + "\";");
            outFile.println("");
            outFile.println("    /** " + realTableName + " */");
            outFile.println("    public static String getTable( )");
            outFile.println("    {");
            outFile.println("        return \"" +
                        realTableName + "\";");
            outFile.println("    }");
            outFile.println("");

            ColumnMap[] columns = tmap.getColumns();
            for (int j = 0; j < columns.length; j++)
            {
                ColumnMap cmap = columns[j];
                String realColumnName = getColumnName(cmap.getColumnName());
                String shortColumnName = removeUnderScores(realColumnName);
                outFile.println("    /** " + realTableName + "." +
                realColumnName + " */");
                outFile.println("    public static String get" +
                shortTableName + "_" + shortColumnName + "()");
                outFile.println("    {");
                outFile.println("        return getTable() + \"." +
                realColumnName + "\";");
                outFile.println("    }");
                outFile.println("");
            }

            // If were implementing MapBuilder ourselves we need to add the
            // following implementations.  Otherwise we extend a superclass
            // which we assume is implementing these...
            if (getExtend().indexOf ("implements") != -1)
            {
                outFile.println("    /**  the database map  */");
                outFile.println("    private DatabaseMap dbMap = null;");
                outFile.println("");
                outFile.println("    /**");
                outFile.println("        tells us if this DatabaseMapBuilder is built so that we don't have");
                outFile.println("        to re-build it every time");
                outFile.println("    */");
                outFile.println("    public boolean isBuilt()");
                outFile.println("    {");
                outFile.println("        if ( dbMap != null )");
                outFile.println("            return true;");
                outFile.println("        return false;");
                outFile.println("    }");
                outFile.println("");
                outFile.println("    /**  gets the databasemap this map builder built.  */");
                outFile.println("    public DatabaseMap getDatabaseMap()");
                outFile.println("    {");
                outFile.println("        return this.dbMap;");
                outFile.println("    }");
            }

            // top of the doBuild() method
            outFile.println("    /** the doBuild() method builds the DatabaseMap */");
            outFile.println("    public void doBuild ( ) throws Exception");
            outFile.println("    {");
            outFile.println("        String string = new String(\"\");");
            outFile.println("        Integer integer = new Integer(0);");
            outFile.println("        java.util.Date date = new Date();");

            outFile.println("");
            outFile.println("        dbMap = DBBroker.getInstance().getDatabaseMap(DBBroker.DEFAULT);");
            outFile.println("        dbMap.addTable(getTable());");
            outFile.println("        TableMap tMap = dbMap.getTable(getTable());");
            outFile.println("");

            // generate relationships
            for (int j = 0; j < columns.length; j++)
            {
                ColumnMap cmap = columns[j];
                String type = getType(cmap.getType());
                String realColumnName = cmap.getColumnName();
                String shortColumnName = removeUnderScores(realColumnName);
                String shortRelatedTableName =
                        removeUnderScores(cmap.getRelatedTableName());
                String shortRelatedColumnName = removeUnderScores(
                        getColumnName(cmap.getRelatedColumnName()));
                if (cmap.isForeignKey() && cmap.isPrimaryKey())
                {
                    outFile.println(
                            "        tMap.addForeignPrimaryKey ( get" +
                            shortTableName + "_" + shortColumnName +
                            "(), " + type + ",");
                    outFile.println("            " +
                            shortRelatedTableName + getSuffix() + ".getTable(),");
                    outFile.println("            " +
                            shortRelatedTableName + getSuffix() +
                            ".get" + shortRelatedTableName + "_" +
                            shortRelatedColumnName + "() );");
                }
                else if (cmap.isForeignKey())
                {
                    outFile.println( "        tMap.addForeignKey ( get" +
                            shortTableName + "_" + shortColumnName +
                            "(), " + type + ",");
                    outFile.println("            " +
                            shortRelatedTableName + getSuffix() + ".getTable(),");
                    outFile.println("            " +
                            shortRelatedTableName + getSuffix() +
                            ".get" + shortRelatedTableName + "_" +
                            shortRelatedColumnName + "() );");
                }
                else if (cmap.isPrimaryKey())
                {
                    outFile.println( "        tMap.addPrimaryKey ( get" +
                            shortTableName + "_" + shortColumnName +
                            "(), " + type + ");");
                }
                else
                {
                    outFile.println("        tMap.addColumn ( get" +
                            shortTableName + "_" + shortColumnName +
                            "(), " + type + ");");
                }

                outFile.println("");
            }
            outFile.println("    }");
            outFile.println("");

            outFile.println("}");
        }
        catch (Exception e)
        {
        }
        finally { if (outFile != null)
                outFile.close();
        } }

    /**
       * Write the peers
       */
    private void writePeers(TableMap tmap) throws Exception
    {
        String realTableName = tmap.getName(); // Includes  Suffix  - TURBINE_FOOBAR
        String tableClassname = removeUnderScores(realTableName); // FooBar
        String peerclassName = tableClassname + "Peer"; // FooBarPeer
        String mapclassName = tableClassname + "MapBuilder"; // FooBarMapBuilder
        String subPackage = getPackage() + ".peer"; // org.apache.turbine.peer
        String primaryKey = "?";
        PrintWriter outFile = null;

        String fullpath = rootdir + getPackageAsPath() + "peer/" +
            peerclassName + ".java";
        try
        {
            outFile = new PrintWriter(new FileWriter(fullpath), true);
            outFile.println("package " + subPackage + ";");
            outFile.println("");
            writeLicense(outFile);
            outFile.println("");
            outFile.println("import com.workingdogs.village.*;");
            outFile.println("import org.apache.turbine.om.peer.*;");
            outFile.println("import org.apache.turbine.util.*;");
            outFile.println("import org.apache.turbine.util.db.*;");
            outFile.println("import org.apache.turbine.util.db.map.*;");
            outFile.println("import java.util.*;");
            outFile.println("import " + getPackage() + ".map.*;"); // import the package with the DO
            outFile.println("import " + getPackage() + ".*;"); // import the package with the DO
            outFile.println("");
            outFile.println (
                    "/** This class was autogenerated by XMLSchemaToObjectModel on: " +
                    new Date().toString() + " */");
            outFile.println("public class " + peerclassName + " extends BasePeer");
                    outFile.println("{");
                    outFile.println("    /** the mapbuilder for this class */");
                    outFile.println("    private static final " +
                        mapclassName + " mapBuilder = (" +
                        mapclassName + ")getMapBuilder("+
                        mapclassName + ".CLASS_NAME);");
                    outFile.println("");
                    outFile.println("    /** the table name for this class */");
                    outFile.println("    public static final String TABLE_NAME = mapBuilder.getTable();");
                    outFile.println("");

                    // Make the static field accessors
                    ColumnMap[] columns = tmap.getColumns();
                    for (int j = 0; j < columns.length; j++)
            {
                ColumnMap cmap = columns[j];
                String realColumnName = getColumnName(cmap.getColumnName());
                String shortColumnName = removeUnderScores(realColumnName);
                if (cmap.isPrimaryKey())
                {
                    primaryKey = realColumnName;
                }
                outFile.println("    /** the column name for the " +
                        realColumnName + " id field */");
                outFile.println("    public static final String " +
                        realColumnName + " = mapBuilder.get" +
                        tableClassname + "_" + shortColumnName + "();");

            }
            outFile.println("");

            // Write the Insert Method
            outFile.println("    /** Method to do inserts */ ");
            outFile.println("    public static Object doInsert( Criteria criteria ) throws Exception");
            outFile.println("    {");
            //            outFile.println( "        criteria.setMapName (TABLE_NAME);");
            outFile.println("        return BasePeer.doInsert( criteria );");
            outFile.println("    }");
            outFile.println("");

            // Write the doSelect
            outFile.println("    /** Method to do selects */ ");
            outFile.println("    public static Vector doSelect( Criteria criteria ) throws Exception");
            outFile.println("    {");
            //            outFile.println( "        criteria.setMapName (TABLE_NAME);");
            for (int j = 0; j < columns.length; j++)
            {
                ColumnMap cmap = columns[j];
                String realColumnName = getColumnName(cmap.getColumnName());
                outFile.println("        criteria.addSelectColumn( " +
                        realColumnName + " );");
            }
            outFile.println("");
            outFile.println("        // BasePeer returns a Vector of Value (Village) arrays.  The array");
            outFile.println("        // order follows the order columns were placed in the Select clause.");
            outFile.println("        Vector rows = BasePeer.doSelect(criteria);");
            outFile.println("        Vector results = new Vector();");
            outFile.println("");

            outFile.println("        // populate the object(s)");
            outFile.println("        for ( int i=0; i<rows.size(); i++ )");
            outFile.println("        {");
            outFile.println("           " +tableClassname + " obj = new "+
                    tableClassname + "();");
            outFile.println("            Record row = (Record)rows.elementAt(i);");

            for (int j = 0; j < columns.length; j++)
            {
                ColumnMap cmap = columns[j];
                //String Fieldname = firstLetterCaps( getColumnName(cmap.getColumnName()) );
                String Fieldname = removeUnderScores (
                        getColumnName(cmap.getColumnName()));
                String type = getVillageType(cmap.getType());
                outFile.println("            obj.set"+Fieldname + "( row.getValue("+
                        (j + 1) + ")."+type + " );");
            }

            outFile.println("            results.addElement( obj );");
            outFile.println("         }");
            outFile.println("         return results;");
            outFile.println("    }");

            outFile.println("");

            // Write the Update
            outFile.println("    /**");
            outFile.println("     * @param Criteria object containing data that is used to create the UPDATE statement.");
            outFile.println("     */");
            outFile.println("     public static void doUpdate(Criteria criteria) throws Exception");
            outFile.println("     {");
            //            outFile.println( "         criteria.setMapName (TABLE_NAME);");
            outFile.println("         Criteria selectCriteria = new Criteria(2);");
            //            outFile.println( "         selectCriteria.setMapName (TABLE_NAME);" );
            outFile.println("         selectCriteria.put( " +
                    primaryKey + ", criteria.remove("+primaryKey + ") );");
            outFile.println("         BasePeer.doUpdate( selectCriteria, criteria );");
            outFile.println("     }");
            outFile.println("");

            // Write the Delete
            outFile.println("    /**");
            outFile.println("     * @param Criteria object containing data that is used DELETE from database.");
            outFile.println("     */");
            outFile.println("     public static void doDelete(Criteria criteria) throws Exception");
            outFile.println("     {");
            //            outFile.println( "         criteria.setMapName (TABLE_NAME);");
            outFile.println("         BasePeer.doDelete ( criteria );");
            outFile.println("     }");
            outFile.println("");


            // Write the Insert(DO) Method
            outFile.println("    /** Method to do inserts */");
            outFile.println("    public static Object doInsert( " +
                    tableClassname + " obj ) throws Exception");
            outFile.println("    {");
            outFile.println("        return " + peerclassName + ".doInsert(buildCriteria(obj));");
                    outFile.println("    }");
                    outFile.println("");

                    // Write the buildCriteria Method
                    outFile.println("    /** Build a Criteria object from the data object for this peer */");
                    outFile.println(
                            "    public static Criteria buildCriteria( " +
                            tableClassname + " obj )");
                    outFile.println("    {");
                    outFile.println("        Criteria criteria = new Criteria();");
                    for (int j = 0; j < columns.length; j++)
            {
                ColumnMap cmap = columns[j];
                String realColumnName = getColumnName(cmap.getColumnName());
                String shortColumnName = removeUnderScores(realColumnName);
                // this only supports integer primary keys right now
                if (cmap.isPrimaryKey())
                {
                    outFile.println("        if( obj.get" +
                            shortColumnName + "() > 0 )");
                    outFile.print("    ");
                }
                outFile.println("        criteria.add( " +
                        peerclassName + "." + realColumnName +
                        ", obj.get" + shortColumnName + "() );");
                    }
                    outFile.println("        return criteria;");
                    outFile.println("    }");
                    outFile.println("");


                    outFile.println("");

                    // Write the Update(DO)
                    outFile.println("    /**");
                    outFile.println("     * @param obj the data object to update in the database.");
                    outFile.println("     */");
                    outFile.println( "    public static void doUpdate(" +
                            tableClassname + " obj) throws Exception");
                    outFile.println("    {");
                    outFile.println("        " + peerclassName + ".doUpdate(buildCriteria(obj));");
                    outFile.println("    }");

                    // Write the Delete(DO)
                    outFile.println("    /**");
                    outFile.println("     * @param obj the data object to delete in the database.");
                    outFile.println("     */");
                    outFile.println( "    public static void doDelete(" +
                            tableClassname + " obj) throws Exception");
                    outFile.println("    {");
                    outFile.println("        " + peerclassName + ".doDelete(buildCriteria(obj));");
                    outFile.println("    }");

                    outFile.println("");
                    outFile.println("}");
                }
                catch (Exception e)
        {
        }
        finally { if (outFile != null)
                outFile.close();
        } }

    /**
       * Write the DataObjects
       */
    private void writeDOs(TableMap tmap)
    {
        String realTableName = tmap.getName(); // Includes  Suffix  - TURBINE_FOOBAR
        String tableClassname = removeUnderScores(realTableName); // FooBar
        String subPackage = getPackage(); // org.apache.turbine
        PrintWriter outFile = null;

        String fullpath =
                rootdir + getPackageAsPath() + tableClassname + ".java";
        try
        {
            outFile = new PrintWriter(new FileWriter(fullpath), true);
            outFile.println("package " + subPackage + ";");
            outFile.println("");
            writeLicense(outFile);
            outFile.println("");
            outFile.println("// Turbine");
            outFile.println("import org.apache.turbine.om.BaseObject;");
            outFile.println("import java.util.*;");
            outFile.println("");
            outFile.println (
                    "/** This class was autogenerated by XMLSchemaToObjectModel on: " +
                    new Date().toString() + " */");
            outFile.println("public class " + tableClassname + " extends BaseObject");
            outFile.println("{");

            // Make the fields
            ColumnMap[] col1 = tmap.getColumns();
            for (int j = 0; j < col1.length; j++)
            {
                ColumnMap cmap = col1[j];
                String type = getJavaType(cmap.getType());

                String realColumnName = getColumnName(cmap.getColumnName());
                String field =
                        removeUnderScores(realColumnName).toLowerCase();

                outFile.println("    /** the value for the " + field +
                        " field */");
                outFile.println("    private " + type + " " + field + ";");

            }
            outFile.println("");

            // Make the accessors
            ColumnMap[] col2 = tmap.getColumns();
            for (int j = 0; j < col2.length; j++)
            {
                ColumnMap cmap = col2[j];
                String realColumnName = getColumnName(cmap.getColumnName());
                String field = removeUnderScores(realColumnName);
                String type = getJavaType(cmap.getType());
                String fieldlower = field.toLowerCase();

                // Get
                outFile.println("    /**");
                outFile.println("     * Get the " + fieldlower);
                outFile.println("     * @return " + type);
                outFile.println("     */");
                outFile.println("     public " + type + " get" +
                        field + "()");
                outFile.println("     {");
                outFile.println("          return " + fieldlower + ";");
                outFile.println("     }");

                outFile.println("");

                // Set
                outFile.println("    /**");
                outFile.println("     * Set the value of " + fieldlower);
                outFile.println("     * @param " + type);
                outFile.println("     */");
                outFile.println("     public void set" + field + "(" +
                        type + " v )");
                outFile.println("     {");
                outFile.println("          this." + fieldlower + " = v;");
                outFile.println("     }");

                outFile.println("");

            }

            outFile.println("}");

        }
        catch (Exception e)
        {
        }
        finally { if (outFile != null)
                outFile.close();
        } }

    /**
       * Write the license header
       */
    private void writeLicense(PrintWriter of) throws Exception
    {
        String filename = getLicenseFile();

        if (filename != null && filename.length() > 0)
        {
            BufferedReader br =
                    new BufferedReader(new FileReader(getLicenseFile()));
            /** the current line */
            String line = null;
            /** loop over every line in the file */
            while ((line = br.readLine()) != null)
            {
                of.println(line);
            }
        }
        else
        {
            of.println("");
        }
    }
}
