package org.apache.turbine.util;

import java.util.*;

/**
Used for adding and accessing messages that relate to a specific form and field. 
Allows to query for messages by form name and field name.
Used together with FormMessage class.

@author <a href="mailto:neeme@one.lv">Neeme Praks</a>
@version $Id: FormMessages.java,v 1.1 2000/08/30 21:08:47 bschneid Exp $
*/
public class FormMessages
{
    private Hashtable forms_messages;
    private Hashtable fields_messages;
    private Hashtable messages_fields;
    private Hashtable forms_fields;

    public FormMessages()
    {
        forms_messages = new Hashtable();
        fields_messages = new Hashtable();
        messages_fields = new Hashtable();
        forms_fields = new Hashtable();
    }

    public void setMessage(String formName, String fieldName, long returnCode)
    {
        setMessage(formName, fieldName, String.valueOf(returnCode) );
    }

    public void setMessage(String formName, String fieldName, String messageName)
    {
        fieldName = formName + "-" + fieldName;
        addValue(forms_messages, formName, messageName);
        addValue(fields_messages, fieldName, messageName);
        addValue(messages_fields, messageName, fieldName);
        addValue(forms_fields, formName, fieldName);
    }

    private void addValue(Hashtable table, String key, String value)
    {
        Vector values;

        if (!table.containsKey(key))
        {
            values = new Vector();
            values.addElement(value);
            table.put(key, values);
        }
        else
        {
            values = ((Vector) table.get(key));
            if (!values.contains(value))
                values.addElement(value);
        }
    }

    private Vector getValues(Hashtable table, String key)
    {
        if (table.containsKey(key))
            return (Vector) table.get(key);
        else
            return null;
    } 

    public FormMessage[] getFormMessages(String formName)
    {
        Vector messages, fields;
        String messageName, fieldName;
        messages = getValues(forms_messages, formName);
        if (messages != null)
        {
            FormMessage[] result = new FormMessage[messages.size()];
            for (int i = 0; i < messages.size(); i++)
            {
                result[i] = new FormMessage( formName );
                messageName = (String) messages.elementAt(i);
                result[i].setMessage( messageName );
                fields = getValues(messages_fields, messageName);
                for (int j = 0; j < fields.size(); j++)
                {
                    fieldName = (String) fields.elementAt(j);
                    if (formHasField( formName, fieldName ))
                    {
                        result[i].setFieldName( fieldName );
                    }
                }
            }
            return result;
        }
        return null;
    }

    private boolean formHasField(String formName, String fieldName)
    {
        Vector fields;
        fields = getValues(forms_fields, formName);
        for (int i = 0; i < fields.size(); i++)
        {
            if (fieldName == (String) fields.elementAt(i))
            {
                return true;
            }
        }
        return false;
    }
}