package org.apache.turbine.util;

/*
 * Copyright (c) 1997-1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.util.Date;
import java.util.Vector;
import java.io.*;

// adapted for ACS by Bill Schneider (bschneid@arsdigita.com)

import com.arsdigita.acs.*;

// import org.apache.turbine.services.resources.*;

/**
 * This is a utility class that is used for logging events.  Entries are
 * allowed in 3 levels - Notice, Warning, Error.
 * Methods are also available to log exceptions, which are always logged
 * at the Error level.
 *<p>
 * The path to a writable file must be given in TurbineResources.properties
 * for the log to be written. If the file does not already exist, then the
 * file will be created. If the path is invalid, then the file will not be
 * created and nothing will happen.
 *<p>
 * To use this class outside of Turbine, all you need to do is set the path
 * to the TurbineResources.properties file first. To do this, you would write
 * code like this:
 *<p>
 * TurbineResources.setPropertiesFileName ("/path/to/TurbineResources.properties");<br>
 * Log.note ("this is my notice!");
 *
 * @author John D. McNally
 * @author Jon S. Stevens <a href="mailto:jon@clearink.com">jon@clearink.com</a>
 * @author John Thorhauer
 */
public class Log
{
    private static String lf = System.getProperty("line.separator", "\n");

    // true if filename is valid and writeable
    private static boolean isValid = true;
	// filename for log file
	private static FileWriter logfile = null;

	// Levels - Notice, Warning, Error
	private static final String NOTICE = "NOTICE ";
	private static final String WARNING = "WARNING";
	private static final String ERROR = " ERROR ";

    static
    {
        init();
    }

	/**
	 * checks to see if log variables are set up properly and
     * the initializes the log(s)
	 */
	private static void init()
	{
		try
		    {
            String logFileString = Parameters.get("LogFile", null);

            if (logFileString == null)
            {
                throw new Exception (
                "The LogFile parameter cannot be found.  Please make sure that you have defined it in the acs.parameters file.\n");
            }

            if (logFileString != null)
            {
                logfile = new FileWriter( logFileString, true );
            }

		    }
        catch (Exception e)
        {
            e.printStackTrace();
            isValid = false;
        }
		catch (Throwable t)
		{
            isValid = false;
		}
	}

    /*
     * Makes a log entry at the NOTICE level
     *
     * @returns true if entry was entered, false otherwise
     */
	public static boolean note(String description)
	{
		return log(NOTICE, description);
	}

    /*
     * Makes a log entry at the WARNING level
     *
     * @returns true if entry was entered, false otherwise
     */
  	public static boolean warn(String description)
	{
		return log(WARNING, description);
	}

    /*
     * Makes a log entry at the ERROR level
     *
     * @returns true if entry was entered, false otherwise
     */
	public static boolean error(String description)
	{
   System.out.print("Error=" + description + "\n");
		return log(ERROR, description);

	}

    /*
     * Makes a log entry at the ERROR level
     *
     * @returns true if entry was entered, false otherwise
     */
	public static boolean error(String description, Throwable t)
	{
		return log(ERROR, description, t);
	}

    /*
     * Makes a log entry at the ERROR level
     *
     * @returns true if entry was entered, false otherwise
     */
 	public static boolean error(Throwable t)
	{
		return log(ERROR, "", t);
	}

    private static boolean log(String level, String description)
    {
        return log(level, description, null);
    }

 	private static boolean log(String level, String description, Throwable t)
 	{
        if ( isValid )
        {
            Date date = new Date();
            StringBuffer logEntry = new StringBuffer();
            logEntry.append ( "[" );
            logEntry.append ( date.toString() );
            logEntry.append ( "] -- " );
            logEntry.append ( level );
            logEntry.append ( " -- " );
            logEntry.append ( description );
            logEntry.append ( lf );
            if (t != null)
            {
                ByteArrayOutputStream ostr = new ByteArrayOutputStream();
                PrintWriter out = new PrintWriter(ostr,true);
                out.write(logEntry.toString());
                out.write("\tException:  ");
                out.write(t.toString());
                out.write(lf+"\tStack Trace follows:"+lf+"\t");
                t.printStackTrace(out);
                logEntry = new StringBuffer( ostr.toString() );
            }
            synchronized( logfile )
            {
                try
                {
                    logfile.write( logEntry.toString(), 0, logEntry.length());
                    logfile.flush();
                }
                catch(IOException ioe)
                {
                    // ignore, skip log, should not occur since we have checked file
                    // but catching the exception allows Log to be used elsewhere in
                    // catch blocks.
                    return false;
                }
            }
            return true;
        }
		return false;
	}

    /**
        Attempt to close the log files when the class is GC'd
    */
    public void destroy()
    {
        try
        {
            if (logfile != null)
            {
                logfile.close();
            }
        }
        catch (Exception e)
        {}
    }
}
