package org.apache.turbine.util;

/*
 * Copyright (c) 1997-1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

import org.apache.turbine.om.security.*;

/**
 * Utility for doing security checks in Screens and Actions.
 * 
 * Sample usage: <br>
 * <code>
 * SecurityCheck mycheck = new SecurityCheck(data, "Unauthorized to do this!", "WrongPermission");
 * if ( !mycheck.hasPermission("add_user");
 *  return;
 *</code>
 * 
 * @author Dave Bryson
 */
public class SecurityCheck
{
    
    private String message;
    private String failScreen;
    private RunData data = null;
    
    /**
        Constructor. The message is the message to display upon failure.
        The failedScreen is the screen to redirect to upon failure.
    */    
    public SecurityCheck(RunData data, String message, String failedScreen)
    {
        this.data=data;
        this.message=message;
        this.failScreen=failedScreen;
 	}
 
 	/**
        Does the user have this role?
    */
    public boolean hasRole(Role role) throws Exception
    {
        boolean value = false;        
        if ( data.getACL() == null || !data.getACL().hasRole(role) )
        {
            data.setScreen(failScreen);
            data.setMessage(message);
        }
        else
        {
            value = true;
        }
        return value;
    }
    
    /**
        Does the user have this role?
    */
    public boolean hasRole(String role) throws Exception
    {
        boolean value = false;
        if ( data.getACL() == null || !data.getACL().hasRole(role) )
        {
            data.setScreen(failScreen);
            data.setMessage(message);
        }
        else
        {
            value = true;
        }
        return value;
    }
		
    /**
        Does the user have this permission?
    */
    public boolean hasPermission(Permission permission) throws Exception
    {
        boolean value = false;
        if ( data.getACL() == null || !data.getACL().hasPermission(permission) )
        {
            data.setScreen(failScreen);
            data.setMessage(message);
        }
        else
        {
            value = true;
        }
        return value;
    }
    
    /**
        Does the user have this permission?
    */
    public boolean hasPermission(String permission) throws Exception
    {
        boolean value = false;
        if ( data.getACL() == null || !data.getACL().hasPermission(permission) )
        {
            data.setScreen(failScreen);
            data.setMessage(message);
        }
        else
        {
            value = true;
        }
        return value;
    }
    
    /**
        Get the message that should be displayed. This is initialized
        in the constructor.
    */
    public String getMessage()
    {
        return message;
    }
    
    /**
        Get the screen that should be displayed. This is initialized
        in the constructor.
    */
    public String getFailScreen()
    {
        return failScreen;
    }    
}