package org.apache.turbine.util.db.map;

/*
 * Copyright (c) 1997-1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

/**
 * ColumnMap is used to model a column of a table in a database.
 *
 * @author John D. McNally
 */
public class ColumnMap
{
    /** type of the column */
    private Object type = null;
    /** size of the column */
    private int size = 0;
    /** is it a primary key? */
    private boolean pk = false;
    /** is null value allowed ?*/
    private boolean notNull = false;
    /** name of the table that this column is related to */
    private String relatedTableName = "";
    /** name of the column that this column is related to */
    private String relatedColumnName = "";
    /** the TableMap for this column */
    private TableMap table;
    /** the name of the column */
    private String columnName;

    /**
        Constructor.
        @params name the name of the column
        @params containingTable TableMap of the table this column is in.
    */
    public ColumnMap( String name, TableMap containingTable )
    {
        this.columnName = name;
        table = containingTable;
    }
    /**
        Get the name of a column
    */
    public String getColumnName()
    {
        return columnName;
    }
    /**
        Get the table name + column name
    */
    public String getFullyQualifiedName()
    {
        return table.getName() + "." + columnName;
    }
    /**
        Get the name of the table this column is in
    */
    public String getTableName()
    {
        return table.getName();
    }  
    /**
        set the type of this column
    */
    public void setType ( Object type )
    {
        this.type = type;
    }
    /**
        set the size of this column
    */
    public void setSize( int size )
    {
        this.size = size;
    }
    /**
        set if this column is a primary key or not
    */
    public void setPrimaryKey( boolean pk )
    {
        this.pk = pk;
    }
    /**
       set if this column may be null
    */
    public void setNotNull( boolean nn )
    {
        this.notNull = nn;
    }
    /**
        set the foreign key for this column
        @params fullyQualifiedName the name of the table.column that is foreign
    */
    public void setForeignKey(String fullyQualifiedName )
    {
        if ( fullyQualifiedName != null && fullyQualifiedName.length() > 0)
        {
            relatedTableName = fullyQualifiedName.substring( 0, fullyQualifiedName.indexOf('.') );
            relatedColumnName = fullyQualifiedName.substring( fullyQualifiedName.indexOf('.')+1 );
        }
        else
        {
            relatedTableName = "";
            relatedColumnName = "";
        }
    }
    /**
        set the foreign key for this column
        @params tableName the name of the table that is foreign        
        @params columnName the name of the column that is foreign        
    */
    public void setForeignKey(String tableName, String columnName)
    {
        if ( tableName != null && tableName.length() > 0
            && columnName != null && columnName.length() > 0 )
        {
            relatedTableName = tableName;
            relatedColumnName = columnName;
        }
        else
        {
            relatedTableName = "";
            relatedColumnName = "";
        }
    }
    /**
        get the type of this column
    */
    public Object getType()
    {
        return type;
    }
    /**
        get the size of this column
    */
    public int getSize()            
    {
        return size;
    }
    /**
        is this column a primary key?
    */
    public boolean isPrimaryKey()            
    {
        return pk;
    }
    /**
       is null value allowed ?
    */
    public boolean isNotNull()
    {
        return (notNull || isPrimaryKey());
    }
    /**
        is this column a foreign key?
    */
    public boolean isForeignKey()            
    {
        if ( relatedTableName != null && relatedTableName.length() > 0)
            return true;
        else
            return false;
    }
    /**
        get the table.column that this column is related to
    */
    public String getRelatedName()
    {
        return relatedTableName+"."+relatedColumnName;
    }
    /**
        get the table name that this column is related to
    */
    public String getRelatedTableName()
    {
        return relatedTableName;
    }
    /**
        get the column name that this column is related to
    */
    public String getRelatedColumnName()
    {
        return relatedColumnName;
    }
}
