package org.apache.turbine.util.db.pool;

/*
 * Copyright (c) 1997-1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.sql.*;
import java.util.*;

import org.apache.turbine.util.Log;

/**
 * This class wraps the JDBC <code>Connection</code> class, providing built-in 
 * exception logging to stderr and a timestamp which indicates how old the 
 * <code>Connection</code> is.  <code>DBConnection</code>'s timestamp is used 
 * by the <code>ConnectionPool</code> to expire connections when they get too 
 * old.
 *
 * @author Frank Y. Kim
 * @author John D. McNally
 */

public class DBConnection
{
    /** The JDBC database connection. */
    private Connection connection = null;
    /** The URL of this connection. */
    private String url = null;
    /** The time in milliseconds at which the connection was created. */
    private long timestamp;

    /**
     * Creates a Turbine <code>DBConnection</code> with the specified 
     * attributes.
     *
     * @param connection The JDBC connection to wrap.
     * @param url        The URL we're connecting to.
     */
    protected DBConnection(Connection connection, String url)
    {
        this.connection = connection;
        this.url = url;
        this.timestamp = System.currentTimeMillis();
    }

    /**
     * Returns a JDBC connection.
     *
     * @return The database connection.
     */
    public Connection getConnection()
    {
        return connection;
    }

    /**
     * Returns a long representing the time this connection was created.
     *
     * @return The time in milliseconds that this connection was created.
     */
    public long getTimestamp()
    {
        return timestamp;
    }

    public String getUrl()
    {
        return url;
    }

    public void commit()
    {
        try
        {
            connection.commit();
        }
        catch(SQLException e)
        {
            e.printStackTrace();
            Log.error(e);
        }
    }

    public void rollback()
    {
        try
        {
            connection.rollback();
        }
        catch(SQLException e)
        {
            e.printStackTrace();
            Log.error(e);
        }
    }

    public void setAutoCommit(boolean b)
    {
        try
        {
            connection.setAutoCommit(b);
        }
        catch(SQLException e)
        {
            e.printStackTrace();
            Log.error(e);
        }
    }

    public java.sql.Statement createStatement()
    {
        java.sql.Statement stmt=null;
        try
        {
            stmt = connection.createStatement();
        }
        catch(SQLException e)
        {
            e.printStackTrace();
            Log.error(e);
        }
        return stmt;
    }
}
