package org.apache.turbine.util.db.pool;

/*
 * Copyright (c) 1997-1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.io.*;
import java.util.*;
import java.sql.*;

import org.apache.turbine.util.Log;

/**
 * This code should be used for an Interbase database pool.
 *
 * @author Frank Conradie <a href="mailto:frank@opticode.co.za">frank@opticode.co.za</a>
*/
public class DBInterbase extends DB
{
    /** This method is used to ignore case */
    public String toUpperCase(String in)
    {
        return new StringBuffer("UPPER(").append(in).append(")").toString();
    }

    /** This method is used to ignore case */
    public String ignoreCase(String in)
    {
        return new StringBuffer("UPPER(").append(in).append(")").toString();
    }
    
    
    /** 
     * This method is used to ignore case in an ORDER BY clause.  Usually it
     * is the same as ignoreCase, but some databases (Interbase for example)
     * does not use the same SQL in ORDER BY and other clauses.
     */
    public String ignoreCaseInOrderBy(String in)
    {
        return in;
    }

    
    /** This method is used to get the string delimiter, usually it is '\'' */
    public char getStringDelimiter()
    {
        return '\'';
    }
    
    /** databases like MySQL will return a result for this, others won't */
    public String getIdSqlForAutoIncrement()
    {
        return null;
    }
    
    /** databases like Oracle will return a result for this, others won't */
    public String getSequenceSql(Object obj)
    {
        return null;
    }
    
    /** The method is used to lock a table. */
    public void lockTable(Connection con, String table) throws SQLException
    {
        Statement statement = con.createStatement();

        StringBuffer stmt = new StringBuffer();
        stmt.append( "SET TRANSACTION " )
        .append( "ISOLATION LEVEL READ COMMITTED " )
        .append( "NO RECORD_VERSION WAIT " )
        .append( "RESERVING " )
        .append( table )
        .append( " FOR PROTECTED WRITE" );

        statement.executeQuery( stmt.toString() );
    }
    
    /** The method is used to unlock a table. */
    public void unlockTable(Connection con, String table) throws SQLException
    {
        con.commit();
    }
    
}
