-- calendar-api.sql
-- PL/SQL functions providing an API for the ArsDigita Calendar

-- @creation-date 2000-08-25
-- @author W. Scott Meeks (smeeks@arsdigita.com)
-- @cvs-id $Id: calendar-api.sql,v 1.1 2000/10/27 16:36:40 deison Exp $

create or replace procedure cal_insert_item_with_ids
	(v_item_id IN integer,
	 v_instance_id IN integer,
	 v_start_date IN date,
	 v_end_date IN date,
	 v_title IN varchar,
	 v_user_id IN integer,
	 v_group_id IN integer,
	 v_description IN varchar,
	 v_html_p IN varchar,
	 v_related_url IN varchar,
	 v_related_url_text IN varchar,
	 v_related_url_p IN varchar,
	 v_creation_user IN integer,
	 v_editable_p IN varchar )
AS
BEGIN
  INSERT INTO cal_items
  (item_id, title, description, html_p,
   related_url, related_url_text, related_url_p,
   creation_user, creation_date, editable_p)
  VALUES
  (v_item_id, v_title, v_description, v_html_p,
   v_related_url, v_related_url_text, v_related_url_p,
   v_creation_user, sysdate, v_editable_p);

  INSERT INTO cal_item_instances
  (item_id, instance_id, start_date, end_date, last_modifying_user)
  VALUES
  (v_item_id, v_instance_id, v_start_date, v_end_date, v_creation_user);

  INSERT INTO cal_item_map 
  (item_id, user_id, group_id)
  VALUES 
  (v_item_id, v_user_id, v_group_id);

END;
.
/
show errors

-- Generates item_id and instance_id, inserts into appropriate tables, 
-- and returns instance_id
create or replace function cal_insert_item 
	(v_start_date IN date,
	 v_end_date IN date,
	 v_title IN varchar,
	 v_user_id IN integer,
	 v_group_id IN integer,
	 v_description IN varchar,
         v_html_p IN varchar,
	 v_related_url IN varchar,
	 v_related_url_text IN varchar,
	 v_related_url_p IN varchar,
	 v_creation_user IN integer,
	 v_editable_p IN varchar )
RETURN INTEGER
AS
  v_item_id INTEGER;
  v_instance_id INTEGER;
BEGIN
  SELECT cal_item_id_sequence.nextval,
	 cal_instance_id_sequence.nextval
  INTO v_item_id, v_instance_id
  FROM DUAL;

  cal_insert_item_with_ids
	(v_item_id, v_instance_id,
	 v_start_date, v_end_date, v_title,
	 v_user_id, v_group_id, v_description, v_html_p,
	 v_related_url, v_related_url_text, v_related_url_p,
	 v_creation_user, v_editable_p);

  RETURN v_instance_id;
END;
.
/
show errors

-- Calls cal_insert_item and adds mapping to given id and table
create or replace function cal_ins_item_on_id_table
	(v_on_which_table IN varchar,
	 v_on_what_id IN integer,
	 v_start_date IN date,
	 v_end_date IN date,
	 v_title IN varchar,
	 v_user_id IN integer,
	 v_group_id IN integer,
	 v_description IN varchar,
	 v_html_p IN varchar,
	 v_related_url IN varchar,
	 v_related_url_text IN varchar,
	 v_related_url_p IN varchar,
	 v_creation_user IN integer,
	 v_editable_p IN varchar )
RETURN INTEGER
AS
  v_instance_id INTEGER;
BEGIN
  v_instance_id := cal_insert_item(
			v_start_date, v_end_date, v_title,
			v_user_id, v_group_id, v_description, v_html_p,
			v_related_url, v_related_url_text, v_related_url_p,
			v_creation_user, v_editable_p);

  INSERT INTO cal_instance_db_row_map
  (instance_id, on_which_table, on_what_id)
  VALUES
  (v_instance_id, upper(trim(v_on_which_table)), v_on_what_id);

  RETURN v_instance_id;
END;
.
/
show errors

create or replace procedure cal_update_item (v_item_id IN integer)
AS
BEGIN
return;
END;
.
/
show errors
