-- ischecker.sql
-- Delphi Enterprise IS Checker
-- by James Buszard-Welcher <james@arsdigita.com>, November 1999
-- mostly synchonized with actual database April 25, 2000 by Dave Abercrombie <abe@arsdigita.com>
-- ischecker.sql,v 3.11 2000/06/26 18:45:00 cmceniry Exp

--
-- SELF MONITORING
--

set scan off

create table is_global_state (
        last_system_reboot	date
);

create table is_test_state (
	test_type		varchar(12) primary key,
	last_starttime		date,
	last_stoptime		date,
	run_count		integer,
	enabled_p		char(1) default 't'
                                   check(enabled_p in ('t','f')),
	run_period_secs		integer
);

INSERT INTO is_global_state (last_system_reboot) VALUES (NULL);

INSERT INTO is_test_state ( test_type ) VALUES ('tcp');
INSERT INTO is_test_state ( test_type ) VALUES ('web');
INSERT INTO is_test_state ( test_type ) VALUES ('ping');
INSERT INTO is_test_state ( test_type ) VALUES ('mail');
-- 'notify' isn't really a test, but we want to track the
-- state of the notify schedule process...
INSERT INTO is_test_state ( test_type ) VALUES ('notify');

create sequence is_mail_run_number start with 1;

create sequence is_test_sequence start with 10000;

create table is_test_proc_log (
	test_id			integer not null primary key,
	test_type		not null references is_test_state,
	proc_id			integer,
	num_up                  integer,
	num_down                integer,
	test_starttime		date not NULL,
	test_stoptime		date
);

-- This table is to track emails that are sent out
create sequence is_sent_email_seq start with 10000;

create table is_sent_email_log (
	sent_email_id		integer not null primary key,
	sent_date		date not null,
	sent_to			varchar(30),
	subject			varchar(80),
	sent_cc			varchar(1000)
);
	


-- how are we going to use this machine
create sequence is_machine_use_id_seq start with 1;
create table is_machine_uses (
	machine_use_id	integer primary key,
	machine_use	varchar(100) not null unique,
	display_order	integer default 1
);

insert into is_machine_uses (machine_use_id, machine_use, display_order)
values (is_machine_use_id_seq.nextval, 'Development', 1);
insert into is_machine_uses (machine_use_id, machine_use, display_order)
values (is_machine_use_id_seq.nextval, 'Staging', 100);
insert into is_machine_uses (machine_use_id, machine_use, display_order)
values (is_machine_use_id_seq.nextval, 'Production', 200);
insert into is_machine_uses (machine_use_id, machine_use, display_order)
values (is_machine_use_id_seq.nextval, '6.916 - MIT', 300);
insert into is_machine_uses (machine_use_id, machine_use, display_order)
values (is_machine_use_id_seq.nextval, '6.916 - Other', 400);
insert into is_machine_uses (machine_use_id, machine_use, display_order)
values (is_machine_use_id_seq.nextval, 'Bootcamp', 500);
insert into is_machine_uses (machine_use_id, machine_use, display_order)
values (is_machine_use_id_seq.nextval, 'R&D', 600);
insert into is_machine_uses (machine_use_id, machine_use, display_order)
values (is_machine_use_id_seq.nextval, 'RocketStart - Solaris', 700);
insert into is_machine_uses (machine_use_id, machine_use, display_order)
values (is_machine_use_id_seq.nextval, 'RocketStart - Linux', 800);


-- =================================================================
-- add a new table is_machines
--
CREATE TABLE is_machines (
       machine_id           integer NOT NULL,
       machine_use_id    integer references is_machine_uses,
       hostname             varchar2(100) NULL,
       ip_address           varchar2(50) NULL,
       os_version           varchar2(50) NULL,
       description          varchar2(4000) NULL,
       model_and_serial     varchar2(4000) NULL,
       street_address       varchar2(4000) NULL,
       remote_console_instructions varchar2(4000) NULL,
       service_phone_number varchar2(100) NULL,
       service_contract     varchar2(4000) NULL,
       facility_phone       varchar2(100) NULL,
       facility_contact     varchar2(4000) NULL,
       backup_strategy      varchar2(4000) NULL,
       rdbms_backup_strategy varchar2(4000) NULL,
       further_docs_url     varchar2(200) NULL,
       PRIMARY KEY (machine_id)
);
comment on table is_machines is 'one row per machine, a machine has many IPs, ports and services';

create sequence is_machines_seq;


-- associate machines with a group
-- ie, an intranet group or projects

create table is_group_machine_map (
	group_id	integer not null references user_groups,
	machine_id	integer not null references is_machines,
	unique (group_id, machine_id)
);


--
-- SERVICES
-- Tables to record 'what' we are monitoring, with mostly one table per
-- 'test'. i.e. we will be testing vanilla TCP, email, web, and ping
-- (icmp) response.
--
create sequence is_services_seq start with 1000;

create table is_services (
	service_id		integer not null primary key,
	ip_or_hostname		varchar(60) not null,
	port			integer not null,
	protocol		varchar(20),
	name			varchar(100),
	tcp_response		varchar(50),
	first_monitored		date,
	timeout			integer default 20 not null,
	company			varchar(100),
	enabled_p		char(1) default 'f'
                                   check(enabled_p in ('t','f')),
        ping_enabled_p		char(1) default 'f'
                                   check(ping_enabled_p in ('t','f')),
        machine_id              integer null,
	unique (ip_or_hostname, port)
);

-- add a nullable foreign key from is_services to is_machines
--
ALTER TABLE IS_SERVICES
       ADD  ( FOREIGN KEY (machine_id)
                             REFERENCES is_machines ) ;
-- might as well index the foreign key
CREATE INDEX XIF24IS_SERVICES ON IS_SERVICES
(
       machine_id                  ASC
);


create or replace view is_services_active
as
select *
from is_services
where enabled_p = 't';


create sequence is_mail_services_seq start with 1000;

create table is_mail_services (
	mail_service_id		integer
        	constraint mail_service_id_pk primary key,
	service_id		integer 
		constraint service_id_null not null
		constraint service_id_is_service_ref references is_services,
	bouncer_email		varchar(100) default 'mmon_bouncer'
		constraint bouncer_email_null not null,
	   -- This is the special email address which should exist
	   -- on the monitored server.  Anything sent to that address
	   -- should return to sender.  It can be an address
	   -- without the '@hostname' part if the monitored server
	   -- will recognize it.
	last_unbounced_emailet_id	varchar(70),
	run_period		integer default 1
		constraint run_period_null not null,	
	   -- might not need run_period
	bounce_timeout_secs	integer default 60
		constraint bounce_timeout_secs_null not null,
	smtp_ok_p		char(1) default 't'
		constraint smtp_ok_boolean check(smtp_ok_p in ('t','f')),
	enabled_p		char(1) default 't'
		constraint enabled_boolean check(enabled_p in ('t','f'))
);

create or replace view is_mail_services_active
as
select 
     MAIL_SERVICE_ID,
     SERVICE_ID,
     BOUNCER_EMAIL,
     LAST_UNBOUNCED_EMAILET_ID,
     RUN_PERIOD,
     BOUNCE_TIMEOUT_SECS,
     SMTP_OK_P,
     ENABLED_P
from is_mail_services
where enabled_p = 't';

	
create sequence is_web_services_seq start with 1000;

create table is_web_services (
	web_service_id		integer not null primary key,
	service_id		integer not null references is_services,
	return_string		varchar(100) default 'success' not null,
	url			varchar(200) not null,
	query_string		varchar(200),
	enabled_p		char(1) default 't'
                                   check(enabled_p in ('t','f')),
	unique (service_id,url,return_string,query_string)
);

create or replace view is_web_services_active
as
select *
from is_web_services
where enabled_p = 't';


-- 
-- Logs and Alerts
--
create sequence is_event_log_sequence start with 1000;

create table is_event_log (
	event_id		integer not null primary key,
	service_id		references is_services,
        sub_service_id		integer,
	event_time		date,
	  -- discoverer is either a user_id for a test identification tag
	discoverer		varchar(30),
	event_description	varchar(40),
	test_type		not null references is_test_state,
	error_message		varchar(200),
	status_ok_p		char(1) default 't'
                                   check(status_ok_p in ('t','f')),
        remarks                 varchar2(1000)
);


-- this is a log for the MTA SMTP test
create table is_mail_log (
	event_id		references is_event_log,
	emailet_id		varchar(70)
);

create sequence is_alert_sequence start with 1000;

create table is_alerts (
	alert_id		integer not null primary key,
	service_id		references is_services not null,
        sub_service_id		integer,
	event_id		references is_event_log not null,
	test_type		not null references is_test_state,
	status_ok_p		char(1) default 't'
                                   check(status_ok_p in ('t','f')),
	notified_p		char(1) default 'f'
				   check(notified_p in ('t','f')),
	unique (service_id, sub_service_id, test_type)
);


--
-- NOTIFICATIONS
--
create sequence is_notification_rules_seq start with 1000;

-- The is_notification_rules table contains the rules and defaults
-- for the creation of a notice.  There is much duplication between
-- the is_notification_rules table and the is_notices table because
-- the 'rules' are 'new notices'.  After a notice is generate, it seems
-- better to allow the attributes of the notice to be changeable as
-- opposed to be controlled by a parent rule.  This way, someone can
-- change the 'notification_mode' of a rule for a service so that 
-- all future notices follow the new mode, but all existing notices
-- remain the same.

create table is_notification_rules (
	rule_id			integer not null primary key,
	service_id		integer not null references is_services,
	sub_service_id		integer,
	user_id			integer not null references users,
	group_id		integer references user_groups,
	test_type		not null references is_test_state,
	  -- does this notification require acknowledgement?
	acknowledge_p		char(1) default 'f'
                                   check(acknowledge_p in ('t','f')),
	  -- should use be informed of server bounces? (quick up/down)
	bounce_notify_p		char(1) default 'f' not null
                                   check(bounce_notify_p in ('t','f')),
	  -- should a trouble ticket be opened?
	open_ticket_p		char(1) default 'f'
                                   check(open_ticket_p in ('t','f')),
	mail_cc			varchar(1000),
  	  -- the following are for people who have beepers
	  -- and need a special tag or something in the subject
	custom_subject	varchar(80),
	custom_body	varchar(500),
	  -- we always send email when the server is down, we can
	  -- also send email when the server comes back up
	notification_mode	varchar(30), 	-- 'down_then_up', 'periodic'
	-- these two are only used when notification_mode is 'periodic'
	notification_interval_hours	number default 2,
	last_notification	date,
	unique (service_id, user_id, group_id, test_type)
);

create sequence is_notices_seq start with 1000;

-- A notice is a reaction to an event created as a result of a
-- notification rule.
create table is_notices (
	notice_id		integer not null primary key,
	rule_id			references is_notification_rules,
	service_id		not NULL references is_services,
        sub_service_id          integer,
	user_id			not NULL references users,
	group_id		references user_groups,
	event_id		references is_event_log,
	test_type		not null references is_test_state,
	note			varchar(500),
	acknowledged_p		char(1) default NULL
				   check(acknowledged_p in ('t','f',NULL)),
	  -- 'f' means it should be acknowledged, but isn't yet
	  -- NULL means it doesn't need acknowledgement
	bounce_notify_p		char(1) default 'f' not null
                                   check(bounce_notify_p in ('t','f')),
	ticket_id		integer,
	mail_cc			varchar(1000),
  	  -- the following are for people who have beepers
	  -- and need a special tag or something in the subject
	custom_subject	varchar(80),
	custom_body	varchar(500),
	acknowledged_id		references users(user_id),
	creation_time		date,
	acknowledged_time	date,
	notification_mode	varchar(30), 	-- 'down_then_up', 'periodic'
	notification_interval_hours	integer default 2,
	last_notification	date
);
	

create table is_users (
     user_id  integer not null,
     disable_is_mail_p char(1) default 'f' not null
                         check( disable_is_mail_p in ('t','f')),
     primary key (user_id)
);
comment on table is_users is 'users who want no IS mail are in here with f, users who want mail have t or are not in table';

alter table is_users add ( foreign key (user_id) references users);

-- we need to add a row to user_group_types
-- I add it in such a way that it will not give 
-- an erro if you do it twice
insert into  user_group_types 
    (GROUP_TYPE,
     PRETTY_NAME,
     PRETTY_PLURAL,
     APPROVAL_POLICY,
     DEFAULT_NEW_MEMBER_POLICY,
     GROUP_MODULE_ADMINISTRATION,
     HAS_VIRTUAL_DIRECTORY_P,
     GROUP_TYPE_PUBLIC_DIRECTORY,
     GROUP_TYPE_ADMIN_DIRECTORY,
     GROUP_PUBLIC_DIRECTORY,
     GROUP_ADMIN_DIRECTORY,
     USER_GROUP_TYPES_ID)
select
     'is_service',
     'IS Checker Service',
     'IS Checker Services',
     'open',
     'open',
     'none',
     'f',
     NULL,
     NULL,
     NULL,
     NULL,	
     user_group_types_seq.nextval
from dual
where not exists
    (select x.group_type
     from user_group_types x
     where x.group_type = 'is_service');


-- this function is needed to compare Intranet and IS Checker
-- hostnames since different conventions are used in these
-- two modules
create or replace function is_parse_hostname
       (
       	input_url in varchar2
       )
       	-- Dave Abercrombie, abe@arsdigita.com, 2000-04-07
       	--
       	-- Example:
       	--
       	-- Input                                Output
       	-- -------------------                  -----------
	-- http://arsdigita.com/index.tcl       arsdigita.com
        -- https://arsdigita.com/index.tcl      arsdigita.com
	-- nonsense_url                         nonsense_url
       	--
       	return varchar2
       	is parsed_hostname varchar2(4000);

begin -- need a nested block to get a declarations section
	declare

		len integer;  -- length of input URL (used for trunaction

		lpos integer; -- position from left as found by a instr search

		tpos integer; -- position from the left for truncating

	begin

		-- length of string is needed during truncation
		len := length(input_url);

		-- starting at the begining of input_url, look
		-- forwards for the first double slash
		lpos := instr(input_url, '//', 1, 1);

		-- if we did not find a double slash
		-- then lpos will be equal to 0

		-- we want to skip the // itself, so we add 2 to 
		-- the lpos to calculate the truncate position. 
		-- Also if did not find a // then we do not want to add 1
		if lpos=0
		then
			tpos := lpos;
		else
			tpos := lpos + 2 ;
		end if;

		-- truncate at the calculated position from the left
		-- starting at tpos and getting all remaining (need to add 1)
		parsed_hostname := substr(input_url,tpos,(len-tpos+1));

		-- now we need to trim off the end of the URL leaving
		-- just the hostname (and the port if present)

		-- starting at begining of input_url, look
		-- forwards for the first double slash
		lpos := instr(parsed_hostname, '/', 1, 1);

		-- if we did not find a slash
		-- then lpos will be equal to 0

		-- we want to skip the // itself, so we add 2 to 
		-- the lpos to calculate the truncate position. 
		-- Also if did not find a // then we do not want to do substr
		if lpos <> 0
		then
			tpos := lpos -1;
			parsed_hostname := substr(parsed_hostname,1,tpos);
		end if;

		-- if we end up with an empty string, then return input
		if length(parsed_hostname)=0
		then
			parsed_hostname := input_url;
		end if;

		-- return the parsed_hostname
		return(parsed_hostname);

	end; -- of nested block

end; -- of function

-- INSTR   http://oradoc.photo.net/ora81/DOC/server.815/a67779/function.htm#1025362
-- SUBSTR  http://oradoc.photo.net/ora81/DOC/server.815/a67779/function.htm#1025168

/ 
show errors

-- Tracking location a little better
-- cmceniry
create table is_host_locations (
	location_id		integer not null,
	location_name		varchar2(100) not null,
	PRIMARY KEY (location_id)
);

alter table is_machines
	add (location_id references is_host_locations);

create sequence is_host_locations_sequence start with 1;

-- Tracking networks for IPs
-- cmceniry

create table is_networks (
	network_id	integer not null,
	network_name	varchar2(40) not null,
	octa		integer not null check (octa >= 0 AND octa < 256),
	octb		integer not null check (octb >= 0 AND octb < 256),
	octc		integer not null check (octc >= 0 AND octc < 256),
	octd		integer not null check (octd >= 0 AND octd < 256),
	submask		integer not null check (submask > 0 AND submask < 32),
	PRIMARY KEY(network_id)
);
create sequence is_network_id_sequence start with 1;

--this won't scale right, but it works for now...
create table is_ip_addresses (
	ip_id		integer not null,
	network_id	references is_networks,
	octa		integer not null check (octa >= 0 AND octa < 256),
	octb		integer not null check (octb >= 0 AND octb < 256),
	octc		integer not null check (octc >= 0 AND octc < 256),
	octd		integer not null check (octd >= 0 AND octd < 256),
	PRIMARY KEY (ip_id)
);
create sequence is_ip_sequence start with 1000;

create table is_machines_ips_map (
	machine_id	references is_machines not null,
	ip_id		references is_ip_addresses not null,
	    constraint is_machines_ips_map_un unique(machine_id, ip_id)
);

create table is_names (
	name_id		integer primary key,
	dns_name	varchar(128) not null
);
create sequence is_names_sequence start with 1000;

create table is_machines_names_map (
	machine_id	references is_machines not null,
	name_id		references is_names not null,
	    constraint is_machines_names_map_un unique(machine_id, name_id)
);


--- At this point, I'm just hacking it...

create table ishack_websites (
	website_id	integer primary key,
	machine_name	varchar(128),
	server_short	varchar(30),
	server_version	varchar(100),
	chrooted_p	char(1) default 'f' check (chrooted_p in ('t', 'f'))
);
create sequence ishack_website_id_sequence start with 1000;
