-- Data model for storing information relevant to pets
-- this is used primarily to let users upload pictures of their pets and to 
--  specify some attributes common to those pets

-- @author minhngo@cory.eecs.berkeley.edu
-- @creation-date 8/15/2000
-- @cvs-id pets.sql,v 1.1.2.3 2000/08/27 19:58:03 mbryzek Exp

-- simple table to store different types of pets (e.g. cat, dog, hamster, etc.)
create sequence pet_category_id_seq start with 1;
create table pet_category (
       category_id       integer 
                         constraint pt_type_id_pk primary key,
       name              varchar(100) not null 
                         constraint pt_name_un unique,
       description       varchar(4000)
);


-- associate pets with owners and store basic attributes about those pets.
create sequence user_pets_id_seq start with 1;
create table user_pets (
        pet_id                  integer
                                constraint up_pet_id_pk primary key
                                constraint up_pet_id_nn not null,
        owner_id                constraint up_owner_id_fk
                                references users(user_id),
        pet_call_name           varchar(100) not null,
        category_id             integer 
                                constraint up_category_id_fk references pet_category(category_id),
        upload_date             date default sysdate,
        -- birth information
        birthdate               date,
        birthplace              varchar(100),
        -- pet's favorite things, not sure if want to create a new table for it
        favorite_toy            varchar(100),
        favorite_food           varchar(100),
        favorite_place          varchar(100),
        favorite_activity       varchar(100),
        -- pet's story
        story                   clob,
        story_html_p            char(1)
                                constraint up_story_html_p_ck
                                check (story_html_p in ('t', 'f')),
        -- approval status
        approved_p              char(1) default 't'
                                constraint up_approved_p_ck
                                check (approved_p in ('t', 'f'))
);

