<%@ include file="../../acs.jspi" %>

<% /*
    UI for editing the permissions for a specific database row

    Given that a row in the database typically represents an
    object (e.g., a web page, a person), this page accepts
    an object_name parameter which we use to build a meaningful
    page title.
    
    @author michael@arsdigita.com
    @creation-date 2000-02-25
   
*/ %>

<ad:page>
<ad:get_user_info id="user" redirect="false"/>
<ad:page_contract vars="on_what_id:integer on_which_table object_name:optional return_url:optional"/>

<% 
if (Utilities.emptyString(object_name)) { 
    object_name = "Row " + on_what_id + " of Table " + on_which_table;
} 
Integer user_id = null;
if (user == null || !user.isSiteWideAdmin()) {
	throw new ForbiddenException("Not Site-Wide Admin", "You must be a site-wide admin to view this page.");
}

// We need to export the current page as return_url for links that
// we create below, so we save the return_url value obtained from
// the form input in a separate variabel and then restore it later.
//

if (return_url == null) {
 return_url = "one-table?table_name=" + on_which_table;
}
String return_url_save = return_url;
return_url = request.getRequestURI() + "?" + request.getQueryString();
//
// Restore the value of return_url to that obtained from the form
// input.
//
request.setAttribute("return_url", return_url);
request.setAttribute("return_url_save", return_url_save);

// This query fetches all of the standard permission grants (i.e.,
// read, comment, write, and administer) for the specified database
// row. Also, it appends rows for the 'registered_users' and
// 'all_users' scopes if no permissions have been granted to those
// scopes (this is what the two queries UNIONed onto the end do).
//

String registered_users_scope_pretty_name = "Registered Users Only";
String all_users_scope_pretty_name = "All Users";
%>

<ad:db_query id="gp_grid_query">
select * from (select
 decode(pg.scope,
  'user',
   '<a href=\"/shared/community-member?user_id=' || u.user_id ||
   '\">' || u.first_names || ' ' || u.last_name || '</a>',
  'group_role', g.group_name || ' ' || pg.role,
  'group', g.group_name,
  'registered_users', '<%= registered_users_scope_pretty_name %>',
  'all_users', '<%= all_users_scope_pretty_name %>') as permission_owner,
 pg.scope as scope, pg.user_id, pg.group_id, pg.role,
 pg.read_permission_p,
 pg.comment_permission_p,
 pg.write_permission_p,
 pg.administer_permission_p,
 decode(scope, 'user', 1, 'group_role', 2, 'group', 3,
        'registered_users', 4, 'all_users', 5) as display_order
from general_permissions_grid pg, users u, user_groups g
where pg.on_what_id = ?
and pg.on_which_table = lower(?)
and pg.user_id = u.user_id (+)
and pg.group_id = g.group_id (+)
union
select
 '<%= registered_users_scope_pretty_name %>' as permission_owner,
 'registered_users' as scope,
 to_number(null) as user_id,
 to_number(null) as group_id,
 to_char(null) as role,
 'f' as read_permission_p,
 'f' as comment_permission_p,
 'f' as write_permission_p,
 'f' as administer_permission_p,
 4 as display_order
from dual
where not exists (select 1
                  from general_permissions_grid
                  where on_what_id = ?
                  and on_which_table = lower(?)
                  and scope = 'registered_users')
union
select
 '<%= all_users_scope_pretty_name %>' as permission_owner,
 'all_users' as scope,
 to_number(null) as user_id,
 to_number(null) as group_id,
 to_char(null) as role,
 'f' as read_permission_p,
 'f' as comment_permission_p,
 'f' as write_permission_p,
 'f' as administer_permission_p,
 5 as display_order
from dual
where not exists (select 1
                  from general_permissions_grid
                  where on_what_id = ?
                  and on_which_table = lower(?)
                  and scope = 'all_users')
order by display_order asc)
<ad:db_bind><%= on_what_id %></ad:db_bind>
<ad:db_bind><%= on_which_table %></ad:db_bind>
<ad:db_bind><%= on_what_id %></ad:db_bind>
<ad:db_bind><%= on_which_table %></ad:db_bind>
<ad:db_bind><%= on_what_id %></ad:db_bind>
<ad:db_bind><%= on_which_table %></ad:db_bind>
</ad:db_query>

<ad:db_1row id="get_table_data" obj="table_data">
select pretty_table_name_singular, pretty_table_name_plural,
  denorm_view_name, lower(id_column_name) as id_column_name
from general_table_metadata
where upper(table_name) = upper(:table_name)
<ad:db_bind><%= on_which_table %></ad:db_bind>
</ad:db_1row>

<% 
Vector column_list = new Vector();
Vector column_name_list = new Vector();
%>

<ad:db_query id="column_data">
select column_pretty_name, column_name, is_date_p
    from table_metadata_denorm_columns
    where upper(table_name) = :table_name
    order by display_ordinal
<ad:db_bind><%= on_which_table %></ad:db_bind>
</ad:db_query>

<ad:db_foreach in="column_data" row="rs">
<%
column_name_list.addElement(rs.getString("column_name"));
%>
</ad:db_foreach>

<ad:db_1row id="get_column_values" obj="column_values">
select <%= Utilities.join(column_name_list, ",") %>
from <%= table_data.get("denorm_view_name") %>
where <%= table_data.get("id_column_name") %> = ?
<ad:db_bind><%= on_what_id %></ad:db_bind>
</ad:db_1row>

<% 
ContextBar navbar = new ContextBar(ContextBar.ADMIN, user);
navbar.add("index", "General Permissions");
navbar.add("one-table?table_name=" + on_which_table, on_which_table);
navbar.add("One Row");

Selection permissions = new Selection(gp_grid_query, db);

request.setAttribute("permissions", permissions );
request.setAttribute("object_name", object_name);
request.setAttribute("grant_permission_to_user_link", "<a href=\"permission-grant-to-user?" + Utilities.exportURLVars(request, "on_what_id on_which_table object_name return_url") + "\">Grant permission to a user</a>");
request.setAttribute("grant_permission_to_group_link", "<a href=\"permission-grant-to-group?" + Utilities.exportURLVars(request, "on_what_id on_which_table object_name return_url") + "\">Grant permission to a user group</a>");

request.setAttribute("column_data", new Selection(column_data, db));
request.setAttribute("navbar", navbar);
%>

<ad:use_template />

</ad:page>
