<%
   /*
   Queues an outgoing spam message to a group of users,
   by adding it to the spam_history table
   @param spam_id id of the message
   @param subject  subject line for header
   @param from_address  from address for header
   @param message text of the message
   @param message_html html version of the message
   @param message_aol aol_html dialect version of the message
   @param send_date when to schedule the message for delivery
   @param from_file_p flag to get message content from filesystem rather than from page vars
   @param users_sql_query custom SQL query to generate the list of target users
   @param users_description English description of target list of user
   @param user_class_id An id of a user-class, mutually exclusive with users_sql_query
   @param template_p If t, perform Tcl substitution on the message body
   @author hqm@arsdigita.com
   @author phong@arsdigita.com (java port)
   @cvs-id $Id: spam.jsp,v 1.5 2000/10/17 17:12:11 luke Exp $
   */
%>

<%@ include file="/acs.jspi" %>
<%@ page import="com.arsdigita.acs.spam.*" %>

<ad:page>
<ad:page_contract id="pc" vars="spam_id subject from_address message message_html:optional message_aol:optional send_date:optional,array from_file_p:optional users_sql_query:optional users_description:optional user_class_id:optional template_p:optional" />

<ad:get_user_info id="user" />

<%
if (message_html == null) { message_html = ""; }
if (message_aol == null) { message_aol = ""; }
if (Utilities.emptyString(from_file_p)) { from_file_p = "f"; }
if (users_sql_query == null) { users_sql_query = ""; }
if (users_description == null) { users_description = ""; }
if (user_class_id == null) { user_class_id = ""; }
if (Utilities.emptyString(template_p)) { template_p = "f"; }

String message_stripped = "";
if (!message.equals("")) {
   //TO DO: need to strip message of all \r
   message_stripped = message;
}

if (from_file_p.equals("t")) {
   message = Spam.getSpamFromFilesystem("plain");
   message_html = Spam.getSpamFromFilesystem("html");
   message_aol = Spam.getSpamFromFilesystem("aol");
}

if ( Utilities.emptyString(subject) && Utilities.emptyString(message_stripped) &&
     Utilities.emptyString(message_html) && Utilities.emptyString(message_aol) ) {

     throw new ACSException("The contents of your message and subject line is the empty string. You must send something in the message body");
}

// get user information
Integer user_id = user.getUserId();
 
// regenerate the users_sql_query and users_description base on user_class_id
if (!Utilities.emptyString(user_class_id)) {
   AdSet setid = new AdSet();
   setid.put("user_class_id", user_class_id);
   
   users_sql_query = Spam.userClassQuery(setid);
   
   String class_name = db.queryToString("select name from user_classes where user_class_id = :user_class_id", new Object[] {user_class_id});
   String sql_description = db.queryToString("select sql_description from user_classes where user_class_id = :user_class_id", new Object[] {user_class_id});

   users_description = class_name + ": " + sql_description;

}
 
String spam_query = users_sql_query;
String send_date_str = Spam.verifyDate(send_date);   

// check to see if the user double clicked first
String double_click = db.queryToString("select count(1) from spam_history where spam_id = :spam_id", new Object[] {spam_id});

// post the new spam message
String spam_status_p = Spam.postNewSpamMessage(spam_id, template_p,
					       from_address, subject,
					       message_stripped, message_html,
					       message_aol, users_description,
					       spam_query, send_date_str,
					       user_id.toString(), "");  


// create the navigation bar
ContextBar navbar = new ContextBar(pageContext);
navbar.add("index", "Spam");
navbar.add("Spam Execution");

// call the spam daemon if the insert was ok
if (spam_status_p.equals("0")) {
   Spam.sendScheduledSpamMessages();
}

// export variables to template
request.setAttribute("navbar", navbar);
request.setAttribute("users_description", users_description);
request.setAttribute("users_sql_query", users_sql_query);
request.setAttribute("from_address", from_address);
request.setAttribute("subject", subject);
request.setAttribute("send_date_str", send_date_str);
request.setAttribute("message_stripped", message_stripped);
request.setAttribute("double_click", double_click);
request.setAttribute("spam_status_p", spam_status_p);
request.setAttribute("spam_id", spam_id);

%>

<ad:use_template />
</ad:page>

