<%@ include file="../acs.jspi" %>
<%@ page import="com.arsdigita.acs.calendar.*" %>

<ad:page>
<%
/**
 * Process calendar form.
 *  We do an insert or an update based on new_item_p.
 *  If the insert fails, we assume a double-click.
 *  If that has no results, we do an update.  If that fails,
 *  we display an error message.
 *  We check that the user either owns the item being modified or
 *  is an admin for the specified group_id.
 *
 * Warning: Note that user_id does not equal loggedInUserId (creation_user)!
 *
 * NOTE that we do not trust the "new_item_p" flag if it is set to t, because this could be a double-click.
 */
 %>
    // start_date and end_date are AdSets containing year, month, day, hours, minutes, seconds, ampm
<ad:page_contract id="pc" vars="item_id:integer instance_id:integer title description all_day_p:boolean,optional duration:integer repeat_p:boolean update_current_or_all:optional current_view current_date group_id:integer user_id:integer scope new_item_p:boolean return_url:optional category_id_list:integer,multiple,optional compress_day_view_p:optional start_date:array end_date:array">
<%
if (item_id == null) {
    pc.addComplaint("You must specify an item ID");
}
if (instance_id == null) {
    pc.addComplaint("You must specify an instance ID");
}
if (Utilities.emptyString(title)) {
    pc.addComplaint("You must specify a title");
}
if (Utilities.emptyString(description)) {
    pc.addComplaint("You must specify a description");
}
// TODO: Make sure this check covers all the bases; what about new items with repeat_p?
if (Utilities.emptyString(update_current_or_all) && (new_item_p == null || !new_item_p.booleanValue()) && db.queryToString("SELECT 1 " +
                                                                               "FROM cal_repeat_items " + 
                                                                               "WHERE item_id = :item_id", new Object[] {item_id}, "0").equals("1")) {
    pc.addComplaint("This is a repeating item.  You must choose to either update only the current instance or all instances.");
}
%>
</ad:page_contract>

<%
    if (new_item_p == null) {
        new_item_p = Boolean.FALSE;
    }
boolean newItemP = new_item_p.booleanValue();
if (group_id != null && group_id.intValue() == 0) {
    group_id = null;
}
if (duration != null) {
    duration = new Integer(CalendarProcs.roundToPrecision(duration.intValue(), 5));
}
if (all_day_p == null) {
    // Checkboxes are only sent if checked, so need to explicitly set defaults on this.
    all_day_p = Boolean.FALSE;
}

// TCL defaults were: 	{all_day_p "f"} {duration ""} {repeat_p ""} {update_current_or_all ""} {group_id "[db_null]"} {user_id "[db_null]"} {scope ""} {new_item_p "f"} {return_url ""} {category_id_list -multiple-list}
%>

<ad:get_user_info id="loggedInUser" redirect="true"/>
<%
    
com.arsdigita.acs.calendar.CalendarInfo calendarInfo=com.arsdigita.acs.calendar.CalendarProcs.initializeCalendarParameters(request, user_id, loggedInUser, group_id, current_view, current_date, compress_day_view_p, "full", "item-add");
request.setAttribute("calendarInfo", calendarInfo);
String returnString = "<A HREF=\"?" + Utilities.exportURLVars(request, "current_view current_date group_id user_id") + "\">Return to calendar</A>";

// Get start_date, end_date
SqlStatement startDate = new SqlStatement();
SqlStatement endDate = new SqlStatement();
if (all_day_p.booleanValue()) {
    // start_date is truncated to just the date
    startDate.setSql("to_date(:startDateValue, 'YYYY:MM:DD HH24:MI:SS')");
    startDate.bind("startDateValue", CalendarProcs.transformDatetime(start_date, true));
    endDate.setSql("NULL");
} else {
    startDate.setSql("to_date(:startDateValue, 'YYYY:MM:DD HH24:MI:SS')");
    startDate.bind("startDateValue", CalendarProcs.transformDatetime(start_date, false));
    if (duration != null) {
        endDate.append("(").append(startDate).append(" + (:duration)/(24*60))");
        endDate.bind("duration", duration);
    } else {
        endDate.setSql("to_date(:endDateValue, 'YYYY:MM:DD HH24:MI:SS')");
        endDate.bind("endDateValue", CalendarProcs.transformDatetime(end_date, false, start_date));
    }
}
//make sure the start dates and end dates are valid
SqlStatement s;
try {
    s = new SqlStatement().append("select ").append(startDate).append(", ").append(endDate).append(" from dual");
    // We actually don't care about the result
    db.queryToList(s);
} catch (SQLException e) {
    throw new ACSException ("You have entered an invalid time");
}
// make sure that the start time is before the end time
s.clear();
s.append("select count(1) " +
         "from dual " +
         "where ").append(startDate).append(" > ").append(endDate);
// vars are already bound from above
String timeCount = db.queryToString(s);
if (Integer.parseInt(timeCount) != 0) {
    throw new ACSException ("Your start time must be before your end time");
}
if (scope.equalsIgnoreCase("group")) {
    // Group item, don't select user_id
    user_id = null;
}

// TODO: What about altering info in cal_item_map on an update?
db.beginTransaction();
try {
    // If the inserts fail due to a unique
    // constraint, assume a double click and switch to doing an update.
    if (newItemP) {
        try {
            CalendarProcs.insertItem(item_id, instance_id, title, startDate, endDate, user_id, group_id, description, loggedInUser.getUserId());
            // Success
        } catch (SQLException e) {
	    // We either know we don't have a new item, or we had an error
	    // on our inserts... if it was a unique constraint error, 
	    // try doing an update instead.
            if (CalendarProcs.parseOracleErrorForUniqueConstraintViolation(e.getMessage()) != null) {
                // This wasn't a new item, we need to do an update (maybe a double-click)
                newItemP=false;
            } else {
                // Oracle error other than a unique constraint violation, re-throw
                throw e;
            }
        }
    }
    // Yes, we just checked newItemP, but we may have set it false above.
    if (!newItemP) {
        // We do some security checking here -anyone can add a new item, but only
        // certain people (owners or group admins) can modify already existing items.
        if (CalendarProcs.isWriteAccessAllowed(loggedInUser, item_id, group_id)) {
            // We've cleared security...

            // NOTE that the cal_item_map is NOT
            // presently updated in this case, so we
            // can't change item ownership here.
		    
            // Okay, this gets complicated with repeating items....
            if ((!Utilities.emptyString(update_current_or_all) && update_current_or_all.equalsIgnoreCase("all")) || !repeat_p.booleanValue()) {
                // Select fields that were actually changed
                ResultSet rs = db.query("SELECT i.title as item_title, " +
                                        "ii.title as instance_title, " +
                                        "i.description as item_description, " +
                                        "ii.description as instance_description " +
                                        "FROM cal_items i, cal_item_instances ii " +
                                        "WHERE i.item_id = :item_id " +
                                        " AND i.item_id = ii.item_id " + 
                                        "AND ii.instance_id = :instance_id ", new Object[] {item_id, instance_id});
                if (!rs.next()) {
                    throw new ACSException ("Invalid item id or instance id specified for item update.");
                }
                SingleSelection itemSelect = new SingleSelection(rs);
                            
                // if the new title is the same as the instance title
                // or the item title if instance is null, then don't
                // change it
                String updateItemTitleValue = null;
                String updateInstanceTitleSql = null;
                String updateItemDescriptionValue = null;
                String updateInstanceDescriptionSql = null;
                if (! (title.equals(itemSelect.get("instance_title")) || 
                         (Utilities.emptyString(itemSelect.get("instance_title")) && title.equals(itemSelect.get("item_title"))))) {
                    updateItemTitleValue = title;
                    updateInstanceTitleSql = "title = NULL \n";
                }
                // if the new description is the same as the instance 
                // description or the item description if instance is
                // null, then don't change it
                if (! (description.equals(itemSelect.get("instance_description")) ||
                       (Utilities.emptyString(itemSelect.get("instance_description")) && description.equals(itemSelect.get("item_description"))))) {
                    updateItemDescriptionValue = description;
                    updateInstanceDescriptionSql = "description = NULL \n";
                }
                
                if (updateItemTitleValue != null || updateItemDescriptionValue != null) {
                    StringBuffer sqlItem = new StringBuffer("UPDATE cal_items SET ");
                    SqlStatement selItem = new SqlStatement();
                    StringBuffer sqlInstance = new StringBuffer("UPDATE cal_item_instances SET ");
                    if (updateItemTitleValue != null) {
                        sqlItem.append("title=:updateItemTitleValue, \n");
                        selItem.bind("updateItemTitleValue", updateItemTitleValue);
                        sqlInstance.append(updateInstanceTitleSql);
                    }
                    if (updateItemDescriptionValue != null) {
                        sqlItem.append("description=:updateItemDescriptionValue, \n");
                        selItem.bind("updateItemDescriptionValue", updateItemDescriptionValue);
                        sqlInstance.append(updateInstanceDescriptionSql);
                    }
                    sqlItem.append("creation_user=:creation_user, \n" +
                                   "creation_date=sysdate \n" + 
                                   "WHERE item_id=:item_id");
                    selItem.bind("creation_user", loggedInUser.getUserId());
                    selItem.bind("item_id", item_id);

                    sqlInstance.append(" WHERE instance_id = :instance_id");

                    // This may not be exactly what is wanted as it doesn't 
                    // update individual instances that have been changed.  
                    // If we did want to blast individualized instances
                    // we'd need to do an update, setting these fields to null.
                    selItem.setSql(sqlItem.toString());
                    Log.note("XXXXXXXX updating cal_item_instances");
                    db.dml(selItem);

                    // Reset the current instance to nulls so that
                    // it inherits properly
                    if (db.dml(sqlInstance.toString(), new Object[] {instance_id}) <= 0) {
                        throw new SQLException ("Abort calendar item add because no cal_items were updated");
                    }
                }
                if (!repeat_p.booleanValue()) {
                    SqlStatement updateItem1 = new SqlStatement();
                    updateItem1.append("UPDATE cal_item_instances \n" + 
                                       "SET start_date=").append(startDate).append(",\n" +
                                       "end_date=").append(endDate).append("\n" +
                                       "WHERE instance_id=:instance_id");
                    updateItem1.bind("instance_id", instance_id);
                    if (db.dml(updateItem1) <= 0) {
                        throw new SQLException ("Abort calendar item add because no cal_item_instances were updated.");
                    }
                } else {
                    // Calculate time offset 
                    // If nonzero then time changed
                    // If greater than 1, then date changed
                    SqlStatement itemInstanceSelect = new SqlStatement();
                    itemInstanceSelect.append("SELECT (").append(startDate).append(" - start_date) as start_offset, \n");
                    itemInstanceSelect.append("(nvl(").append(endDate).append(", end_date) - end_date) as end_offset \n");
                    itemInstanceSelect.append("FROM cal_item_instances \n" +
                                              "WHERE instance_id = :instance_id");
                    itemInstanceSelect.bind("instance_id", instance_id);
                    ResultSet offsets = db.query(itemInstanceSelect);
                    offsets.next();
                    // TODO: Verify that this returns an integer in Java.  It does in SQLPlus, but that 
                    // may not always be the case.
                    Integer startOffset = new Integer(offsets.getInt("start_offset"));
                    Integer endOffset = new Integer(offsets.getInt("end_offset"));
                    if (Math.abs(startOffset.intValue()) > 1) {
                        Log.warn("item-aev-2.jsp: start_offset was > 1, should change date but presently is not implemented.  instance_id=" + instance_id);
                        // TODO: Figure out how to change date
                    } else if ((endOffset == null) || 
                               endDate.getSql().equals("NULL")) {
                        // Update each instance with start_offset and
                        // new end_date
                        SqlStatement updateSE = new SqlStatement();
                        updateSE.append("UPDATE cal_item_instances " + 
                                        "SET start_date = (start_date + :start_offset), " +
                                        "end_date = ").append(endDate).append(" WHERE item_id = :item_id");
                        updateSE.bind("start_offset", startOffset);
                        updateSE.bind("item_id", item_id);
                        Log.note("XXXXXXXX updating offsets");
                        db.dml(updateSE);
                        // TODO: Figure out why this one doesn't abort while all of the rest seem to...
                    } else if (startOffset.intValue() != 0 || endOffset.intValue() != 0) {
                        // Update each instance with offsets
                        SqlStatement updateOffsets = new SqlStatement();
                        updateOffsets.append("UPDATE cal_item_instances " +
                                             "SET start_date = (start_date + :start_offset), " +
                                             "end_date = (end_date + :end_offset) " +
                                             "WHERE item_id = :item_id");
                        updateOffsets.bind("start_offset", startOffset);
                        updateOffsets.bind("end_offset", endOffset);
                        updateOffsets.bind("item_id", item_id);
                        if (db.dml(updateOffsets) <= 0) {
                            throw new SQLException ("Abort calendar item add because no cal_item_instances' offsets were updated.");
                        }
                    }
                    // Done w/ repeat_p & !repeat_p check
                }
            } else if (!Utilities.emptyString(update_current_or_all) && update_current_or_all.equalsIgnoreCase("current")) {
                // Have to figure out what's changed
                // If time, then update instance
                SqlStatement instances = new SqlStatement();
                instances.append("SELECT 1 \n" + 
                                 "FROM cal_item_instances \n" +
                                 "WHERE instance_id = :instance_id \n" +
                                 "AND (start_date != ").append(startDate).append(" \n" +
                                 "OR end_date != ").append(endDate).append(")");
                instances.bind("instance_id", instance_id);
                if (db.queryToString(instances, "0").equals("1")) {
                    SqlStatement update = new SqlStatement();
                    update.append("UPDATE cal_item_instances \n" + 
                                  "SET start_date=").append(startDate).append(", " +
                                  "end_date=").append(endDate).append(" WHERE instance_id=:instance_id");
                    update.bind("instance_id", instance_id);
                    if (db.dml(update) <= 0) {
                        throw new SQLException ("Abort calendar item add because no cal_item_instances start & end dates were updated.");
                    }
                }
                // Other fields require updating the current instance
                if (db.queryToString("SELECT 1 \n" +
                                     "FROM cal_items \n" +
                                     "WHERE item_id = :item_id \n" +
                                     "AND (title != :title \n" +
                                     "OR description != :description)", new Object[] {item_id, title, description}, "0").equals("1")) {
                    if (db.dml("UPDATE cal_item_instances \n" +
                               "SET title=:title, \n" +
                               "description=:description \n" +
                               "WHERE instance_id = :instance_id", new Object[] {title, description, instance_id}) <= 0) {
                        throw new SQLException ("Abort calendar item add because no cal_item_instances title & description were updated.");
                    }
                }
            }
        } else {
	    throw new ForbiddenException ("Insufficient permission", "Insufficient permission to edit this calendar item - must be item owner or group administrator.<br>" + returnString);
        }
    }
    // deal with category_id_list
    // This is a selection box named category_id_list that the user can select categories with.
    // We want to get the category IDs.
    if (category_id_list != null) {
        db.dml ("delete from cal_category_map where instance_id = :instance_id", new Object[] {instance_id});
        // Yes, we're putting user input directly into the SQL, which is generally bad.  But we validated that these 
        // are numbers up in the page_contract, so it should be ok.
        db.dml ("insert into cal_category_map \n" +
	    "(instance_id, category_id) \n" +
	    "select :instance_id, category_id \n" +
	    "from cal_categories \n" +
	    "where enabled_p = 't' \n" +
	    "and category_id in ('" + Utilities.join(category_id_list, "', '") + "')", new Object[] {instance_id});
    }
} catch (SQLException e) {
    Log.warn("Aborting calendar item add for item " + item_id + ", instance " + instance_id + ": " + e.toString());
    db.abortTransaction();
    throw new SQLException(("Aborting calendar item add for item " + item_id + ", instance " + instance_id + ": " + e.getMessage() + "<br>" + returnString));
} finally {
    // This has an implicit commit in it.  We should have done a rollback in the catch above in the event of a problem.
    // We need to endTransaction to get back into auto-commit mode.
    db.endTransaction();
}
db.close();

if (Utilities.emptyString(return_url)) {
    if (repeat_p.booleanValue()) {
        response.sendRedirect("index?" + Utilities.exportURLVars(request, "group_id item_id instance_id current_view current_date") + "&action=item-repeat");
    } else {
        response.sendRedirect("index?" + Utilities.exportURLVars(request, "group_id item_id instance_id current_view current_date") + "&action=item-view");
    }
} else {
    response.sendRedirect(return_url);
}
%>
