<%
/*  
    insert both a file and a version into the database

    ADDED 8/20: the ability to upload a url. We take either
    a url or a file, but not both. We look for an URL first.

    @author aure@arsdigita.com
    @creation-date July 1999
    @cvs-id $Id: upload-new-2.jsp,v 1.9 2000/10/17 17:12:22 luke Exp $

    modified by randyg@arsdigita.com, January 2000 
    to use the general permissions system
    
    files, urls and folders all have at least one record in fs_vesion so that the
    permissions will work properly.  In addition, this leaves itself open to the
    options of placing permissions on the folders

*/
%>

<%@ include file="/acs.jspi" %>
<%@ page import="com.arsdigita.acs.fs.*" %>

<ad:page>

<ad:get_user_info id="user" />
<% Integer user_id = user.getUserId(); %>

<ad:page_contract id="pc" vars="parent_id:integer file_id:integer file_title:optional return_url:optional upload_file:optional,file group_id:optional,integer url:optional version_id:integer,optional version_description:optional public_p:optional">

<%
// check if the url starts with http:// or https:// or ftp://
if (url != null && url.length() > 0 && 
   !((url.toLowerCase().startsWith("http://")) || 
    (url.toLowerCase().startsWith("https://")) || 
    (url.toLowerCase().startsWith("ftp://"))))
{
    pc.addComplaint("The URL must start with http://, https:// or ftp://\n");
}

if (file_title == null || file_title.length() == 0) {
    pc.addComplaint("You must give a title\n");
}

if ((url == null || url.length() == 0) && (upload_file == null || upload_file.length() == 0)) {
    pc.addComplaint("You need to upload a file or enter a URL\n");
}

if (!Utilities.emptyString(url) && !Utilities.emptyString(upload_file)) {
    pc.addComplaint("You can not both add a url and upload a file");
}

if ("t".equals(public_p) && !(parameters.getBoolean("PublicDocumentTreeP","fs"))) {
    pc.addComplaint(parameters.get("SystemName", "acs") + " does not support a public directory tree. \n");
} 

if (group_id != null) {
    if (!user.isGroupMember(group_id)) {
	pc.addComplaint("You are not a member of this group.\n");
    }
} else {
    group_id = null;
}

// return errors
%>

</ad:page_contract>

<%


// Set default public permissions

String public_read_p = null;
String public_write_p = null;
String public_comment_p = null;
String client_filename = null;
String guessed_file_type = null;
String file_extension = null;
java.io.File tmp_file = null;
long n_bytes = 0;

if ("t".equals(public_p)) {
    public_read_p = "t";
    public_write_p = "t";
    public_comment_p = "t";
} else {
    public_read_p = "f";
    public_write_p = "f";
    public_comment_p = "f";
}

if (Utilities.emptyString(url)) {
    // We are processing a file, not a URL.
    // get the file from the user.
    // number_of_bytes is the upper-limit
        
    int max_n_bytes = parameters.getInt("MaxNumberOfBytes", "fs", -1);
    tmp_file = new java.io.File(upload_file_tmpfile);

    if (max_n_bytes != -1 && (tmp_file.length() > max_n_bytes)) {
	out.println("Your file is larger than the maximum permissible upload size:  " + max_n_bytes + "bytes");
	return;
    }

    file_extension = upload_file.substring(upload_file.lastIndexOf('.')+1);

    // guess file type based on servlet container
    guessed_file_type = getServletContext().getMimeType(upload_file);
    if (guessed_file_type == null) {
	// default for unknown
	guessed_file_type = "text/plain";
    }

    n_bytes = tmp_file.length();

    // strip off the C:\directories... crud and just get the file name
    com.oroinc.text.perl.Perl5Util re =  new com.oroinc.text.perl.Perl5Util();
    
    if (re.match("@([^//\\\\]+)$@", upload_file)) {
	client_filename = re.group(1);
    } else {
	client_filename = upload_file;
    }
}
    
%>

<ad:db_query id="version_insert_file">
begin 
    insert into fs_versions (
        version_id,
        file_id, 
        version_description, 
        creation_date, 
        author_id, 
        client_file_name, 
        file_type, 
        file_extension, 
        n_bytes, 
        version_content)
    values (?, ?, ?, sysdate, ?, ?, ?, ?, ?, empty_blob())
    returning version_content into ?;
end;
<ad:db_bind><%= version_id %></ad:db_bind>
<ad:db_bind><%= file_id %></ad:db_bind>
<ad:db_bind><%= db.bindObj(version_description) %></ad:db_bind>
<ad:db_bind><%= user_id %></ad:db_bind>
<ad:db_bind><%= client_filename %></ad:db_bind>
<ad:db_bind><%= db.bindObj(guessed_file_type) %></ad:db_bind>
<ad:db_bind><%= db.bindObj(file_extension) %></ad:db_bind>
<ad:db_bind><%= n_bytes %></ad:db_bind>
</ad:db_query>

<ad:db_query id="version_insert_blob">
 select version_content from fs_versions
 where version_id=?
<ad:db_bind><%= version_id %></ad:db_bind>
</ad:db_query>

<ad:db_query id="dup_check"> 
 select count(*) from fs_files where file_id = ?
<ad:db_bind><%= file_id %></ad:db_bind>
</ad:db_query>

<ad:db_query id="version_insert_url">
	insert into fs_versions
	(version_id, file_id, version_description, creation_date, author_id, url)
	values
	(:version_id, :file_id, :version_description, sysdate, :user_id, :url)
<ad:db_bind><%= version_id %></ad:db_bind>
<ad:db_bind><%= file_id %></ad:db_bind>
<ad:db_bind><%= db.bindObj(version_description) %></ad:db_bind>
<ad:db_bind><%= user_id %></ad:db_bind>
<ad:db_bind><%= url %></ad:db_bind>
</ad:db_query>

<ad:db_query id="file_insert">
 insert into fs_files (	file_id, 
		       	file_title, 
		       	owner_id, 
		       	parent_id, 
			sort_key, 
			depth, 
		        group_id, 
		        public_p)
	values(?, ?, ?, ?, 0, 0, ?, ?)
 <ad:db_bind><%= file_id %></ad:db_bind>
 <ad:db_bind><%= file_title %></ad:db_bind>
 <ad:db_bind><%= user_id %></ad:db_bind>
 <ad:db_bind><%= db.bindObj(parent_id) %></ad:db_bind>
 <ad:db_bind><%= db.bindObj(group_id) %></ad:db_bind>
 <ad:db_bind><%= db.bindObj(public_p) %></ad:db_bind>
</ad:db_query>

<ad:db_transaction>
<%
    // first check for duplicates
    try {
	String ct = db.queryToString(dup_check);	
	if (!"0".equals(ct)) {
	    // redirect to return URL
	    response.sendRedirect(return_url);
	    return;
	}
    } catch (ACSException e) {
	throw new JspException("oops: " + e.getMessage());
    }

    db.dml(file_insert);
    if (Utilities.emptyString(url)) {
	out.println(version_insert_file.getSql());
	db.blobDmlFile(version_insert_file, tmp_file);
    } else {
	db.dml(version_insert_url);
    }

    // now that the version has been inserted, let's set up the permissions

    if ("t".equals(public_p)) {
      GeneralPermissions.grantPermission(db, "all_users", null, null, null, 
       "read", version_id, "FS_VERSIONS");
      GeneralPermissions.grantPermission(db, "all_users", null, null, null, 
       "write", version_id, "FS_VERSIONS");
      GeneralPermissions.grantPermission(db, "all_users", null, null, null, 
       "comment", version_id, "FS_VERSIONS");
    }
 
    GeneralPermissions.grantPermission(db, "user", user_id, null, null, 
     "read", version_id, "FS_VERSIONS");
    GeneralPermissions.grantPermission(db, "user", user_id, null, null, 
     "write", version_id, "FS_VERSIONS");
    GeneralPermissions.grantPermission(db, "user", user_id, null, null, 
     "comment", version_id, "FS_VERSIONS");
    GeneralPermissions.grantPermission(db, "user", user_id, null, null, 
     "administer", version_id, "FS_VERSIONS");

FileStorageProcs.orderFiles(db);

%>

</ad:db_transaction>

<%
	
// Code common to inserting a file and a url.
return_url = "/file-storage/" + return_url;
String object_name = file_title;
Integer on_what_id = version_id;
String on_which_table = "FS_VERSIONS";
AdSet set = new AdSet();
set.put("return_url", return_url);
set.put("object_name", object_name);
set.put("on_what_id", on_what_id);
set.put("on_which_table", on_which_table);

return_url = "/gp/administer-permissions?" + Utilities.exportURLVars(set);
response.sendRedirect(return_url);

%>

</ad:page>
