<%@ include file="../acs.jspi" %>

<% /*
    UI for editing the permissions for a specific database row

    Given that a row in the database typically represents an
    object (e.g., a web page, a person), this page accepts
    an object_name parameter which we use to build a meaningful
    page title.
    
    @author michael@arsdigita.com
    @creation-date 2000-02-25
   
*/ %>

<ad:page>
<ad:get_user_info id="user" redirect="false"/>
<ad:page_contract vars="on_what_id:integer on_which_table object_name:optional return_url"/>

<% 
if (Utilities.emptyString(object_name)) { 
    object_name = "Row " + on_what_id + " of Table " + on_which_table;
} 
Integer user_id = null;
if (user != null)
  user_id = user.getUserId();
if (!GeneralPermissions.userHasPermission(user_id, "administer", on_what_id.toString(), on_which_table)) { 
    if (user_id == null) {
	Security.redirectForRegistration(pageContext);
	return;
    } else {
	response.sendError(response.SC_FORBIDDEN, "Access denied!");
	return;
    }
}

// We need to export the current page as return_url for links that
// we create below, so we save the return_url value obtained from
// the form input in a separate variabel and then restore it later.
//
String return_url_save = return_url;
return_url = request.getRequestURI() + "?" + request.getQueryString();
//
// Restore the value of return_url to that obtained from the form
// input.
//
request.setAttribute("return_url", return_url);
request.setAttribute("return_url_save", return_url_save);

// This query fetches all of the standard permission grants (i.e.,
// read, comment, write, and administer) for the specified database
// row. Also, it appends rows for the 'registered_users' and
// 'all_users' scopes if no permissions have been granted to those
// scopes (this is what the two queries UNIONed onto the end do).
//

String registered_users_scope_pretty_name = "Registered Users Only";
String all_users_scope_pretty_name = "All Users";
%>

<ad:db_query id="gp_grid_query">
select * from (select
 decode(pg.scope,
  'user',
   '<a href=\"/shared/community-member?user_id=' || u.user_id ||
   '\">' || u.first_names || ' ' || u.last_name || '</a>',
  'group_role', g.group_name || ' ' || pg.role,
  'group', g.group_name,
  'registered_users', '<%= registered_users_scope_pretty_name %>',
  'all_users', '<%= all_users_scope_pretty_name %>') as permission_owner,
 pg.scope as scope, pg.user_id, pg.group_id, pg.role,
 pg.read_permission_p,
 pg.comment_permission_p,
 pg.write_permission_p,
 pg.administer_permission_p,
 decode(scope, 'user', 1, 'group_role', 2, 'group', 3,
        'registered_users', 4, 'all_users', 5) as display_order
from general_permissions_grid pg, users u, user_groups g
where pg.on_what_id = ?
and pg.on_which_table = lower(?)
and pg.user_id = u.user_id (+)
and pg.group_id = g.group_id (+)
union
select
 '<%= registered_users_scope_pretty_name %>' as permission_owner,
 'registered_users' as scope,
 to_number(null) as user_id,
 to_number(null) as group_id,
 to_char(null) as role,
 'f' as read_permission_p,
 'f' as comment_permission_p,
 'f' as write_permission_p,
 'f' as administer_permission_p,
 4 as display_order
from dual
where not exists (select 1
                  from general_permissions_grid
                  where on_what_id = ?
                  and on_which_table = lower(?)
                  and scope = 'registered_users')
union
select
 '<%= all_users_scope_pretty_name %>' as permission_owner,
 'all_users' as scope,
 to_number(null) as user_id,
 to_number(null) as group_id,
 to_char(null) as role,
 'f' as read_permission_p,
 'f' as comment_permission_p,
 'f' as write_permission_p,
 'f' as administer_permission_p,
 5 as display_order
from dual
where not exists (select 1
                  from general_permissions_grid
                  where on_what_id = ?
                  and on_which_table = lower(?)
                  and scope = 'all_users')
order by display_order asc)
<ad:db_bind><%= on_what_id %></ad:db_bind>
<ad:db_bind><%= on_which_table %></ad:db_bind>
<ad:db_bind><%= on_what_id %></ad:db_bind>
<ad:db_bind><%= on_which_table %></ad:db_bind>
<ad:db_bind><%= on_what_id %></ad:db_bind>
<ad:db_bind><%= on_which_table %></ad:db_bind>
</ad:db_query>

<% 
Selection permissions = new Selection(gp_grid_query, db);

request.setAttribute("permissions", permissions );
request.setAttribute("object_name", object_name);
request.setAttribute("grant_permission_to_user_link", "<a href=\"permission-grant-to-user?" + Utilities.exportURLVars(request, "on_what_id on_which_table object_name return_url") + "\">Grant permission to a user</a>");
request.setAttribute("grant_permission_to_group_link", "<a href=\"permission-grant-to-group?" + Utilities.exportURLVars(request, "on_what_id on_which_table object_name return_url") + "\">Grant permission to a user group</a>");
%>

<ad:use_template />

</ad:page>
