<%@ include file="/acs.jspi" %>

<%
  /**
   * Things not done: mbryzek - 9/6/2000
   *   "_returnas" variables ignored
   *   "limit_to_users_in_group_id" ignored
   *   only user_id_from_search is passed to target (Passthrough is ignored)
   */


  /**  
   * /user-search.tcl
   * 
   * Reusable page for searching the users table.
   * 
   * Takes email or last_name as search arguments.  Can be constrained with the argument 
   * limit_to_users_in_group_id, which accepts a comma-separated list.
   *
   * Generates a list of users who match, and prints the names of the groups searched.
   * 
   * Each user is a link to target, with user_id, email, last_name, and first_names passed as URL vars.
   * 
   * By default these values are passed as user_id_from_search, etc. but the variable names can be set by
   * specifying userid_returnas, etc.
   * 
   * @param email (search string)
   * @param last_name (search strings)
   * @param target (URL to return to)
   * @param userid_returnas For returning the user_id into a variable named something other than "user_id_from_search"
   * @param firstnames_returnas Likewise 
   * @param lastname_returnas
   * @param email_returnas
   * @param passthrough  (form variables to pass along from caller)
   * @param display_title (if you're doing a passthrough, this title can help inform users for what we searched)
   * @param limit_to_users_in_group_id (optional, limits our search to
   *        users in the specified group id. can be a comma separated list.)
   * @param subgroups_p t/f - optional. If specified along with
   *        limit_to_users_in_group_id, searches users who are members of a
   *        subgroup of the specified group_id
   * 
   * @author philg@mit.edu and others
   * @cvs-id $Id: user-search.jsp,v 1.6 2000/10/17 17:12:05 luke Exp $ 
   */
%>

<ad:page>

<ad:page_contract id="pc" vars="target:optional userid_returnas:optional firstnames_returnas:optional lastname_returnas:optional email_returnas:optional email:optional last_name:optional passthrough:optional display_title:optional limit_to_users_in_group_id:optional subgroups_p:optional">
<%
  if (target == null) {
    pc.addComplaint("Target was not specified. This shouldn't have happened, please contact the administrator and let them know what happened.\n");
  }

  // The redundant sets below are necessary to keep the set values from being overwritten by the JSP engine pulling values back from the pageContext after the tag exits.
  if (userid_returnas == null) {
    userid_returnas = "user_id_from_search";
    pageContext.setAttribute("userid_returnas", userid_returnas);
  }
  if (firstnames_returnas == null) {
    firstnames_returnas = "first_names_from_search";
    pageContext.setAttribute("firstnames_returnas", firstnames_returnas);
  }
  if (lastname_returnas == null) {
    lastname_returnas = "last_names_from_search";
    pageContext.setAttribute("lastname_returnas", lastname_returnas);
  }
  if (email_returnas == null) {
    email_returnas = "email_from_search";
    pageContext.setAttribute("email_returnas", email_returnas);
  }
  if (Utilities.emptyString(email) && Utilities.emptyString(last_name)) {
    pc.addComplaint("You must specify either an email address or last name for which to search.\n");
  }
  if (Utilities.emptyString(email) && Utilities.emptyString(last_name)) {
    pc.addComplaint("You can only specify either email or last name, not both.\n");
  }
  if (display_title == null) {
    display_title = "Member Search";
    pageContext.setAttribute("display_title", display_title);
  }

%>    
</ad:page_contract>

<%
    // Input is reasonably good. Set up the search clause to use based
    // on email or last_name

    String query_string = "";
    String search_html = "";
    String search_clause = "";

    if ( !Utilities.emptyString(email) ) {
      query_string += "%" + email.toLowerCase() + "%";
      search_html += "email \"" + email + "\"";
      search_clause += "lower(u.email) like :query_string";
    } else {
      query_string += "%" + last_name.toLowerCase() + "%";
      search_html += "last name \"" + last_name + "\"";
      search_clause += "lower(u.last_name) like :query_string";
    }
%>




<ad:db_query id="group_search">
    select u.user_id,
           u.first_names || ' ' || u.last_name as name,
           u.email
      from users_active u
     where <%= search_clause %>
     order by lower(name)
  <ad:db_bind><%=query_string%></ad:db_bind>
</ad:db_query>


<%

  ContextBar navbar = new ContextBar(pageContext);
  navbar.add(display_title);

  // set up request before including template

  request.setAttribute("search_query", new Selection(group_search, db));
  request.setAttribute("title", display_title);
  request.setAttribute("navbar", navbar);
  request.setAttribute("target", target);
  request.setAttribute("passthrough", passthrough);
  request.setAttribute("search_html", search_html);
  request.setAttribute("group_html", ""); // for later
%>

<ad:use_template/>

</ad:page>

